@@ -36,9 +36,14 @@ namespace vapp {
     
     pLine->P0 = P0;
     pLine->P1 = P1;
-    _ComputeNormal( pLine );
-    
-    m_Lines.push_back( pLine );
+
+    try {
+      _ComputeNormal( pLine );
+      m_Lines.push_back( pLine );
+    } catch(Exception &e) {
+      /* can't compute normal because points are too near */
+      /* just forget this line */
+    }
   }
 
   /*===========================================================================
@@ -47,14 +52,16 @@ namespace vapp {
   std::vector<BSPPoly *> &BSP::compute(void) {
     /* Start by creating the root polygon - i.e. the quad that covers
        the entire region enclosed by the input linedefs */
-    Vector2f GlobalMin(1000000,1000000),GlobalMax(-1000000,-1000000);
-    
+    AABB GlobalBox;
+
     for(int i=0;i<m_Lines.size();i++) {
-      _UpdateAABB(m_Lines[i]->P0,GlobalMin,GlobalMax);
-      _UpdateAABB(m_Lines[i]->P1,GlobalMin,GlobalMax);
+      GlobalBox.addPointToAABB2f(m_Lines[i]->P0);
+      GlobalBox.addPointToAABB2f(m_Lines[i]->P1);
     }
     
     BSPPoly RootPoly;
+    Vector2f GlobalMin = GlobalBox.getBMin();
+    Vector2f GlobalMax = GlobalBox.getBMax();
     RootPoly.Vertices.push_back( new BSPVertex( Vector2f(GlobalMin.x,GlobalMin.y),Vector2f(0,0) ) ); 
     RootPoly.Vertices.push_back( new BSPVertex( Vector2f(GlobalMin.x,GlobalMax.y),Vector2f(0,0) ) ); 
     RootPoly.Vertices.push_back( new BSPVertex( Vector2f(GlobalMax.x,GlobalMax.y),Vector2f(0,0) ) ); 
@@ -64,7 +71,7 @@ namespace vapp {
     
     /* Start the recursion */
     _Recurse(&RootPoly,m_Lines);
-    
+
     /* Return polygon list */
     return m_Polys;
   }
@@ -97,6 +104,12 @@ namespace vapp {
         
         /* Cut */
         BSPPoly *pTempPoly = new BSPPoly;
+
+	if(pPoly->Vertices.empty()) {
+	  Log("** Warning ** : _Recurse(), try to split an empty poly (no BestSplitter)");
+	  Log("** Warning ** : Line causing the trouble is (%.5f %.5f ; %.5f %.5f)",
+	      Lines[i]->P0.x, Lines[i]->P0.y, Lines[i]->P1.x, Lines[i]->P1.y);
+	}
         _SplitPoly(pPoly,NULL,pTempPoly,&Splitter);
         delete pPoly;
         pPoly = pTempPoly;
@@ -105,6 +118,13 @@ namespace vapp {
       if(pPoly->Vertices.size() > 0)
         m_Polys.push_back( pPoly );
       else {
+
+	Log("** Warning ** : Lines causing the trouble are :");
+	for(int i=0;i<Lines.size();i++) {
+	  Log("%.5f %.5f ; %.5f %.5f",
+	      Lines[i]->P0.x, Lines[i]->P0.y, Lines[i]->P1.x, Lines[i]->P1.y);
+	}
+
       /*  printf("Lines: %d   (subspace size: %d)\n",Lines.size(),pSubSpace->Vertices.size());
         for(int i=0;i<Lines.size();i++)
           printf("    (%f,%f)  ->  (%f,%f)\n",Lines[i]->P0.x,Lines[i]->P0.y,Lines[i]->P1.x,Lines[i]->P1.y);*/
@@ -123,6 +143,9 @@ namespace vapp {
       
       /* Also split the convex subspace */
       BSPPoly FrontSpace,BackSpace;
+      if(pSubSpace->Vertices.empty()) {
+	Log("** Warning ** : _Recurse(), try to split an empty poly (BestSplitter is set)");
+      }
       _SplitPoly(pSubSpace,&FrontSpace,&BackSpace,pBestSplitter);
       
       //if(FrontSpace.Vertices.empty()) {
