@@ -27,14 +27,33 @@ Info
 =============================================================================*/
 
 #define BUILD_MAJORVERSION    0
-#define BUILD_VERSION         2
-#define BUILD_MINORVERSION    4
+#define BUILD_VERSION         3
+#define BUILD_MINORVERSION    2
 #define BUILD_EXTRAINFO       ""
 
 /*=============================================================================
 Build configuration
 =============================================================================*/
 
+/* Rendering engine configuration. xmoto currently supports two rendering engines
+ * one is openGl based and the other one is based on SDL_gfx. Here we define
+ * ENABLE_OPENGL , ENABLE_SDLGFX for easy use in 
+ * the code.
+ */
+#ifndef USE_OPENGL
+  #define USE_OPENGL 1
+#endif
+#if USE_OPENGL == 1
+  #define ENABLE_OPENGL
+#endif
+
+#ifndef USE_SDLGFX
+  #define USE_SDLGFX 0
+#endif
+#if USE_SDLGFX == 1
+  #define ENABLE_SDLGFX
+#endif
+
 /* DO_NOT_LOAD_TEXTURES - Do not load texture files. This speeds up the init
    time enormously */
 //#define DO_NOT_LOAD_TEXTURES
@@ -50,10 +69,6 @@ Build configuration
 /* HIDE_JOYSTICK_SUPPORT - Disable joystick config UI */
 #define HIDE_JOYSTICK_SUPPORT
 
-/* ALLOW_GHOST - Allow user to see replay ghosts while playing */
-/* (BUG: disabling this will make it impossible to compile) */
-#define ALLOW_GHOST
-
 /* BREAK_ON_EXCEPTION - (Visual C++ debug-mode only) Will break program if an
    exception occurs */
 //#define BREAK_ON_EXCEPTION
@@ -68,13 +83,6 @@ Misc, don't touch
   #undef EMUL_800x600 /* editor don't want that */
 #endif
 
-#ifndef ALLOW_WEBACCESS
-#define ALLOW_WEBACCESS 1
-#endif
-#if HAVE_LIBCURL == 1 && ALLOW_WEBACCESS == 1
-  #define SUPPORT_WEBACCESS
-#endif
-
 #ifndef ALLOW_ZOOMING
 #define ALLOW_ZOOMING 0
 #endif
@@ -86,5 +94,7 @@ Misc, don't touch
   #define USE_GETTEXT
 #endif
 
+
+
 #endif
 
