@@ -88,36 +88,6 @@ namespace vapp {
     int nX,nY,nWidth,nHeight;
   };
 
-	/*===========================================================================
-	Font character
-  ===========================================================================*/
-  struct UIFontChar {
-    UIFontChar() {
-      bAvail = false;
-      nIncX = nIncY = nWidth = nHeight = nOffsetX = nOffsetY = 0;
-      fX1 = fY1 = fX2 = fY2 = 0.0f;
-    }
-  
-    bool bAvail;
-    int nIncX,nIncY;
-    int nWidth,nHeight;
-    int nOffsetX,nOffsetY;
-    float fX1,fY1,fX2,fY2;
-  };
-
-	/*===========================================================================
-	Font
-  ===========================================================================*/
-  struct UIFont {
-    UIFont() {
-      pTexture = NULL;
-    }
-  
-    std::string Name;
-    UIFontChar Chars[256];
-    Texture *pTexture;
-  };
-
 	/*===========================================================================
 	Text style
   ===========================================================================*/
@@ -149,26 +119,6 @@ namespace vapp {
       static Texture *m_pUIElemTextureA;
   };
 
-	/*===========================================================================
-	UI text drawing static class
-  ===========================================================================*/
-  class UITextDraw {
-    public:
-      static void initTextDrawing(App *pApp);
-      static void uninitTextDrawing(void);
-      static UIFont *getFont(std::string Name);
-      static void printRaw(UIFont *pFont,int x,int y,std::string Text,Color c);
-      static void printRawGrad(UIFont *pFont,int x,int y,std::string Text,Color c1,Color c2,Color c3,Color c4,bool bRotated=false);
-      static int getRefCount(void);
-      static App *getApp(void);
-      static void getTextExt(UIFont *pFont,std::string Text,int *pnMinX,int *pnMinY,int *pnMaxX,int *pnMaxY);
-    
-    private:
-      static std::vector<UIFont *> m_Fonts;
-      static App *m_pApp;
-      static int m_nRefCount;
-  };
-
 	/*===========================================================================
 	UI window
   ===========================================================================*/
@@ -185,7 +135,6 @@ namespace vapp {
     protected:
       void _InitWindow(void) {
         m_pParent=NULL;m_pApp=NULL;
-        m_pCurFont=NULL;
         m_Pos.nX=m_Pos.nY=m_Pos.nWidth=m_Pos.nHeight=0;        
         m_bHide = false;
         m_bDisable = false;
@@ -197,13 +146,15 @@ namespace vapp {
       }
       
     public:
-      UIWindow() {}
+      UIWindow() {m_curFont = NULL;}
       UIWindow(UIWindow *pParent,int x=0,int y=0,std::string Caption="",int nWidth=0,int nHeight=0) {
         initW(pParent,x,y,Caption,nWidth,nHeight);
       }
       virtual ~UIWindow(void) {
         freeW();
       }
+
+      static void setDrawLib(DrawLib* i_drawLib);
     
       /* Methods */
       virtual void paint(void) {}
@@ -215,24 +166,24 @@ namespace vapp {
       virtual void mouseRDown(int x,int y) {}
       virtual void mouseRUp(int x,int y) {}
       virtual void mouseHover(int x,int y) {}
-      virtual bool keyDown(int nKey,int nChar) {return false;}
-      virtual bool keyUp(int nKey) {return false;}
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar) {return false;}
+      virtual bool keyUp(int nKey, SDLMod mod) {return false;}
       virtual bool offerActivation(void) {return false;}
       virtual bool offerMouseEvent(void) {return true;}
       virtual std::string subContextHelp(int x,int y) {return "";}
 
-      /* Painting */            
-      void putText(int x,int y,std::string Text,bool bRotated=false);      
+      /* Painting */
+      void putText(int x,int y, std::string Text,
+		   float i_xper = 0.0, float i_yper = 0.0);
+      void putTextS(int x,int y, std::string Text,
+		    int& o_width, int &o_height,
+		    float i_xper = 0.0, float i_yper = 0.0);
       void putImage(int x,int y,int nWidth,int nHeight,Texture *pImage);
       void putElem(int x,int y,int nWidth,int nHeight,UIElem Elem,bool bDisabled,bool bActive=false);
       void putRect(int x,int y,int nWidth,int nHeight,Color c);
       void setScissor(int x,int y,int nWidth,int nHeight);
       void getScissor(int *px,int *py,int *pnWidth,int *pnHeight);
       
-      /* Utils */
-      void getTextExt(std::string Text,int *pnMinX,int *pnMinY,int *pnMaxX,int *pnMaxY);
-      void getTextSize(std::string Text,int *pnX,int *pnY);
-      
       bool isMouseLDown(void);
       bool isMouseRDown(void);
       
@@ -251,8 +202,8 @@ namespace vapp {
       /* Data interface */
       UIWindow *getPrimaryChild(void) {return m_pPrimaryChild;}
       void setPrimaryChild(UIWindow *p) {m_pPrimaryChild = p;}
-      UIFont *getFont(void) {return m_pCurFont;}
-      void setFont(UIFont *pFont) {m_pCurFont=pFont;}
+      FontManager *getFont() {return m_curFont;}
+      void setFont(FontManager *pFont) {m_curFont = pFont;}
       UIWindow *getParent(void) {return m_pParent;}
       App *getApp(void) {return m_pApp;}
       void setApp(App *pApp) {m_pApp=pApp;}
@@ -262,7 +213,7 @@ namespace vapp {
       UIRect &getPosition(void) {return m_Pos;}
       void setPosition(int x,int y,int nWidth,int nHeight) {m_Pos.nX=x; m_Pos.nY=y; m_Pos.nWidth=nWidth; m_Pos.nHeight=nHeight;}
       std::string getCaption(void) {return m_Caption;}
-      void setCaption(std::string Caption) {m_Caption=Caption;}
+      virtual void setCaption(std::string Caption) {m_Caption=Caption;}
       UITextStyle &getTextStyle(void) {return m_TextStyle;}
       void setTextSolidColor(Color c) {m_TextStyle.c0=m_TextStyle.c1=m_TextStyle.c2=m_TextStyle.c3=c;}
       void setTextGradientColors(Color a,Color b) {m_TextStyle.c0=m_TextStyle.c1=a; m_TextStyle.c2=m_TextStyle.c3=b;}
@@ -291,6 +242,8 @@ namespace vapp {
       bool haveChildW(UIWindow *pWindow);
       void initW(UIWindow *pParent,int x,int y,std::string Caption,int nWidth,int nHeight);
       void freeW(void);
+      
+      static DrawLib* m_drawLib;
     
     private:
       /* Data */
@@ -304,7 +257,7 @@ namespace vapp {
       App *m_pApp;                              /* Application */      
       UIRect m_Pos;                             /* Position */
       std::string m_Caption;                    /* Caption */
-      UIFont *m_pCurFont;                       /* Current font */
+      FontManager* m_curFont;
       UITextStyle m_TextStyle;                  /* Current text style */
       float m_fOpacity;                         /* Opacity (0-100) */
       Texture *m_pUIElemTexture;                /* UI element texture */
@@ -326,19 +279,26 @@ namespace vapp {
         initW(pParent,x,y,Caption,nWidth,nHeight);
         m_nCursorPos = 0;
 	m_hideText = false;
+  m_hasChanged = false;
       }      
     
       /* Methods */
       virtual void paint(void);
-      virtual bool keyDown(int nKey,int nChar);
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar);
       
       virtual bool offerActivation(void) {return true;}
       void hideText(bool bHideText) {m_hideText=bHideText;} 
 
+			void setCaption(std::string Caption);
+
+			void setHasChanged(bool b_value);
+			bool hasChanged();
+
     private:
       /* Data */
       int m_nCursorPos;
       bool m_hideText;    
+			bool m_hasChanged;
   };
 
 	/*===========================================================================
@@ -402,7 +362,7 @@ namespace vapp {
       void setup(std::string Header,int n1,int n2) {m_Header=Header; m_nHighlight1=n1; m_nHighlight2=n2;}
       void addRow1(std::string Col1,std::string Col2) {m_Col1.push_back(Col1); m_Col2.push_back(Col2);}
       void addRow2(std::string Col1,std::string Col2) {m_Col3.push_back(Col1); m_Col4.push_back(Col2);}
-      void setHFont(UIFont *pFont) {m_pHFont=pFont;}
+      void setHFont(FontManager* pFont) {m_hFont=pFont;}
       void clear(void);
       
     private:
@@ -413,7 +373,7 @@ namespace vapp {
       std::vector<std::string> m_Col3;
       std::vector<std::string> m_Col4;
       int m_nHighlight1,m_nHighlight2;
-      UIFont *m_pHFont;
+      FontManager* m_hFont;
   };
 
 	/*===========================================================================
@@ -426,7 +386,7 @@ namespace vapp {
         initW(pParent,x,y,Caption,nWidth,nHeight);
 
         setStyle(UI_FRAMESTYLE_TRANS);
-        m_pTextInputFont = NULL;
+        m_textInputFont = NULL;
         m_bTextInput = false;
         m_nNumButtons = 0;
       }      
@@ -434,7 +394,7 @@ namespace vapp {
     
       /* Virtual methods */
       virtual void paint(void);      
-      virtual bool keyDown(int nKey,int nChar);
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar);
       virtual bool offerActivation(void) {if(m_bTextInput) return true; return false;}
     
       /* Methods */
@@ -443,7 +403,7 @@ namespace vapp {
       void enableTextInput(void) {m_bTextInput=true;}
       std::string getTextInput(void) {return m_TextInput;}
       void setTextInput(std::string s) {m_TextInput=s;}
-      void setTextInputFont(UIFont *pFont) {m_pTextInputFont = pFont;}
+      void setTextInputFont(FontManager* pFont) {m_textInputFont = pFont;}
       
       /* Data interface */
       void addButton(UIButton *p) {m_pButtons[m_nNumButtons++] = p;}
@@ -458,7 +418,7 @@ namespace vapp {
       
       bool m_bTextInput;
       std::string m_TextInput;
-      UIFont *m_pTextInputFont;
+      FontManager* m_textInputFont;
       
       /* Helpers */
       void _ReEnableSiblings(void);
@@ -478,7 +438,7 @@ namespace vapp {
       }      
 
       /* Virtual methods */
-      virtual bool keyDown(int nKey,int nChar);
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar);
   };
 
 	/*===========================================================================
@@ -497,7 +457,7 @@ namespace vapp {
         
         m_pDarkBlobTexture = NULL;
 	Sprite *pSprite;
-	pSprite = getApp()->m_theme.getSprite(SPRITE_TYPE_UI, "DarkBlob");
+	pSprite = getApp()->getTheme()->getSprite(SPRITE_TYPE_UI, "DarkBlob");
 	if(pSprite != NULL) {
 	  m_pDarkBlobTexture = pSprite->getTexture(false,true, FM_NEAREST);
 	}
@@ -562,7 +522,7 @@ namespace vapp {
       virtual void mouseRUp(int x,int y);
       virtual void mouseHover(int x,int y);      
       virtual bool offerActivation(void);
-      virtual bool keyDown(int nKey,int nChar);
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar);
       
       /* Data interface */
       UIButtonState getState(void) {return m_State;}
@@ -576,9 +536,8 @@ namespace vapp {
         m_bChecked=b;
       }
       bool getChecked(void) {return m_bChecked;}
-      
-    private:
-      /* Data */
+
+    protected:
       UIButtonState m_State;
       UIButtonType m_Type;            
       bool m_bHover,m_bClicked,m_bChecked;
@@ -587,12 +546,41 @@ namespace vapp {
       void _UncheckGroup(int nGroup);
   };
 
+class UIButtonDrawn : public UIButton {
+ public:
+  UIButtonDrawn(UIWindow *pParent,
+		const std::string& i_spriteUnpressed,
+		const std::string& i_spritePressed,
+		const std::string& i_spriteHover,
+		int x=0, int y=0,
+		std::string Caption="",
+		int nWidth=0, int nHeight=0);
+  ~UIButtonDrawn();
+
+  virtual void paint();
+  void setBorder(int i_border);
+
+ private:
+  int m_border;
+  Texture* m_texturePressed;
+  Texture* m_textureUnpressed;
+  Texture* m_textureHover;
+};
+
 	/*===========================================================================
 	UI list
   ===========================================================================*/
   struct UIListEntry {
     std::vector<std::string> Text;
     void *pvUser;
+    bool bFiltered;
+
+    bool bUseOwnProperties;
+    Color ownTextColor;
+    Color ownSelectedColor;
+    Color ownUnSelectedColor;
+    int   ownXOffset;
+    int   ownYOffset;
   };
   
   class UIList : public UIWindow {
@@ -612,18 +600,22 @@ namespace vapp {
       virtual void mouseWheelDown(int x,int y);
       virtual void mouseHover(int x,int y);      
       virtual bool offerActivation(void);
-      virtual bool keyDown(int nKey,int nChar);
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar);
 
       virtual std::string subContextHelp(int x,int y);
       
-      UIListEntry *addEntry(std::string Text,void *pvUser=NULL);
-      void clear(void);
+      /* if position != -1, force the entry to this position */
+      UIListEntry *addEntry(std::string Text,void *pvUser=NULL, int i_position = -1);
+      virtual void clear();
       
       /* Data interface */
       std::vector<UIListEntry *> &getEntries();
       std::vector<std::string> &getColumns();
-      int getSelected();
-      void setSelected(int n);
+      virtual int getSelected();
+      int getRowAtPosition(int x, int y);    /* return -1 if none is found */
+      int getColumnAtPosition(int x, int y); /* return -1 if none is found */
+      void setRealSelected(int n);
+      void setVisibleSelected(int n);
       void addColumn(std::string Title,int nWidth,const std::string &Help = "");
       void setEnterButton(UIButton *pButton);
       bool isItemActivated();
@@ -631,6 +623,9 @@ namespace vapp {
       void unhideAllColumns(void);
       void setSort(bool bSort, int(*f)(void *pvUser1, void *pvUser2) = NULL);
       void setNumeroted(bool bNumeroted);
+      std::string getSelectedEntry();
+
+      void randomize();
 
       bool isClicked(void);
       void setClicked(bool b);
@@ -638,6 +633,8 @@ namespace vapp {
       bool isChanged(void);
       void setChanged(bool b);
 
+      void setFilter(std::string i_filter);
+
     private:
       /* Data */
       bool m_bChanged;
@@ -650,7 +647,8 @@ namespace vapp {
       std::vector<std::string> m_ColumnHelpStrings;
       std::vector<int> m_ColumnWidths;
       unsigned int m_nColumnHideFlags;
-      int m_nSelected;
+      int m_nRealSelected;
+			int m_nVisibleSelected;
       bool m_bItemActivated;
       UIButton *m_pEnterButton; /* if not null this is the "default" button of the list, i.e. the one 
                                    that gets pressed if the list gets an enter */
@@ -662,6 +660,9 @@ namespace vapp {
       bool m_bClicked;
       bool  m_bScrolling;
 
+			std::string m_filter;
+			int m_filteredItems;
+
       /* draw */
       int HeaderHeight();
       int HeaderSubBorderHeight();
@@ -683,7 +684,8 @@ namespace vapp {
       int ScrollBarScrollerStartY();
       void setScrollBarScrollerStartY(float y);
       float ScrollNbVisibleItems();
-      
+      bool isScrollBarRequired();
+
       int m_headerHeight;
       int m_headerSubBorderHeight;
       int m_rowHeight;
@@ -736,6 +738,9 @@ namespace vapp {
       bool isChanged(void) {return m_bChanged;}
       void setChanged(bool b) {m_bChanged=b;}
 
+      void selectChildren(unsigned int i);
+      void selectChildrenById(const std::string& i_id);
+
     private:
       /* Data */
       bool m_bChanged;
@@ -783,8 +788,8 @@ namespace vapp {
       virtual void mouseHover(int x,int y);
       virtual void mouseWheelUp(int x,int y);
       virtual void mouseWheelDown(int x,int y);
-      virtual bool keyDown(int nKey,int nChar);      
-      virtual bool keyUp(int nKey);  
+      virtual bool keyDown(int nKey, SDLMod mod,int nChar);      
+      virtual bool keyUp(int nKey, SDLMod mod);  
       
       void deactivate(UIWindow *pWindow);
 
@@ -802,7 +807,7 @@ namespace vapp {
       std::string m_CurrentContextHelp;
             
       /* Helpers */      
-      bool _RootKeyEvent(UIWindow *pWindow,UIRootKeyEvent Event,int nKey,int nChar);
+      bool _RootKeyEvent(UIWindow *pWindow,UIRootKeyEvent Event,int nKey, SDLMod mod,int nChar);
       bool _RootMouseEvent(UIWindow *pWindow,UIRootMouseEvent Event,int x,int y);
       void _RootPaint(int x,int y,UIWindow *pWindow,UIRect *pRect);
       void _ClipRect(UIRect *pRect,UIRect *pClipWith);
