@@ -24,11 +24,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include "VXml.h"
 #include "GUI.h"
+#include "helpers/utf8.h"
 
 namespace vapp {
 
   UIFrame::UIFrame() {
     m_bMinimizable = false;
+    m_bHover = false;
   }
 
   UIFrame::UIFrame(UIWindow *pParent,
@@ -38,6 +40,7 @@ namespace vapp {
     initW(pParent,x,y,Caption,nWidth,nHeight);
     
     m_bMinimizable = false;
+    m_bHover = false;
     m_fMinMaxTime = 0.0f;
     
     m_Style=UI_FRAMESTYLE_TRANS;
@@ -48,22 +51,22 @@ namespace vapp {
     m_pMenuBR = NULL;
     
     Sprite *pSprite;
-    pSprite = getApp()->m_theme.getSprite(SPRITE_TYPE_UI, "MenuTL");
+    pSprite = getApp()->getTheme()->getSprite(SPRITE_TYPE_UI, "MenuTL");
     if(pSprite != NULL) {
       m_pMenuTL = pSprite->getTexture(false,true, FM_NEAREST);
     }
 
-    pSprite = getApp()->m_theme.getSprite(SPRITE_TYPE_UI, "MenuTR");
+    pSprite = getApp()->getTheme()->getSprite(SPRITE_TYPE_UI, "MenuTR");
     if(pSprite != NULL) {
       m_pMenuTR = pSprite->getTexture(false,true, FM_NEAREST);
     }
 
-    pSprite = getApp()->m_theme.getSprite(SPRITE_TYPE_UI, "MenuBL");
+    pSprite = getApp()->getTheme()->getSprite(SPRITE_TYPE_UI, "MenuBL");
     if(pSprite != NULL) {
       m_pMenuBL = pSprite->getTexture(false,true, FM_NEAREST);
     }
 
-    pSprite = getApp()->m_theme.getSprite(SPRITE_TYPE_UI, "MenuBR");
+    pSprite = getApp()->getTheme()->getSprite(SPRITE_TYPE_UI, "MenuBR");
     if(pSprite != NULL) {
       m_pMenuBR = pSprite->getTexture(false,true, FM_NEAREST);
     }
@@ -85,6 +88,12 @@ namespace vapp {
       }
         break;
       case UI_FRAMESTYLE_TRANS:
+      if(isUglyMode()) {
+	putRect(0, 0, 2, getPosition().nHeight, MAKE_COLOR(188,186,67,255));
+	putRect(0, 0, getPosition().nWidth, 2, MAKE_COLOR(188,186,67,255));
+	putRect(getPosition().nWidth-2, 0, 2, getPosition().nHeight, MAKE_COLOR(188,186,67,255));
+	putRect(0, getPosition().nHeight-2, getPosition().nWidth, 2, MAKE_COLOR(188,186,67,255));
+      } else {
         putElem(0,0,-1,-1,UI_ELEM_FRAME_TL,false);
         putElem(getPosition().nWidth-8,0,-1,-1,UI_ELEM_FRAME_TR,false);
         putElem(getPosition().nWidth-8,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BR,false);
@@ -94,19 +103,27 @@ namespace vapp {
         putElem(0,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_ML,false);
         putElem(getPosition().nWidth-8,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_MR,false);
         putRect(8,8,getPosition().nWidth-16,getPosition().nHeight-16,MAKE_COLOR(0,0,0,127));
-        break;
+      }
+      break;
       case UI_FRAMESTYLE_LEFTTAG: {
           int nTagWidth = 20;      
       
-          putElem(0,0,-1,-1,UI_ELEM_FRAME_TL,false);
-          putElem(getPosition().nWidth-8,0,-1,-1,UI_ELEM_FRAME_TR,false);
-          putElem(getPosition().nWidth-8,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BR,false);
-          putElem(0,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BL,false);
-          putElem(8,0,getPosition().nWidth-16,-1,UI_ELEM_FRAME_TM,false);
-          putElem(8,getPosition().nHeight-8,getPosition().nWidth-16,-1,UI_ELEM_FRAME_BM,false);
-          putElem(0,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_ML,false);
-          putElem(getPosition().nWidth-8,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_MR,false);
-          putRect(8,8,getPosition().nWidth-16,getPosition().nHeight-16,MAKE_COLOR(0,0,0,127));
+	  if(isUglyMode()) {
+	    putRect(0, 0, 2, getPosition().nHeight, MAKE_COLOR(188,186,67,255));
+	    putRect(0, 0, getPosition().nWidth, 2, MAKE_COLOR(188,186,67,255));
+	    putRect(getPosition().nWidth-2, 0, 2, getPosition().nHeight, MAKE_COLOR(188,186,67,255));
+	    putRect(0, getPosition().nHeight-2, getPosition().nWidth, 2, MAKE_COLOR(188,186,67,255));
+	  } else {
+	    putElem(0,0,-1,-1,UI_ELEM_FRAME_TL,false);
+	    putElem(getPosition().nWidth-8,0,-1,-1,UI_ELEM_FRAME_TR,false);
+	    putElem(getPosition().nWidth-8,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BR,false);
+	    putElem(0,getPosition().nHeight-8,-1,-1,UI_ELEM_FRAME_BL,false);
+	    putElem(8,0,getPosition().nWidth-16,-1,UI_ELEM_FRAME_TM,false);
+	    putElem(8,getPosition().nHeight-8,getPosition().nWidth-16,-1,UI_ELEM_FRAME_BM,false);
+	    putElem(0,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_ML,false);
+	    putElem(getPosition().nWidth-8,8,-1,getPosition().nHeight-16,UI_ELEM_FRAME_MR,false);
+	  }
+	  putRect(8,8,getPosition().nWidth-16,getPosition().nHeight-16,MAKE_COLOR(0,0,0,127));
 
           putRect(2,2,getPosition().nWidth-4,getPosition().nHeight-4,MAKE_COLOR(0,0,70,150));
           
@@ -115,7 +132,7 @@ namespace vapp {
           else
             setTextSolidColor(MAKE_COLOR(188,186,67,255));
 
-          putText(4,14,getCaption(),true);
+          putText(4,14, utf8::txt2vertical(getCaption()));
         }
         break;
     }
