@@ -25,163 +25,93 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VCommon.h"
 #include "VApp.h"
 #include "xmscene/Level.h"
-#include "MotoGame.h"
+#include "xmscene/Bike.h"
+#include "xmscene/Scene.h"
 #include "VTexture.h"
 #include "Renderer.h"
 #include "Replay.h"
-#include "UserConfig.h"
 #include "GameText.h"
-#include "PlayerData.h"
 #include "Sound.h"
 #include "Input.h"
 #include "WWW.h"
 #include "WWWAppInterface.h"
 #include "GUIXMoto.h"
-#include "Stats.h"
 #include "Credits.h"
 #include "LevelsManager.h"
+#include "XMotoLoadLevelsInterface.h"
+#include "db/xmDatabaseUpdateInterface.h"
+#include "SysMessage.h"
 
 #define PRESTART_ANIMATION_TIME 2.0
+#define INPLAY_ANIMATION_TIME 1.0
+#define INPLAY_ANIMATION_SPEED 10
 #define PRESTART_ANIMATION_LEVEL_MSG_DURATION 1.0
 #define PRESTART_ANIMATION_MARGIN_SIZE 5
 #define PRESTART_ANIMATION_CURVE 3.0
 
-namespace vapp {
+/* logf(PRESTART_ANIMATION_CURVE + 1.0) = 1.386294361*/
+#define LOGF_PRE_ANIM_TIME_ADDED_ONE 1.386294361
 
-  /*===========================================================================
-  Overall game states
-  ===========================================================================*/
-  enum GameState {
-    GS_UNASSIGNED = 0,
-    GS_MENU,                  /* In the game menu */
-    GS_PREPLAYING,            /* Just before the game start */
-    GS_PLAYING,               /* Playing the game */
-    GS_PAUSE,                 /* Paused from GS_PLAYING */
-    GS_DEADJUST,              /* just dead */
-    GS_DEADMENU,              /* Head-banging too much */
-    GS_EDIT_PROFILES,         /* In profile editor */
-    GS_FINISHED,              /* Finished a level */
-    GS_REPLAYING,             /* Replaying */
-    GS_LEVEL_INFO_VIEWER,     /* In level info viewer */
-    GS_LEVELPACK_VIEWER,      /* In level pack viewer */
-    GS_CREDITSMODE,           /* Credits/replay */
-#if defined(SUPPORT_WEBACCESS)
-    GS_EDIT_WEBCONFIG         /* Editing internet configuration */
-#endif
-  };
+class xmDatabase;
 
-  /*===========================================================================
-  Menu background graphical settings
-  ===========================================================================*/
-  enum MenuBackgroundGraphics {
-    MENU_GFX_OFF,
-    MENU_GFX_LOW,
-    MENU_GFX_HIGH
-  };
-  
+ namespace vapp {
 
-#if defined(ALLOW_GHOST) 
-  enum GhostSearchStrategy {
-    GHOST_STRATEGY_MYBEST,
-    GHOST_STRATEGY_THEBEST,
-    GHOST_STRATEGY_BESTOFROOM
-  };
-#endif
+   /*===========================================================================
+   Overall game states
+   ===========================================================================*/
+   enum GameState {
+     GS_UNASSIGNED = 0,
+     GS_MENU,                  /* In the game menu */
+     GS_PREPLAYING,            /* Just before the game start */
+     GS_PLAYING,               /* Playing the game */
+     GS_PAUSE,                 /* Paused from GS_PLAYING */
+     GS_DEADJUST,              /* just dead */
+     GS_DEADMENU,              /* Head-banging too much */
+     GS_EDIT_PROFILES,         /* In profile editor */
+     GS_FINISHED,              /* Finished a level */
+     GS_REPLAYING,             /* Replaying */
+     GS_LEVEL_INFO_VIEWER,     /* In level info viewer */
+     GS_LEVELPACK_VIEWER,      /* In level pack viewer */
+     GS_CREDITSMODE,           /* Credits/replay */
+     GS_EDIT_WEBCONFIG         /* Editing internet configuration */
+   };
 
-  /*===========================================================================
-  Game application
-  ===========================================================================*/
-  #if defined(SUPPORT_WEBACCESS)
-  class GameApp : public App, public WWWAppInterface {
-  #else
-  class GameApp : public App {
-  #endif
-    public:
-      virtual ~GameApp() {}
-      GameApp() {m_pCredits = NULL;
-                 m_bShowMiniMap=true;
-                 m_bDebugMode=false;
-                 m_bListLevels=false;
-                 m_bListReplays=false;
-                 m_bTimeDemo=false;
-                 m_bShowFrameRate=false;
-                 m_bEnableLevelCache=true;
-                 m_bEnableMenuMusic=false;
-                 m_bEnableInitZoom=false;
-                 m_bCleanCache=false;
-                 m_bEnableDeathAnim=true;
-                 m_pQuitMsgBox=NULL;
-                 m_pNotifyMsgBox=NULL;
-                 m_pInfoMsgBox=NULL;
-#if defined(SUPPORT_WEBACCESS)
-                 m_pNewLevelsAvailIcon=NULL;
-                 m_pWebConfEditor=NULL;
-                 m_pWebConfMsgBox=NULL;
-#endif
-                 m_pNewProfileMsgBox=NULL;
-                 m_pDeleteProfileMsgBox=NULL;
-                 m_pDeleteReplayMsgBox=NULL;
-                 m_pSaveReplayMsgBox=NULL;
-                 m_pReplaysWindow=NULL;
-                 m_pLevelPacksWindow=NULL;
-                 m_pStatsReport=NULL;
-                 m_pLevelPackViewer=NULL;  
-                 m_pActiveLevelPack=NULL;
-                 m_pGameInfoWindow=NULL;
-                 m_fFrameTime = 0;
-                 m_fFPS_Rate = 0;
-                 m_b50FpsMode = false;
-                 m_bUglyMode = false;
-                 m_bTestThemeMode = false;
-                 m_pReplay = NULL;
-                 m_pMenuMusic = NULL;
-		 m_updateAutomaticallyLevels = false;
-#if defined(ALLOW_GHOST)
-     m_pGhostReplay = NULL;
-     m_lastGhostReplay = "";
-     GhostSearchStrategies[0] = GHOST_STRATEGY_MYBEST;
-     GhostSearchStrategies[1] = GHOST_STRATEGY_THEBEST;
-     GhostSearchStrategies[2] = GHOST_STRATEGY_BESTOFROOM;
-     m_bEnableGhost = true;
-     m_bShowGhostTimeDiff = true;
-     m_GhostSearchStrategy = GHOST_STRATEGY_MYBEST;
-                 m_bEnableGhostInfo = false;
-                 m_bGhostMotionBlur = true;
-#endif
-     m_bAutosaveHighscoreReplays = true;
-                 m_bRecordReplays = true;
-                 m_bShowCursor = true;
-                 m_bEnableEngineSound = true;
-                 m_bCompressReplays = true;
-                 m_bBenchmark = false;
-                 m_bEnableContextHelp = true;     
-
-#if defined(SUPPORT_WEBACCESS)
-                 m_bShowWebHighscoreInGame = false;
-                 m_bEnableWebHighscores = true;
-     m_pWebHighscores = NULL;
-     m_pWebLevels = NULL;
-     m_pWebRooms = NULL;
-     m_fDownloadTaskProgressLast = 0;
-     m_bWebHighscoresUpdatedThisSession = false;
-     m_bWebLevelsToDownload = false;
-
-     m_bEnableCheckNewLevelsAtStartup  = true;
-     m_bEnableCheckHighscoresAtStartup = true;
-#endif
-     m_fLastSqueekTime = 0.0f;
+   /*===========================================================================
+   Menu background graphical settings
+   ===========================================================================*/
+   enum MenuBackgroundGraphics {
+     MENU_GFX_OFF,
+     MENU_GFX_LOW,
+     MENU_GFX_HIGH
+   };
 
 
-     m_Renderer.setTheme(&m_theme);
-     m_MotoGame.setRenderer(&m_Renderer);
+   enum GhostSearchStrategy {
+     GHOST_STRATEGY_MYBEST,
+     GHOST_STRATEGY_THEBEST,
+     GHOST_STRATEGY_BESTOFROOM
+   };
 
-     m_bPrePlayAnim = true;
-     m_bShowEngineCounter = true;
+   class XMMotoGameHooks : public MotoGameHooks {
+   public:
+     XMMotoGameHooks();
+     ~XMMotoGameHooks();
+     void setGameApps(GameApp *i_GameApp, MotoGame *i_MotoGame);
+     void OnTakeEntity();
 
-     m_currentPlayingList = NULL;
-      }
-                 
-#if defined(SUPPORT_WEBACCESS)                 
+   private:
+     GameApp  *m_GameApp;
+     MotoGame *m_MotoGame;
+   };
+
+   /*===========================================================================
+   Game application
+   ===========================================================================*/
+   class GameApp : public App, public XMotoLoadLevelsInterface, public XmDatabaseUpdateInterface, public WWWAppInterface {
+     public:
+     GameApp();
+     virtual ~GameApp();
+           
         /* WWWAppInterface implementation */ 
         virtual void setTaskProgress(float fPercent);
 
@@ -190,24 +120,23 @@ namespace vapp {
         
         virtual bool shouldLevelBeUpdated(const std::string &LevelID);
 
-        virtual std::string levelPathForUpdate(const std::string &p_LevelId);
-        virtual std::string levelMD5Sum(const std::string &LevelID);
-	virtual bool doesLevelExist(const std::string &p_LevelId);
-#endif
-      
+	/* load level */
+	void loadLevelHook(std::string i_level, int i_percentage);
+	void updatingDatabase(std::string i_message);
+
       /* Virtual methods */
       virtual void drawFrame(void);
       virtual void userInit(void);
-      virtual void userPreInit(void);
       virtual void userShutdown(void);
-      virtual void keyDown(int nKey,int nChar);
-      virtual void keyUp(int nKey);
+      virtual void keyDown(int nKey, SDLMod mod,int nChar);
+      virtual void keyUp(int nKey, SDLMod mod);
       virtual void mouseDown(int nButton);
       virtual void mouseDoubleClick(int nButton);
       virtual void mouseUp(int nButton);
       virtual void parseUserArgs(std::vector<std::string> &UserArgs);
       virtual void helpUserArgs(void);
       virtual void selectDisplayMode(int *pnWidth,int *pnHeight,int *pnBPP,bool *pbWindowed);
+      virtual std::string selectDrawLibMode();
       virtual std::string getConfigThemeName(ThemeChoicer *p_themeChoicer);
 
       /* Methods */
@@ -216,16 +145,36 @@ namespace vapp {
       void setPrePlayAnim(bool pEnabled);
       void reloadTheme();
 
+      void PlaySpecificLevel(std::string i_level);
+      void PlaySpecificLevelFile(std::string i_levelFile);
+      void PlaySpecificReplay(std::string i_replay);
+
       /* Data interface */
       bool isUglyMode() {return m_bUglyMode;}
       bool isTestThemeMode(void) {return m_bTestThemeMode;}
-    
-    private: 
-      EngineSoundSimulator m_EngineSound;
-    
+      bool isUglyOverMode() {return m_bUglyOverMode;}
+
+      void setAutoZoom(bool bValue);
+      bool AutoZoom();
+      void setAutoZoomStep(int n);
+      int AutoZoomStep();
+
+      void TeleportationCheatTo(int i_player, Vector2f i_position);
+
+      bool creditsModeActive();
+
+      void initCameras(int nbPlayer);
+
+   protected:
+      void createDefaultConfig();
+
+    private:   
       /* Data */
-      ReplayList m_ReplayList;                  /* Replay list */
+      bool m_bMultiStopWhenOneFinishes;         /* in multiplayer, stop the game when one finishes the level */
       bool m_bEnableInitZoom;                   /* true: Perform initial level scroll/zoom */
+      bool m_autoZoom;                          /* true : the key is pressed so that it zooms out to see the level */
+      int m_autoZoomStep;
+      bool m_bAutoZoomInitialized;
       bool m_bEnableDeathAnim;                  /* true: Bike falls apart at when dead */
       bool m_bEnableMenuMusic;                  /* true: Play menu music */      
       bool m_bEnableContextHelp;                /* true: Show context help */
@@ -235,56 +184,60 @@ namespace vapp {
       bool m_bListReplays;                      /* true: list replays */
       bool m_bTimeDemo;                         /* true: (valid for replaying) - performance benchmark */
       bool m_bDebugMode;                        /* true: show debug info */
+      bool m_sqlTrace;                          /* true: show sql traces */
       bool m_bUglyMode;                         /* true: fast 'n ugly graphics */
       bool m_bCleanCache;                       /* true: clean the level cache at startup */
-      bool m_bShowEngineCounter;
+      bool m_bDisplayInfosReplay;               /* true: just display infos of a replay */
+      std::string m_InfosReplay;                /* name of the replay to display information */
 
       bool m_bTestThemeMode;
+      bool m_bUglyOverMode;
       bool m_bEnableEngineSound;                /* true: engine sound is enabled */
       bool m_bCompressReplays;                  /* true: compress replays with zlib */
-      bool m_bEnableLevelCache;                 /* true: cache levels for faster loading */
       bool m_bAutosaveHighscoreReplays;
       std::string m_PlaySpecificLevel;          /* If set, we only want to 
                                                    play this level */
       std::string m_PlaySpecificReplay;         /* If set, we only want to
-                                                   play this replay */                                                   
+                                                   play this replay */
+      std::string m_PlaySpecificLevelFile;      /* If set, we only want to
+                                                   play this level */
+
+      ReplayBiker* m_replayBiker; /* link to the replay biker in REPLAYING state */
+      bool m_stopToUpdateReplay;
+      bool m_allowReplayInterpolation;
+
       std::string m_ForceProfile;               /* Force this player profile */    
       std::string m_GraphDebugInfoFile;
       InputHandler m_InputHandler;              /* The glorious input handler */
       GameState m_State;                        /* Current state */      
       GameState m_StateAfterPlaying;            /* State that should be used later */
       MotoGame m_MotoGame;                      /* Game object */      
+      XMMotoGameHooks m_MotoGameHooks;
       GameRenderer m_Renderer;                  /* Renderer */
       int m_nFrame;                             /* Frame # */
-      PlayerProfile *m_pPlayer;                 /* The player's profile */
-      
-      Mix_Music *m_pMenuMusic;
-      
+      std::string m_profile;
+       
       double m_fLastFrameTime;                  /* When the last frama was initiated */
-      double m_fLastPerfStateTime;
-      float m_fLastStateSerializationTime;    
+      double m_fLastPerfStateTime;   
       double m_fLastPhysTime;                  /* When the last physic was computed */
       double m_fStartTime;                      
-      double m_fLastSqueekTime;
       
+      std::string m_playingMusic; /* name of the music played to not restart it if the same must be played on an action */
+
       bool m_b50FpsMode;
+      bool m_reloadingLevelsUser;
       
-      Replay *m_pReplay;
-#if defined(ALLOW_GHOST) 
-      Replay *m_pGhostReplay;
-      String m_lastGhostReplay;
-      int m_nGhostFrame;
-      enum GhostSearchStrategy GhostSearchStrategies[3];
+      Replay *m_pJustPlayReplay;
+
+      std::vector<GhostSearchStrategy> GhostSearchStrategies;
+
       bool m_bEnableGhost;
       GhostSearchStrategy m_GhostSearchStrategy;
       bool m_bShowGhostTimeDiff;
       bool m_bGhostMotionBlur;                  /* true: apply fancy motion blur to ghosts */
       bool m_bEnableGhostInfo;
-#endif
-      std::string m_ReplayPlayerName;
-      
+
       /* WWW */
-#if defined(SUPPORT_WEBACCESS)
       bool m_bShowWebHighscoreInGame;           /* true: Show world highscore inside the game */
       WebRoom *m_pWebHighscores;
       WebRooms *m_pWebRooms;
@@ -294,13 +247,14 @@ namespace vapp {
       std::string m_DownloadingMessage;
       float m_fDownloadTaskProgressLast;
 
+      std::string m_WebHighscoresIdRoom;
+      std::string m_WebHighscoresURL;
+
       bool m_bEnableWebHighscores;              /* true: Read world highscores from website */
       bool m_bWebHighscoresUpdatedThisSession;  /* true: Updated this session */
       bool m_bWebLevelsToDownload;              /* true: there are new levels to download */
       bool m_bEnableCheckNewLevelsAtStartup;
       bool m_bEnableCheckHighscoresAtStartup;
-
-#endif
       
       bool m_bCreditsModeActive;
       
@@ -322,9 +276,19 @@ namespace vapp {
       float m_zoomU;
       float static_time;
 
-      Stats m_GameStats;
+      float fAnimPlayStartZoom;
+      float fAnimPlayStartCameraX; 
+      float fAnimPlayStartCameraY;
+      float fAnimPlayFinalZoom;
+      float fAnimPlayFinalCameraX1;
+      float fAnimPlayFinalCameraY1;
+      float fAnimPlayFinalCameraX2;
+      float fAnimPlayFinalCameraY2;
+
       UIWindow *m_pStatsReport;
       
+      UIQuickStartButton *m_pQuickStart;
+
       /* Sound effects */
       SoundSample *m_pEndOfLevelSFX;
       SoundSample *m_pStrawberryPickupSFX;
@@ -340,10 +304,10 @@ namespace vapp {
       /* Main menu background / title */
       Texture *m_pTitleBL,*m_pTitleBR,*m_pTitleTL,*m_pTitleTR;
       Texture *m_pCursor;
-#if defined(SUPPORT_WEBACCESS)
-      Texture *m_pNewLevelsAvailIcon;
-#endif
       bool m_bShowCursor;
+
+      /* */
+      UIButtonDrawn* m_pNewLevelsAvailable;
       
       /* Main menu buttons and stuff */
       int m_nNumMainMenuButtons;
@@ -358,11 +322,9 @@ namespace vapp {
       /* LEVEL lists */
       UILevelList *m_currentPlayingList;
       UILevelList *m_pAllLevelsList;
-#if defined(SUPPORT_WEBACCESS)
       UILevelList *m_pPlayNewLevelsList;
-#endif
+      UILevelList *m_quickStartList;
 
-#if defined(SUPPORT_WEBACCESS)
       UIWindow *m_pLevelInfoFrame;
       UIButton *m_pLevelInfoViewReplayButton;      
       UIStatic *m_pBestPlayerText;
@@ -372,7 +334,6 @@ namespace vapp {
       UIStatic *m_pPackBestPlayerText;
 
       String    m_pLevelToShowOnViewHighscore;
-#endif
 
       /* if true, don't ask for updating levels */
       bool m_updateAutomaticallyLevels;
@@ -382,7 +343,7 @@ namespace vapp {
       int m_nPauseShade;
       UIButton *m_pPauseMenuButtons[10];
       int m_nNumPauseMenuButtons;
-      
+
       /* In-game JUSTDEAD menu fun */
       UIFrame *m_pJustDeadMenu;
       int m_nJustDeadShade;
@@ -404,11 +365,9 @@ namespace vapp {
       UIMsgBox *m_pNewProfileMsgBox;    
       UIMsgBox *m_pDeleteProfileMsgBox;
       
-#if defined(SUPPORT_WEBACCESS)
       /* Internet connection configurator */
       UIFrame *m_pWebConfEditor;      
       UIMsgBox *m_pWebConfMsgBox;    
-#endif
 
       /* Level pack viewer fun */
       UIFrame *m_pLevelPackViewer;  
@@ -420,10 +379,6 @@ namespace vapp {
       
       /* Replay saving UI fun */
       UIMsgBox *m_pSaveReplayMsgBox;    
-      
-      /* Config & profiles */
-      UserConfig m_Config;
-      PlayerData m_Profiles;
 
       LevelsManager m_levelsManager;
       
@@ -432,38 +387,37 @@ namespace vapp {
       bool m_bShowMiniMap;
       bool m_bRecordReplays;
       float m_fReplayFrameRate;
-      float m_fCurrentReplayFrameRate;
       
       /* Credits */
       Credits *m_pCredits;         
       
+      SysMessage m_sysMsg;
+
       /* Main loop statics */
       double m_fFrameTime;
       float m_fFPS_Rate;
 
+      bool m_bLockMotoGame;
+      xmDatabase *m_db;
+
       /* Helpers */
-#if defined(SUPPORT_WEBACCESS) 
       void _UpdateWorldRecord(const std::string &LevelID);
-#endif
       void _HandleMainMenu(void);  
       void _HandlePauseMenu(void);
       void _HandleJustDeadMenu(void);
       void _HandleFinishMenu(void);
-#if defined(SUPPORT_WEBACCESS) 
       void _HandleWebConfEditor(void);
-#endif
       void _HandleProfileEditor(void);
       void _HandleLevelInfoViewer(void);
       void _HandleLevelPackViewer(void);
+      void _CreateLevelListsSql(UILevelList *pAllLevels, const std::string& i_sql);
       void _CreateLevelLists(UILevelList *pAllLevels, std::string i_packageName);
       void _CreateReplaysList(UIList *pList);
       void _CreateThemesList(UIList *pList);
-#if defined(SUPPORT_WEBACCESS) 
       void _CreateRoomsList(UIList *pList);
-#endif
       void _CreateProfileList(void);
-      void _CreateDefaultConfig(void);
-      void _CreateLevelPackLevelList(void);
+      void _CreateLevelPackLevelList();
+      void _UpdateLevelPackLevelList(const std::string& v_levelPack);
       void _UpdateActionKeyList(void);
       void _UpdateLevelPackList(void);
       void _UpdateLevelInfoViewerBestTimes(const std::string &LevelID);     
@@ -487,67 +441,75 @@ namespace vapp {
       void _UpdateThemesLists(void);
 
       void _UpdateLevelsLists();
+      void _UpdateCurrentPackList(const std::string& i_id_level, float i_playerHighscore);
 
-#if defined(SUPPORT_WEBACCESS) 
       void _UpdateRoomsLists(void);
-#endif
       void _GameScreenshot(void);
       void _SaveReplay(const std::string &Name);
     
-      void _UpdateLoadingScreen(float fDone,Texture *pScreen,const std::string &NextTask);      
+      void _UpdateLoadingScreen(float fDone, const std::string &NextTask);
       
       void _SimpleMessage(const std::string &Msg,UIRect *pRect=NULL,bool bNoSwap=false);
       
       int _IsKeyInUse(const std::string &Key);
       
-      std::string _DetermineNextLevel(Level *pLevelSrc);
-      bool _IsThereANextLevel(Level *pLevelSrc);
+      std::string _DetermineNextLevel(const std::string& i_id_level);
+      bool _IsThereANextLevel(const std::string& i_id_level);
       
-      bool _IsReplayScripted(Replay *p_pReplay);
       void _RestartLevel(bool i_reloadLevel = false);
   
-#if defined(SUPPORT_WEBACCESS)
       void _InitWebConf(void);
       void _CheckForExtraLevels(void);
       void _UpdateWebHighscores(bool bSilent);
-      void _UpdateWebLevels(bool bSilent);
+      void _UpdateWebLevels(bool bSilent, bool bEnableWeb = true);
       void _UpdateWebThemes(bool bSilent);
-      void _UpdateWebTheme(ThemeChoice* pThemeChoice, bool bNotify);
+      void _UpdateWebTheme(const std::string& i_id_theme, bool bNotify);
       void _UpgradeWebHighscores();
       void _UpdateWebRooms(bool bSilent);
       void _UpgradeWebRooms(bool bUpdateMenus);
       void _DownloadExtraLevels(void);
-      void _UploadHighscore(std::string p_replayname);
+      void _UploadHighscore(std::string p_replayname, bool b_notify = true);
+      void _UploadAllHighscores();
       void _ConfigureProxy(void);
-#endif
 
-#if defined(SUPPORT_WEBACCESS)
       void setLevelInfoFrameBestPlayer(String pLevelID,
 				       UIWindow *i_pLevelInfoFrame,
 				       UIButton *i_pLevelInfoViewReplayButton,
 				       UIStatic *i_pBestPlayerText
 				       );
       void viewHighscoreOf();
-#endif
+      void enableWWW(bool bValue);
+
+      UILevelList* buildQuickStartList();
 
-#if defined(ALLOW_GHOST) 
       std::string _getGhostReplayPath_bestOfThePlayer(std::string p_levelId, float &p_time);
       std::string _getGhostReplayPath_bestOfLocal(std::string p_levelId, float &p_time);
-#if defined(SUPPORT_WEBACCESS)
       std::string _getGhostReplayPath_bestOfTheRoom(std::string p_levelId, float &p_time);
-#endif
       std::string _getGhostReplayPath(std::string p_levelId,
               GhostSearchStrategy p_strategy);
-#endif
 
       void switchUglyMode(bool bUgly);
       void switchTestThemeMode(bool mode);
+      void switchUglyOverMode(bool mode);
 
       void statePrestart_init();
       void statePrestart_step();
       void prestartAnimation_init();
       void prestartAnimation_step();
       
+      void zoomAnimation1_init();
+      bool zoomAnimation1_step();
+      void zoomAnimation1_abort();
+      
+      void zoomAnimation2_init();
+      bool zoomAnimation2_step();
+      void zoomAnimation2_init_unzoom();
+      bool zoomAnimation2_unstep();
+      void zoomAnimation2_abort();
+
+      void lockMotoGame(bool bLock);
+      bool isLockedMotoGame() const;
+
       std::string splitText(const std::string &str, int p_breakLineLength);
       
       /* Main loop utility functions */
@@ -565,7 +527,20 @@ namespace vapp {
       void _PostUpdateJustDead(void);
       void _PostUpdateFinished(void);
 
-      int getNumberOfFinishedLevelsOfPack(LevelsPack *i_pack);
+      void autoZoom();
+
+      int getNumberOfPlayersToPlay();
+      TColor getColorFromPlayerNumber(int i_player);
+      TColor getUglyColorFromPlayerNumber(int i_player);
+
+      UIWindow* stats_generateReport(const std::string &PlayerName, vapp::UIWindow *pParent,
+				     int x, int y, int nWidth, int nHeight, FontManager* pFont);
+
+      void initReplaysFromDir();
+      void addReplay(const std::string& i_file);
+
+      static UIFrame* makeOptionsWindow(DrawLib* i_drawLib, UIWindow* io_parent, UserConfig* i_Config, std::vector<GhostSearchStrategy>& i_ghostStrategies);
+      static UIFrame* makeHelpWindow(DrawLib* i_drawLib, UIWindow* io_parent, UserConfig* i_Config);
   };
 
 }
