@@ -30,12 +30,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "Sound.h"
 #include "PhysSettings.h"
 #include "Input.h"
+#include "xmscene/Bike.h"
+#include "xmscene/BikeGhost.h"
+#include "xmscene/BikePlayer.h"
 
-#if defined(SUPPORT_WEBACCESS)
-  #include <curl/curl.h>
-#endif
-
-#include "helpers/HighPrecisionTimer.h"
+#include <curl/curl.h>
 
 /* Set the following #defines to simulate slow systems */
 #define SIMULATE_SLOW_RENDERING     0 /* extra ms to add to rendering */
@@ -50,38 +49,29 @@ namespace vapp {
     /* This function is called by the framework as fast as possible */
     bool bIsPaused = false;
     bool bDrawFPS = false;
-    
-    HighPrecisionTimer::reset();
 
     /* Prepare frame rendering / game update */
     _PrepareFrame();    
-    HighPrecisionTimer::checkTime("_PrepareFrame");
 
     /* Check some GUI controls */
     _PreUpdateGUI();     
-    HighPrecisionTimer::checkTime("_PreUpdateGUI");
      
     /* Update FPS stuff */
     _UpdateFPSCounter();
-    HighPrecisionTimer::checkTime("_UpdateFPSCounter");
             
     /* What state? */
     switch(m_State) {
       case GS_MENU:
         /* Menu specifics */
         _PreUpdateMenu();
-        HighPrecisionTimer::checkTime("_PreUpdateMenu");
         /* Note the lack of break; the following are done for GS_MENU too */
 
       case GS_LEVEL_INFO_VIEWER:
       case GS_LEVELPACK_VIEWER:
-#if defined(SUPPORT_WEBACCESS)
       case GS_EDIT_WEBCONFIG:
-#endif
       case GS_EDIT_PROFILES:
         /* Following is done for all the above states */
         _DrawMainGUI();
-        HighPrecisionTimer::checkTime("_DrawMainGUI");
         
         /* Show frame rate? */
         if(m_bShowFrameRate) bDrawFPS = true;
@@ -103,63 +93,73 @@ namespace vapp {
            the game, not the main menu) */
         try {
           int nPhysSteps = 0;
-          bool bValidGameState = true;
         
           /* When did the frame start? */
           double fStartFrameTime = getTime();                    
+	  int numberCam = m_MotoGame.getNumberCameras();
 
           if(m_State == GS_PREPLAYING) {
             /* If "preplaying" / "initial-zoom" is enabled, this is where it's done */
+	    if(numberCam > 1){
+	      m_MotoGame.setCurrentCamera(numberCam);
+	    }
             statePrestart_step();
-            HighPrecisionTimer::checkTime("statePrestart_step");
-          } 
-          else if(m_State == GS_PLAYING || ((m_State == GS_DEADMENU || m_State == GS_DEADJUST) && m_bEnableDeathAnim)) {
+
+	    if(!m_bTimeDemo) {
+	      /* limit framerate while PREPLAY (100 fps)*/
+	      double timeElapsed = getTime() - fStartFrameTime;
+	      if(timeElapsed < 0.01)
+		setFrameDelay(10 - (int)(timeElapsed*1000.0));
+	    }
+          } else if(m_State == GS_PLAYING ||
+		    ((m_State == GS_DEADMENU || m_State == GS_DEADJUST) && m_bEnableDeathAnim)
+		    ) {
             /* When actually playing or when dead and the bike is falling apart, 
                a physics update is required */
-            nPhysSteps = _UpdateGamePlaying();            
-            HighPrecisionTimer::checkTime("_UpdateGamePlaying");
+	    if(isLockedMotoGame()) {
+	      nPhysSteps = 0;
+	    } else {
+	      nPhysSteps = _UpdateGamePlaying();            
+	    }
           }
           else if(m_State == GS_REPLAYING) {
             /* When playing back a replay, no physics update is requried - instead
                the game state is streamed out of a binary .rpl file */
-            nPhysSteps = _UpdateGameReplaying();
-            HighPrecisionTimer::checkTime("_UpdateGameReplaying");
-            bValidGameState = nPhysSteps > 0;
-            //printf("%d ",nPhysSteps);
+	    nPhysSteps = _UpdateGamePlaying();
           }
   
+	  if(m_State == GS_PLAYING) {
+	    if(numberCam > 1){
+	      m_MotoGame.setCurrentCamera(numberCam);
+	    }
+	    autoZoom();
+	  }
+
           /* Render */
-          if(!isNoGraphics() && bValidGameState) {
-            m_Renderer.render(bIsPaused);
-            HighPrecisionTimer::checkTime("m_Renderer.render");
-      
-            if(m_bShowMiniMap && !m_bCreditsModeActive) {
-              if(m_MotoGame.getBikeState()->Dir == DD_LEFT &&
-                 (m_bShowEngineCounter == false || m_State == GS_REPLAYING)) {
-                m_Renderer.renderMiniMap(getDispWidth()-150,getDispHeight()-100,150,100);
-                HighPrecisionTimer::checkTime("m_Renderer.renderMiniMap");
-              } 
-              else {
-                m_Renderer.renderMiniMap(0,getDispHeight()-100,150,100);
-                HighPrecisionTimer::checkTime("m_Renderer.renderMiniMap");
-              }
-            }             
-      
-            if(m_bShowEngineCounter && m_bUglyMode == false && m_State != GS_REPLAYING) {
-              m_Renderer.renderEngineCounter(getDispWidth()-128,getDispHeight()-128,128,128,
-                                             m_MotoGame.getBikeEngineSpeed());
-              HighPrecisionTimer::checkTime("m_Renderer.renderEngineCounter");
-            } 
-          }
+          if(!getDrawLib()->isNoGraphics()) {
+	    try {
+	      if((m_autoZoom || (m_bPrePlayAnim && m_bUglyMode == false)) && numberCam > 1){
+		m_Renderer.render(bIsPaused);
+	      }else{
+		for(int i=0; i<numberCam; i++){
+		  m_MotoGame.setCurrentCamera(i);
+		  m_Renderer.render(bIsPaused);
+		}
+	      }
+	    } catch(Exception &e) {
+	      m_MotoGame.endLevel();
+	      setState(m_StateAfterPlaying);
+	      notifyMsg(splitText(e.getMsg(), 50));
+	    }
+	    getDrawLib()->getMenuCamera()->setCamera2d();
+	  }
 #if SIMULATE_SLOW_RENDERING
           SDL_Delay(SIMULATE_SLOW_RENDERING);
-          HighPrecisionTimer::checkTime("(dummy)");
 #endif
   
           /* When actually playing, check if something happened (like dying or finishing) */
           if(m_State == GS_PLAYING) {        
             _PostUpdatePlaying();
-            HighPrecisionTimer::checkTime("_PostUpdatePlaying");
           }
         
           /* When did frame rendering end? */
@@ -169,22 +169,23 @@ namespace vapp {
              desired frame rate */
           int nADelay = 0;    
           
-          if(m_State == GS_REPLAYING) {
-            /* When replaying... */
-            //printf("{ %f } ",m_fCurrentReplayFrameRate);
-            if(nPhysSteps <= 1)                        
-              nADelay = ((1.0f/m_fCurrentReplayFrameRate - (fEndFrameTime-fStartFrameTime)) * 1000.0f) * 0.5;
-            //printf(" { %f }  %d\n",fEndFrameTime-fStartFrameTime,nADelay);            
-          }
-          else if ((m_State == GS_FINISHED) || (m_State == GS_DEADMENU || m_State == GS_DEADJUST) || (m_State == GS_PAUSE)) {
+	  if ((m_State == GS_FINISHED) || (m_State == GS_DEADMENU || m_State == GS_DEADJUST) || (m_State == GS_PAUSE)) {
             setFrameDelay(10);
-          }
-          else {
+          } else if(m_State == GS_REPLAYING && m_stopToUpdateReplay == true) {
+	    nADelay = 10;
+	  } else {
             /* become idle only if we hadn't to skip any frame, recently, and more globaly (80% of fps) */
             if((nPhysSteps <= 1) && (m_fFPS_Rate > (0.8f / PHYS_STEP_SIZE)))
               nADelay = ((m_fLastPhysTime + PHYS_STEP_SIZE) - fEndFrameTime) * 1000.0f;
+
+	    if(m_autoZoom){
+	      /* limit framerate while zooming (100 fps)*/
+	      double timeElapsed = getTime() - fStartFrameTime;
+	      if(timeElapsed < 0.01)
+		nADelay = 10 - (int)(timeElapsed*1000.0);
+	    }
           }
-                  
+
           if(nADelay > 0) {
             if(!m_bTimeDemo) {
               setFrameDelay(nADelay);
@@ -201,53 +202,37 @@ namespace vapp {
               nFrameCnt = 0;
               m_fLastPerfStateTime = m_fFrameTime;
             }
-            drawText(Vector2f(0,100),cBuf,MAKE_COLOR(0,0,0,255),-1);        
+	    FontManager* v_fm = getDrawLib()->getFontSmall();
+	    FontGlyph* v_fg = v_fm->getGlyph(cBuf);
+	    v_fm->printString(v_fg,
+			      0, 100,
+			      MAKE_COLOR(0,0,0,255));
             nFrameCnt++;
           }
 
-          HighPrecisionTimer::checkTime("(misc)");
 
           if(m_State == GS_PAUSE) {
             /* Okay, nifty thing. Paused! */
             _PostUpdatePause();
-            HighPrecisionTimer::checkTime("_PostUpdatePause");
           }        
           else if(m_State == GS_DEADJUST) {
             /* Hmm, you're dead and you know it. */
             _PostUpdateJustDead();
-            HighPrecisionTimer::checkTime("_PostUpdateJustDead");
           }
           else if(m_State == GS_DEADMENU) {
             /* Hmm, you're dead and you know it. */
             _PostUpdateMenuDead();
-            HighPrecisionTimer::checkTime("_PostUpdateMenuDead");
           }
           else if(m_State == GS_FINISHED) {
             /* Hmm, you've won and you know it. */
             _PostUpdateFinished();
-            HighPrecisionTimer::checkTime("_PostUpdateFinished");
           }        
 
           /* Level and player name to draw? */
           if(!m_bCreditsModeActive &&
             (m_State == GS_DEADMENU || m_State == GS_DEADJUST || m_State == GS_PAUSE || m_State == GS_FINISHED || m_State == GS_REPLAYING) &&
             m_MotoGame.getLevelSrc() != NULL) {
-            UIFont *v_font = m_Renderer.getMediumFont();
-            std::string v_infos;
-            
-            v_infos = m_MotoGame.getLevelSrc()->Name();
-
-            if(m_State == GS_REPLAYING && m_pReplay != NULL) {
-              v_infos += " (" + std::string(GAMETEXT_BY) + " " + m_pReplay->getPlayerName() + ")";
-            }
-
-            if(v_font != NULL) {
-              UITextDraw::printRaw(v_font,0,getDispHeight()-4,
-                v_infos,
-                MAKE_COLOR(255,255,255,255));
-            }
-            HighPrecisionTimer::checkTime("(level and player name)");
-          }
+	  }
          
           /* Context menu? */
           if(m_State == GS_PREPLAYING || m_State == GS_PLAYING || m_State == GS_REPLAYING || !m_bEnableContextHelp)
@@ -256,23 +241,27 @@ namespace vapp {
             m_Renderer.getGUI()->enableContextMenuDrawing(true);
           
           /* Draw GUI */
-          m_Renderer.getGUI()->paint();        
-          HighPrecisionTimer::checkTime("m_Renderer.getGUI()->paint");
+	  // only if it's not the autozoom camera
+	  if(m_MotoGame.getCurrentCamera() != m_MotoGame.getNumberCameras()){
+	    m_Renderer.getGUI()->paint();        
+	  }
         
           /* Credits? */
           if(m_State == GS_REPLAYING && m_bCreditsModeActive && m_pCredits!=NULL) {
             m_pCredits->render(m_MotoGame.getTime());
-            HighPrecisionTimer::checkTime("m_pCredits->render");
           }
 
           /* Show frame rate */
           if(m_bShowFrameRate) bDrawFPS = true;
 
           break;
-        } 
+        }
         catch(Exception &e) {
-    setState(m_StateAfterPlaying);
-          notifyMsg(splitText(e.getMsg(), 50));
+	  Log("** Warning ** : drawFrame failed !");
+	  // it doesn't work
+	  //m_MotoGame.endLevel();
+	  //setState(m_StateAfterPlaying);
+          //notifyMsg(splitText(e.getMsg(), 50));
         }
       }
     }
@@ -280,46 +269,31 @@ namespace vapp {
     /* Draw a little FPS counter */
     if(bDrawFPS) {
       char cTemp[256];        
-      sprintf(cTemp,"%f",m_fFPS_Rate);
-      drawText(Vector2f(130,0),cTemp);
+      sprintf(cTemp,"%.1f",m_fFPS_Rate);
+
+      FontManager* v_fm = getDrawLib()->getFontSmall();
+      FontGlyph* v_fg = v_fm->getGlyph(cTemp);
+      v_fm->printString(v_fg,
+			130, 0,
+			MAKE_COLOR(255,255,255,255));
     }    
-    
-    /* Profiling? */
-#if defined(PROFILE_MAIN_LOOP)
-    int nCurY = 60;
-    float fTotal = 0;
-    for(int i=0;i<HighPrecisionTimer::numTimeChecks();i++) {
-      HighPrecisionTimer::TimeCheck *pTc = HighPrecisionTimer::getTimeCheck(i);
-      if(pTc != NULL) {
-        fTotal += pTc->fTime;
-      }
-    }
-    for(int i=0;i<HighPrecisionTimer::numTimeChecks();i++) {
-      char cTemp[256];
-      HighPrecisionTimer::TimeCheck *pTc = HighPrecisionTimer::getTimeCheck(i);
-      if(pTc != NULL) {
-        sprintf(cTemp,"%-20s : %.0f",pTc->cWhere,pTc->fTime);
-        drawText(Vector2f(30,nCurY),cTemp);
-        sprintf(cTemp,"%.0f%%",(pTc->fTime * 100) / fTotal);
-        drawText(Vector2f(350,nCurY),cTemp);
-        nCurY += 16;
-      }
-    }
-#endif   
-    
+       
     /* Draw mouse cursor */
     if(m_bShowCursor)
       _DrawMouseCursor();
+
+    /* system message */
+    m_sysMsg.render();
   }
 
   /*===========================================================================
   Main loop utility functions
   ===========================================================================*/
   void GameApp::_DrawMouseCursor(void) {
-    if(!isNoGraphics() && m_pCursor != NULL && m_bUglyMode == false) {
+    if(!getDrawLib()->isNoGraphics() && m_pCursor != NULL && m_bUglyMode == false) {
       int nMX,nMY;
       getMousePos(&nMX,&nMY);      
-      drawImage(Vector2f(nMX-2,nMY-2),Vector2f(nMX+30,nMY+30),m_pCursor);
+      getDrawLib()->drawImage(Vector2f(nMX-2,nMY-2),Vector2f(nMX+30,nMY+30),m_pCursor);
     }
   }
   
@@ -330,21 +304,17 @@ namespace vapp {
     setFrameDelay(0);
     
     /* Update sound system and input */
-    if(!isNoGraphics()) {        
-      m_EngineSound.update(getRealTime());
-      m_EngineSound.setRPM(0); /* per default don't have engine sound */
+    if(!getDrawLib()->isNoGraphics()) {        
       Sound::update();
-      
-      m_InputHandler.updateInput(m_MotoGame.getBikeController());
+
+      m_InputHandler.updateInput(m_MotoGame.Players());
     }    
     
     /* Whether or not we should have a mouse cursor? */
     switch(m_State) {
       case GS_MENU:
       case GS_EDIT_PROFILES:
-#if defined(SUPPORT_WEBACCESS)
       case GS_EDIT_WEBCONFIG:
-#endif
       case GS_LEVEL_INFO_VIEWER:
       case GS_PAUSE:
       case GS_DEADMENU:
@@ -359,12 +329,6 @@ namespace vapp {
       case GS_REPLAYING:
         m_bShowCursor = false;
         //SDL_ShowCursor(SDL_DISABLE);      
-
-        /* Music playing? Not anymore that is! */
-        if(m_pMenuMusic != NULL && Mix_PlayingMusic()) {
-          Mix_FadeOutMusic(500);
-        }
-        
         break;
       case GS_DEADJUST:
       break;
@@ -377,7 +341,11 @@ namespace vapp {
       UIMsgBoxButton Button = m_pQuitMsgBox->getClicked();
       if(Button == UI_MSGBOX_YES) {
         if(m_State == GS_PAUSE) {
-          m_GameStats.abortedLevel(m_pPlayer->PlayerName,m_MotoGame.getLevelSrc()->Id(),m_MotoGame.getLevelSrc()->Name(),m_MotoGame.getTime()); 
+	  if(m_MotoGame.Players().size() == 1) {
+	    m_db->stats_abortedLevel(m_profile,
+				     m_MotoGame.getLevelSrc()->Id(),
+				     m_MotoGame.getTime()); 
+	  }
         }
       
         quit();      
@@ -397,7 +365,6 @@ namespace vapp {
         m_pNotifyMsgBox = NULL;
       }
     }
-#if defined(SUPPORT_WEBACCESS)
     /* And the download-levels box? */
     else if(m_pInfoMsgBox != NULL) {
       UIMsgBoxButton Button = m_pInfoMsgBox->getClicked();
@@ -410,18 +377,13 @@ namespace vapp {
         
         /* current theme should be updated when there are new levels */
         _UpdateWebThemes(true);
-        ThemeChoice* v_ThemeChoice;
-        v_ThemeChoice = m_themeChoicer->getChoiceByName(m_Config.getString("Theme"));
-        if(v_ThemeChoice != NULL) {
-          _UpdateWebTheme(v_ThemeChoice, false);      
-        }
+	_UpdateWebTheme(m_Config.getString("Theme"), false);      
       }
       else if(Button == UI_MSGBOX_NO) {
         delete m_pInfoMsgBox;
         m_pInfoMsgBox = NULL;
       }
     }
-#endif       
   }
   
   void GameApp::_UpdateFPSCounter(void) {
@@ -455,41 +417,6 @@ namespace vapp {
         /* Don't do this again, please */
         m_Config.setBool("NotifyAtInit",false); 
       }
-    }        
-
-    if(m_bEnableMenuMusic && Sound::isEnabled()) {
-      /* No music playing? If so, playback time! */
-      if(m_pMenuMusic == NULL) {
-        /* No music available, try loading */
-        std::string MenuMusicPath = FS::getDataDir() + std::string("/xmoto.ogg");
-        const char *pc = MenuMusicPath.c_str();
-        #if defined(_MSC_VER) /* this works around a bug in SDL_mixer 1.2.7 on Windows */
-          SDL_RWops *rwfp;              
-          rwfp = SDL_RWFromFile(pc, "rb");                          
-          m_pMenuMusic = Mix_LoadMUS_RW(rwfp);
-        #else
-          m_pMenuMusic = Mix_LoadMUS(pc);
-        #endif
-        /* (Don't even complain the slightest if music isn't found...) */          
-      }
-      
-      if(m_pMenuMusic != NULL) {
-        if(!Mix_PlayingMusic()) {
-          /* Not playing, start it */
-          if(Mix_PlayMusic(m_pMenuMusic,-1) < 0) {
-            Log("** Warning ** : Mix_PlayMusic() failed, disabling music");
-            m_bEnableMenuMusic = false;                
-          }
-        }
-      }
-    }
-    else {
-      /* Hmm, no music please. If it's playing, stop it */
-      if(m_pMenuMusic != NULL) {
-        if(Mix_PlayingMusic()) {
-          Mix_FadeOutMusic(500);              
-        }
-      }
     }
   }
 
@@ -505,10 +432,8 @@ namespace vapp {
       _HandleMainMenu();
     else if(m_State == GS_EDIT_PROFILES)
       _HandleProfileEditor();
-#if defined(SUPPORT_WEBACCESS) 
     else if(m_State == GS_EDIT_WEBCONFIG)
       _HandleWebConfEditor();
-#endif
     else if(m_State == GS_LEVEL_INFO_VIEWER)
       _HandleLevelInfoViewer();
     else if(m_State == GS_LEVELPACK_VIEWER)
@@ -524,6 +449,14 @@ namespace vapp {
   }
   
   int GameApp::_UpdateGamePlaying(void) {
+
+    if(m_State == GS_REPLAYING) {
+      if(m_stopToUpdateReplay) {
+	return 0;
+      }
+    }
+    
+
     /* Increase frame counter */
     m_nFrame++;
 
@@ -537,11 +470,8 @@ namespace vapp {
     /* Update game until we've catched up with the real time */
     int nPhysSteps = 0;
     do {
-      if(m_State == GS_PLAYING) {
-        m_MotoGame.updateLevel( PHYS_STEP_SIZE,NULL,m_pReplay );
-      } 
-      else {
-        m_MotoGame.updateLevel( PHYS_STEP_SIZE,NULL, NULL );
+      if(m_State == GS_PLAYING || m_State == GS_DEADJUST || m_State == GS_REPLAYING) {
+        m_MotoGame.updateLevel(PHYS_STEP_SIZE, m_pJustPlayReplay);
       }
       m_fLastPhysTime += PHYS_STEP_SIZE;
       nPhysSteps++;
@@ -550,76 +480,39 @@ namespace vapp {
       SDL_Delay(SIMULATE_SLOW_PHYSICS);
 #endif
 
-      if(m_Config.getBool("LimitFramerate")) {
-        SDL_Delay(1);
-      }     
-
       /* don't do this infinitely, maximum miss 10 frames, then give up */
     } while ((m_fLastPhysTime + PHYS_STEP_SIZE <= getTime()) && (nPhysSteps < 10));
+
+		for(int i=0; i<m_MotoGame.getNumberCameras(); i++){
+			m_MotoGame.setCurrentCamera(i);
+			m_MotoGame.getCamera()->setSpeedMultiplier(nPhysSteps);
+		}
   
-    m_Renderer.setSpeedMultiplier(nPhysSteps);
-  
-    if(!m_bTimeDemo) {
+    if(m_bTimeDemo == false) {
       /* Never pass this point while being ahead of time, busy wait until it's time */
       if(nPhysSteps <= 1) {  
         while (m_fLastPhysTime > getTime());
       }
     }
 
-    if(m_bEnableEngineSound) {
-      /* Update engine RPM */
-      m_EngineSound.setRPM( m_MotoGame.getBikeEngineRPM() ); 
-    }
-  
-    /* Squeeking? */
-    if(m_MotoGame.isSqueeking()) {
-      if(getTime() - m_fLastSqueekTime > 0.1f) {
-        /* (this is crappy, not enabled) */
-        //Sound::playSampleByName("Sounds/Squeek.ogg",m_MotoGame.howMuchSqueek());
-        m_fLastSqueekTime = getTime();
-      }
-    }
-  
-#if defined(ALLOW_GHOST)
-    /* Read replay state */
-    if(m_pGhostReplay != NULL) {
-      static SerializedBikeState GhostBikeState;
-      static SerializedBikeState previousGhostBikeState;
-      
-      m_pGhostReplay->peekState(previousGhostBikeState);
-      if(previousGhostBikeState.fGameTime < m_MotoGame.getTime() && m_pGhostReplay->endOfFile() == false) {
-        do {
-          m_pGhostReplay->loadState(GhostBikeState);
-        } while(GhostBikeState.fGameTime < m_MotoGame.getTime() && m_pGhostReplay->endOfFile() == false);
+    if(m_State == GS_REPLAYING) {
+      if(m_replayBiker->isDead() || m_replayBiker->isFinished()) {
+	m_stopToUpdateReplay = true;
 
-        if(m_nGhostFrame%2 || m_nGhostFrame==1) {
-          /* NON-INTERPOLATED FRAME */
-          m_MotoGame.UpdateGhostFromReplay(&GhostBikeState);
-        } 
-        else {
-          /* INTERPOLATED FRAME */
-          SerializedBikeState ibs;
-          m_MotoGame.interpolateGameState(&previousGhostBikeState,&GhostBikeState,&ibs,0.5f);
-          m_MotoGame.UpdateGhostFromReplay(&ibs);
-        }
-        m_nGhostFrame++;
-      }
-    }
-#endif    
+    	if(m_bBenchmark) {
+    	  double fBenchmarkTime = getRealTime() - m_fStartTime;
+    	  printf("\n");
+    	  printf(" * %d frames rendered in %.0f seconds\n",m_nFrame,fBenchmarkTime);
+    	  printf(" * Average framerate: %.1f fps\n",((double)m_nFrame) / fBenchmarkTime);
+    	  quit();
+    	}
 
-    if(m_State == GS_PLAYING) {
-      /* We'd like to serialize the game state 25 times per second for the replay */
-      if(getRealTime() - m_fLastStateSerializationTime >= 1.0f/m_fReplayFrameRate) {
-        m_fLastStateSerializationTime = getRealTime();
-        
-        /* Get it */
-        SerializedBikeState BikeState;
-        m_MotoGame.getSerializedBikeState(&BikeState);
-        if(m_pReplay != NULL)
-          m_pReplay->storeState(BikeState);              
+    	if(m_replayBiker->isFinished()) {
+    	  m_MotoGame.setTime(m_replayBiker->finishTime());
+	}
       }
     }
-    
+
     return nPhysSteps;
   }  
 
@@ -627,117 +520,48 @@ namespace vapp {
     int nPhysSteps = 1;
     m_nFrame++;
     static float fGTime = 0, fRTime = 0;
-
-    /* Following (automatic replay speed adjustment) is only enabled 
-       when showing credits.  */     
-    if(m_pReplay != NULL && m_bCreditsModeActive) {
-      /* Compare game time with real time */
-      float fGCurTime = m_MotoGame.getTime();
-      float fRCurTime = getRealTime();
-      float fGDiff = fGCurTime - fGTime;
-      float fRDiff = fRCurTime - fRTime;
-      fGTime = fGCurTime;
-      fRTime = fRCurTime;
-      
-      if(fabs(fRDiff - fGDiff) > 0.01f) {            
-        if(fGDiff + 0.01f < fRDiff && !m_pReplay->isPaused() ||
-          fGDiff - 0.01f > fRDiff && !m_pReplay->isPaused()) {
-          float fControl = (fRDiff - fGDiff)*0.6;
-          if(fControl < -0.1f) fControl = -0.1f;
-          if(fControl > 0.1f) fControl = 0.1f;
-          m_pReplay->setSpeed(m_pReplay->getSpeed() + fControl);
-        }
-        
-        if(m_pReplay->getSpeed() < 0.2f) m_pReplay->setSpeed(0.2f);
-        if(m_pReplay->getSpeed() > 8.0f) m_pReplay->setSpeed(8.0);
-      }
-    }
-    
-    /* Read replay state */
-    static SerializedBikeState BikeState;          
-    if(m_pReplay != NULL) {       
-      /* Even frame number: Read the next state */
-      if(m_nFrame%2 || m_nFrame==1) {       
-        /* REAL NON-INTERPOLATED FRAME */
-        if(m_pReplay->endOfFile() == false) {
-          m_pReplay->loadState(BikeState);
-        
-          /* Update game */
-          m_MotoGame.updateLevel( 1/m_pReplay->getFrameRate(),&BikeState,m_pReplay ); 
-          //m_MotoGame.updateLevel( PHYS_STEP_SIZE,&BikeState,m_pReplay ); 
-        
-          if(m_bEnableEngineSound) {
-            /* Update engine RPM */
-            m_EngineSound.setRPM( m_MotoGame.getBikeEngineRPM() ); 
-          }
-        }
-        else {
-          if(m_pReplay->didFinish()) {
-            /* Make sure that it's the same finish time */
-            m_MotoGame.setTime(m_pReplay->getFinishTime());
-          }
-        }
-      }
-      else {                          
-        /* INTERPOLATED FRAME */
-        SerializedBikeState NextBikeState,ibs;
-        
-        if(m_pReplay->endOfFile() == false) {
-          m_pReplay->peekState(NextBikeState);
-          /* Nice. Interpolate the states! */
-          m_MotoGame.interpolateGameState(&BikeState,&NextBikeState,&ibs,0.5f);
-
-          /* Update game */
-          //m_MotoGame.updateLevel( PHYS_STEP_SIZE,&ibs,m_pReplay );                 
-          //printf("[%f]\n",m_pReplay->getFrameRate());
-          m_MotoGame.updateLevel( 1/m_pReplay->getFrameRate(),&ibs,m_pReplay );                 
-        }
-      }
-    
-      /* Benchmarking and finished? If so, print the report and quit */
-      if(m_pReplay->endOfFile() && m_bBenchmark) {
-        double fBenchmarkTime = getRealTime() - m_fStartTime;
-        printf("\n");
-        printf(" * %d frames rendered in %.0f seconds\n",m_nFrame,fBenchmarkTime);
-        printf(" * Average framerate: %.1f fps\n",((double)m_nFrame) / fBenchmarkTime);
-        quit();
-      }
-    }
-    else return 0; /* something went wrong */
-  
-    //if(m_pReplay->getSpeed() < 0) {
-    //m_Renderer.clearAllParticles();
-    //m_Renderer.skipBackTime(100000);            
-    //}
-  
-    m_Renderer.setSpeedMultiplier(nPhysSteps);    
+		for(int i=0; i<m_MotoGame.getNumberCameras(); i++){
+			m_MotoGame.setCurrentCamera(i);
+			m_MotoGame.getCamera()->setSpeedMultiplier(nPhysSteps);
+		}
 
     return nPhysSteps;
   }
 
   void GameApp::_PostUpdatePlaying(void) {
-    /* News? */
-    if(m_MotoGame.isDead()) {
-      /* You're dead maan! */
-      setState(GS_DEADJUST);
-    }
-    else if(m_MotoGame.isFinished()) {
-      /* You're done maaaan! :D */
-      std::string TimeStamp = getTimeStamp();
-      m_Profiles.addFinishTime(m_pPlayer->PlayerName,"",
-                              m_MotoGame.getLevelSrc()->Id(),m_MotoGame.getFinishTime(),TimeStamp); 
-      _MakeBestTimesWindow(m_pBestTimes,m_pPlayer->PlayerName,m_MotoGame.getLevelSrc()->Id(),
-                          m_MotoGame.getFinishTime(),TimeStamp);
+    bool v_all_dead = true;
+    bool v_one_still_play = false;
+    bool v_one_finished = false;
+ 
+    for(unsigned int i=0; i<m_MotoGame.Players().size(); i++) {
+      if(m_MotoGame.Players()[i]->isDead() == false) {
+	v_all_dead = false;
+      }
+      if(m_MotoGame.Players()[i]->isFinished()) {
+	v_one_finished = true;
+      }
 
-      _UpdateLevelsLists();     
-      setState(GS_FINISHED);
+      if(m_MotoGame.Players()[i]->isFinished() == false && m_MotoGame.Players()[i]->isDead() == false) {
+	v_one_still_play = true;
+      }
+    }
+    
+    /* News? */
+    if(v_one_still_play == false || m_bMultiStopWhenOneFinishes) { // let people continuing when one finished or not
+      if(v_one_finished) {
+	/* You're done maaaan! :D */
+	setState(GS_FINISHED);
+      } else if(v_all_dead) {
+	/* You're dead maan! */
+	setState(GS_DEADJUST);
+      }
     }
   }
 
   void GameApp::_PostUpdatePause(void) {
     if(!m_bUglyMode) {
       if(m_nPauseShade < 150) m_nPauseShade+=8;
-      drawBox(Vector2f(0,0),Vector2f(getDispWidth(),getDispHeight()),0,MAKE_COLOR(0,0,0,m_nPauseShade));                                        
+      getDrawLib()->drawBox(Vector2f(0,0),Vector2f(getDrawLib()->getDispWidth(),getDrawLib()->getDispHeight()),0,MAKE_COLOR(0,0,0,m_nPauseShade));                                        
     }
 
     /* Update mouse stuff */
@@ -750,14 +574,14 @@ namespace vapp {
   void GameApp::_PostUpdateJustDead(void) {
     if(!m_bUglyMode) {
       if(m_nJustDeadShade < 150) m_nJustDeadShade+=8;
-      drawBox(Vector2f(0,0),Vector2f(getDispWidth(),getDispHeight()),0,MAKE_COLOR(0,0,0,m_nJustDeadShade));     
+      getDrawLib()->drawBox(Vector2f(0,0),Vector2f(getDrawLib()->getDispWidth(),getDrawLib()->getDispHeight()),0,MAKE_COLOR(0,0,0,m_nJustDeadShade));     
     }
   }
 
   void GameApp::_PostUpdateMenuDead(void) {
     if(!m_bUglyMode) {
       if(m_nJustDeadShade < 150) m_nJustDeadShade+=8;
-      drawBox(Vector2f(0,0),Vector2f(getDispWidth(),getDispHeight()),0,MAKE_COLOR(0,0,0,m_nJustDeadShade));     
+      getDrawLib()->drawBox(Vector2f(0,0),Vector2f(getDrawLib()->getDispWidth(),getDrawLib()->getDispHeight()),0,MAKE_COLOR(0,0,0,m_nJustDeadShade));     
     }
     
     /* Update mouse stuff */
@@ -772,7 +596,7 @@ namespace vapp {
   void GameApp::_PostUpdateFinished(void) {
     if(!m_bUglyMode) {
       if(m_nFinishShade < 150) m_nFinishShade+=8;
-      drawBox(Vector2f(0,0),Vector2f(getDispWidth(),getDispHeight()),0,MAKE_COLOR(0,0,0,m_nFinishShade));     
+      getDrawLib()->drawBox(Vector2f(0,0),Vector2f(getDrawLib()->getDispWidth(),getDrawLib()->getDispHeight()),0,MAKE_COLOR(0,0,0,m_nFinishShade));     
     }
 
     /* Update mouse stuff */
