@@ -20,6 +20,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "GameEvents.h"
+#include "helpers/SwapEndian.h"
 
 namespace vapp {
 
@@ -35,24 +36,51 @@ namespace vapp {
     Buffer << (int) (this->getType());
   }
 
-  MotoGameEvent* MotoGameEvent::getUnserialized(DBuffer &Buffer) {
+  MotoGameEvent* MotoGameEvent::getUnserialized(DBuffer &Buffer, bool bDisplayInformation) {
     MotoGameEvent* v_event;
     float v_fEventTime;
     GameEventType v_eventType;
     int i_tmp;
 
+    /**** to be able to debug the replays files */
+    //int   EType;
+    //float ETime;
+    //int   SNameL;
+    //
+    //Buffer >> ETime;
+    //Buffer >> EType;
+    //Buffer >> SNameL;
+    //
+    //printf("%i\n", EType);
+    //
+    //int n = 256;
+    //char c[n];
+    //Buffer.readBuf(c, n);
+    //c[n-1] = '\0';
+    //for(int i=0; i<n-1; i++) {
+    //  if(c[i] == '\0') {
+    //	c[i] = '0';
+    //  } else {
+    //	if( ! ( (c[i] >= 'a' && c[i] <= 'z') || (c[i] >= 'A' && c[i] <= 'Z') ) ) {
+    //	  c[i] = '@';
+    //	}
+    //  }
+    //}
+    //printf("+%s+\n", c);
+    /*****/
+
     Buffer >> v_fEventTime;
     Buffer >> i_tmp;
     v_eventType = (GameEventType) i_tmp;
 
-    if(MGE_PlayerDies::SgetType() == v_eventType) {
-      v_event = new MGE_PlayerDies(v_fEventTime);
-    } else if(MGE_PlayerEntersZone::SgetType() == v_eventType) {
-      v_event = new MGE_PlayerEntersZone(v_fEventTime);
-    } else if(MGE_PlayerLeavesZone::SgetType() == v_eventType) {
-      v_event = new MGE_PlayerLeavesZone(v_fEventTime);
-    } else if(MGE_PlayerTouchesEntity::SgetType() == v_eventType) {
-      v_event = new MGE_PlayerTouchesEntity(v_fEventTime);
+    if(MGE_PlayersDie::SgetType() == v_eventType) {
+      v_event = new MGE_PlayersDie(v_fEventTime);
+    } else if(MGE_PlayersEnterZone::SgetType() == v_eventType) {
+      v_event = new MGE_PlayersEnterZone(v_fEventTime);
+    } else if(MGE_PlayersLeaveZone::SgetType() == v_eventType) {
+      v_event = new MGE_PlayersLeaveZone(v_fEventTime);
+    } else if(MGE_PlayersToucheEntity::SgetType() == v_eventType) {
+      v_event = new MGE_PlayersToucheEntity(v_fEventTime);
     } else if(MGE_EntityDestroyed::SgetType() == v_eventType) {
       v_event = new MGE_EntityDestroyed(v_fEventTime);
     } else if(MGE_ClearMessages::SgetType() == v_eventType) {
@@ -71,8 +99,8 @@ namespace vapp {
       v_event = new MGE_SetBlockPos(v_fEventTime);
     } else if(MGE_SetGravity::SgetType() == v_eventType) {
       v_event = new MGE_SetGravity(v_fEventTime);
-    } else if(MGE_SetPlayerPosition::SgetType() == v_eventType) {
-      v_event = new MGE_SetPlayerPosition(v_fEventTime);
+    } else if(MGE_SetPlayersPosition::SgetType() == v_eventType) {
+      v_event = new MGE_SetPlayersPosition(v_fEventTime);
     } else if(MGE_SetEntityPos::SgetType() == v_eventType) {
       v_event = new MGE_SetEntityPos(v_fEventTime);
     } else if(MGE_SetBlockCenter::SgetType() == v_eventType) {
@@ -97,13 +125,34 @@ namespace vapp {
       v_event = new MGE_CameraZoom(v_fEventTime);
     } else if(MGE_PenalityTime::SgetType() == v_eventType) {
       v_event = new MGE_PenalityTime(v_fEventTime);
+    } else if(MGE_SetPlayerPosition::SgetType() == v_eventType) {
+      v_event = new MGE_SetPlayerPosition(v_fEventTime);
+    } else if(MGE_PlayerDies::SgetType() == v_eventType) {
+      v_event = new MGE_PlayerDies(v_fEventTime);
+    } else if(MGE_PlayerEntersZone::SgetType() == v_eventType) {
+      v_event = new MGE_PlayerEntersZone(v_fEventTime);
+    } else if(MGE_PlayerLeavesZone::SgetType() == v_eventType) {
+      v_event = new MGE_PlayerLeavesZone(v_fEventTime);
+    } else if(MGE_PlayerTouchesEntity::SgetType() == v_eventType) {
+      v_event = new MGE_PlayerTouchesEntity(v_fEventTime);
+    } else if(MGE_SetDynamicBlockSelfRotation::SgetType() == v_eventType) {
+      v_event = new MGE_SetDynamicBlockSelfRotation(v_fEventTime);
+    } else if(MGE_SetDynamicEntitySelfRotation::SgetType() == v_eventType) {
+      v_event = new MGE_SetDynamicEntitySelfRotation(v_fEventTime);
+    } else if(MGE_CameraRotate::SgetType() == v_eventType) {
+      v_event = new MGE_CameraRotate(v_fEventTime);
+    } else if(MGE_CameraAdaptToGravity::SgetType() == v_eventType) {
+      v_event = new MGE_CameraAdaptToGravity(v_fEventTime);
     } else {
       std::ostringstream error_type;
       error_type << (int) v_eventType;
       throw Exception("Can't unserialize ! (event of type " + error_type.str() + ")");
     }
-
     v_event->unserialize(Buffer);
+    if(bDisplayInformation) {
+      printf("   %6.2f %-27s\n", v_fEventTime, v_event->toString().c_str());
+    }
+
     return v_event;
   }
 
@@ -115,21 +164,61 @@ namespace vapp {
   }
 
   //////////////////////////////
-  MGE_PlayerDies::MGE_PlayerDies(float p_fEventTime) 
+  MGE_PlayersDie::MGE_PlayersDie(float p_fEventTime) 
   : MotoGameEvent(p_fEventTime) {
     m_bKilledByWrecker = false;
   }
 
-  MGE_PlayerDies::MGE_PlayerDies(float p_fEventTime, bool p_bKilledByWrecker) 
+  MGE_PlayersDie::MGE_PlayersDie(float p_fEventTime, bool p_bKilledByWrecker) 
     : MotoGameEvent(p_fEventTime) {
       m_bKilledByWrecker = p_bKilledByWrecker;
   }
 
+  MGE_PlayersDie::~MGE_PlayersDie() {
+  } 
+  
+  void MGE_PlayersDie::doAction(MotoGame *p_pMotoGame) {
+    for(unsigned int i=0; i<p_pMotoGame->Players().size(); i++) {
+      p_pMotoGame->killPlayer(i);
+    }
+  }
+
+  void MGE_PlayersDie::serialize(DBuffer &Buffer) {
+  }
+  
+  void MGE_PlayersDie::unserialize(DBuffer &Buffer) {
+  }
+
+  GameEventType MGE_PlayersDie::SgetType() {
+    return GAME_EVENT_PLAYERS_DIE;
+  }
+
+  GameEventType MGE_PlayersDie::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_PlayersDie::toString() {
+    return "Players die";
+  }
+
+  //////////////////////////////
+  MGE_PlayerDies::MGE_PlayerDies(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+    m_bKilledByWrecker = false;
+    m_player           = 0;
+  }
+
+  MGE_PlayerDies::MGE_PlayerDies(float p_fEventTime, bool p_bKilledByWrecker, int i_player) 
+    : MotoGameEvent(p_fEventTime) {
+      m_bKilledByWrecker = p_bKilledByWrecker;
+      m_player           = i_player;
+  }
+
   MGE_PlayerDies::~MGE_PlayerDies() {
   } 
   
   void MGE_PlayerDies::doAction(MotoGame *p_pMotoGame) {
-    p_pMotoGame->killPlayer();
+    p_pMotoGame->killPlayer(m_player);
   }
 
   void MGE_PlayerDies::serialize(DBuffer &Buffer) {
@@ -146,24 +235,70 @@ namespace vapp {
     return SgetType();
   }
 
+  std::string MGE_PlayerDies::toString() {
+    std::ostringstream v_txt_player;
+    v_txt_player << m_player;
+    return "Player " + v_txt_player.str() + " dies";
+  }
 
   //////////////////////////////
-  MGE_PlayerEntersZone::MGE_PlayerEntersZone(float p_fEventTime) 
+  MGE_PlayersEnterZone::MGE_PlayersEnterZone(float p_fEventTime) 
     : MotoGameEvent(p_fEventTime) {
       m_zone = NULL;
     }
 
-  MGE_PlayerEntersZone::MGE_PlayerEntersZone(float p_fEventTime, Zone *p_zone) 
+  MGE_PlayersEnterZone::MGE_PlayersEnterZone(float p_fEventTime, Zone *p_zone) 
     : MotoGameEvent(p_fEventTime) {
       m_zone = p_zone;
     }
 
 
+  MGE_PlayersEnterZone::~MGE_PlayersEnterZone() {
+  } 
+  
+  void MGE_PlayersEnterZone::doAction(MotoGame *p_pMotoGame) {
+    for(unsigned int i=0; i<p_pMotoGame->Players().size(); i++) {
+      p_pMotoGame->playerEntersZone(i, m_zone);
+    }
+  }
+
+  void MGE_PlayersEnterZone::serialize(DBuffer &Buffer) {
+  }
+  
+  void MGE_PlayersEnterZone::unserialize(DBuffer &Buffer) {
+  }
+
+  GameEventType MGE_PlayersEnterZone::SgetType() {
+     return GAME_EVENT_PLAYERS_ENTER_ZONE;
+  }
+
+  GameEventType MGE_PlayersEnterZone::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_PlayersEnterZone::toString() {
+    return "Players enter on zone " + m_zone->Id();
+  }
+
+  //////////////////////////////
+  MGE_PlayerEntersZone::MGE_PlayerEntersZone(float p_fEventTime) 
+    : MotoGameEvent(p_fEventTime) {
+      m_zone   = NULL;
+      m_player = 0;
+    }
+
+  MGE_PlayerEntersZone::MGE_PlayerEntersZone(float p_fEventTime, Zone *p_zone, int i_player) 
+    : MotoGameEvent(p_fEventTime) {
+      m_zone   = p_zone;
+      m_player = i_player;
+    }
+
+
   MGE_PlayerEntersZone::~MGE_PlayerEntersZone() {
   } 
   
   void MGE_PlayerEntersZone::doAction(MotoGame *p_pMotoGame) {
-    p_pMotoGame->playerEntersZone(m_zone);
+    p_pMotoGame->playerEntersZone(m_player, m_zone);
   }
 
   void MGE_PlayerEntersZone::serialize(DBuffer &Buffer) {
@@ -180,23 +315,68 @@ namespace vapp {
     return SgetType();
   }
 
+  std::string MGE_PlayerEntersZone::toString() {
+    std::ostringstream v_txt_player;
+    v_txt_player << m_player;
+    return "Player " + v_txt_player.str() + " enter on zone " + m_zone->Id();
+  }
 
   //////////////////////////////
-  MGE_PlayerLeavesZone::MGE_PlayerLeavesZone(float p_fEventTime) 
+  MGE_PlayersLeaveZone::MGE_PlayersLeaveZone(float p_fEventTime) 
   : MotoGameEvent(p_fEventTime) {
     m_zone = NULL;
   }
 
-  MGE_PlayerLeavesZone::MGE_PlayerLeavesZone(float p_fEventTime, Zone *p_zone) 
+  MGE_PlayersLeaveZone::MGE_PlayersLeaveZone(float p_fEventTime, Zone *p_zone) 
     : MotoGameEvent(p_fEventTime) {
       m_zone = p_zone;
   }
 
+  MGE_PlayersLeaveZone::~MGE_PlayersLeaveZone() {
+  } 
+  
+  void MGE_PlayersLeaveZone::doAction(MotoGame *p_pMotoGame) {
+    for(unsigned int i=0; i<p_pMotoGame->Players().size(); i++) {
+      p_pMotoGame->playerLeavesZone(i, m_zone);
+    }
+  }
+
+  void MGE_PlayersLeaveZone::serialize(DBuffer &Buffer) {
+  }
+  
+  void MGE_PlayersLeaveZone::unserialize(DBuffer &Buffer) {
+  }
+
+  GameEventType MGE_PlayersLeaveZone::SgetType() {
+    return GAME_EVENT_PLAYERS_LEAVE_ZONE;
+  }
+
+  GameEventType MGE_PlayersLeaveZone::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_PlayersLeaveZone::toString() {
+    return "Players leave the zone " + m_zone->Id();
+  }
+
+  //////////////////////////////
+  MGE_PlayerLeavesZone::MGE_PlayerLeavesZone(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+    m_zone   = NULL;
+    m_player = 0;
+  }
+
+  MGE_PlayerLeavesZone::MGE_PlayerLeavesZone(float p_fEventTime, Zone *p_zone, int i_player) 
+    : MotoGameEvent(p_fEventTime) {
+      m_zone   = p_zone;
+      m_player = i_player;
+  }
+
   MGE_PlayerLeavesZone::~MGE_PlayerLeavesZone() {
   } 
   
   void MGE_PlayerLeavesZone::doAction(MotoGame *p_pMotoGame) {
-     p_pMotoGame->playerLeavesZone(m_zone);
+    p_pMotoGame->playerLeavesZone(m_player, m_zone);
   }
 
   void MGE_PlayerLeavesZone::serialize(DBuffer &Buffer) {
@@ -213,25 +393,73 @@ namespace vapp {
     return SgetType();
   }
 
+  std::string MGE_PlayerLeavesZone::toString() {
+    std::ostringstream v_txt_player;
+    v_txt_player << m_player;
+    return "Player " + v_txt_player.str() + " leaves the zone " + m_zone->Id();
+  }
+
+  //////////////////////////////
+  MGE_PlayersToucheEntity::MGE_PlayersToucheEntity(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+    m_entityID = "";
+    m_bTouchedWithHead = false;
+  }
+
+  MGE_PlayersToucheEntity::MGE_PlayersToucheEntity(float p_fEventTime, std::string p_entityID, bool p_bTouchedWithHead) 
+    : MotoGameEvent(p_fEventTime) {
+      m_entityID = p_entityID;
+      m_bTouchedWithHead = p_bTouchedWithHead;
+    }
+
+  MGE_PlayersToucheEntity::~MGE_PlayersToucheEntity() {
+  } 
+  
+  void MGE_PlayersToucheEntity::doAction(MotoGame *p_pMotoGame) {
+    for(unsigned int i=0; i<p_pMotoGame->Players().size(); i++) {
+      p_pMotoGame->playerTouchesEntity(i, m_entityID, m_bTouchedWithHead);
+    }
+  }
+
+  void MGE_PlayersToucheEntity::serialize(DBuffer &Buffer) {
+  }
+  
+  void MGE_PlayersToucheEntity::unserialize(DBuffer &Buffer) {
+  }
+
+  GameEventType MGE_PlayersToucheEntity::SgetType() {
+    return GAME_EVENT_PLAYERS_TOUCHE_ENTITY;
+  }
+
+  GameEventType MGE_PlayersToucheEntity::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_PlayersToucheEntity::toString() {
+    return "Players touche entity " + m_entityID;
+  }
 
   //////////////////////////////
   MGE_PlayerTouchesEntity::MGE_PlayerTouchesEntity(float p_fEventTime) 
   : MotoGameEvent(p_fEventTime) {
     m_entityID = "";
     m_bTouchedWithHead = false;
+    m_player = 0;
   }
 
-  MGE_PlayerTouchesEntity::MGE_PlayerTouchesEntity(float p_fEventTime, std::string p_entityID, bool p_bTouchedWithHead) 
+  MGE_PlayerTouchesEntity::MGE_PlayerTouchesEntity(float p_fEventTime, std::string p_entityID,
+						   bool p_bTouchedWithHead, int i_player) 
     : MotoGameEvent(p_fEventTime) {
       m_entityID = p_entityID;
       m_bTouchedWithHead = p_bTouchedWithHead;
+      m_player = i_player;
     }
 
   MGE_PlayerTouchesEntity::~MGE_PlayerTouchesEntity() {
   } 
   
   void MGE_PlayerTouchesEntity::doAction(MotoGame *p_pMotoGame) {
-    p_pMotoGame->playerTouchesEntity(m_entityID, m_bTouchedWithHead);
+    p_pMotoGame->playerTouchesEntity(m_player, m_entityID, m_bTouchedWithHead);
   }
 
   void MGE_PlayerTouchesEntity::serialize(DBuffer &Buffer) {
@@ -248,6 +476,12 @@ namespace vapp {
     return SgetType();
   }
 
+  std::string MGE_PlayerTouchesEntity::toString() {
+    std::ostringstream v_txt_player;
+    v_txt_player << m_player;
+
+    return "Player " + v_txt_player.str() + " touches entity " + m_entityID;
+  }
 
   //////////////////////////////
   MGE_EntityDestroyed::MGE_EntityDestroyed(float p_fEventTime) 
@@ -264,15 +498,15 @@ namespace vapp {
     }
 
   MGE_EntityDestroyed::~MGE_EntityDestroyed() {
-  } 
+  }
   
   void MGE_EntityDestroyed::doAction(MotoGame *p_pMotoGame) {
-     p_pMotoGame->entityDestroyed(m_entityId);
+    p_pMotoGame->entityDestroyed(m_entityId);
   }
 
   void MGE_EntityDestroyed::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_entityId);
+    Buffer << m_entityId;
     Buffer << (int) (m_entityType);
     Buffer << m_entitySize;
     Buffer << m_entityPosition.x;
@@ -283,11 +517,13 @@ namespace vapp {
 
     Vector2f v_position;
     std::string v_entityId;
+    int i_tmp;
 
-    Buffer.read(m_entityId);
+    Buffer >> m_entityId;
 
     /* no more used, kept just for compatibiliy */    
-    Buffer >> m_entityType;
+    Buffer >> i_tmp;
+    m_entityType = (EntitySpeciality) i_tmp;
     switch(m_entityType) {
     case ET_NONE:
     case ET_ISSTART:
@@ -297,7 +533,9 @@ namespace vapp {
     case ET_PARTICLES_SOURCE:
       break;
     default:
-      throw Exception("Invalid entity type"); // with some compilator, an invalid value causes a segfault (on my linux box)
+      std::ostringstream error_type;
+      error_type << (int) m_entityType;
+      throw Exception("Invalid entity type (" + error_type.str() + ")"); // with some compilator, an invalid value causes a segfault (on my linux box)
     }
     Buffer >> m_entitySize;
     Buffer >> m_entityPosition.x;
@@ -316,6 +554,10 @@ namespace vapp {
     return SgetType();
   }
 
+  std::string MGE_EntityDestroyed::toString() {
+    return "Player destroys entity " + m_entityId;
+  }
+
   std::string MGE_EntityDestroyed::EntityId() {
     return m_entityId;
   }
@@ -340,13 +582,16 @@ namespace vapp {
   }
 
   GameEventType MGE_ClearMessages::SgetType() {
-    return GAME_EVENT_LUA_CALL_CLEARMESSAGES;
+    return GAME_EVENT_CLEARMESSAGES;
   }
 
   GameEventType MGE_ClearMessages::getType() {
     return SgetType();
   }
 
+  std::string MGE_ClearMessages::toString() {
+    return "Messages are cleared";
+  }
 
   //////////////////////////////
   MGE_PlaceInGameArrow::MGE_PlaceInGameArrow(float p_fEventTime) 
@@ -386,13 +631,16 @@ namespace vapp {
   }
 
   GameEventType MGE_PlaceInGameArrow::SgetType() {
-    return GAME_EVENT_LUA_CALL_PLACEINGAMEARROW;
+    return GAME_EVENT_PLACEINGAMEARROW;
   }
 
   GameEventType MGE_PlaceInGameArrow::getType() {
     return SgetType();
   }
 
+  std::string MGE_PlaceInGameArrow::toString() {
+    return "Place in game arrow";
+  }
 
   //////////////////////////////
   MGE_PlaceScreenarrow::MGE_PlaceScreenarrow(float p_fEventTime) 
@@ -432,13 +680,16 @@ namespace vapp {
   }
 
   GameEventType MGE_PlaceScreenarrow::SgetType() {
-    return GAME_EVENT_LUA_CALL_PLACESCREENARROW;
+    return GAME_EVENT_PLACESCREENARROW;
   }
 
   GameEventType MGE_PlaceScreenarrow::getType() {
     return SgetType();
   }
 
+  std::string MGE_PlaceScreenarrow::toString() {
+    return "Place screen arrow";
+  }
 
   //////////////////////////////
   MGE_HideArrow::MGE_HideArrow(float p_fEventTime) 
@@ -460,13 +711,16 @@ namespace vapp {
   }
 
   GameEventType MGE_HideArrow::SgetType() {
-    return GAME_EVENT_LUA_CALL_HIDEARROW;
+    return GAME_EVENT_HIDEARROW;
   }
 
   GameEventType MGE_HideArrow::getType() {
     return SgetType();
   }
 
+  std::string MGE_HideArrow::toString() {
+    return "Hide arrow";
+  }
 
   //////////////////////////////
   MGE_Message::MGE_Message(float p_fEventTime) 
@@ -488,21 +742,24 @@ namespace vapp {
 
   void MGE_Message::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_message);
+    Buffer << m_message;
   }
   
   void MGE_Message::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_message);
+    Buffer >> m_message;
   }
 
   GameEventType MGE_Message::SgetType() {
-    return GAME_EVENT_LUA_CALL_MESSAGE;
+    return GAME_EVENT_MESSAGE;
   }
 
   GameEventType MGE_Message::getType() {
     return SgetType();
   }
 
+  std::string MGE_Message::toString() {
+    return "Display message '" + m_message + "'";
+  }
 
   //////////////////////////////
   MGE_MoveBlock::MGE_MoveBlock(float p_fEventTime) 
@@ -530,25 +787,28 @@ namespace vapp {
 
   void MGE_MoveBlock::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
     Buffer << m_x;
     Buffer << m_y;
   }
   
   void MGE_MoveBlock::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
     Buffer >> m_x;
     Buffer >> m_y;
   }
 
   GameEventType MGE_MoveBlock::SgetType() {
-    return GAME_EVENT_LUA_CALL_MOVEBLOCK;
+    return GAME_EVENT_MOVEBLOCK;
   }
 
   GameEventType MGE_MoveBlock::getType() {
     return SgetType();
   }
 
+  std::string MGE_MoveBlock::toString() {
+    return "Block " + m_blockID + " is moved";
+  }
 
   //////////////////////////////
   MGE_SetBlockPos::MGE_SetBlockPos(float p_fEventTime) 
@@ -576,25 +836,28 @@ namespace vapp {
 
   void MGE_SetBlockPos::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
     Buffer << m_x;
     Buffer << m_y;
   }
   
   void MGE_SetBlockPos::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
     Buffer >> m_x;
     Buffer >> m_y;
   }
 
   GameEventType MGE_SetBlockPos::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETBLOCKPOS;
+    return GAME_EVENT_SETBLOCKPOS;
   }
 
   GameEventType MGE_SetBlockPos::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetBlockPos::toString() {
+    return "Block " + m_blockID + "'s position is changed";
+  }
 
   //////////////////////////////
   MGE_SetGravity::MGE_SetGravity(float p_fEventTime) 
@@ -628,23 +891,26 @@ namespace vapp {
   }
 
   GameEventType MGE_SetGravity::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETGRAVITY;
+    return GAME_EVENT_SETGRAVITY;
   }
 
   GameEventType MGE_SetGravity::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetGravity::toString() {
+    return "Gravity is changed";
+  }
 
   //////////////////////////////
-  MGE_SetPlayerPosition::MGE_SetPlayerPosition(float p_fEventTime) 
+  MGE_SetPlayersPosition::MGE_SetPlayersPosition(float p_fEventTime) 
   : MotoGameEvent(p_fEventTime) {
     m_x = 0.0;
     m_y = 0.0;
     m_bRight = 0.0;
   }
 
-  MGE_SetPlayerPosition::MGE_SetPlayerPosition(float p_fEventTime,
+  MGE_SetPlayersPosition::MGE_SetPlayersPosition(float p_fEventTime,
                                                float p_x, float p_y,
                                                bool p_bRight) 
     : MotoGameEvent(p_fEventTime) {
@@ -653,11 +919,66 @@ namespace vapp {
       m_bRight = p_bRight;
     }
 
+  MGE_SetPlayersPosition::~MGE_SetPlayersPosition() {
+  } 
+  
+  void MGE_SetPlayersPosition::doAction(MotoGame *p_pMotoGame) {
+    for(unsigned int i=0; i<p_pMotoGame->Players().size(); i++) {
+      p_pMotoGame->setPlayerPosition(i, m_x, m_y, m_bRight);
+    }
+  }
+
+  void MGE_SetPlayersPosition::serialize(DBuffer &Buffer) {
+    MotoGameEvent::serialize(Buffer);
+    Buffer << m_x;
+    Buffer << m_y;
+    Buffer << m_bRight;
+  }
+  
+  void MGE_SetPlayersPosition::unserialize(DBuffer &Buffer) {
+    Buffer >> m_x;
+    Buffer >> m_y;
+    Buffer >> m_bRight;
+  }
+
+  GameEventType MGE_SetPlayersPosition::SgetType() {
+    return GAME_EVENT_SETPLAYERSPOSITION;
+  }
+
+  GameEventType MGE_SetPlayersPosition::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_SetPlayersPosition::toString() {
+    return "Teleportation of the players";
+  }
+
+  //////////////////////////////
+  MGE_SetPlayerPosition::MGE_SetPlayerPosition(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+    m_x = 0.0;
+    m_y = 0.0;
+    m_bRight = 0.0;
+    m_player = 0;
+  }
+
+  MGE_SetPlayerPosition::MGE_SetPlayerPosition(float p_fEventTime,
+                                               float p_x, float p_y,
+                                               bool p_bRight,
+					       int i_player
+					       ) 
+    : MotoGameEvent(p_fEventTime) {
+      m_x = p_x;
+      m_y = p_y;
+      m_bRight = p_bRight;
+      m_player = i_player;
+    }
+
   MGE_SetPlayerPosition::~MGE_SetPlayerPosition() {
   } 
   
   void MGE_SetPlayerPosition::doAction(MotoGame *p_pMotoGame) {
-    p_pMotoGame->setPlayerPosition(m_x, m_y, m_bRight);
+    p_pMotoGame->setPlayerPosition(m_player, m_x, m_y, m_bRight);
   }
 
   void MGE_SetPlayerPosition::serialize(DBuffer &Buffer) {
@@ -665,22 +986,27 @@ namespace vapp {
     Buffer << m_x;
     Buffer << m_y;
     Buffer << m_bRight;
+    Buffer << m_player;
   }
   
   void MGE_SetPlayerPosition::unserialize(DBuffer &Buffer) {
     Buffer >> m_x;
     Buffer >> m_y;
     Buffer >> m_bRight;
+    Buffer >> m_player;
   }
 
   GameEventType MGE_SetPlayerPosition::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETPLAYERPOSITION;
+    return GAME_EVENT_SETPLAYERPOSITION;
   }
 
   GameEventType MGE_SetPlayerPosition::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetPlayerPosition::toString() {
+    return "Teleportation of the player " + m_player;
+  }
 
   //////////////////////////////
   MGE_SetEntityPos::MGE_SetEntityPos(float p_fEventTime) 
@@ -708,25 +1034,28 @@ namespace vapp {
 
   void MGE_SetEntityPos::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_entityID);
+    Buffer << m_entityID;
     Buffer << m_x;
     Buffer << m_y;
   }
   
   void MGE_SetEntityPos::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_entityID);
+    Buffer >> m_entityID;
     Buffer >> m_x;
     Buffer >> m_y;
   }
 
   GameEventType MGE_SetEntityPos::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETENTITYPOS;
+    return GAME_EVENT_SETENTITYPOS;
   }
 
   GameEventType MGE_SetEntityPos::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetEntityPos::toString() {
+    return "Entity " + m_entityID + "'s position is changed";
+  }
 
   //////////////////////////////
   MGE_SetBlockCenter::MGE_SetBlockCenter(float p_fEventTime) 
@@ -754,25 +1083,28 @@ namespace vapp {
 
   void MGE_SetBlockCenter::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
     Buffer << m_x;
     Buffer << m_y;
   }
   
   void MGE_SetBlockCenter::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
     Buffer >> m_x;
     Buffer >> m_y;
   }
 
   GameEventType MGE_SetBlockCenter::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETBLOCKCENTER;
+    return GAME_EVENT_SETBLOCKCENTER;
   }
 
   GameEventType MGE_SetBlockCenter::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetBlockCenter::toString() {
+    return "Block " + m_blockID + "'s center is changed";
+  }
 
   //////////////////////////////
   MGE_SetBlockRotation::MGE_SetBlockRotation(float p_fEventTime) 
@@ -798,23 +1130,26 @@ namespace vapp {
 
   void MGE_SetBlockRotation::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
     Buffer << m_angle;
   }
   
   void MGE_SetBlockRotation::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
     Buffer >> m_angle;
   }
 
   GameEventType MGE_SetBlockRotation::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETBLOCKROTATION ;
+    return GAME_EVENT_SETBLOCKROTATION ;
   }
 
   GameEventType MGE_SetBlockRotation::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetBlockRotation::toString() {
+    return "Block " + m_blockID + "'s rotation is changed";
+  }
 
   //////////////////////////////
   MGE_SetDynamicEntityRotation::MGE_SetDynamicEntityRotation(float p_fEventTime) 
@@ -855,7 +1190,7 @@ namespace vapp {
 
   void MGE_SetDynamicEntityRotation::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_entityID);
+    Buffer << m_entityID;
     Buffer << m_fInitAngle;
     Buffer << m_fRadius;
     Buffer << m_fPeriod;
@@ -864,7 +1199,7 @@ namespace vapp {
   }
   
   void MGE_SetDynamicEntityRotation::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_entityID);
+    Buffer >> m_entityID;
     Buffer >> m_fInitAngle;
     Buffer >> m_fRadius;
     Buffer >> m_fPeriod;
@@ -873,13 +1208,16 @@ namespace vapp {
   }
 
   GameEventType MGE_SetDynamicEntityRotation::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETDYNAMICENTITYROTATION ;
+    return GAME_EVENT_SETDYNAMICENTITYROTATION ;
   }
 
   GameEventType MGE_SetDynamicEntityRotation::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetDynamicEntityRotation::toString() {
+    return "Dynamic rotation is set for entity " + m_entityID;
+  }
 
   //////////////////////////////
   MGE_SetDynamicEntityTranslation::MGE_SetDynamicEntityTranslation(float p_fEventTime) 
@@ -920,7 +1258,7 @@ namespace vapp {
 
   void MGE_SetDynamicEntityTranslation::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_entityID);
+    Buffer << m_entityID;
     Buffer << m_x;
     Buffer << m_y;
     Buffer << m_fPeriod;
@@ -929,7 +1267,7 @@ namespace vapp {
   }
   
   void MGE_SetDynamicEntityTranslation::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_entityID);
+    Buffer >> m_entityID;
     Buffer >> m_x;
     Buffer >> m_y;
     Buffer >> m_fPeriod;
@@ -938,13 +1276,16 @@ namespace vapp {
   }
 
   GameEventType MGE_SetDynamicEntityTranslation::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETDYNAMICENTITYTRANSLATION;
+    return GAME_EVENT_SETDYNAMICENTITYTRANSLATION;
   }
 
   GameEventType MGE_SetDynamicEntityTranslation::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetDynamicEntityTranslation::toString() {
+    return "Dynamic translation is set for entity " + m_entityID;
+  }
 
   //////////////////////////////
   MGE_SetDynamicEntityNone::MGE_SetDynamicEntityNone(float p_fEventTime) 
@@ -966,21 +1307,25 @@ namespace vapp {
 
   void MGE_SetDynamicEntityNone::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_entityID);
+    Buffer << m_entityID;
   }
   
   void MGE_SetDynamicEntityNone::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_entityID);
+    Buffer >> m_entityID;
   }
 
   GameEventType MGE_SetDynamicEntityNone::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETDYNAMICENTITYNONE;
+    return GAME_EVENT_SETDYNAMICENTITYNONE;
   }
 
   GameEventType MGE_SetDynamicEntityNone::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetDynamicEntityNone::toString() {
+    return "Remove dynamic for entity " + m_entityID;
+  }
+
   //////////////////////////////
   MGE_SetDynamicBlockRotation::MGE_SetDynamicBlockRotation(float p_fEventTime) 
   : MotoGameEvent(p_fEventTime) {
@@ -1020,7 +1365,7 @@ namespace vapp {
 
   void MGE_SetDynamicBlockRotation::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
     Buffer << m_fInitAngle;
     Buffer << m_fRadius;
     Buffer << m_fPeriod;
@@ -1029,7 +1374,7 @@ namespace vapp {
   }
   
   void MGE_SetDynamicBlockRotation::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
     Buffer >> m_fInitAngle;
     Buffer >> m_fRadius;
     Buffer >> m_fPeriod;
@@ -1038,13 +1383,16 @@ namespace vapp {
   }
 
   GameEventType MGE_SetDynamicBlockRotation::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKROTATION;
+    return GAME_EVENT_SETDYNAMICBLOCKROTATION;
   }
 
   GameEventType MGE_SetDynamicBlockRotation::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetDynamicBlockRotation::toString() {
+    return "Dynamic rotation is set for block " + m_blockID;
+  }
 
   //////////////////////////////
   MGE_SetDynamicBlockTranslation::MGE_SetDynamicBlockTranslation(float p_fEventTime) 
@@ -1085,7 +1433,7 @@ namespace vapp {
 
   void MGE_SetDynamicBlockTranslation::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
     Buffer << m_x;
     Buffer << m_y;
     Buffer << m_fPeriod;
@@ -1094,7 +1442,7 @@ namespace vapp {
   }
   
   void MGE_SetDynamicBlockTranslation::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
     Buffer >> m_x;
     Buffer >> m_y;
     Buffer >> m_fPeriod;
@@ -1103,13 +1451,16 @@ namespace vapp {
   }
 
   GameEventType MGE_SetDynamicBlockTranslation::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKTRANSLATION;
+    return GAME_EVENT_SETDYNAMICBLOCKTRANSLATION;
   }
 
   GameEventType MGE_SetDynamicBlockTranslation::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetDynamicBlockTranslation::toString() {
+    return "Dynamic translation is set for block " + m_blockID;
+  }
 
   //////////////////////////////
   MGE_SetDynamicBlockNone::MGE_SetDynamicBlockNone(float p_fEventTime) 
@@ -1131,21 +1482,25 @@ namespace vapp {
 
   void MGE_SetDynamicBlockNone::serialize(DBuffer &Buffer) {
     MotoGameEvent::serialize(Buffer);
-    Buffer.write(m_blockID);
+    Buffer << m_blockID;
   }
   
   void MGE_SetDynamicBlockNone::unserialize(DBuffer &Buffer) {
-    Buffer.read(m_blockID);
+    Buffer >> m_blockID;
   }
 
   GameEventType MGE_SetDynamicBlockNone::SgetType() {
-    return GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKNONE;
+    return GAME_EVENT_SETDYNAMICBLOCKNONE;
   }
 
   GameEventType MGE_SetDynamicBlockNone::getType() {
     return SgetType();
   }
 
+  std::string MGE_SetDynamicBlockNone::toString() {
+    return "Remove dynamic for block " + m_blockID;
+  }
+
   //////////////////////////////
   MGE_CameraMove::MGE_CameraMove(float p_fEventTime) 
   : MotoGameEvent(p_fEventTime) {
@@ -1178,13 +1533,16 @@ namespace vapp {
   }
 
   GameEventType MGE_CameraMove::SgetType() {
-    return GAME_EVENT_LUA_CALL_CAMERAMOVE;
+    return GAME_EVENT_CAMERAMOVE;
   }
 
   GameEventType MGE_CameraMove::getType() {
     return SgetType();
   }
 
+  std::string MGE_CameraMove::toString() {
+    return "Camera moves";
+  }
 
   //////////////////////////////
   MGE_CameraZoom::MGE_CameraZoom(float p_fEventTime) 
@@ -1214,13 +1572,17 @@ namespace vapp {
   }
 
   GameEventType MGE_CameraZoom::SgetType() {
-    return GAME_EVENT_LUA_CALL_CAMERAZOOM;
+    return GAME_EVENT_CAMERAZOOM;
   }
 
   GameEventType MGE_CameraZoom::getType() {
     return SgetType();
   }
 
+  std::string MGE_CameraZoom::toString() {
+    return "Camera zoom is changed";
+  }
+
   //////////////////////////////
 
   MGE_PenalityTime::MGE_PenalityTime(float p_fEventTime)
@@ -1249,11 +1611,249 @@ namespace vapp {
   }
  
   GameEventType MGE_PenalityTime::SgetType() {
-    return GAME_EVENT_LUA_CALL_PENALITY_TIME;
+    return GAME_EVENT_PENALITY_TIME;
   }
 
   GameEventType MGE_PenalityTime::getType() {
     return SgetType();
   }
 
+  std::string MGE_PenalityTime::toString() {
+    return "Time penality";
+  }
+
+  //////////////////////////////
+
+  MGE_SetDynamicBlockSelfRotation::MGE_SetDynamicBlockSelfRotation(float p_fEventTime)
+  : MotoGameEvent(p_fEventTime) {
+    m_blockID   = "";
+    m_fPeriod    = 0.0;
+    m_startTime  = 0;
+    m_endTime    = 0;
+  }
+  
+  MGE_SetDynamicBlockSelfRotation::MGE_SetDynamicBlockSelfRotation(float p_fEventTime,
+								   std::string p_blockID,
+								   float p_fPeriod,
+								   int   p_startTime,
+								 int   p_endTime)
+  : MotoGameEvent(p_fEventTime){
+    m_blockID   = p_blockID;
+    m_fPeriod   = p_fPeriod;
+    m_startTime = p_startTime;
+    m_endTime   = p_endTime;
+  }
+  
+  MGE_SetDynamicBlockSelfRotation::~MGE_SetDynamicBlockSelfRotation() {
+  }
+  
+  void MGE_SetDynamicBlockSelfRotation::doAction(MotoGame *p_pMotoGame) {
+    p_pMotoGame->addDynamicObject(new SDynamicBlockSelfRotation(m_blockID,
+								m_fPeriod,
+								m_startTime, m_endTime));  
+  }
+  
+  void MGE_SetDynamicBlockSelfRotation::serialize(DBuffer &Buffer) {
+    MotoGameEvent::serialize(Buffer);
+    Buffer << m_blockID;
+    Buffer << m_fPeriod;
+    Buffer << m_startTime;
+    Buffer << m_endTime;
+  }
+  
+  void MGE_SetDynamicBlockSelfRotation::unserialize(DBuffer &Buffer) {
+    Buffer >> m_blockID;
+    Buffer >> m_fPeriod;
+    Buffer >> m_startTime;
+    Buffer >> m_endTime;
+  }
+  
+  GameEventType MGE_SetDynamicBlockSelfRotation::SgetType() {
+    return GAME_EVENT_SETDYNAMICBLOCKSELFROTATION;
+  }
+  
+  GameEventType MGE_SetDynamicBlockSelfRotation::getType() {
+    return SgetType();
+  }
+  
+  std::string MGE_SetDynamicBlockSelfRotation::toString() {
+    return "Dynamic self rotation is set for block " + m_blockID;
+  }
+
+  //////////////////////////////
+
+  MGE_SetDynamicEntitySelfRotation::MGE_SetDynamicEntitySelfRotation(float p_fEventTime)
+  : MotoGameEvent(p_fEventTime) {
+    m_entityID   = "";
+    m_fPeriod    = 0.0;
+    m_startTime  = 0;
+    m_endTime    = 0;
+  }
+  
+  MGE_SetDynamicEntitySelfRotation::MGE_SetDynamicEntitySelfRotation(float p_fEventTime,
+								     std::string p_entityID,
+								     float p_fPeriod,
+								     int   p_startTime,
+								     int   p_endTime)
+  : MotoGameEvent(p_fEventTime){
+    m_entityID   = p_entityID;
+    m_fPeriod   = p_fPeriod;
+    m_startTime = p_startTime;
+    m_endTime   = p_endTime;
+  }
+  
+  MGE_SetDynamicEntitySelfRotation::~MGE_SetDynamicEntitySelfRotation() {
+  }
+  
+  void MGE_SetDynamicEntitySelfRotation::doAction(MotoGame *p_pMotoGame) {
+    p_pMotoGame->addDynamicObject(new SDynamicEntitySelfRotation(m_entityID,
+								 m_fPeriod,
+								 m_startTime, m_endTime));  
+  }
+  
+  void MGE_SetDynamicEntitySelfRotation::serialize(DBuffer &Buffer) {
+    MotoGameEvent::serialize(Buffer);
+    Buffer << m_entityID;
+    Buffer << m_fPeriod;
+    Buffer << m_startTime;
+    Buffer << m_endTime;
+  }
+  
+  void MGE_SetDynamicEntitySelfRotation::unserialize(DBuffer &Buffer) {
+    Buffer >> m_entityID;
+    Buffer >> m_fPeriod;
+    Buffer >> m_startTime;
+    Buffer >> m_endTime;
+  }
+  
+  GameEventType MGE_SetDynamicEntitySelfRotation::SgetType() {
+    return GAME_EVENT_SETDYNAMICENTITYSELFROTATION;
+  }
+  
+  GameEventType MGE_SetDynamicEntitySelfRotation::getType() {
+    return SgetType();
+  }
+  
+  std::string MGE_SetDynamicEntitySelfRotation::toString() {
+    return "Dynamic self rotation is set for entity " + m_entityID;
+  }
+
+  //////////////////////////////
+  MGE_CameraRotate::MGE_CameraRotate(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+    m_angle = 0.0;
+  }
+
+  MGE_CameraRotate::MGE_CameraRotate(float p_fEventTime, float p_angle)
+  : MotoGameEvent(p_fEventTime) {
+    m_angle = p_angle;
+  }
+
+  MGE_CameraRotate::~MGE_CameraRotate() {
+  } 
+  
+  void MGE_CameraRotate::doAction(MotoGame *p_pMotoGame) {
+    p_pMotoGame->CameraRotate(m_angle);
+  }
+
+  void MGE_CameraRotate::serialize(DBuffer &Buffer) {
+    MotoGameEvent::serialize(Buffer);
+    Buffer << m_angle;
+  }
+  
+  void MGE_CameraRotate::unserialize(DBuffer &Buffer) {
+    Buffer >> m_angle;
+  }
+
+  GameEventType MGE_CameraRotate::SgetType() {
+    return GAME_EVENT_CAMERAROTATE;
+  }
+
+  GameEventType MGE_CameraRotate::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_CameraRotate::toString() {
+    return "Camera rotates";
+  }
+
+  //////////////////////////////
+  MGE_CameraAdaptToGravity::MGE_CameraAdaptToGravity(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+  }
+
+  MGE_CameraAdaptToGravity::~MGE_CameraAdaptToGravity() {
+  } 
+  
+  void MGE_CameraAdaptToGravity::doAction(MotoGame *p_pMotoGame) {
+    p_pMotoGame->CameraAdaptToGravity();
+  }
+
+  void MGE_CameraAdaptToGravity::serialize(DBuffer &Buffer) {
+    MotoGameEvent::serialize(Buffer);
+  }
+  
+  void MGE_CameraAdaptToGravity::unserialize(DBuffer &Buffer) {
+  }
+
+  GameEventType MGE_CameraAdaptToGravity::SgetType() {
+    return GAME_EVENT_CAMERAADAPTTOGRAVITY;
+  }
+
+  GameEventType MGE_CameraAdaptToGravity::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_CameraAdaptToGravity::toString() {
+    return "Camera is adapted to the gravity";
+  }
+
+  //////////////////////////////
+  MGE_AddForceToPlayer::MGE_AddForceToPlayer(float p_fEventTime) 
+  : MotoGameEvent(p_fEventTime) {
+    m_force = Vector2f(0.0, 0.0);
+    m_player = 0;
+  }
+
+  MGE_AddForceToPlayer::MGE_AddForceToPlayer(float p_fEventTime,
+					     const Vector2f& i_force,
+					     int i_player
+					     ) 
+    : MotoGameEvent(p_fEventTime) {
+      m_force = i_force;
+      m_player = i_player;
+    }
+
+  MGE_AddForceToPlayer::~MGE_AddForceToPlayer() {
+  } 
+  
+  void MGE_AddForceToPlayer::doAction(MotoGame *p_pMotoGame) {
+    p_pMotoGame->addForceToPlayer(m_player, m_force);
+  }
+
+  void MGE_AddForceToPlayer::serialize(DBuffer &Buffer) {
+    MotoGameEvent::serialize(Buffer);
+    Buffer << m_force.x;
+    Buffer << m_force.y;
+    Buffer << m_player;
+  }
+  
+  void MGE_AddForceToPlayer::unserialize(DBuffer &Buffer) {
+    Buffer >> m_force.x;
+    Buffer >> m_force.y;
+    Buffer >> m_player;
+  }
+
+  GameEventType MGE_AddForceToPlayer::SgetType() {
+    return GAME_EVENT_ADDFORCETOPLAYER;
+  }
+
+  GameEventType MGE_AddForceToPlayer::getType() {
+    return SgetType();
+  }
+
+  std::string MGE_AddForceToPlayer::toString() {
+    return "Add force to the player " + m_player;
+  }
+
 }
