@@ -34,36 +34,56 @@ namespace vapp {
      ALWAYS ADD VALUES at the end : (otherwise, old replays will not work !)
   */
   enum GameEventType {
-    GAME_EVENT_PLAYER_DIES                          =  0,
-    GAME_EVENT_PLAYER_ENTERS_ZONE                   =  1,
-    GAME_EVENT_PLAYER_LEAVES_ZONE                   =  2,
-    GAME_EVENT_PLAYER_TOUCHES_ENTITY                =  3,
-    GAME_EVENT_ENTITY_DESTROYED                     =  4,    
-    GAME_EVENT_LUA_CALL_CLEARMESSAGES               =  5,
-    GAME_EVENT_LUA_CALL_PLACEINGAMEARROW            =  6,
-    GAME_EVENT_LUA_CALL_PLACESCREENARROW            =  7,
-    GAME_EVENT_LUA_CALL_HIDEARROW                   =  8,
-    GAME_EVENT_LUA_CALL_MESSAGE                     =  9,
-    GAME_EVENT_LUA_CALL_MOVEBLOCK                   = 10,
-    GAME_EVENT_LUA_CALL_SETBLOCKPOS                 = 11,
-    GAME_EVENT_LUA_CALL_SETGRAVITY                  = 12,
-    GAME_EVENT_LUA_CALL_SETPLAYERPOSITION           = 13,
-    GAME_EVENT_LUA_CALL_SETENTITYPOS                = 14,
-    GAME_EVENT_LUA_CALL_SETBLOCKCENTER              = 15,
-    GAME_EVENT_LUA_CALL_SETBLOCKROTATION            = 16, 
-    GAME_EVENT_LUA_CALL_SETDYNAMICENTITYROTATION    = 17,
-    GAME_EVENT_LUA_CALL_SETDYNAMICENTITYTRANSLATION = 18,
-    GAME_EVENT_LUA_CALL_SETDYNAMICENTITYNONE        = 19,
-    GAME_EVENT_LUA_CALL_CAMERAZOOM                  = 20,
-    GAME_EVENT_LUA_CALL_CAMERAMOVE                  = 21,
-    GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKROTATION     = 22,
-    GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKTRANSLATION  = 23,
-    GAME_EVENT_LUA_CALL_SETDYNAMICBLOCKNONE         = 24,
-    GAME_EVENT_LUA_CALL_PENALITY_TIME               = 25
+    GAME_EVENT_PLAYERS_DIE                  =  0, // to able able to read replay made with xmoto < 0.3.0
+    GAME_EVENT_PLAYERS_ENTER_ZONE           =  1, // to able able to read replay made with xmoto < 0.3.0
+    GAME_EVENT_PLAYERS_LEAVE_ZONE           =  2, // to able able to read replay made with xmoto < 0.3.0
+    GAME_EVENT_PLAYERS_TOUCHE_ENTITY        =  3, // to able able to read replay made with xmoto < 0.3.0
+    GAME_EVENT_ENTITY_DESTROYED             =  4,    
+    GAME_EVENT_CLEARMESSAGES                =  5,
+    GAME_EVENT_PLACEINGAMEARROW             =  6,
+    GAME_EVENT_PLACESCREENARROW             =  7,
+    GAME_EVENT_HIDEARROW                    =  8,
+    GAME_EVENT_MESSAGE                      =  9,
+    GAME_EVENT_MOVEBLOCK                    = 10,
+    GAME_EVENT_SETBLOCKPOS                  = 11,
+    GAME_EVENT_SETGRAVITY                   = 12,
+    GAME_EVENT_SETPLAYERSPOSITION           = 13, // to able able to read replay made with xmoto < 0.3.0
+    GAME_EVENT_SETENTITYPOS                 = 14,
+    GAME_EVENT_SETBLOCKCENTER               = 15,
+    GAME_EVENT_SETBLOCKROTATION             = 16, 
+    GAME_EVENT_SETDYNAMICENTITYROTATION     = 17,
+    GAME_EVENT_SETDYNAMICENTITYTRANSLATION  = 18,
+    GAME_EVENT_SETDYNAMICENTITYNONE         = 19,
+    GAME_EVENT_CAMERAZOOM                   = 20,
+    GAME_EVENT_CAMERAMOVE                   = 21,
+    GAME_EVENT_SETDYNAMICBLOCKROTATION      = 22,
+    GAME_EVENT_SETDYNAMICBLOCKTRANSLATION   = 23,
+    GAME_EVENT_SETDYNAMICBLOCKNONE          = 24,
+    GAME_EVENT_PENALITY_TIME                = 25,
+    GAME_EVENT_PLAYER_DIES                  = 26,
+    GAME_EVENT_PLAYER_ENTERS_ZONE           = 27,
+    GAME_EVENT_PLAYER_LEAVES_ZONE           = 28,
+    GAME_EVENT_PLAYER_TOUCHES_ENTITY        = 29,
+    GAME_EVENT_SETPLAYERPOSITION            = 30,
+    GAME_EVENT_SETDYNAMICBLOCKSELFROTATION  = 31,
+    GAME_EVENT_SETDYNAMICENTITYSELFROTATION = 32,
+    GAME_EVENT_CAMERAROTATE                 = 33,
+    GAME_EVENT_CAMERAADAPTTOGRAVITY         = 34,
+    GAME_EVENT_ADDFORCETOPLAYER             = 35
   };
 }
 
-#include "MotoGame.h"
+namespace vapp {
+class MotoGameEvent;
+
+struct RecordedGameEvent {
+  vapp::MotoGameEvent *Event;     /* Event itself */
+  bool bPassed;                   /* Whether we have passed it */
+};
+
+}
+
+#include "xmscene/Scene.h"
 
 namespace vapp {
 
@@ -78,17 +98,37 @@ class MotoGameEvent {
   virtual void revert(MotoGame *p_pMotoGame);
   virtual GameEventType getType()                       = 0;
 
-  static MotoGameEvent* getUnserialized(DBuffer &Buffer);
+  virtual std::string toString() = 0;
+
+  static MotoGameEvent* getUnserialized(DBuffer &Buffer, bool bDisplayInformation = false);
   float getEventTime();
 
   protected:
   float m_fEventTime;
 };
 
+class MGE_PlayersDie : public MotoGameEvent {
+ public:
+  MGE_PlayersDie(float p_fEventTime);
+  MGE_PlayersDie(float p_fEventTime, bool p_bKilledByWrecker);
+  ~MGE_PlayersDie();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  bool m_bKilledByWrecker;
+};
+
 class MGE_PlayerDies : public MotoGameEvent {
  public:
   MGE_PlayerDies(float p_fEventTime);
-  MGE_PlayerDies(float p_fEventTime, bool p_bKilledByWrecker);
+  MGE_PlayerDies(float p_fEventTime, bool p_bKilledByWrecker, int i_player);
   ~MGE_PlayerDies();
 
   void doAction(MotoGame *p_pMotoGame);
@@ -97,14 +137,35 @@ class MGE_PlayerDies : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   bool m_bKilledByWrecker;
+  int  m_player;
+};
+
+class MGE_PlayersEnterZone : public MotoGameEvent {
+ public:
+  MGE_PlayersEnterZone(float p_fEventTime);
+  MGE_PlayersEnterZone(float p_fEventTime, Zone *p_zone);
+  ~MGE_PlayersEnterZone();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  Zone *m_zone;
 };
 
 class MGE_PlayerEntersZone : public MotoGameEvent {
  public:
   MGE_PlayerEntersZone(float p_fEventTime);
-  MGE_PlayerEntersZone(float p_fEventTime, Zone *p_zone);
+  MGE_PlayerEntersZone(float p_fEventTime, Zone *p_zone, int i_player);
   ~MGE_PlayerEntersZone();
 
   void doAction(MotoGame *p_pMotoGame);
@@ -113,6 +174,27 @@ class MGE_PlayerEntersZone : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
+ private:
+  Zone *m_zone;
+  int   m_player;
+};
+
+class MGE_PlayersLeaveZone : public MotoGameEvent {
+ public:
+  MGE_PlayersLeaveZone(float p_fEventTime);
+  MGE_PlayersLeaveZone(float p_fEventTime, Zone *p_zone);
+  ~MGE_PlayersLeaveZone();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
  private:
   Zone *m_zone;
 };
@@ -120,7 +202,7 @@ class MGE_PlayerEntersZone : public MotoGameEvent {
 class MGE_PlayerLeavesZone : public MotoGameEvent {
  public:
   MGE_PlayerLeavesZone(float p_fEventTime);
-  MGE_PlayerLeavesZone(float p_fEventTime, Zone *p_zone);
+  MGE_PlayerLeavesZone(float p_fEventTime, Zone *p_zone, int i_player);
   ~MGE_PlayerLeavesZone();
 
   void doAction(MotoGame *p_pMotoGame);
@@ -129,14 +211,36 @@ class MGE_PlayerLeavesZone : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   Zone *m_zone;
+  int   m_player;
+};
+
+class MGE_PlayersToucheEntity : public MotoGameEvent {
+ public:
+  MGE_PlayersToucheEntity(float p_fEventTime);
+  MGE_PlayersToucheEntity(float p_fEventTime, std::string p_entityID, bool p_bTouchedWithHead);
+  ~MGE_PlayersToucheEntity();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+    std::string m_entityID;
+    bool m_bTouchedWithHead;
 };
 
 class MGE_PlayerTouchesEntity : public MotoGameEvent {
  public:
   MGE_PlayerTouchesEntity(float p_fEventTime);
-  MGE_PlayerTouchesEntity(float p_fEventTime, std::string p_entityID, bool p_bTouchedWithHead);
+  MGE_PlayerTouchesEntity(float p_fEventTime, std::string p_entityID, bool p_bTouchedWithHead, int i_player);
   ~MGE_PlayerTouchesEntity();
 
   void doAction(MotoGame *p_pMotoGame);
@@ -145,9 +249,12 @@ class MGE_PlayerTouchesEntity : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
     std::string m_entityID;
     bool m_bTouchedWithHead;
+    int  m_player;
 };
 
 class MGE_EntityDestroyed : public MotoGameEvent {
@@ -163,6 +270,8 @@ class MGE_EntityDestroyed : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
   std::string EntityId();
 
  private:
@@ -183,6 +292,8 @@ class MGE_ClearMessages : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
 };
 
@@ -198,6 +309,8 @@ class MGE_PlaceInGameArrow : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_x, m_y;
   float m_angle;
@@ -215,6 +328,8 @@ class MGE_PlaceScreenarrow : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_x, m_y;
   float m_angle;
@@ -231,6 +346,8 @@ class MGE_HideArrow : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
 };
 
@@ -246,6 +363,8 @@ class MGE_Message : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_message;
 };
@@ -262,6 +381,8 @@ class MGE_MoveBlock : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
   float m_x, m_y;
@@ -279,6 +400,8 @@ class MGE_SetBlockPos : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
   float m_x, m_y;
@@ -296,14 +419,35 @@ class MGE_SetGravity : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_x, m_y;
 };
 
+class MGE_SetPlayersPosition : public MotoGameEvent {
+ public:
+  MGE_SetPlayersPosition(float p_fEventTime);
+  MGE_SetPlayersPosition(float p_fEventTime, float p_x, float p_y, bool p_bRight);
+  ~MGE_SetPlayersPosition();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  float m_x, m_y;
+  bool  m_bRight;
+};
+
 class MGE_SetPlayerPosition : public MotoGameEvent {
  public:
   MGE_SetPlayerPosition(float p_fEventTime);
-  MGE_SetPlayerPosition(float p_fEventTime, float p_x, float p_y, bool p_bRight);
+  MGE_SetPlayerPosition(float p_fEventTime, float p_x, float p_y, bool p_bRight, int i_player);
   ~MGE_SetPlayerPosition();
 
   void doAction(MotoGame *p_pMotoGame);
@@ -312,9 +456,31 @@ class MGE_SetPlayerPosition : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_x, m_y;
   bool  m_bRight;
+  int   m_player;
+};
+
+class MGE_AddForceToPlayer : public MotoGameEvent {
+ public:
+  MGE_AddForceToPlayer(float p_fEventTime);
+  MGE_AddForceToPlayer(float p_fEventTime, const Vector2f& i_force, int i_player);
+  ~MGE_AddForceToPlayer();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  Vector2f m_force;
+  int      m_player;
 };
 
 class MGE_SetEntityPos : public MotoGameEvent {
@@ -329,6 +495,8 @@ class MGE_SetEntityPos : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_entityID;
   float m_x, m_y;
@@ -346,6 +514,8 @@ class MGE_SetBlockCenter : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
   float m_x, m_y;
@@ -363,11 +533,39 @@ class MGE_SetBlockRotation : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
   float m_angle;
 };
 
+class MGE_SetDynamicBlockSelfRotation : public MotoGameEvent {
+ public:
+  MGE_SetDynamicBlockSelfRotation(float p_fEventTime);
+  MGE_SetDynamicBlockSelfRotation(float p_fEventTime,
+				  std::string p_blockID,
+				  float p_fPeriod,
+				  int   p_startTime,
+				  int   p_endTime);
+  ~MGE_SetDynamicBlockSelfRotation();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  std::string m_blockID;
+  float m_angle;
+  float m_fPeriod;
+  int   m_startTime;
+  int   m_endTime;
+};
+
 class MGE_SetDynamicEntityRotation : public MotoGameEvent {
  public:
   MGE_SetDynamicEntityRotation(float p_fEventTime);
@@ -386,6 +584,8 @@ class MGE_SetDynamicEntityRotation : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_entityID;
   float m_fInitAngle;
@@ -395,6 +595,32 @@ class MGE_SetDynamicEntityRotation : public MotoGameEvent {
   int   m_endTime;
 };
 
+class MGE_SetDynamicEntitySelfRotation : public MotoGameEvent {
+ public:
+  MGE_SetDynamicEntitySelfRotation(float p_fEventTime);
+  MGE_SetDynamicEntitySelfRotation(float p_fEventTime,
+				  std::string p_entityID,
+				  float p_fPeriod,
+				  int   p_startTime,
+				  int   p_endTime);
+  ~MGE_SetDynamicEntitySelfRotation();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  std::string m_entityID;
+  float m_angle;
+  float m_fPeriod;
+  int   m_startTime;
+  int   m_endTime;
+};
+
 class MGE_SetDynamicEntityTranslation : public MotoGameEvent {
  public:
   MGE_SetDynamicEntityTranslation(float p_fEventTime);
@@ -413,6 +639,8 @@ class MGE_SetDynamicEntityTranslation : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_entityID;
   float m_x, m_y;
@@ -433,6 +661,8 @@ class MGE_SetDynamicEntityNone : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_entityID;
 };
@@ -455,6 +685,8 @@ class MGE_SetDynamicBlockRotation : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
   float m_fInitAngle;
@@ -482,6 +714,8 @@ class MGE_SetDynamicBlockTranslation : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
   float m_x, m_y;
@@ -502,6 +736,8 @@ class MGE_SetDynamicBlockNone : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   std::string m_blockID;
 };
@@ -518,6 +754,8 @@ class MGE_CameraMove : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_moveX, m_moveY;
 };
@@ -534,6 +772,8 @@ class MGE_CameraZoom : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_zoom;
 };
@@ -550,10 +790,46 @@ class MGE_PenalityTime : public MotoGameEvent {
   static GameEventType SgetType();
   GameEventType getType();
 
+  std::string toString();
+
  private:
   float m_penalityTime;
 };
 
+class MGE_CameraRotate : public MotoGameEvent {
+ public:
+  MGE_CameraRotate(float p_fEventTime);
+  MGE_CameraRotate(float p_fEventTime, float p_angle);
+  ~MGE_CameraRotate();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+  float m_angle;
+};
+
+class MGE_CameraAdaptToGravity : public MotoGameEvent {
+ public:
+  MGE_CameraAdaptToGravity(float p_fEventTime);
+  ~MGE_CameraAdaptToGravity();
+
+  void doAction(MotoGame *p_pMotoGame);
+  void serialize(DBuffer &Buffer);
+  void unserialize(DBuffer &Buffer);
+  static GameEventType SgetType();
+  GameEventType getType();
+
+  std::string toString();
+
+ private:
+};
+
 }
 
 #endif /* __MOTOGAMEEVENT_H__ */
