@@ -30,60 +30,67 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "Sound.h"
 #include "PhysSettings.h"
 #include "Input.h"
+#include "db/xmDatabase.h";
 
-#if defined(SUPPORT_WEBACCESS)
-  #include <curl/curl.h>
-#endif
+#include <curl/curl.h>
+
+#define DATABASE_FILE vapp::FS::getUserDirUTF8() + "/" + "xm.db"
 
 namespace vapp {
 
-  /*===========================================================================
-  Pre-initialize game
-  ===========================================================================*/
-  void GameApp::userPreInit(void) {
-    /* Config */
-    _CreateDefaultConfig();
-    m_Config.loadFile();
-  }
-  
   /*===========================================================================
   Select display mode
   ===========================================================================*/
   void GameApp::selectDisplayMode(int *pnWidth,int *pnHeight,int *pnBPP,bool *pbWindowed) {
-    if(!isCmdDispWidth() && !isCmdDispHeight()) {
-      *pnWidth = m_Config.getInteger("DisplayWidth");
-      *pnHeight = m_Config.getInteger("DisplayHeight");
-    }
-    
-    if(!isCmdDispBPP()) {
-      *pnBPP = m_Config.getInteger("DisplayBPP");
-    }
-    
-    if(!isCmdDispWindowed()) {
-      *pbWindowed = m_Config.getBool("DisplayWindowed");
-    }
+    *pnWidth = m_Config.getInteger("DisplayWidth");
+    *pnHeight = m_Config.getInteger("DisplayHeight");
+    *pnBPP = m_Config.getInteger("DisplayBPP");
+    *pbWindowed = m_Config.getBool("DisplayWindowed");
   }  
   
+  std::string GameApp::selectDrawLibMode() {
+    return m_Config.getString("DrawLib");
+  }
+
   /*===========================================================================
   Update loading screen
   ===========================================================================*/
-  void GameApp::_UpdateLoadingScreen(float fDone,Texture *pLoadingScreen,const std::string &NextTask) {
-    if(pLoadingScreen != NULL) {
-      glClear(GL_COLOR_BUFFER_BIT);
-      drawImage(Vector2f(getDispWidth()/2 - 256,getDispHeight()/2 - 40),
-                Vector2f(getDispWidth()/2 + 256,getDispHeight()/2 + 40),
-                pLoadingScreen,MAKE_COLOR(255,255,255,255));
-      drawBox(Vector2f(getDispWidth()/2 + 256 - (512.0f*(1-fDone)),getDispHeight()/2 - 40),              
-              Vector2f(getDispWidth()/2 + 256,getDispHeight()/2 - 25),              
-              0,MAKE_COLOR(0,0,0,128));
-      drawBox(Vector2f(getDispWidth()/2 + 256 - (512.0f*(1-fDone)),getDispHeight()/2 + 25),              
-              Vector2f(getDispWidth()/2 + 256,getDispHeight()/2 + 40),              
-              0,MAKE_COLOR(0,0,0,128));
-             
-      drawText(Vector2f(getDispWidth()/2 - 256,getDispHeight()/2 + 40 + 3),NextTask);
-              
-      SDL_GL_SwapBuffers();
-    }
+  void GameApp::_UpdateLoadingScreen(float fDone, const std::string &NextTask) {
+    FontManager* v_fm;
+    FontGlyph* v_fg;
+    int v_border = 3;
+    int v_fh;
+
+    getDrawLib()->clearGraphics();
+    getDrawLib()->resetGraphics();
+
+    v_fm = getDrawLib()->getFontBig();
+    v_fg = v_fm->getGlyph(GAMETEXT_LOADING);
+    v_fh = v_fg->realHeight();
+    v_fm->printString(v_fg,
+		      getDrawLib()->getDispWidth()/2 - 256,
+		      getDrawLib()->getDispHeight()/2 - 30,
+		      MAKE_COLOR(255,255,255, 255));
+
+    getDrawLib()->drawBox(Vector2f(getDrawLib()->getDispWidth()/2 - 256,
+				   getDrawLib()->getDispHeight()/2 - 30),              
+			  Vector2f(getDrawLib()->getDispWidth()/2 - 256 + (512.0f*fDone),
+				   getDrawLib()->getDispHeight()/2 - 30 + v_border),
+			  0,MAKE_COLOR(255,255,255,255));
+
+    getDrawLib()->drawBox(Vector2f(getDrawLib()->getDispWidth()/2 - 256,
+				   getDrawLib()->getDispHeight()/2 -30 + v_fh),              
+			  Vector2f(getDrawLib()->getDispWidth()/2 - 256 + (512.0f*fDone),
+				   getDrawLib()->getDispHeight()/2 - 30 + v_fh + v_border),
+			  0,MAKE_COLOR(255,255,255,255));
+    
+    v_fm = getDrawLib()->getFontSmall();
+    v_fg = v_fm->getGlyph(NextTask);
+    v_fm->printString(v_fg,
+		      getDrawLib()->getDispWidth()/2 - 256,
+		      getDrawLib()->getDispHeight()/2 -30 + v_fh + 2,
+		      MAKE_COLOR(255,255,255,255));      
+    getDrawLib()->flushGraphics();
   }
   
   /*===========================================================================
@@ -108,56 +115,28 @@ namespace vapp {
     /* Init some config */
     _UpdateSettings();
     
-    /* load theme */
-    m_themeChoicer = new ThemeChoicer(
-#if defined(SUPPORT_WEBACCESS)
-              this,
-              &m_ProxySettings
-#endif
-              );
-    reloadTheme();
-
-    /* Profiles */
-    Log("Loading profiles...");
-    m_Profiles.loadFile();
-    Log(" %d profile%s loaded",m_Profiles.getProfiles().size(),m_Profiles.getProfiles().size()==1?"":"s");
-
     /* Select profile */
-    m_pPlayer = NULL;
+    m_profile = "";
     if(m_ForceProfile != "") {
-      m_pPlayer = m_Profiles.getProfile(m_ForceProfile);
-      if(m_pPlayer == NULL)
-        Log("** Warning ** : unknown profile '%s'",m_ForceProfile.c_str());       
+      m_profile = m_ForceProfile;
     }
-    if(m_pPlayer == NULL)
-      m_pPlayer = m_Profiles.getProfile(m_Config.getString("DefaultProfile"));
-    if(m_pPlayer == NULL && !m_Profiles.getProfiles().empty()) {
-      /* OK, use the first then */
-      m_pPlayer = m_Profiles.getProfiles()[0];
-    }
-    
-    /* Update stats */
-    m_GameStats.loadXML("stats.xml");
-    if(m_pPlayer != NULL)
-      m_GameStats.xmotoStarted(m_pPlayer->PlayerName);
-   
+    if(m_profile == "") {
+      m_profile = m_Config.getString("DefaultProfile");
+    }     
+
     /* Init sound system */
-    if(!isNoGraphics()) {
-      Log("Initializing sound system...");
+    if(m_useGraphics) {
       Sound::init(&m_Config);
       if(!Sound::isEnabled()) {
-        Log(" (sound is disabled)\n");
       }    
     }
-              
+      
     /* Init renderer */
     m_Renderer.setParent( (App *)this );
     m_Renderer.setGameObject( &m_MotoGame );        
     m_Renderer.setDebug( m_bDebugMode );
 
-#if defined(ALLOW_GHOST)
     m_Renderer.setGhostMotionBlur( m_bGhostMotionBlur );
-#endif
     
     /* Tell collision system whether we want debug-info or not */
     m_MotoGame.getCollisionHandler()->setDebug( m_bDebugMode );
@@ -167,119 +146,141 @@ namespace vapp {
 
     if(m_GraphDebugInfoFile != "") m_Renderer.loadDebugInfo(m_GraphDebugInfoFile);
 
-    Texture *pLoadingScreen = NULL;
-    if(!isNoGraphics()) {    
-      /* Show loading screen */
-      pSprite = m_theme.getSprite(SPRITE_TYPE_UI, "Loading");
-
-      if(pSprite != NULL) {
-	pLoadingScreen = pSprite->getTexture(false, true);
-      }
+    /* database */
+    m_db = new xmDatabase(DATABASE_FILE,
+			  m_profile == "" ? std::string("") : m_profile,
+			  FS::getDataDir(), FS::getUserDir(), FS::binCheckSum(),
+			  m_useGraphics ? this : NULL);
+    if(m_sqlTrace) {
+      m_db->setTrace(m_sqlTrace);
     }
 
-    /* Update replays */
-    _UpdateLoadingScreen((1.0f/9.0f) * 0,pLoadingScreen,GAMETEXT_LOADINGREPLAYS);
-    
+    /* load theme */
+    m_themeChoicer = new ThemeChoicer(
+              this,
+              &m_ProxySettings
+              );
+    m_themeChoicer->setURLBase(m_Config.getString("WebThemesURLBase"));
+    if(m_db->themes_isIndexUptodate() == false) {
+      m_themeChoicer->initThemesFromDir(m_db);
+    }
     try {
-      m_ReplayList.initFromCache();
+      reloadTheme();
     } catch(Exception &e) {
-      m_ReplayList.initFromDir();
-    }    
+      /* if the theme cannot be loaded, try to reload from files */
+      /* perhaps that the xm.db comes from an other computer */
+      Log("** warning ** : Theme cannot be reload, try to update themes into the database");
+      m_themeChoicer->initThemesFromDir(m_db);
+      reloadTheme();
+    }
+
+    /* Update stats */
+    if(m_profile != "")
+      m_db->stats_xmotoStarted(m_profile);
+
+    /* load levels */
+    if(m_db->levels_isIndexUptodate() == false) {
+      m_levelsManager.reloadLevelsFromLvl(m_db, m_useGraphics ? this : NULL);
+    }
+    m_levelsManager.reloadExternalLevels(m_db, m_useGraphics ? this : NULL);
+
+    /* Update replays */
     
+    if(m_db->replays_isIndexUptodate() == false) {
+      initReplaysFromDir();
+    }
+
     /* List replays? */  
     if(m_bListReplays) {
-      std::vector<ReplayInfo *> *Replays = m_ReplayList.findReplays();
+      char **v_result;
+      int nrow;
+
       printf("\nReplay                    Level                     Player\n");
       printf("-----------------------------------------------------------------------\n");
-      for(int i=0;i<Replays->size();i++) {
-	std::string LevelDesc;
-	
-	if((*Replays)[i]->Level.length() == 6 &&
-	   (*Replays)[i]->Level[0] == '_' && (*Replays)[i]->Level[1] == 'i' &&
-	   (*Replays)[i]->Level[2] == 'L' && (*Replays)[i]->Level[5] == '_') {
-	  int nNum;
-	  sscanf((*Replays)[i]->Level.c_str(),"_iL%d_",&nNum);
-	  char cBuf[256];
-	  sprintf(cBuf,"#%d",nNum+1);
-	  LevelDesc = cBuf;
+
+      v_result = m_db->readDB("SELECT a.name, a.id_profile, b.name "
+			      "FROM replays AS a INNER JOIN levels AS b "
+			      "ON a.id_level = b.id_level;", nrow);
+      if(nrow == 0) {
+	printf("(none)\n");
+      } else {
+	for(unsigned int i=0; i<nrow; i++) {
+	  //m_db->getResult(v_result, 4, i, 0)
+	  printf("%-25s %-25s %-25s\n",
+		 m_db->getResult(v_result, 3, i, 0),
+		 m_db->getResult(v_result, 3, i, 2),
+		 m_db->getResult(v_result, 3, i, 1)
+		 );
 	}
-	else LevelDesc = (*Replays)[i]->Level;
-	
-	printf("%-25s %-25s %-25s\n",
-	       (*Replays)[i]->Name.c_str(),
-	       LevelDesc.c_str(),
-	       (*Replays)[i]->Player.c_str());
       }
-	if(Replays->empty()) printf("(none)\n");
-      delete Replays;
+      m_db->read_DB_free(v_result);
       quit();
       return;
     }
     
-    if(!isNoGraphics()) {  
-      _UpdateLoadingScreen((1.0f/9.0f) * 0,pLoadingScreen,GAMETEXT_LOADINGSOUNDS);
+    if(m_bDisplayInfosReplay) {
+      Replay v_replay;
+      std::string v_levelId;
+      std::string v_player;
+      
+      v_levelId = v_replay.openReplay(m_InfosReplay, v_player, true);
+      if(v_levelId == "") {
+	throw Exception("Invalid replay");
+      }
+      
+      quit();
+      return;	
+    }
+    
+    if(m_useGraphics) {  
+      _UpdateLoadingScreen((1.0f/9.0f) * 0,GAMETEXT_LOADINGSOUNDS);
       
       if(Sound::isEnabled()) {
         /* Load sounds */
-        Sound::loadSample("Sounds/NewHighscore.ogg");
-        //m_pDieSFX = Sound::loadSample("Sounds/Die.ogg");
-        
-        Sound::loadSample("Sounds/Button1.ogg");
-//        Sound::loadSample("Sounds/Button2.ogg");
-        Sound::loadSample("Sounds/Button3.ogg");
-        
-        Sound::loadSample("Sounds/PickUpStrawberry.ogg");        
-        Sound::loadSample("Sounds/Headcrash.ogg");
-        //Sound::loadSample("Sounds/Squeek.ogg");
-
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/00.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/01.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/02.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/03.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/04.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/05.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/06.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/07.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/08.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/09.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/10.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/11.wav"));
-        m_EngineSound.addBangSample(Sound::loadSample("Sounds/Engine/12.wav"));
-        
+	try {
+	  for(unsigned int i=0; i<m_theme.getSoundsList().size(); i++) {
+	    Sound::loadSample(m_theme.getSoundsList()[i]->FilePath());
+	  }
+	} catch(Exception &e) {
+	  Log("*** Warning *** : %s\n", e.getMsg().c_str());
+	  /* hum, not cool */
+	}
+	
         Log(" %d sound%s loaded",Sound::getNumSamples(),Sound::getNumSamples()==1?"":"s");
       }
 
-      _UpdateLoadingScreen((1.0f/9.0f) * 1,pLoadingScreen,GAMETEXT_INITTEXT);
+      _UpdateLoadingScreen((1.0f/9.0f) * 1,GAMETEXT_INITTEXT);
           
       /* Find all files in the textures dir and load them */     
-      UITextDraw::initTextDrawing(this);
       UITexture::setApp(this);
+      UIWindow::setDrawLib(getDrawLib());
+      m_sysMsg.setDrawLib(getDrawLib());
 
-      _UpdateLoadingScreen((1.0f/9.0f) * 3,pLoadingScreen,GAMETEXT_LOADINGMENUGRAPHICS);
+      _UpdateLoadingScreen((1.0f/9.0f) * 2,GAMETEXT_LOADINGMENUGRAPHICS);
         
       /* Load title screen textures + cursor + stuff */
       m_pTitleBL = NULL;
       pSprite = m_theme.getSprite(SPRITE_TYPE_UI, "TitleBL");
       if(pSprite != NULL) {
-        m_pTitleBL = pSprite->getTexture(false, true);
+        m_pTitleBL = pSprite->getTexture(false, true, FM_LINEAR);
       }
 
       m_pTitleBR = NULL;
       pSprite = m_theme.getSprite(SPRITE_TYPE_UI, "TitleBR");
       if(pSprite != NULL) {
-        m_pTitleBR = pSprite->getTexture(false, true);
+        m_pTitleBR = pSprite->getTexture(false, true, FM_LINEAR);
       }
 
       m_pTitleTL = NULL;
       pSprite = m_theme.getSprite(SPRITE_TYPE_UI, "TitleTL");
       if(pSprite != NULL) {
-        m_pTitleTL = pSprite->getTexture(false, true);
+        m_pTitleTL = pSprite->getTexture(false, true, FM_LINEAR);
       }
 
       m_pTitleTR = NULL;
       pSprite = m_theme.getSprite(SPRITE_TYPE_UI, "TitleTR");
       if(pSprite != NULL) {
-        m_pTitleTR = pSprite->getTexture(false, true);
+        m_pTitleTR = pSprite->getTexture(false, true, FM_LINEAR);
       }
 
       m_pCursor = NULL;
@@ -288,100 +289,104 @@ namespace vapp {
         m_pCursor = pSprite->getTexture(false, true, FM_LINEAR);
       }
 
-#if defined(SUPPORT_WEBACCESS)  
-      m_pNewLevelsAvailIcon = NULL;
-      pSprite = m_theme.getSprite(SPRITE_TYPE_UI, "NewLevelsAvailable");
-      if(pSprite != NULL) {
-        m_pNewLevelsAvailIcon = pSprite->getTexture(false, true, FM_LINEAR);
+      /* Fetch highscores from web? */
+      if(m_pWebHighscores != NULL) delete m_pWebHighscores;
+      m_pWebHighscores = new WebRoom(&m_ProxySettings);      
+      m_pWebHighscores->setWebsiteInfos(m_WebHighscoresIdRoom,
+					m_WebHighscoresURL);
+      
+    if(m_bEnableWebHighscores && m_PlaySpecificLevelFile == "" && m_PlaySpecificReplay == "") {  
+      bool bSilent = true;
+      try {
+	if(m_bEnableCheckHighscoresAtStartup) {
+	  _UpdateLoadingScreen((1.0f/9.0f) * 3,GAMETEXT_DLHIGHSCORES);      
+	  _UpdateWebHighscores(bSilent);
+	  _UpgradeWebHighscores();
+	}
+      } catch(Exception &e) {
+	/* No internet connection, probably... (just use the latest times, if any) */
+	Log("** Warning ** : Failed to update web-highscores [%s]",e.getMsg().c_str());              
+	if(!bSilent)
+	  notifyMsg(GAMETEXT_FAILEDDLHIGHSCORES);
       }
-#endif
-
-      _UpdateLoadingScreen((1.0f/9.0f) * 4,pLoadingScreen,GAMETEXT_LOADINGLEVELS);
+      
+      if(m_bEnableCheckNewLevelsAtStartup) {
+	try {
+	  _UpdateLoadingScreen((1.0f/9.0f) * 4,GAMETEXT_DLLEVELSCHECK);      
+	  _UpdateWebLevels(bSilent);       
+	} catch(Exception &e) {
+	  Log("** Warning ** : Failed to update web-levels [%s]",e.getMsg().c_str());              
+	  if(!bSilent)
+	    notifyMsg(GAMETEXT_FAILEDDLHIGHSCORES);
+	}
+      }
+      
     }
-    
+
+      if(m_pWebRooms != NULL) delete m_pWebRooms;
+      m_pWebRooms = new WebRooms(&m_ProxySettings);
+      
+    }
+
     /* Test level cache directory */
-    LevelsManager::checkPrerequires(m_bEnableLevelCache);
-    
+    if(m_useGraphics) {  
+      _UpdateLoadingScreen((1.0f/9.0f) * 5,GAMETEXT_LOADINGLEVELS);
+    }
+    LevelsManager::checkPrerequires();
+    m_levelsManager.makePacks(m_db,
+			      m_profile,
+			      m_Config.getString("WebHighscoresIdRoom"),
+			      m_bDebugMode);     
+
     /* Should we clean the level cache? (can also be done when disabled) */
     if(m_bCleanCache) {
       LevelsManager::cleanCache();
     }
-     
-    try {
-      m_levelsManager.loadLevelsFromIndex();
-    } catch(Exception &e) {
-      Log(std::string("Unable to load levels from index:\n" + e.getMsg()).c_str());
-      m_levelsManager.reloadLevelsFromFiles(m_bEnableLevelCache);
-    }
-    
+
     /* -listlevels? */
     if(m_bListLevels) {
-      m_levelsManager.printLevelsList();
+      m_levelsManager.printLevelsList(m_db);
     }
 
-    _UpdateLoadingScreen((1.0f/9.0f) * 5,pLoadingScreen,GAMETEXT_INITRENDERER);
-    
+    if(m_useGraphics) {  
+      _UpdateLoadingScreen((1.0f/9.0f) * 6,GAMETEXT_INITRENDERER);
+    }    
+
     if(m_bListLevels) {
       quit();
       return;
     }
-    
-    #if defined(SUPPORT_WEBACCESS)    
-      /* Fetch highscores from web? */
-      if(m_pWebHighscores != NULL) delete m_pWebHighscores;
-      m_pWebHighscores = new WebRoom(&m_ProxySettings);      
-      m_pWebHighscores->setWebsiteURL(m_Config.getString("WebHighscoresURL"));
-      
-      if(m_bEnableWebHighscores) {  
-  bool bSilent = true;
-          
-  try {
-    if(m_bEnableCheckHighscoresAtStartup) {
-      _UpdateLoadingScreen((1.0f/9.0f) * 6,pLoadingScreen,GAMETEXT_DLHIGHSCORES);      
-      _UpdateWebHighscores(bSilent);
-    }
-    if(m_bEnableCheckNewLevelsAtStartup) {
-      _UpdateLoadingScreen((1.0f/9.0f) * 6,pLoadingScreen,GAMETEXT_DLLEVELSCHECK);      
-      _UpdateWebLevels(bSilent);       
-    }
-  } catch(Exception &e) {
-    /* No internet connection, probably... (just use the latest times, if any) */
-    Log("** Warning ** : Failed to update web-highscores [%s]",e.getMsg().c_str());              
-    if(!bSilent)
-      notifyMsg(GAMETEXT_FAILEDDLHIGHSCORES);
-  }
-      }
 
-      _UpgradeWebHighscores();
-
-    if(m_pWebRooms != NULL) delete m_pWebRooms;
-    m_pWebRooms = new WebRooms(&m_ProxySettings);
-    _UpgradeWebRooms(false);
-
-    #endif
-        
-    if(!isNoGraphics()) {
+    if(m_useGraphics) {
       /* Initialize renderer */
       m_Renderer.init();
-      _UpdateLoadingScreen((1.0f/9.0f) * 7,pLoadingScreen,GAMETEXT_INITMENUS);
+      _UpdateLoadingScreen((1.0f/9.0f) * 7,GAMETEXT_INITMENUS);
       
       /* Initialize menu system */
       _InitMenus();    
-      _UpdateLoadingScreen((1.0f/9.0f) * 8,pLoadingScreen,GAMETEXT_UPDATINGLEVELS);
+      _UpdateLoadingScreen((1.0f/9.0f) * 8,GAMETEXT_UPDATINGLEVELS);
 
       _UpdateLevelsLists();
-      _UpdateLoadingScreen((1.0f/9.0f) * 9,pLoadingScreen,GAMETEXT_INITINPUT);      
+      _UpdateLoadingScreen((1.0f/9.0f) * 9,GAMETEXT_INITINPUT);      
       
       /* Init input system */
       m_InputHandler.init(&m_Config);
     }
         
     /* What to do? */
-    if(m_PlaySpecificLevel != "" && !isNoGraphics()) {
+    if(m_PlaySpecificLevelFile != "") {
+      try {
+	m_levelsManager.addExternalLevel(m_db, m_PlaySpecificLevelFile);
+	m_PlaySpecificLevel = m_levelsManager.LevelByFileName(m_db, m_PlaySpecificLevelFile);
+      } catch(Exception &e) {
+	m_PlaySpecificLevel = m_PlaySpecificLevelFile;
+      }
+    }
+    if((m_PlaySpecificLevel != "") && m_useGraphics) {
       /* ======= PLAY SPECIFIC LEVEL ======= */
       m_StateAfterPlaying = GS_MENU;
       setState(GS_PREPLAYING);
-      Log("Playing as '%s'...",m_pPlayer->PlayerName.c_str());
+      Log("Playing as '%s'...", m_profile.c_str());
     }
     else if(m_PlaySpecificReplay != "") {
       /* ======= PLAY SPECIFIC REPLAY ======= */
@@ -390,82 +395,95 @@ namespace vapp {
     }
     else {
       /* Graphics? */
-      if(isNoGraphics())
+      if(m_useGraphics == false)
         throw Exception("menu requires graphics");
         
       /* Do we have a player profile? */
-      if(m_pPlayer == NULL) {
+      if(m_profile == "") {
         setState(GS_EDIT_PROFILES);
       }
-#if defined(SUPPORT_WEBACCESS)
       else if(m_Config.getBool("WebConfAtInit")) {
         /* We need web-config */
         _InitWebConf();
         setState(GS_EDIT_WEBCONFIG);
       }
-#endif
       else {
         /* Enter the menu */
         setState(GS_MENU);
       }
-    }            
+    }
+
+    Log("UserInit ended at %.3f", App::getTime());
   }
     
   /*===========================================================================
   Shutdown game
   ===========================================================================*/
   void GameApp::userShutdown(void) {  
-    #if defined(SUPPORT_WEBACCESS)
-      if(m_pWebHighscores != NULL)
-        delete m_pWebHighscores;
+    if(m_pWebHighscores != NULL)
+    delete m_pWebHighscores;
         
-      if(m_pWebLevels != NULL)
-        delete m_pWebLevels;
-
-      if(m_pWebRooms != NULL)
-      delete m_pWebRooms;  
-    #endif
-  
+    if(m_pWebLevels != NULL)
+    delete m_pWebLevels;
+    
+    if(m_pWebRooms != NULL)
+    delete m_pWebRooms;  
+    
     if(m_pCredits != NULL)
-      delete m_pCredits;
-  
-    m_levelsManager.saveXml();
-    m_GameStats.saveXML("stats.xml");
-      
-    if(!isNoGraphics()) {
+    delete m_pCredits;
+    
+    if(m_useGraphics) {
       m_Renderer.unprepareForNewLevel(); /* just to be sure, shutdown can happen quite hard */
       m_Renderer.shutdown();
       m_InputHandler.uninit();
     }
-
+    
     delete m_themeChoicer;
     
-    if(m_pReplay != NULL)
-      delete m_pReplay;
-
-#if defined(ALLOW_GHOST)
-      if(m_pGhostReplay != NULL)
-      delete m_pGhostReplay;
-#endif  
-
-    if(m_pPlayer != NULL) 
-      m_Config.setString("DefaultProfile",m_pPlayer->PlayerName);
-
-    if(m_pMenuMusic != NULL) {
-      Mix_FreeMusic(m_pMenuMusic);
-    }
-
+    if(m_pJustPlayReplay != NULL)
+    delete m_pJustPlayReplay;
+    
+    if(m_profile != "") 
+    m_Config.setString("DefaultProfile", m_profile);
+    
     Sound::uninit();
+    
+    m_Config.setInteger("QSQualityMIN",    m_pQuickStart->getQualityMIN());
+    m_Config.setInteger("QSDifficultyMIN", m_pQuickStart->getDifficultyMIN());
+    m_Config.setInteger("QSQualityMAX",    m_pQuickStart->getQualityMAX());
+    m_Config.setInteger("QSDifficultyMAX", m_pQuickStart->getDifficultyMAX());
 
     m_Config.saveFile();
-
-    m_Profiles.saveFile();
-
-    if(!isNoGraphics()) {
-      UITextDraw::uninitTextDrawing();  
-    }
   }  
   
+  void GameApp::PlaySpecificLevel(std::string i_level) {
+    m_PlaySpecificLevel = i_level;
+          
+    /* If it is a plain number, it's for a internal level */
+    bool v_isANumber =  true;
+    for(int i=0; i<m_PlaySpecificLevel.length(); i++) {
+      if(m_PlaySpecificLevel[i] < '0' || m_PlaySpecificLevel[i] > '9') {
+	v_isANumber = false;
+      }
+    }
+    if(v_isANumber) {
+      int nNum = atoi(m_PlaySpecificLevel.c_str());
+      if(nNum > 0) {
+	char cBuf[256];
+	sprintf(cBuf,"_iL%02d_",nNum-1);
+	m_PlaySpecificLevel = cBuf;
+      }
+    }
+  }
+
+  void GameApp::PlaySpecificReplay(std::string i_replay) {
+    m_PlaySpecificReplay = i_replay;
+  }
+
+  void GameApp::PlaySpecificLevelFile(std::string i_levelFile) {
+    m_PlaySpecificLevelFile = i_levelFile;
+  }
+
   /*===========================================================================
   Handle a command-line passed argument
   ===========================================================================*/
@@ -474,7 +492,7 @@ namespace vapp {
     for(int i=0;i<UserArgs.size();i++) {
       if(UserArgs[i] == "-replay") {
         if(i+1<UserArgs.size()) {
-          m_PlaySpecificReplay = UserArgs[i+1];
+          PlaySpecificReplay(UserArgs[i+1]);
         }
         else
           throw SyntaxError("no replay specified");        
@@ -482,31 +500,24 @@ namespace vapp {
       }
       else if(UserArgs[i] == "-level") {
         if(i+1<UserArgs.size()) {
-          m_PlaySpecificLevel = UserArgs[i+1];
-          
-          /* If it is a plain number, it's for a internal level */
-    bool v_isANumber =  true;
-    for(int i=0; i<m_PlaySpecificLevel.length(); i++) {
-      if(m_PlaySpecificLevel[i] < '0' || m_PlaySpecificLevel[i] > '9') {
-        v_isANumber = false;
+	  PlaySpecificLevel(UserArgs[i+1]);
+        } else
+	throw SyntaxError("no level specified");        
+        i++;
       }
-    }
-    if(v_isANumber) {
-      int nNum = atoi(m_PlaySpecificLevel.c_str());
-      if(nNum > 0) {
-        char cBuf[256];
-        sprintf(cBuf,"_iL%02d_",nNum-1);
-        m_PlaySpecificLevel = cBuf;
-      }
-    }
-        }
-        else
-          throw SyntaxError("no level specified");        
+      else if(UserArgs[i] == "-levelFile") {
+        if(i+1<UserArgs.size()) {
+	  PlaySpecificLevelFile(UserArgs[i+1]);
+        } else
+	throw SyntaxError("no level file specified");        
         i++;
       }
       else if(UserArgs[i] == "-debug") {
         m_bDebugMode = true;
       }
+      else if(UserArgs[i] == "-sqlTrace") {
+	m_sqlTrace = true;
+      }
       else if(UserArgs[i] == "-profile") {
         if(i+1<UserArgs.size())
           m_ForceProfile = UserArgs[i+1];
@@ -523,11 +534,11 @@ namespace vapp {
       }      
       else if(UserArgs[i] == "-listlevels") {
         m_bListLevels = true;
-        setNoGraphics(true);
+	m_useGraphics = false;
       }
       else if(UserArgs[i] == "-listreplays") {
         m_bListReplays = true;
-        setNoGraphics(true);
+	m_useGraphics = false;
       }
       else if(UserArgs[i] == "-timedemo") {
         m_bTimeDemo = true;
@@ -546,7 +557,28 @@ namespace vapp {
       }
       else if(UserArgs[i] == "-cleancache") {
         m_bCleanCache = true;
-      }
+     } else if(UserArgs[i] == "-replayInfos") {
+       if(i+1<UserArgs.size()) {
+	 m_useGraphics = false;
+	 m_bDisplayInfosReplay = true;
+	 m_InfosReplay = UserArgs[i+1];
+       } else
+       throw SyntaxError("no replay specified");        
+       i++;
+     } else {
+       /* check if the parameter is a file */
+       std::string v_extension = FS::getFileExtension(UserArgs[i]);
+
+       /* replays */
+       if(v_extension == "rpl") {
+	   PlaySpecificReplay(UserArgs[i]);
+       } else if(v_extension == "lvl") {
+	 PlaySpecificLevelFile(UserArgs[i]);
+       } else {
+	 /* unknown extension */
+	 throw SyntaxError("Invalid argument");
+       }
+     }
     }
   }
 
@@ -565,8 +597,8 @@ namespace vapp {
     printf("\t-ugly\n\t\tEnable 'ugly' mode, suitable for computers without\n");
              printf("\t\ta good OpenGL-enabled video card.\n");
     printf("\t-testTheme\n\t\tDisplay forms around the theme to check it.\n");
-    printf("\t-benchmark\n\t\tOnly meaningful when combined with -replay and\n");
-    printf("\t\t-timedemo. Useful to determine the graphics\n");
+    printf("\t-benchmark\n\t\tOnly meaningful when combined with -replay\n");
+    printf("\t\tand -timedemo. Useful to determine the graphics\n");
              printf("\t\tperformance.\n");
     printf("\t-cleancache\n\t\tDeletes the content of the level cache.\n");
   }  
@@ -574,7 +606,8 @@ namespace vapp {
   /*===========================================================================
   Create the default config
   ===========================================================================*/
-  void GameApp::_CreateDefaultConfig(void) {
+  void GameApp::createDefaultConfig(void) {
+    m_Config.createVar( "Language" , "");
     m_Config.createVar( "Theme",                  THEME_DEFAULT_THEMENAME);    
 
     /* Display */
@@ -584,6 +617,7 @@ namespace vapp {
     m_Config.createVar( "DisplayWindowed",        "false" );
     m_Config.createVar( "MenuBackgroundGraphics", "High" );
     m_Config.createVar( "GameGraphics",           "High" );
+    m_Config.createVar( "DrawLib",                "OPENGL" );
         
     /* Audio */
     m_Config.createVar( "AudioEnable",            "true" );
@@ -599,9 +633,26 @@ namespace vapp {
     m_Config.createVar( "KeyFlipLeft1",           "Left" );
     m_Config.createVar( "KeyFlipRight1",          "Right" );
     m_Config.createVar( "KeyChangeDir1",          "Space" );
+    m_Config.createVar( "ControllerMode2",        "Keyboard" );
+    m_Config.createVar( "KeyDrive2",              "A" );
+    m_Config.createVar( "KeyBrake2",              "Q" );
+    m_Config.createVar( "KeyFlipLeft2",           "Z" );
+    m_Config.createVar( "KeyFlipRight2",          "E" );
+    m_Config.createVar( "KeyChangeDir2",          "W" );
+    m_Config.createVar( "ControllerMode3",        "Keyboard" );
+    m_Config.createVar( "KeyDrive3",              "R" );
+    m_Config.createVar( "KeyBrake3",              "F" );
+    m_Config.createVar( "KeyFlipLeft3",           "T" );
+    m_Config.createVar( "KeyFlipRight3",          "Y" );
+    m_Config.createVar( "KeyChangeDir3",          "V" );
+    m_Config.createVar( "ControllerMode4",        "Keyboard" );
+    m_Config.createVar( "KeyDrive4",              "Y" );
+    m_Config.createVar( "KeyBrake4",              "H" );
+    m_Config.createVar( "KeyFlipLeft4",           "U" );
+    m_Config.createVar( "KeyFlipRight4",          "I" );
+    m_Config.createVar( "KeyChangeDir4",          "N" );
     
     m_Config.createVar( "AutosaveHighscoreReplays", "true");
-    m_Config.createVar("LimitFramerate", "false");
 
     #if defined(ENABLE_ZOOMING)
       m_Config.createVar( "KeyZoomIn",              "PageUp" );
@@ -611,8 +662,10 @@ namespace vapp {
       m_Config.createVar( "KeyCameraMoveXDown",     "Pad 4" );
       m_Config.createVar( "KeyCameraMoveYUp",       "Pad 8" );
       m_Config.createVar( "KeyCameraMoveYDown",     "Pad 2" );
+      m_Config.createVar( "KeyAutoZoom",            "Pad 5" );
     #endif
-        
+       
+    /* joystick */
     m_Config.createVar( "JoyIdx1",                "0" );
     m_Config.createVar( "JoyAxisPrim1",           "1" );
     m_Config.createVar( "JoyAxisPrimMax1",        "32760" );
@@ -625,7 +678,6 @@ namespace vapp {
     m_Config.createVar( "JoyAxisSecUL1",          "1024" );
     m_Config.createVar( "JoyAxisSecLL1",          "-1024" );
     m_Config.createVar( "JoyButtonChangeDir1",    "0" );
-    m_Config.createVar( "JoystickLimboArea",      "0.07" );
 
     /* Misc */
     m_Config.createVar( "DefaultProfile",         "" );
@@ -637,13 +689,11 @@ namespace vapp {
     m_Config.createVar( "StoreReplays",           "true" );
     m_Config.createVar( "ReplayFrameRate",        "25" );
     m_Config.createVar( "CompressReplays",        "true" );
-    m_Config.createVar( "LevelCache",             "true" );
     m_Config.createVar( "ContextHelp",            "true" );
     m_Config.createVar( "MenuMusic",              "true" );    
-    m_Config.createVar( "InitZoom",               "false" );
+    m_Config.createVar( "InitZoom",               "true" );
     m_Config.createVar( "DeathAnim",              "true" );
 
-#if defined(SUPPORT_WEBACCESS)
     m_Config.createVar( "WebHighscores",            "false" );
     m_Config.createVar( "CheckHighscoresAtStartup", "true" );
     m_Config.createVar( "CheckNewLevelsAtStartup",  "true" );
@@ -656,6 +706,7 @@ namespace vapp {
     m_Config.createVar( "WebThemesURL",           DEFAULT_WEBTHEMES_URL);
     m_Config.createVar( "WebThemesURLBase",       DEFAULT_WEBTHEMES_SPRITESURLBASE);
     m_Config.createVar( "WebRoomsURL",            DEFAULT_WEBROOMS_URL);
+    m_Config.createVar( "WebHighscoresIdRoom",     DEFAULT_WEBROOM_ID);
 
     /* Proxy */
     m_Config.createVar( "ProxyType",              "" ); /* (blank), HTTP, SOCKS4, or SOCKS5 */
@@ -665,20 +716,24 @@ namespace vapp {
     m_Config.createVar( "ProxyAuthPwd",           "" );
 
     /* auto upload */
-    m_Config.createVar( "WebHighscoreUploadURL"      , DEFAULT_UPLOADREPLAY_URL);
-    m_Config.createVar( "WebHighscoreUploadIdRoom"   , DEFAULT_WEBROOM_ID);
+    m_Config.createVar( "WebHighscoreUploadURL", DEFAULT_UPLOADREPLAY_URL);
     m_Config.createVar( "WebHighscoreUploadLogin"    , "");
     m_Config.createVar( "WebHighscoreUploadPassword" , ""); 
 
-#endif
-    
-#if defined(ALLOW_GHOST)
     m_Config.createVar( "EnableGhost"        , "true");
     m_Config.createVar( "GhostSearchStrategy", "0");
     m_Config.createVar( "ShowGhostTimeDiff"  , "true");
     m_Config.createVar( "DisplayGhostInfo"   , "false");
     m_Config.createVar( "GhostMotionBlur"    , "true" );
-#endif
+
+    /* quick start button */
+    m_Config.createVar("QSQualityMIN",    "1");
+    m_Config.createVar("QSDifficultyMIN", "1");
+    m_Config.createVar("QSQualityMAX",    "5");
+    m_Config.createVar("QSDifficultyMAX", "5");
+
+    /* multi */
+    m_Config.createVar("MultiStopWhenOneFinishes" , "true");
   }
   
 }
