@@ -30,6 +30,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Central place to keep many in-game text strings
 =============================================================================*/
 
+// set the web lang to use
+#define WEB_LANGUAGE _("en_EN")
+
 #define GAMETEXT_11KHZ                _("11 kHz")
 #define GAMETEXT_16BIT                _("16-bit")
 #define GAMETEXT_16BPP                _("16 bits per pixel")
@@ -37,7 +40,7 @@ Central place to keep many in-game text strings
 #define GAMETEXT_32BPP                _("32 bits per pixel")
 #define GAMETEXT_44KHZ                _("44 kHz")
 #define GAMETEXT_8BIT                 _("8-bit")
-#define GAMETEXT_ABORT                _("Abort Playing")
+#define GAMETEXT_ABORT                _("Quit level")
 #define GAMETEXT_ADDTOFAVORITE       _("Add to favorite")
 #define GAMETEXT_ACTION               _("Action")
 #define GAMETEXT_ALL                  _("All (on this computer)")
@@ -47,15 +50,15 @@ Central place to keep many in-game text strings
                                         "and best times of other players?")
 #define GAMETEXT_ALLRECORDS           _("All (on this computer)")
 #define GAMETEXT_ALREADYUSED          _("Already used!")
-#define GAMETEXT_AND                  _("and")
 #define GAMETEXT_AUDIO                _("Audio")
 #define GAMETEXT_AUTHOR               _("Author")
 #define GAMETEXT_AUTOSAVEREPLAYS      _("Automatic saving of replays")
+#define GAMETEXT_AUTOZOOM             _("Auto zoom")
 #define GAMETEXT_BESTTIMES            _("Best Times")
 #define GAMETEXT_BESTPLAYER           _("Best player")
 #define GAMETEXT_BRAKE                _("Brake")
 #define GAMETEXT_BUILTINLEVELS        _("Built-In Levels")
-#define GAMETEXT_BY                   _("by")
+#define GAMETEXT_BY_PLAYER            _("(by %s)")
 #define GAMETEXT_CAMERAMOVEXDOWN      _("Move Camera to left")
 #define GAMETEXT_CAMERAMOVEXUP        _("Move Camera to right")
 #define GAMETEXT_CAMERAMOVEYDOWN      _("Move Camera down")
@@ -69,68 +72,73 @@ Central place to keep many in-game text strings
 #define GAMETEXT_CLOSE                _("Close")
 #define GAMETEXT_CONFIGUREJOYSTICK    _("Configure Joystick...")
 #define GAMETEXT_CONTROLS             _("Controls")
-#define GAMETEXT_CREDITS              _("Lead Programming:Rasmus Neckelmann;" \
-                                        ":Nicolas Adenis-Lamarre;" \
-                                        ":;" \
-                                        "Additional Programming:Eric Piel;" \
-                                        ":Emmanuel Gorse;" \
-                                        ":Christoph Sommer;" \
-                                        ":Frederic Huard;" \
-                                        ":;" \
-                                        "Graphics:Rasmus Neckelmann;" \
-                                        ":;" \
-                                        "Audio:Jens Erler;" \
-                                        ":Brandon Ross;" \
-                                        ":;" \
-                                        "Built-in Levels:Rasmus Neckelmann;" \
-                                        ":Pasi Kallinen;" \
-                                        ":Germain Gagnerot;" \
-                                        ":Clive Crous;" \
-                                        ":Torben Green;" \
-                                        ":Thomas Kjaerulff;" \
-                                        ":;" \
-                                        "MacOS X Support:Dave Vasilevsky;" \
-                                        ":Maxime Biais;" \
-                                        ":;" \
-                                        "Website Programming:Nicolas Adenis-Lamarre;" \
-                                        ":Nx;" \
-                                        ":;" \
-                                        "Website Administration:Nicolas Adenis-Lamarre;" \
-                                        ":Valentin;" \
-                                        ":Nx;" \
-                                        ":Alrj;" \
-                                        ":;" \
-                                        "Website Localization:Felix Schl;" \
-                                        ":Nicolas Adenis-Lamarre;" \
-                                        ":Trullo;" \
-                                        ":Afaland;" \
-                                        ":Jj;" \
-                                        ":Vertigo;" \
-                                        ":Pol Vinogradov;" \
-                                        ":;" \
-                                        "Forum:Valentin;" \
-                                        ":;" \
-                                        "Main testers:Jose Alberto Coelho;" \
-                                        ":Frederic Huard;" \
-                                        ":;" \
-                                        "Thanks to:SnowBear (for KDE/gnome integration);" \
-                                        ":Jonathan Sieber (for german translation);"\
-                                        ":Samuel Mimram (for Debian packaging);" \
-                                        ":Dark (for Gentoo packaging);" \
-                                        ":Olivier Blin (for Mandriva packaging);" \
-                                        ":Balazs Rozsa (for across/elma);" \
-                                        ":Yves Berquin (for making tinyxml);" \
-                                        ":L. Peter Deutsch (for the MD5 code);" \
-                                        ":Petter Reinholdtsen;" \
-                                        ":Jes Vestervang (for providing web space);" \
-                                        ":Kenneth (for being such a little girl);"\
-                                        ":Everyone who have made levels;" \
-                                        ":People who have provided bug reports;" \
-                                        ":Anyone who has helped in any way;")                                        
+
+#define GAMETEXT_CREDITS                                                                     \
+  std::string(_("Lead Programming")) +                                                       \
+    ":Rasmus Neckelmann;"                                                                    \
+    ":Nicolas Adenis-Lamarre;"                                                               \
+    ":Emmanuel Gorse;"                                                                       \
+    ":Kees Jongenburger;"                                                                    \
+    ":;"                                                                                     \
+    + std::string(_("Additional Programming")) +                                             \
+    ":Eric Piel;"                                                                            \
+    ":Christoph Sommer;"                                                                     \
+    ":Frederic Huard;"                                                                       \
+    ":Brice Goglin;"                                                                         \
+    ":;"                                                                                     \
+    + std::string(_("Graphics")) +                                                           \
+    ":Rasmus Neckelmann;"                                                                    \
+    ":Jens Erler;"                                                                           \
+    ":;"                                                                                     \
+    + std::string(_("Audio")) +                                                              \
+    ":Jens Erler;"                                                                           \
+    ":Brandon Ross;"                                                                         \
+    ":;"                                                                                     \
+    + std::string(_("MacOS X Support")) +                                                    \
+    ":Dave Vasilevsky;"                                                                      \
+    ":Maxime Biais;"                                                                         \
+    ":;"                                                                                     \
+    + std::string(_("Localization")) +                                                       \
+    ":Jozef Riha (" + std::string(_("Slovak")) + ");"                                        \
+    ":Nicolas Adenis-Lamarre (" + std::string(_("French")) + ");"                            \
+    ":Wilhelm Francke (" + std::string(_("Norwegian")) + ");"                                \
+    ":Jonathan Sieber and Jens Erler (" + std::string(_("German")) + ");"                    \
+    ":Maximum/Tuomo Koistinen (" + std::string(_("Finnish")) + ");"                                          \
+    ":Trullo (" + std::string(_("Catalan")) + ");"                                           \
+    ":Trullo (" + std::string(_("Spanish")) + ");"                                           \
+    ":Mateusz Kosibowicz (" + std::string(_("Polish")) + ");"                                \
+    ":Lucas Manzari (" + std::string(_("Italian")) + ");"                                    \
+    ":Lukas Klingsbo (" + std::string(_("Swedish")) + ");"                                   \
+    ":Jan Kaláb (" + std::string(_("Czech")) + ");"                                          \
+    ":" + std::string(_("Mikhail Brinchuk")) + " (" + std::string(_("Russian")) + ");"       \
+    ":Lucas Hermann Negri (" + std::string(_("Brazilian Portuguese")) + ");"                 \
+    ":Jānis Rūcis (" + std::string(_("Latvian")) + ");"                 \
+    ":;"                                                                                     \
+    + std::string(_("Main testers")) +                                                       \
+    ":Jose Alberto Coelho;"                                                                  \
+    ":Frederic Huard;"                                                                       \
+    ":Jens Erler;"                                                                           \
+    ":;"                                                                                     \
+    + std::string(_("Thanks to")) +                                                          \
+    ":SnowBear ("           + std::string(_("for KDE/gnome integration")) 	     + ");"  \
+    ":Samuel Mimram ("      + std::string(_("for Debian packaging"))      	     + ");"  \
+    ":Dark ("               + std::string(_("for Gentoo packaging"))      	     + ");"  \
+    ":Olivier Blin (" 	    + std::string(_("for Mandriva packaging"))    	     + ");"  \
+    ":Balazs Rozsa (" 	    + std::string(_("for across/elma"))           	     + ");"  \
+    ":Yves Berquin (" 	    + std::string(_("for making tinyxml"))        	     + ");"  \
+    ":L. Peter Deutsch ("   + std::string(_("for the MD5 code"))          	     + ");"  \
+    ":Petter Reinholdtsen;"                                                                  \
+    ":" 		    + std::string(_("Everyone who have made levels"))        + ";"   \
+    ":" 		    + std::string(_("People who have provided bug reports")) + ";"   \
+    ":" 		    + std::string(_("Anyone who has helped in any way"))     + ";"
+
 #define GAMETEXT_CREDITSBUTTON        _("Credits")
 #define GAMETEXT_CURPLAYER            _("Player")
 #define GAMETEXT_DATE                 _("Date")
-#define GAMETEXT_DEATHANIM            _("Bike Breaks")
+#define GAMETEXT_DB_UPGRADING               _("Upgrading database")
+#define GAMETEXT_DB_UPGRADING_STATS_PROFILE _("Upgrading database stats for the profile")
+#define GAMETEXT_DB_UPGRADING_PROFILE _("Upgrading database profile")
+#define GAMETEXT_DEATHANIM            _("Death Animation")
 #define GAMETEXT_DEFAULTS             _("Defaults")
 #define GAMETEXT_DELETEPLAYERMESSAGE  _("Do you really want to delete player?")
 #define GAMETEXT_DELETEREPLAYMESSAGE  _("Do you really want to delete replay?")
@@ -175,6 +183,7 @@ Central place to keep many in-game text strings
 #define GAMETEXT_FAILEDTOSAVEREPLAY   _("Failed to save replay!\nMaybe you should try with another name?")
 #define GAMETEXT_FAILEDUPDATETHEMESLIST _("Failed to update the theme list\nCheck your Internet connection!")
 #define GAMETEXT_FILE                 _("File")
+#define GAMETEXT_FILTER               _("Filter")
 #define GAMETEXT_FINISH               _("Finished!")
 #define GAMETEXT_FINISHTIME           _("Finish Time")
 #define GAMETEXT_FLIPLEFT             _("Flip left")
@@ -183,11 +192,13 @@ Central place to keep many in-game text strings
 #define GAMETEXT_GENERAL              _("General")
 #define GAMETEXT_GENERALINFO          _("General Info")
 #define GAMETEXT_GETSELECTEDTHEME     _("Get this theme")
+#define GAMETEXT_GHOST_LOCAL          _("Local best")
+#define GAMETEXT_GHOST_BEST           _("Your best")
 #define GAMETEXT_GHOST_STRATEGIES_TYPE  _("Type")
 #define GAMETEXT_GHOST_STRATEGY_BESTOFROOM _("The highscore of the room")
 #define GAMETEXT_GHOST_STRATEGY_MYBEST  _("The best of my replays (on this computer)")
 #define GAMETEXT_GHOST_STRATEGY_THEBEST _("The best of the replays (on this computer)")
-#define GAMETEXT_GHOSTOF              _("Ghost of")
+#define GAMETEXT_GHOSTOF              _("Ghost of %s")
 #define GAMETEXT_GHOSTTAB             _("Ghost")
 #define GAMETEXT_HELP                 _("Help")
 
@@ -202,12 +213,13 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 "  Esc - " + std::string(_("Pause")) + "\n" \
 "  Return - " + std::string(_("Restart the level")) + "\n" \
 "  F12 - " + std::string(_("Take a screenshot")) + "\n" \
+"  F5 - " + std::string(_("Reload levels, themes and replays from directories")) + "\n" \
 "\n" \
  + std::string(_("Find all the strawberries and touch the flower to finish\nthe level.")) + "\n" \
-"\n" \
- + std::string(_("Read the README file or check out the website at\nhttp://xmoto.sourceforge.net for more information."))
+ + std::string(_("Read the README file or check out the website at\nhttp://xmoto.tuxfamily.org for more information."))
 
 #define GAMETEXT_HIGH                 _("High")
+#define GAMETEXT_LOAD_LEVEL_HOOK      _("Levels are being added into the database. Please wait.")
 #define GAMETEXT_INETCONF             _("Internet Configuration")
 #define GAMETEXT_INITINPUT            _("Initializing input system...")
 #define GAMETEXT_INITMENUS            _("Initializing menus...")
@@ -223,12 +235,13 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_LEVEL                _("Level")
 #define GAMETEXT_LEVELINFO            _("Level Info...")
 #define GAMETEXT_LEVELNAME            _("Level Name")
-#define GAMETEXT_LEVELNOTFOUND        _("Level '%s' not found!")
+#define GAMETEXT_LEVELCANNOTBELOADED  _("Level '%s' cannot be loaded")
 #define GAMETEXT_LEVELPACK            _("Level Pack")
 #define GAMETEXT_LEVELPACKS           _("Level Packs")
 #define GAMETEXT_LEVELREQUIREDBYREPLAY _("Level '%s' required by replay!")
 #define GAMETEXT_LEVELS               _("Levels")
 #define GAMETEXT_LISTALL              _("List All")
+#define GAMETEXT_LOADING              _("Loading...")
 #define GAMETEXT_LOADINGLEVELS        _("Loading levels...")
 #define GAMETEXT_LOADINGMENUGRAPHICS  _("Loading menu graphics...")
 #define GAMETEXT_LOADINGREPLAYS       _("Loading replays...")
@@ -242,7 +255,11 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_MONO                 _("Mono")
 #define GAMETEXT_MOSTPLAYEDLEVELSFOLLOW _("Following are your most played levels")
 #define GAMETEXT_MOTIONBLURGHOST      _("Motion blur ghost")
+#define GAMETEXT_MULTI                _("Multi")
+#define GAMETEXT_MULTISTOPWHENONEFINISHES _("Stop the game once a player ends the level")
 #define GAMETEXT_NAME                 _("Name")
+#define GAMETEXT_NB_PLAYERS           _("Number of players")
+#define GAMETEXT_NEW                  _("New")
 #define GAMETEXT_NEWERXMOTOREQUIRED   _("X-Moto %s or newer required to load level")
 #define GAMETEXT_NEWHIGHSCORE         _("New highscore!")
 #define GAMETEXT_NEWHIGHSCOREPERSONAL _("New personal highscore!")
@@ -256,26 +273,34 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_NONEXTLEVEL          _("No level following this one, sorry.")
 #define GAMETEXT_NOSTATS              _("No statistics for this profile.")
 #define GAMETEXT_NOTFINISHED          _("Not finished")
-#define GAMETEXT_NOTIFYATINIT         _("Important note!\n"                                                \
-                                      "\n"                                                               \
-                                      "This is an in development version of X-Moto!\n"                                     \
-                                      "All kinds of feedback are highly appreciated, so the game\n"      \
-                                      "can get better.\n"                                                \
-                                      "Mail bugs, ideas, comments, feature requests, hatemail, etc\n"    \
-                                      "to neckelmann@gmail.com\n"                                        \
-                                      "\n"                                                               \
-                                      "Also visit http://xmoto.sourceforge.net to make sure you've\n"    \
+#define GAMETEXT_NOTIFYATINIT         _("Important note!\n" \
+                                      "\n" \
+                                      "This is an in development version of X-Moto!\n" \
+                                      "All kinds of feedback are highly appreciated, so the game\n" \
+                                      "can get better.\n" \
+                                      "Mail bugs, ideas, comments, feature requests, hatemail, etc\n" \
+                                      "to nicolas at adenis-lamarre dot fr\n" \
+                                      "\n" \
+                                      "Also visit http://xmoto.tuxfamily.org to make sure you've\n" \
                                       "got the latest version.")
 #define GAMETEXT_NUMLEVELS            _("# Levels")
 #define GAMETEXT_OK                   _("OK")  
 #define GAMETEXT_OPEN                 _("Open")
 #define GAMETEXT_OPTIONS              _("Options")
 #define GAMETEXT_OPTIONSREQURERESTART _("Some options will not take effect before next restart!")
+#define GAMETEXT_PACK_AUTHORS 	      _("By author")
+#define GAMETEXT_PACK_MEDALS 	      _("Medals")
+#define GAMETEXT_PACK_ROOM 	      _("Room")
+#define GAMETEXT_PACK_SPECIAL 	      _("Special")
+#define GAMETEXT_PACK_STANDARD 	      _("Standard")
+#define GAMETEXT_PACK_WEBVOTES 	      _("Web votes")
+#define GAMETEXT_PACK_STATS           _("Statistics")
+#define GAMETEXT_PACK_BEST_DRIVERS    _("By best driver")
 #define GAMETEXT_PAUSE                _("Pause")
 #define GAMETEXT_PASSWORD             _("Password")
 #define GAMETEXT_PERSONAL             _("Personal")
 #define GAMETEXT_PERSONALRECORDS      _("Personal")
-#define GAMETEXT_PLAYER               _("Player")
+#define GAMETEXT_PLAYER(A)            ngettext("Player", "Players", A)
 #define GAMETEXT_PLAYERPROFILE        _("Player Profile")
 #define GAMETEXT_PLAYERPROFILES       _("Player Profiles")
 #define GAMETEXT_PLAYNEXT             _("Play Next Level")
@@ -283,15 +308,17 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_PRESSANYKEYTO        _("Press key you want to '%s' or ESC to cancel...")
 #define GAMETEXT_PROXYSERVER          _("Proxy Server")
 #define GAMETEXT_PROXYCONFIG          _("Configure Proxy...")
+#define GAMETEXT_QUICKSTART           _("Quick start")
 #define GAMETEXT_QUIT                 _("Quit Game")
 #define GAMETEXT_QUITMESSAGE          _("Do you really want to quit?")
+#define GAMETEXT_RANDOMIZE            _("Randomize")
 #define GAMETEXT_RELOADINGLEVELS      _("Reloading levels...")
 #define GAMETEXT_RELOADINGREPLAYS     _("Reloading replays...")
 #define GAMETEXT_REPLAY               _("Replay")
 #define GAMETEXT_REPLAYHELPTEXT(current_speed) _("Stop[esc] ||[space]  << >>[left/right keys]  < >[up/down keys]   Speed:") + current_speed + "x"
 #define GAMETEXT_REPLAYHELPTEXTNOREWIND(current_speed) _("Stop[esc] ||[space] >>[right key] < >[up/down keys]   Speed:") + current_speed + "x"
   // + "a" + "X"
-#define GAMETEXT_REPLAYNOTFOUND       _("Replay '%s' can't be played!")
+#define GAMETEXT_REPLAYNOTFOUND       _("The replay can't be played!")
 #define GAMETEXT_REPLAYS              _("Replays")
 #define GAMETEXT_RESTART              _("Restart This Level")
 #define GAMETEXT_RESUME               _("Resume Playing")
@@ -322,11 +349,13 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_UNPACKED_LEVELS_PACK _("Unpacked levels")
 #define GAMETEXT_UNUPDATABLETHEMEONWEB _("Can't update this theme !\nThe theme is not avaible on the web\nor your theme list is not up to date")
 #define GAMETEXT_UPDATE               _("Update")
+#define GAMETEXT_UPDATED              _("Updated")
 #define GAMETEXT_UPDATEHIGHSCORES     _("Check WWW")
 #define GAMETEXT_UPDATEROOMSSLIST     _("Update the rooms list")
 #define GAMETEXT_UPDATETHEMESLIST     _("Update the theme list")
 #define GAMETEXT_UPDATINGLEVELS       _("Updating level lists...")
 #define GAMETEXT_UPLOAD_HIGHSCORE     _("Upload highscore")
+#define GAMETEXT_UPLOAD_ALL_HIGHSCORES _("Upload all highscores")
 #define GAMETEXT_UPLOAD_HIGHSCORE_ERROR _("An unexcepted error occured\nThe website has perhaps some troubles")
 #define GAMETEXT_UPLOAD_HIGHSCORE_WEB_WARNING_BEFORE _("Oh no !")
 #define GAMETEXT_UPLOADING_HIGHSCORE  _("Uploading the highscore...")
@@ -372,7 +401,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_TRY_LEVEL_AGAIN _("Try this level again")
 #define CONTEXTHELP_BUILT_IN_AND_EXTERNALS _("Built-in and stand-alone external levels")
 #define CONTEXTHELP_LEVEL_PACKS _("Levels grouped together in level packs")
-#define CONTEXTHELP_LEVELS _("Browse levels avaiable to you")
+#define CONTEXTHELP_LEVELS _("Browse levels available to you")
 #define CONTEXTHELP_REPLAY_LIST _("View list of recorded replays")
 #define CONTEXTHELP_OPTIONS _("Configure X-Moto preferences")
 #define CONTEXTHELP_HELP _("Instructions of how to play X-Moto")
@@ -418,7 +447,6 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_SELECT_PLAYER_PROFILE _("Select a player profile to use")
 #define CONTEXTHELP_USE_PLAYER_PROFILE _("Use the selected player profile")
 #define CONTEXTHELP_CREATE_PLAYER_PROFILE _("Create new player profile")
-#define CONTEXTHELP_CLOSE_PROFILE_EDITOR _("Close profile editor")
 #define CONTEXTHELP_DELETE_PROFILE _("Permanently delete selected player profile, including best times")
 #define CONTEXTHELP_SELECT_LEVEL_IN_LEVEL_PACK _("Select a level in the level pack to play")
 #define CONTEXTHELP_CLOSE_LEVEL_PACK _("Close level pack")
@@ -460,6 +488,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_UPDATETHEMESLIST _("Check for new themes on the web")
 #define CONTEXTHELP_GETSELECTEDTHEME _("Download or update the selected theme")
 #define CONTEXTHELP_UPLOAD_HIGHSCORE _("Upload the replay on the website of highscores")
+#define CONTEXTHELP_UPLOAD_HIGHSCORE_ALL _("Upload your best replays to your room (useful when it is just created)")
 #define CONTEXTHELP_WWW_MAIN_TAB _("Configure the main www options")
 #define CONTEXTHELP_WWW_ROOMS_TAB _("Choose where to upload/download replays, highscore lists (for more information, check the website)")
 #define CONTEXTHELP_WWW_ROOMS_LIST _("Choose your room")
@@ -478,15 +507,81 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_LEVELPACK _("Name of level pack")
 #define CONTEXTHELP_ADDTOFAVORITE _("Add the level to the favorite levels list")
 #define CONTEXTHELP_DELETEFROMFAVORITE _("Delete the level from the favorite levels list")
+#define CONTEXTHELP_RANDOMIZE _("Puzzle the levels pack list")
+#define CONTEXTHELP_LEVEL_FILTER _("Put a text to limit the level list")
+#define CONTEXTHELP_MULTI _("Choose the number of players")
+#define CONTEXTHELP_PROXYLOGIN    _("Your login in the room")
+#define CONTEXTHELP_PROXYPASSWORD _("Your password associated with your login")
+#define CONTEXTHELP_REPLAYS_FILTER _("Filter on replay list")
+#define CONTEXTHELP_QUICKSTART _("Press to play immediatly to X-Moto")
+#define CONTEXTHELP_QUICKSTART_QUALITY    _("Increase or dicrease the quality of the levels selected")
+#define CONTEXTHELP_QUICKSTART_DIFFICULTY _("Increase or dicrease the difficulty of the levels selected")
+#define CONTEXTHELP_MULTISTOPWHENONEFINISHES _("Stop the game once a player ends the level or continue while a player is running")
 
 #define VPACKAGENAME_LEVELS_WITH_NO_HIGHSCORE   _("Levels with no highscore")
-#define VPACKAGENAME_RANDOM_LEVELS              _("Random levels")
 #define VPACKAGENAME_INCOMPLETED_LEVELS         _("Levels you have not completed")
 #define VPACKAGENAME_YOU_HAVE_NOT_THE_HIGHSCORE _("You're not the highscore holder")
 #define VPACKAGENAME_ALL_LEVELS                 _("All levels")
-#define VPACKAGENAME_NEW_LEVELS                 _("New levels")
-#define VPACKAGENAME_UPDATED_LEVELS             _("Updated levels")
-#define VPACKAGENAME_MOST_PLAYED_LEVELS         _("Most played levels")
+#define VPACKAGENAME_NEW_LEVELS                 _("New and updated levels")
 #define VPACKAGENAME_FAVORITE_LEVELS            _("Favorite levels")
+#define VPACKAGENAME_NICEST_LEVELS              _("Nicest levels")
+#define VPACKAGENAME_HARDEST_LEVELS             _("Hardest levels")
+#define VPACKAGENAME_EASIEST_LEVELS             _("Easiest levels")
+#define VPACKAGENAME_CRAPIEST_LEVELS            _("Crapiest levels")
+#define VPACKAGENAME_SCRIPTED                   _("Scripted levels")
+#define VPACKAGENAME_MUSICAL                    _("Musical")
+#define VPACKAGENAME_BEST_DRIVER                _("By best driver")
+#define VPACKAGENAME_LAST_PLAYED                _("Last played levels")
+#define VPACKAGENAME_NEVER_PLAYED               _("Never played levels")
+#define VPACKAGENAME_MOST_PLAYED                _("Most played levels")
+#define VPACKAGENAME_LESS_PLAYED                _("Less played levels (but played)")
+#define VPACKAGENAME_LAST_HIGHSCORES            _("Last highscores")
+#define VPACKAGENAME_LAST_LEVELS                _("Last levels")
+#define VPACKAGENAME_OLDEST_HIGHSCORES          _("Oldest highscores")
+#define VPACKAGENAME_MEDAL_PLATINIUM            _("Platinium")
+#define VPACKAGENAME_MEDAL_GOLD                 _("Gold")
+#define VPACKAGENAME_MEDAL_SILVER               _("Silver")
+#define VPACKAGENAME_MEDAL_BRONZE               _("Bronze")
+#define VPACKAGENAME_MEDAL_NONE                 _("No medal")
+#define VPACKAGENAME_MY_LEVELS                  _("My levels")
+
+#define VPACKAGENAME_DESC_LEVELS_WITH_NO_HIGHSCORE   _("Levels with no highscore in you room")
+#define VPACKAGENAME_DESC_INCOMPLETED_LEVELS         _("Levels you have not completed")
+#define VPACKAGENAME_DESC_YOU_HAVE_NOT_THE_HIGHSCORE _("Somebody in the room made a better highscore than yours")
+#define VPACKAGENAME_DESC_ALL_LEVELS                 _("All X-Moto levels")
+#define VPACKAGENAME_DESC_NEW_LEVELS                 _("New and updated levels since your last check")
+#define VPACKAGENAME_DESC_FAVORITE_LEVELS            _("Your favorite levels")
+#define VPACKAGENAME_DESC_NICEST_LEVELS          _("Nicest X-Moto levels according to the web votes")
+#define VPACKAGENAME_DESC_HARDEST_LEVELS         _("Hardest X-Moto levels according to the web votes")
+#define VPACKAGENAME_DESC_EASIEST_LEVELS         _("Easiest X-Moto levels according to the web votes")
+#define VPACKAGENAME_DESC_CRAPIEST_LEVELS        _("Crapiest X-Moto levels according to the web votes")
+#define VPACKAGENAME_DESC_SCRIPTED                   _("X-Moto levels which are dynamic")
+#define VPACKAGENAME_DESC_MUSICAL                    _("X-Moto levels with a background music")
+#define VPACKAGENAME_DESC_LAST_PLAYED                _("Last levels you have played")
+#define VPACKAGENAME_DESC_NEVER_PLAYED               _("X-Moto levels you never played")
+#define VPACKAGENAME_DESC_MOST_PLAYED                _("X-Moto levels you have played the most")
+#define VPACKAGENAME_DESC_LESS_PLAYED                _("X-Moto levels you have played only a few times")
+#define VPACKAGENAME_DESC_LAST_HIGHSCORES        _("X-Moto levels having a recent highscore in you room")
+#define VPACKAGENAME_DESC_LAST_LEVELS                _("Last created X-Moto levels")
+#define VPACKAGENAME_DESC_OLDEST_HIGHSCORES      _("X-Moto levels having an old highscore in you room")
+#define VPACKAGENAME_DESC_MEDAL_PLATINIUM        _("You have the room highscore")
+#define VPACKAGENAME_DESC_MEDAL_GOLD             _("You highscore finish time is almost the room highscore (95%)")
+#define VPACKAGENAME_DESC_MEDAL_SILVER           _("You highscore finish time is almost the room highscore (90%-95%)")
+#define VPACKAGENAME_DESC_MEDAL_BRONZE           _("You highscore finish time is almost the room highscore (80%-90%)")
+#define VPACKAGENAME_DESC_MEDAL_NONE             _("The room highscore is really better than your own highscore")
+#define VPACKAGENAME_DESC_STANDARD               _("Level pack")
+#define VPACKAGENAME_DESC_BEST_DRIVERS           _("Highscores by")
+#define VPACKAGENAME_DESC_MY_LEVELS              _("External levels (the one you put into the 'My levels' directory)")
+
+#define SYS_MSG_UGLY_MODE_ENABLED      	_("Ugly mode enabled")
+#define SYS_MSG_UGLY_MODE_DISABLED     	_("Ugly mode disabled")
+#define SYS_MSG_THEME_MODE_ENABLED     	_("Theme mode enabled")
+#define SYS_MSG_THEME_MODE_DISABLED    	_("Theme mode disabled")
+#define SYS_MSG_UGLY_OVER_MODE_ENABLED  _("UglyOver mode enabled")
+#define SYS_MSG_UGLY_OVER_MODE_DISABLED _("UglyOver mode disabled")
+#define SYS_MSG_WWW_ENABLED   	       	_("Web connexion enabled")
+#define SYS_MSG_WWW_DISABLED  	       	_("Web connexion disabled")
+#define SYS_MSG_INTERPOLATION_ENABLED  	_("Replay interpolation enabled")
+#define SYS_MSG_INTERPOLATION_DISABLED 	_("Replay interpolation disabled")
 
 #endif
