@@ -24,12 +24,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "VCommon.h"
 #include "VApp.h"
-#include "MotoGame.h"
+#include "xmscene/Scene.h"
 #include "UserConfig.h"
 #include "Renderer.h"
 
 namespace vapp {
 
+	class GameApp;
+
   /*===========================================================================
   Controller modes
   ===========================================================================*/
@@ -82,7 +84,8 @@ namespace vapp {
     ACTION_CAMERAMOVEXUP,
     ACTION_CAMERAMOVEXDOWN,
     ACTION_CAMERAMOVEYUP,
-    ACTION_CAMERAMOVEYDOWN
+    ACTION_CAMERAMOVEYDOWN,
+    ACTION_AUTOZOOMOUT        /* Param: None */
   };
     
   struct InputAction {
@@ -103,10 +106,17 @@ namespace vapp {
     public:
       
       /* Methods */
+    InputHandler();
+    void reset();
+    void setMirrored(bool i_value);
+
       void configure(UserConfig *pConfig);
-      void handleInput(InputEventType Type,int nKey,BikeController *pController, GameRenderer *pGameRender);      
+      void handleInput(InputEventType Type,int nKey,SDLMod mod,
+		       std::vector<Biker*>& i_bikers,
+		       std::vector<Camera*>& i_cameras,
+		       GameApp *pGameApp);      
       std::string waitForKey(void);
-      void updateInput(BikeController *pController);
+      void updateInput(std::vector<Biker*>& i_bikers);
       void init(UserConfig *pConfig);
       void uninit(void);
       
@@ -121,20 +131,21 @@ namespace vapp {
       int m_nNumScriptKeyHooks;
       InputScriptKeyHook m_ScriptKeyHooks[MAX_SCRIPT_KEY_HOOKS];
       
-      ControllerModeID m_ControllerModeID1;      
+      ControllerModeID m_ControllerModeID[4];
       InputAction m_ActiveAction;
 
       std::vector<SDL_Joystick *> m_Joysticks;
       
       /* For ControllerMode1 = CONTROLLER_MODE_KEYBOARD */
-      int m_nDriveKey1;
-      int m_nBrakeKey1;
-      int m_nPullBackKey1;
-      int m_nPushForwardKey1;
-      int m_nChangeDirKey1;
+      int m_nDriveKey[4];
+      int m_nBrakeKey[4];
+      int m_nPullBackKey[4];
+      int m_nPushForwardKey[4];
+      int m_nChangeDirKey[4];
       int m_nZoomIn;
       int m_nZoomOut;
       int m_nZoomInit;
+      int m_nAutoZoom;
       int m_nCameraMoveXUp;
       int m_nCameraMoveXDown;
       int m_nCameraMoveYUp;
@@ -147,15 +158,15 @@ namespace vapp {
       int m_nJoyAxisPrimMin1;
       int m_nJoyAxisPrimUL1;
       int m_nJoyAxisPrimLL1;
-      
       int m_nJoyAxisSec1; /**< Secondary axis: Pulling and pushing the handle bar */
       int m_nJoyAxisSecMax1;
       int m_nJoyAxisSecMin1;
       int m_nJoyAxisSecUL1;
       int m_nJoyAxisSecLL1;
-
       int m_nJoyButtonChangeDir1;
-      
+     
+      bool m_mirrored;
+
       std::vector<bool> m_JoyButtonsPrev;
       
       /* Static data */
