@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VCommon.h"
 #include "Packager.h"
 #include "helpers/SwapEndian.h"
+#include "md5sum/md5file.h"
 
 /* 
    How to create an xmoto.bin file:
@@ -82,14 +83,28 @@ namespace vapp {
     FILE *fp = fopen(BinFile.c_str(),"rb");
     if(fp != NULL) {    
       char cBuf[256];
+      char md5sum[256];
+      int md5sumLen;
+
       fread(cBuf,4,1,fp);
-      if(!strncmp(cBuf,"XBI1",4)) {
+      if(!strncmp(cBuf,"XBI3",4)) {
+
+	/* get bin sum */
+	md5sumLen = fgetc(fp);
+	fread(md5sum,md5sumLen,1,fp);
+	md5sum[md5sumLen] = '\0';	
+	/* this is not usefull for unpack */
+
         int nNameLen;
         while((nNameLen=fgetc(fp)) >= 0) {  
           /* Extract name of file to extract */
           fread(cBuf,nNameLen,1,fp);
           cBuf[nNameLen] = '\0';
           
+	  md5sumLen = fgetc(fp);
+          fread(md5sum,md5sumLen,1,fp);
+          md5sum[md5sumLen] = '\0';
+
           /* Write name to package.lst file */
           if(fp_lst != NULL)
             fprintf(fp_lst,"%s\n",cBuf);
@@ -163,7 +178,7 @@ namespace vapp {
   /*===========================================================================
   Packager main - note that this is a VERY simplistic approach to data files
   ===========================================================================*/
-  void Packager::go(void) {
+  void Packager::go(const std::string &BinFile, const std::string &DataDir) {
     char cBuf[256];
 
     printf("X-Moto Data Packager!\n");
@@ -171,8 +186,8 @@ namespace vapp {
     printf("Finding out which files to package...\n");
     
     FILE *fp;
-
-    if( (fp=fopen("package.lst","r")) == NULL) {
+    std::string pkglist = DataDir + "/package.lst";
+    if( (fp=fopen(pkglist.c_str(),"r")) == NULL) {
       printf("Can't find the file package.lst...\n");
       return;
     }
@@ -180,20 +195,19 @@ namespace vapp {
     std::vector<std::string> FileList;
     
     while(!feof(fp) && fgets(cBuf,sizeof(cBuf)-1,fp)!=NULL) {
-      /* Strip leading and tailing white-spaces */
-      while(cBuf[0] == ' ' || cBuf[0] == '\t') {
-        char cTemp[256];
-        strcpy(cTemp,&cBuf[1]); strcpy(cBuf,cTemp);
-      }
-      for(int i=strlen(cBuf)-1;i>=0;i--) {
-        if(cBuf[i] == ' ' || cBuf[i] == '\t' || cBuf[i] == '\n' || cBuf[i] == '\r') cBuf[i]='\0';
-        else break;
-      }
+      std::string s = cBuf;
+      bool bOk = true;
       
-      if(cBuf[0]) {
+      /* Strip leading and tailing white-spaces */
+      std::string::size_type lead = s.find_first_not_of("\t ");
+      std::string::size_type trail = s.find_last_not_of("\t \n\r");
+      
+      if(trail != std::string::npos) {
+      	s = s.substr(lead, trail - lead + 1);
+        printf("  %s\n",s.c_str());
+      	
         /* Add it to the list */
-        printf("  %s\n",cBuf);
-        FileList.push_back(cBuf);
+        FileList.push_back(s);
       }      
     }
     fclose(fp);
@@ -203,11 +217,19 @@ namespace vapp {
     printf("Creating package 'xmoto.bin'...\n");
     
     /* Do it */
-    fp = fopen("xmoto.bin","wb");
-    fwrite("XBI1",4,1,fp);
+    fp = fopen(BinFile.c_str(),"wb");
+    fwrite("XBI3",4,1,fp);
+    
+    /* control sum */
+    unsigned char c;
+    std::string sum = md5file(pkglist);
+    c = sum.length();
+    fputc(c, fp);
+    fwrite(sum.c_str(), c, 1, fp);
+
     for(int i=0;i<FileList.size();i++) {
       /* Open and load entire file into memory (yikes!! (but all files a pretty small :P)) */
-      FILE *in = fopen(FileList[i].c_str(),"rb");
+      FILE *in = fopen((DataDir + "/" + FileList[i]).c_str(),"rb");
       if(in != NULL) {
         fseek(in,0,SEEK_END);
         int nSize = ftell(in);
@@ -220,6 +242,12 @@ namespace vapp {
         unsigned char c = FileList[i].length();
         fputc(c,fp);
         fwrite(FileList[i].c_str(),c,1,fp);
+
+	std::string md5sum = md5file((DataDir + "/" + FileList[i]));
+        c = md5sum.length();
+        fputc(c,fp);
+        fwrite(md5sum.c_str(),c,1,fp);
+
 	int LnSize = SwapEndian::LittleLong(nSize);
         fwrite(&LnSize,4,1,fp);        
         fwrite(pcBuf,nSize,1,fp);
