@@ -24,7 +24,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include "VXml.h"
 #include "VFileIO.h"
-#include "MotoGame.h"
+#include "xmscene/Scene.h"
 #include "Renderer.h"
 #include "PhysSettings.h"
 #include "xmscene/BasicSceneStructs.h"
@@ -47,45 +47,59 @@ namespace vapp {
     p3 = C + p3 * fSize;
     p4 = C + p4 * fSize;
     
-    glEnable(GL_BLEND); 
-    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);         
-    glBindTexture(GL_TEXTURE_2D,pTexture->nID);
-    glEnable(GL_TEXTURE_2D);
-    glBegin(GL_POLYGON);
-    glColor4ub(c.Red(), c.Green(), c.Blue(), c.Alpha());
-    glTexCoord2f(0,0);
-    _Vertex(p1);
-    glTexCoord2f(1,0);
-    _Vertex(p2);
-    glTexCoord2f(1,1);
-    _Vertex(p3);
-    glTexCoord2f(0,1);
-    _Vertex(p4);
-    glEnd();
-    glDisable(GL_TEXTURE_2D);
-    glDisable(GL_BLEND);            
+    getParent()->getDrawLib()->setTexture(pTexture,BLEND_MODE_A);
+    getParent()->getDrawLib()->startDraw(DRAW_MODE_POLYGON);
+    //convert the TColor to a Color 
+    getParent()->getDrawLib()->setColor(MAKE_COLOR(c.Red(), c.Green(), c.Blue(), c.Alpha()));
+    getParent()->getDrawLib()->glTexCoord(0.01, 0.01);
+    getParent()->getDrawLib()->glVertex(p1);
+    getParent()->getDrawLib()->glTexCoord(0.99, 0.01);
+    getParent()->getDrawLib()->glVertex(p2);
+    getParent()->getDrawLib()->glTexCoord(0.99, 0.99);
+    getParent()->getDrawLib()->glVertex(p3);
+    getParent()->getDrawLib()->glTexCoord(0.01, 0.99);
+    getParent()->getDrawLib()->glVertex(p4);
+    getParent()->getDrawLib()->endDraw();
   }
 
   void GameRenderer::_RenderParticle(ParticlesSource *i_source) {
-    AnimationSprite *pStarAnim = (AnimationSprite *)getParent()->m_theme.getSprite(SPRITE_TYPE_ANIMATION,"Star");
-    EffectSprite* pFireType = (EffectSprite*) getParent()->m_theme.getSprite(SPRITE_TYPE_EFFECT, "Fire1");
-    EffectSprite* pSmoke1Type = (EffectSprite*) getParent()->m_theme.getSprite(SPRITE_TYPE_EFFECT, "Smoke1");
-    EffectSprite* pSmoke2Type = (EffectSprite*) getParent()->m_theme.getSprite(SPRITE_TYPE_EFFECT, "Smoke2");
-    
-    EffectSprite* pDebrisType = (EffectSprite*) getParent()->m_theme.getSprite(SPRITE_TYPE_EFFECT, "Debris1");
 
     if(i_source->SpriteName() == "Star") {
-      if(pStarAnim != NULL) {
+
+      AnimationSprite *pStarAnimation;
+      pStarAnimation = (AnimationSprite*) getParent()->getTheme()
+      ->getSprite(SPRITE_TYPE_ANIMATION, getGameObject()->getLevelSrc()->SpriteForStar());
+
+      if(pStarAnimation != NULL) {
 	
 	for(unsigned j = 0; j < i_source->Particles().size(); j++) {
 	  _RenderParticleDraw(i_source->Particles()[j]->DynamicPosition(),
-			      pStarAnim->getTexture(),
-			      pStarAnim->getWidth(),
+			      pStarAnimation->getTexture(),
+			      pStarAnimation->getWidth(),
 			      i_source->Particles()[j]->Angle(),
 			      i_source->Particles()[j]->Color());
 	}
+      } else {
+	DecorationSprite *pStarDecoration;
+
+	/* search as a simple decoration, not nice, crappy crappy */
+	pStarDecoration = (DecorationSprite*) getParent()->getTheme()
+	->getSprite(SPRITE_TYPE_DECORATION, getGameObject()->getLevelSrc()->SpriteForStar());
+	if(pStarDecoration != NULL) {
+	
+	  for(unsigned j = 0; j < i_source->Particles().size(); j++) {
+	    _RenderParticleDraw(i_source->Particles()[j]->DynamicPosition(),
+				pStarDecoration->getTexture(),
+				pStarDecoration->getWidth(),
+				i_source->Particles()[j]->Angle(),
+				i_source->Particles()[j]->Color());
+	  }
+	}
       }
     } else if(i_source->SpriteName() == "Fire") {
+      EffectSprite* pFireType = (EffectSprite*) getParent()->getTheme()
+      ->getSprite(SPRITE_TYPE_EFFECT, "Fire1");
+
       if(pFireType != NULL) {
 	for(unsigned j = 0; j < i_source->Particles().size(); j++) {
 	  _RenderParticleDraw(i_source->Particles()[j]->DynamicPosition(),
@@ -96,6 +110,11 @@ namespace vapp {
 	}
       }
     } else if(i_source->SpriteName() == "Smoke") {
+      EffectSprite* pSmoke1Type = (EffectSprite*) getParent()->getTheme()
+      ->getSprite(SPRITE_TYPE_EFFECT, "Smoke1");
+      EffectSprite* pSmoke2Type = (EffectSprite*) getParent()->getTheme()
+      ->getSprite(SPRITE_TYPE_EFFECT, "Smoke2");
+
       if(pSmoke1Type != NULL && pSmoke2Type != NULL) {
 	for(unsigned j = 0; j < i_source->Particles().size(); j++) {
 	  if(i_source->Particles()[j]->SpriteName() == "Smoke1") {
@@ -114,6 +133,9 @@ namespace vapp {
 	}
       }
     } else if(i_source->SpriteName() == "Debris1") {
+      EffectSprite* pDebrisType = (EffectSprite*) getParent()->getTheme()
+      ->getSprite(SPRITE_TYPE_EFFECT, "Debris1");
+
       if(pDebrisType != NULL) {
 	for(unsigned j = 0; j < i_source->Particles().size(); j++) {
 	  if(i_source->Particles()[j]->SpriteName() == "Debris1") {
@@ -129,8 +151,20 @@ namespace vapp {
   }
   
   void GameRenderer::_RenderParticles(bool bFront) {
-    for(unsigned int i = 0; i < getGameObject()->getLevelSrc()->Entities().size(); i++) {
-      Entity* v_entity = getGameObject()->getLevelSrc()->Entities()[i];
+    AABB screenBigger;
+    Vector2f screenMin = m_screenBBox.getBMin();
+    Vector2f screenMax = m_screenBBox.getBMax();
+    /* to avoid sprites being clipped out of the screen,
+       we draw also the nearest one */
+#define ENTITY_OFFSET 5.0f
+    screenBigger.addPointToAABB2f(screenMin.x-ENTITY_OFFSET,
+				  screenMin.y-ENTITY_OFFSET);
+    screenBigger.addPointToAABB2f(screenMax.x+ENTITY_OFFSET,
+				  screenMax.y+ENTITY_OFFSET);
+
+    std::vector<Entity*> Entities = getGameObject()->getCollisionHandler()->getEntitiesNearPosition(screenBigger);
+    for(unsigned int i = 0; i < Entities.size(); i++) {
+      Entity* v_entity = Entities[i];
       if(v_entity->Speciality() == ET_PARTICLES_SOURCE) {
 	_RenderParticle((ParticlesSource*) v_entity);
       }
