@@ -34,7 +34,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <iostream>
 //#include <locale> /* can this safely be removed? */
 
-#include <helpers/SwapEndian.h>
+#include "helpers/SwapEndian.h"
 
 namespace vapp {
 
@@ -46,8 +46,6 @@ namespace vapp {
     m_fFinishTime = 0.0f;
     m_pcInputEventsData = NULL;
     m_nInputEventsDataSize = 0;
-    m_speed_factor = 1.0;
-    m_is_paused = false;
   }
       
   Replay::~Replay() {
@@ -55,6 +53,7 @@ namespace vapp {
   }        
   
   void Replay::_FreeReplay(void) {
+
     /* Get rid of replay events */
     for(int i=0;i<m_ReplayEvents.size();i++) {
       delete m_ReplayEvents[i]->Event;
@@ -64,19 +63,22 @@ namespace vapp {
 
     /* Dealloc chunks */
     for(int i=0;i<m_Chunks.size();i++) {
-      if(m_Chunks[i].pcChunkData != NULL) {
-        delete [] m_Chunks[i].pcChunkData;
+      if(m_Chunks[i]->pcChunkData != NULL) {
+        delete [] m_Chunks[i]->pcChunkData;
       }
+      delete m_Chunks[i];
     }
     m_Chunks.clear();
     
-    if(m_pcInputEventsData != NULL) delete [] m_pcInputEventsData;
+    if(m_pcInputEventsData != NULL) {
+      delete [] m_pcInputEventsData;
+      m_pcInputEventsData = NULL;
+    }
   }
   
   void Replay::finishReplay(bool bFinished,float fFinishTime) {
     m_fFinishTime = fFinishTime;
     m_bFinished = bFinished;
-  
     saveReplay();
   }
   
@@ -140,38 +142,33 @@ namespace vapp {
     }
     
     /* Chunks */
-    if(m_Chunks.empty()) FS::writeInt_LE(pfh,0);
-    else {        
-      FS::writeInt_LE(pfh,m_Chunks.size());
+    FS::writeInt_LE(pfh,m_Chunks.size());
     
-      /* Write chunks */    
-      for(int i=0;i<m_Chunks.size();i++) {
-        FS::writeInt_LE(pfh,m_Chunks[i].nNumStates);
-
-        /* Compression enabled? */
-        if(m_bEnableCompression) {
-          /* Try compressing the chunk with zlib */        
-          unsigned char *pcCompressed = new unsigned char[m_nStateSize * m_Chunks[i].nNumStates * 2 + 12];
-          uLongf nDestLen = m_nStateSize * m_Chunks[i].nNumStates * 2 + 12;
-          uLongf nSrcLen = m_nStateSize * m_Chunks[i].nNumStates;
-          int nZRet = compress2((Bytef *)pcCompressed,&nDestLen,(Bytef *)m_Chunks[i].pcChunkData,nSrcLen,9);
-          if(nZRet != Z_OK) {
-            /* Failed to compress... Save uncompressed chunk then */
-            FS::writeBool(pfh,false); /* compression: false */
-            FS::writeBuf(pfh,m_Chunks[i].pcChunkData,m_nStateSize * m_Chunks[i].nNumStates);
-          }
-          else {
-            /* Compressed ok */
-            FS::writeBool(pfh,true); /* compression: true */
-            FS::writeInt_LE(pfh,nDestLen);
-            FS::writeBuf(pfh,(char *)pcCompressed,nDestLen);
-          }
-          delete [] pcCompressed;        
-        }
-        else {
-          FS::writeBool(pfh,false); /* compression: false */
-          FS::writeBuf(pfh,m_Chunks[i].pcChunkData,m_nStateSize * m_Chunks[i].nNumStates);
-        }
+    /* Write chunks */    
+    for(int i=0;i<m_Chunks.size();i++) {
+      FS::writeInt_LE(pfh,m_Chunks[i]->nNumStates);
+      
+      /* Compression enabled? */
+      if(m_bEnableCompression) {
+	/* Try compressing the chunk with zlib */        
+	unsigned char *pcCompressed = new unsigned char[m_nStateSize * m_Chunks[i]->nNumStates * 2 + 12];
+	uLongf nDestLen = m_nStateSize * m_Chunks[i]->nNumStates * 2 + 12;
+	uLongf nSrcLen = m_nStateSize * m_Chunks[i]->nNumStates;
+	int nZRet = compress2((Bytef *)pcCompressed,&nDestLen,(Bytef *)m_Chunks[i]->pcChunkData,nSrcLen,9);
+	if(nZRet != Z_OK) {
+	  /* Failed to compress... Save uncompressed chunk then */
+	  FS::writeBool(pfh,false); /* compression: false */
+	  FS::writeBuf(pfh,m_Chunks[i]->pcChunkData,m_nStateSize * m_Chunks[i]->nNumStates);
+	} else {
+	  /* Compressed ok */
+	  FS::writeBool(pfh,true); /* compression: true */
+	  FS::writeInt_LE(pfh,nDestLen);
+	  FS::writeBuf(pfh,(char *)pcCompressed,nDestLen);
+	}
+	delete [] pcCompressed;        
+      } else {
+	FS::writeBool(pfh,false); /* compression: false */
+	FS::writeBuf(pfh,m_Chunks[i]->pcChunkData,m_nStateSize * m_Chunks[i]->nNumStates);
       }
     }
     
@@ -179,10 +176,10 @@ namespace vapp {
     FS::closeFile(pfh);
   }
   
-  std::string Replay::openReplay(const std::string &FileName,float *pfFrameRate,std::string &Player) {
+  std::string Replay::openReplay(const std::string &FileName, std::string &Player, bool bDisplayInformation) {
     /* Try opening as if it is a full path */
-    FileHandle *pfh = FS::openIFile(FileName);
-    if(pfh == NULL) {        
+    FileHandle *pfh = FS::openIFile(FileName, true);
+    if(pfh == NULL) {
       /* Open file for input */
       pfh = FS::openIFile(std::string("Replays/") + FileName);
       if(pfh == NULL) {
@@ -190,55 +187,78 @@ namespace vapp {
         pfh = FS::openIFile(std::string("Replays/") + FileName + std::string(".rpl"));
         if(pfh == NULL) {    
           Log("** Warning ** : Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
-          return "";
+          throw Exception("Unable to open the replay");
         }
       }
     }
-    
+
     /* Read header */
     int nVersion = FS::readByte(pfh); 
-        
+    if(bDisplayInformation) {
+      printf("%-30s: %i\n", "Replay file version", nVersion);
+    }
+       
     /* Supported version? */
     if(nVersion != 0 && nVersion != 1) {
       FS::closeFile(pfh);
       Log("** Warning ** : Unsupported replay file version (%d): %s",nVersion,(std::string("Replays/") + FileName).c_str());
-      return "";
-    }
-    else {
+      throw Exception("Unable to open the replay");
+    } else {
       /* Little/big endian safety check */
       if(FS::readInt_LE(pfh) != 0x12345678) {
         FS::closeFile(pfh);
         Log("** Warning ** : Sorry, the replay you're trying to open are not endian-compatible with your computer!");
-        return "";        
+        throw Exception("Unable to open the replay");        
       }
-    
+  
       /* Read level ID */
       m_LevelID = FS::readString(pfh);
+      if(bDisplayInformation) {
+	printf("%-30s: %s\n", "Level Id", m_LevelID.c_str());
+      }
 
       /* Read player name */
       Player = m_PlayerName = FS::readString(pfh);
-      
+      if(bDisplayInformation) {
+	printf("%-30s: %s\n", "Player", Player.c_str());
+      }      
+
       /* Read replay frame rate */
       m_fFrameRate = FS::readFloat_LE(pfh);
-      if(pfFrameRate != NULL) *pfFrameRate = m_fFrameRate;            
 
       /* Read state size */
       m_nStateSize = FS::readInt_LE(pfh);
+      if(bDisplayInformation) {
+	printf("%-30s: %i\n", "State size", m_nStateSize);
+      } 
       
       /* Read finish time if any */
       m_bFinished = FS::readBool(pfh);
       m_fFinishTime = FS::readFloat_LE(pfh);
+      if(bDisplayInformation) {
+	if(m_bFinished) {
+	  printf("%-30s: %.2f (%f)\n", "Finish time", m_fFinishTime, m_fFinishTime);
+	} else {
+	  printf("%-30s: %s\n", "Finish time", "unfinished");
+	}
+      }
 
       /* Version 1 includes event data */
       if(nVersion == 1) {
         /* Read uncompressed size */
         m_nInputEventsDataSize = FS::readInt_LE(pfh);
+	if(bDisplayInformation) {
+	  printf("%-30s: %i\n", "Events data size", m_nInputEventsDataSize);
+	} 
         m_pcInputEventsData = new char [m_nInputEventsDataSize];
         
         /* Compressed? */
         if(FS::readBool(pfh)) {
           /* Compressed */          
           int nCompressedEventsSize = FS::readInt_LE(pfh);
+	  if(bDisplayInformation) {
+	    printf("%-30s: %i\n", "Compressed events data size", nCompressedEventsSize);
+	  } 
           
           char *pcCompressedEvents = new char [nCompressedEventsSize];
           FS::readBuf(pfh,pcCompressedEvents,nCompressedEventsSize);
@@ -253,7 +273,7 @@ namespace vapp {
             FS::closeFile(pfh);
             _FreeReplay();
             Log("** Warning ** : Failed to uncompress events in replay: %s",FileName.c_str());
-            return "";
+            throw Exception("Unable to open the replay");
           }
           
           /* Clean up */
@@ -270,39 +290,69 @@ namespace vapp {
       
       /* Read chunks */
       int nNumChunks = FS::readInt_LE(pfh);
-      for(int i=0;i<nNumChunks;i++) {        
-        ReplayStateChunk Chunk;        
-        Chunk.nNumStates = FS::readInt_LE(pfh);
-        Chunk.pcChunkData = new char [Chunk.nNumStates * m_nStateSize];
+      if(bDisplayInformation) {
+	printf("%-30s: %i\n", "Number of chunks", nNumChunks);
+      }
+      if(nNumChunks == 0) {
+	FS::closeFile(pfh);
+	_FreeReplay();
+	Log("** Warning ** : try to open a replay with no chunk");
+	throw Exception("Replay with no chunk !");
+      }
+
+      for(int i=0;i<nNumChunks;i++) {
+	if(bDisplayInformation) {
+	  printf("Chunk %02i\n", i);
+	}  
+  
+        ReplayStateChunk* Chunk = new ReplayStateChunk();
+        Chunk->nNumStates = FS::readInt_LE(pfh);
+
+	if(bDisplayInformation) {
+	  printf("   %-27s: %i\n", "Number of states", Chunk->nNumStates);
+	} 
+
+        Chunk->pcChunkData = new char [Chunk->nNumStates * m_nStateSize];
         
         /* Compressed or not compressed? */
         if(FS::readBool(pfh)) {
+	  if(bDisplayInformation) {
+	    printf("   %-27s: %s\n", "Compressed data", "true");
+	  } 
+
           /* Compressed! - read compressed size */
           int nCompressedSize = FS::readInt_LE(pfh);
-          
+	  if(bDisplayInformation) {
+	    printf("   %-27s: %i\n", "Compressed states size", nCompressedSize);
+	  }
+
           /* Read compressed data */
           unsigned char *pcCompressed = new unsigned char [nCompressedSize];
           FS::readBuf(pfh,(char *)pcCompressed,nCompressedSize);
          
           /* Uncompress it */           
-          uLongf nDestLen = Chunk.nNumStates * m_nStateSize;
+          uLongf nDestLen = Chunk->nNumStates * m_nStateSize;
           uLongf nSrcLen = nCompressedSize;
-          int nZRet = uncompress((Bytef *)Chunk.pcChunkData,&nDestLen,(Bytef *)pcCompressed,nSrcLen);
-          if(nZRet != Z_OK || nDestLen != Chunk.nNumStates * m_nStateSize) {
+          int nZRet = uncompress((Bytef *)Chunk->pcChunkData,&nDestLen,(Bytef *)pcCompressed,nSrcLen);
+          if(nZRet != Z_OK || nDestLen != Chunk->nNumStates * m_nStateSize) {
+            Log("** Warning ** : Failed to uncompress chunk %d in replay: %s",i,FileName.c_str());
             delete [] pcCompressed;
-            delete [] Chunk.pcChunkData;
+            Chunk->pcChunkData = NULL;
             FS::closeFile(pfh);
             _FreeReplay();
-            Log("** Warning ** : Failed to uncompress chunk %d in replay: %s",i,FileName.c_str());
-            return "";
+            throw Exception("Unable to open the replay");
           }
-          
+
           /* Clean up */
           delete [] pcCompressed;
         }
         else {
+	  if(bDisplayInformation) {
+	    printf("   %-27s: %s\n", "Compressed data", "false");
+	  } 
+
           /* Not compressed! */
-          FS::readBuf(pfh,Chunk.pcChunkData,m_nStateSize*Chunk.nNumStates);
+          FS::readBuf(pfh,Chunk->pcChunkData,m_nStateSize*Chunk->nNumStates);
         }
         
         m_Chunks.push_back(Chunk);
@@ -315,34 +365,41 @@ namespace vapp {
     m_nCurChunk = 0;
     m_nCurState = 0.0;
     
-            
     /* Reconstruct game events that are going to happen during the replay */
-    MotoGame::unserializeGameEvents(this, &m_ReplayEvents);
+    if(bDisplayInformation) {
+      printf("%-30s:\n", "Game Events");
+    }
+
+    /* unserialize events */
+    MotoGame::unserializeGameEvents(this, &m_ReplayEvents, bDisplayInformation);
+    initOutput(1024);
+    for(int i=0; i<m_ReplayEvents.size(); i++) {
+      m_ReplayEvents[i]->Event->serialize(*this);
+    }
 
     return m_LevelID;
   }
       
   void Replay::storeState(const SerializedBikeState& state) {
     char *addr;
-    
-    if(m_Chunks.empty()) {
-      ReplayStateChunk Chunk;
-      Chunk.pcChunkData = new char [STATES_PER_CHUNK * m_nStateSize];
-      Chunk.nNumStates = 1;
-      addr = Chunk.pcChunkData;
+
+    if(m_Chunks.size() == 0) {
+      ReplayStateChunk* Chunk = new ReplayStateChunk();
+      Chunk->pcChunkData = new char [STATES_PER_CHUNK * m_nStateSize];
+      Chunk->nNumStates = 1;
+      addr = Chunk->pcChunkData;
       m_Chunks.push_back(Chunk);
-    }
-    else {
+    } else {
       int i = m_Chunks.size() - 1;
-      if(m_Chunks[i].nNumStates < STATES_PER_CHUNK) {
-        addr = &m_Chunks[i].pcChunkData[m_Chunks[i].nNumStates * m_nStateSize];
-        m_Chunks[i].nNumStates++;
+      if(m_Chunks[i]->nNumStates < STATES_PER_CHUNK) {
+        addr = &m_Chunks[i]->pcChunkData[m_Chunks[i]->nNumStates * m_nStateSize];
+        m_Chunks[i]->nNumStates++;
       }
       else {
-        ReplayStateChunk Chunk;
-        Chunk.pcChunkData = new char [STATES_PER_CHUNK * m_nStateSize];
-        Chunk.nNumStates = 1;
-        addr = Chunk.pcChunkData;
+        ReplayStateChunk* Chunk = new ReplayStateChunk();
+        Chunk->pcChunkData = new char [STATES_PER_CHUNK * m_nStateSize];
+        Chunk->nNumStates = 1;
+        addr = Chunk->pcChunkData;
         m_Chunks.push_back(Chunk);        
       }
     }
@@ -351,15 +408,16 @@ namespace vapp {
   SwapEndian::LittleSerializedBikeState(*(SerializedBikeState*)addr);
   }
   
+  int Replay::CurrentFrame() const {
+    return m_nCurChunk * STATES_PER_CHUNK + m_nCurState + 1;
+  }
+
   bool Replay::nextNormalState() {
-    return nextState(m_speed_factor);
+    return nextState(1);
   }
 
   bool Replay::nextState(float p_frames) {
     m_bEndOfFile = false;
-    if(m_is_paused) {
-      return ! m_bEndOfFile;
-    }
 
     m_nCurState += p_frames;
 
@@ -369,13 +427,13 @@ namespace vapp {
       /* go on the following chunk */
       if(m_nCurState >= STATES_PER_CHUNK) { /* end of a chunk */
   if(m_nCurChunk < m_Chunks.size() -1) {
-    m_nCurState = m_nCurState - m_Chunks[m_nCurChunk].nNumStates -1;
+    m_nCurState = m_nCurState - m_Chunks[m_nCurChunk]->nNumStates -1;
     m_nCurChunk++;
   }
       } else {
   if(m_nCurState < 0.0) { /* start of a chunk */
     if(m_nCurChunk > 0) {
-      m_nCurState = m_nCurState + m_Chunks[m_nCurChunk-1].nNumStates -1;
+      m_nCurState = m_nCurState + m_Chunks[m_nCurChunk-1]->nNumStates -1;
       m_nCurChunk--;
     }
   }
@@ -387,8 +445,8 @@ namespace vapp {
       }
 
       /* if that's end */
-      if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
-  m_nCurState = m_Chunks[m_nCurChunk].nNumStates -1;
+      if(m_nCurState >= m_Chunks[m_nCurChunk]->nNumStates) {
+  m_nCurState = m_Chunks[m_nCurChunk]->nNumStates -1;
   return false;
       }
       
@@ -397,8 +455,8 @@ namespace vapp {
       /* if that's the beginning, do nothing */
 
       /* if that's end */
-      if(m_nCurState >= m_Chunks[m_nCurChunk].nNumStates) {
-  m_nCurState = m_Chunks[m_nCurChunk].nNumStates -1;
+      if(m_nCurState >= m_Chunks[m_nCurChunk]->nNumStates) {
+  m_nCurState = m_Chunks[m_nCurChunk]->nNumStates -1;
   return false;
       }
     }
@@ -412,7 +470,7 @@ namespace vapp {
     peekState(state);
 
     m_bEndOfFile = (m_nCurChunk       == m_Chunks.size()-1 && 
-        (int)m_nCurState  == m_Chunks[m_nCurChunk].nNumStates-1);
+        (int)m_nCurState  == m_Chunks[m_nCurChunk]->nNumStates-1);
 
     if(m_bEndOfFile == false) {
       if(nextNormalState()) { /* do nothing */ }
@@ -422,46 +480,11 @@ namespace vapp {
   void Replay::peekState(SerializedBikeState& state) {
     /* Like loadState() but this one does not advance the cursor... it just takes a peek */
     memcpy((char *)&state,
-     &m_Chunks[m_nCurChunk].pcChunkData[((int)m_nCurState)*m_nStateSize],
+     &m_Chunks[m_nCurChunk]->pcChunkData[((int)m_nCurState)*m_nStateSize],
      m_nStateSize);
     SwapEndian::LittleSerializedBikeState(state);
   }
   
-  void Replay::pause() {
-    m_is_paused = ! m_is_paused;
-  }
-
-  void Replay::faster() {
-    if(m_is_paused == false) {
-      m_speed_factor += REPLAY_SPEED_INCREMENT;
-    }
-  }
-
-  void Replay::slower() {
-    if(m_is_paused == false) {
-      m_speed_factor -= REPLAY_SPEED_INCREMENT;
-    }
-  }
-
-  float Replay::getSpeed() const {
-    if(m_is_paused) {
-      return 0.0;
-    }
-    return m_speed_factor;
-  }
-
-  void Replay::fastforward(float fSeconds) {
-    /* How many states should we move forward? */
-    float nNumStates = (fSeconds * m_fFrameRate);
-    nextState(nNumStates);
-  }
-
-  void Replay::fastrewind(float fSeconds) {
-    /* How many states should we move forward? */
-    float nNumStates = (fSeconds * m_fFrameRate);
-    nextState(-nNumStates);
-  }  
-
   std::string Replay::giveAutomaticName() {
     time_t date;
     time(&date);
@@ -477,8 +500,6 @@ namespace vapp {
     m_nCurChunk = 0;
     m_nCurState = 0.0;
     m_bEndOfFile = false;
-    m_speed_factor = 1;
-    m_is_paused = false;
 
     /* resetting events */
     for(int i=0;i<m_ReplayEvents.size();i++) {
@@ -501,18 +522,18 @@ namespace vapp {
       /* Try opening it */
       FileHandle *pfh = FS::openIFile("Replays/" + p_ReplayName + ".rpl");
       if(pfh == NULL) {
-  return NULL;
+	return NULL;
       }
-
+      
       int nVersion = FS::readByte(pfh);
       if(nVersion != 0 && nVersion != 1) {
-  FS::closeFile(pfh);
-  return NULL;
+	FS::closeFile(pfh);
+	return NULL;
       }
-
+      
       if(FS::readInt_LE(pfh) != 0x12345678) {   
-  FS::closeFile(pfh);
-  return NULL;
+	FS::closeFile(pfh);
+	return NULL;
       }
   
       std::string LevelID = FS::readString(pfh);
@@ -523,23 +544,33 @@ namespace vapp {
       float fFinishTime   = FS::readFloat_LE(pfh);
                 
       ReplayInfo *pRpl     = new ReplayInfo;
-      int nTimeStamp       = FS::getFileTimeStamp("Replays/" + p_ReplayName + ".rpl");
 
       /* Set members */
       pRpl->Level = LevelID;
       pRpl->Name = p_ReplayName;
       pRpl->Player = Player;
-      pRpl->nTimeStamp = nTimeStamp;
-      pRpl->fFrameRate = fFrameRate;
+      pRpl->IsFinished = bFinished;
 
       if(bFinished) {
-  pRpl->fFinishTime = fFinishTime;
+	pRpl->fFinishTime = fFinishTime;
       } else {
-  pRpl->fFinishTime = -1;                
+	pRpl->fFinishTime = -1;                
       }
           
       FS::closeFile(pfh);
       return pRpl;
   }
 
+  void Replay::fastforward(float fSeconds) {
+    /* How many states should we move forward? */
+    float nNumStates = (fSeconds * m_fFrameRate);
+    nextState(nNumStates);
+  }
+
+  void Replay::fastrewind(float fSeconds) {
+    /* How many states should we move forward? */
+    float nNumStates = (fSeconds * m_fFrameRate);
+    nextState(-nNumStates);
+  } 
+
 }
