@@ -26,9 +26,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VApp.h"
 #include "VFileIO.h"
 #include "DBuffer.h"
-#include "MotoGame.h"
+#include "xmscene/Scene.h"
 
-#define REPLAY_SPEED_INCREMENT 0.25
 #define STATES_PER_CHUNK 512
 
 namespace vapp {
@@ -42,10 +41,8 @@ namespace vapp {
     std::string Name;
     std::string Player;
     std::string Level;
-    float fFrameRate;
+    bool  IsFinished;
     float fFinishTime;
-    
-    int nTimeStamp;   /* replay file time stamp */
   };
   
   /* Replays states (frames) are grouped together in chunks for easy
@@ -55,31 +52,6 @@ namespace vapp {
     int nNumStates;
   };
 
-  /*===========================================================================
-  ReplayList class
-  ===========================================================================*/  
-  class ReplayList {
-    public:
-      ~ReplayList() {clear();} /* destructor */
-    
-      /* Methods */
-      void initFromCache();
-      void initFromDir();
-      void addReplay(const std::string &Replay, bool i_saveCache = true);
-      void delReplay(const std::string &Replay, bool i_saveCache = true);
-      void clear(bool i_saveCache = true);
-
-      /* you must free the list after this call */
-      std::vector<ReplayInfo *>* findReplays(const std::string &PlayerName = "",
-               const std::string &LevelID = "");
-
-    private:
-      /* Data */
-      std::vector<ReplayInfo *> m_Replays;
-      void saveCache();
-      static std::string ReplayIndexFileName();
-  };
-   
   /*===========================================================================
   Replay class
   ===========================================================================*/  
@@ -96,21 +68,16 @@ namespace vapp {
       
       void createReplay(const std::string &FileName,const std::string &LevelID,const std::string &Player,float fFrameRate,int nStateSize);
       void saveReplay(void);
-      std::string openReplay(const std::string &FileName,float *pfFrameRate,std::string &Player);
+      std::string openReplay(const std::string &FileName, std::string &Player, bool bDisplayInformation = false);
       static void deleteReplay(std::string ReplayName);
       void reinitialize();
       std::string getLevelId();
 
       void finishReplay(bool bFinished,float fFinishTime);
+      int CurrentFrame() const;
+
       void fastforward(float fSeconds);
       void fastrewind(float fSeconds);
-      void pause();
-      void faster();
-      void slower();
-      float getSpeed() const; /* get multiple factor of the replay */
-      
-      void setSpeed(float f) {m_speed_factor = f;}
-      bool isPaused(void) {return m_is_paused;}
 
       /* Data interface */
       bool didFinish(void) {return m_bFinished;}
@@ -131,7 +98,7 @@ namespace vapp {
 
     private: 
       /* Data */ 
-      std::vector<ReplayStateChunk> m_Chunks;
+      std::vector<ReplayStateChunk*> m_Chunks;
       int m_nCurChunk;
       float m_nCurState; /* is a float so that manage slow */   
       std::string m_FileName,m_LevelID,m_PlayerName;
@@ -142,10 +109,6 @@ namespace vapp {
       float m_fFinishTime;
       char *m_pcInputEventsData;
       int m_nInputEventsDataSize;
-
-      float m_speed_factor; /* nb frame to increment each time ;
-             is a float so that manage slow */
-      bool  m_is_paused;
  
       /* Helpers */
       void _FreeReplay(void);
