@@ -20,7 +20,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "ScriptDynamicObjects.h"
-#include "MotoGame.h"
+#include "xmscene/Scene.h"
 #include "math.h"
 
 SDynamicObject::SDynamicObject(int p_startTime, int p_endTime, float pPeriod) {
@@ -33,16 +33,30 @@ SDynamicObject::SDynamicObject(int p_startTime, int p_endTime, float pPeriod) {
 SDynamicObject::~SDynamicObject() {
 }
 
-bool SDynamicObject::nextState(vapp::MotoGame* v_motoGame) {
+bool SDynamicObject::nextState(vapp::MotoGame* v_motoGame, int i_nbCents) {
+  int v_realNbCents;
+
+  v_realNbCents = i_nbCents;
+
   if(m_time >= m_endTime && m_endTime != 0.0) {
     return false;
   }
 
-  if(isTimeToMove() == true) {
-    performMove(v_motoGame);
+  if(m_startTime > m_time) {
+    v_realNbCents -= m_startTime - m_time;
+  }
+
+  if(m_time + i_nbCents > m_endTime && m_endTime != 0.0) {
+    v_realNbCents -= m_time + i_nbCents - m_endTime;
+  }
+  
+  performMove(v_motoGame, v_realNbCents);
+
+  m_time += i_nbCents;
+  if(m_time >= m_endTime && m_endTime != 0.0) {
+    return false;
   }
 
-  m_time++;
   return true;
 }
 
@@ -50,6 +64,9 @@ bool SDynamicObject::isTimeToMove() {
   return m_time >= m_startTime && (m_time <= m_endTime || m_endTime == 0.0);
 }
 
+float SDynamicObject::Period() const {
+  return m_period;
+}
 
 void SDynamicTranslation::performXY(float *vx, float *vy) {
   *vx = m_sensUp ? m_moveX : -m_moveX;
@@ -127,6 +144,30 @@ void SDynamicRotation::performXY(float *vx, float *vy) {
   m_Angle += m_Speed;
 }
 
+SDynamicSelfRotation::SDynamicSelfRotation(float i_period) {
+  m_period = i_period;
+  m_totalAngle = 0.0;
+  m_incr = 0;
+}
+
+SDynamicSelfRotation::~SDynamicSelfRotation() {
+}
+
+void SDynamicSelfRotation::performXY(float *vAngle) {
+  *vAngle = (2 * M_PI) / m_period;
+  m_incr++;
+  m_totalAngle += *vAngle;
+
+  /* to limit sum error */
+  if( (m_period > 0.0 && m_incr >= ((int)m_period)) ||
+      (m_period < 0.0 && m_incr >= ((int)-m_period)) ) {
+    float v_realSum = (((float) m_incr) * (2 * M_PI)) / m_period;
+    *vAngle += v_realSum - m_totalAngle;
+    m_incr = 0;
+    m_totalAngle = 0.0;
+  }
+}
+
 /* entity */
 
 SDynamicEntityMove::SDynamicEntityMove(std::string pEntity, int p_startTime, int p_endTime, float pPeriod) : SDynamicObject(p_startTime, p_endTime, pPeriod){
@@ -136,13 +177,32 @@ SDynamicEntityMove::SDynamicEntityMove(std::string pEntity, int p_startTime, int
 SDynamicEntityMove::~SDynamicEntityMove() {
 }
 
-void SDynamicEntityMove::performMove(vapp::MotoGame* v_motoGame) {
-  Entity *p = &(v_motoGame->getLevelSrc()->getEntityById(m_entity));
-  float vx, vy;
-  performXY(&vx, &vy);
-  v_motoGame->SetEntityPos(p->Id(),
-			   vx + p->DynamicPosition().x,
-			   vy + p->DynamicPosition().y);
+void SDynamicEntityMove::performMove(vapp::MotoGame* v_motoGame, int i_nbCents) {
+  Entity* p = &(v_motoGame->getLevelSrc()->getEntityById(m_entity));
+  if(! p->isAlive()){
+    return;
+  }
+  float vx, vy, vAngle;
+  float addvx = 0.0, addvy = 0.0, addvAngle = 0.0;
+
+  if(i_nbCents > 0) {
+    for(int i=0; i<i_nbCents; i++) {
+      performXY(&vx, &vy, &vAngle);
+      addvx += vx;
+      addvy += vy;
+      addvAngle = vAngle;
+    }
+
+    if(addvx != 0.0 || addvy != 0) { /* a simple fast test because it's probably the main case */
+      v_motoGame->SetEntityPos(p,
+			       addvx + p->DynamicPosition().x,
+			       addvy + p->DynamicPosition().y);
+    }
+
+    if(vAngle != 0.0) { /* a simple fast test because it's probably the main case */
+      v_motoGame->SetEntityDrawAngle(p->Id(), addvAngle + p->DrawAngle());
+    }
+  }
 }
 
 std::string SDynamicEntityMove::getObjectId() {
@@ -161,11 +221,13 @@ SDynamicEntityTranslation::SDynamicEntityTranslation(std::string pEntity, float
 SDynamicEntityTranslation::~SDynamicEntityTranslation() {
 }
 
-void SDynamicEntityRotation::performXY(float *vx, float *vy) {
+void SDynamicEntityRotation::performXY(float *vx, float *vy, float *vAngle) {
+  *vAngle = 0.0;
   SDynamicRotation::performXY(vx, vy);
 }
 
-void SDynamicEntityTranslation::performXY(float *vx, float *vy) {
+void SDynamicEntityTranslation::performXY(float *vx, float *vy, float *vAngle) {
+  *vAngle = 0.0;
   SDynamicTranslation::performXY(vx, vy);
 }
 
@@ -178,13 +240,29 @@ SDynamicBlockMove::SDynamicBlockMove(std::string pBlock, int p_startTime, int p_
 SDynamicBlockMove::~SDynamicBlockMove() {
 }
 
-void SDynamicBlockMove::performMove(vapp::MotoGame* v_motoGame) {
+void SDynamicBlockMove::performMove(vapp::MotoGame* v_motoGame, int i_nbCents) {
   Block *p = &(v_motoGame->getLevelSrc()->getBlockById(m_block));
-  float vx, vy;
-  performXY(&vx, &vy);
-  v_motoGame->SetBlockPos(p->Id(),
-			  vx + p->DynamicPosition().x,
-			  vy + p->DynamicPosition().y);
+  float vx, vy, vAngle;
+  float addvx = 0.0, addvy = 0.0, addvAngle = 0.0;
+
+  if(i_nbCents > 0) {
+    for(int i=0; i<i_nbCents; i++) {
+      performXY(&vx, &vy, &vAngle);
+      addvx += vx;
+      addvy += vy;
+      addvAngle = vAngle;
+    }
+
+    if(addvx != 0.0 || addvy != 0) { /* a simple fast test because it's probably the main case */
+      v_motoGame->MoveBlock(p->Id(),
+			    addvx,
+			    addvy);
+    }
+
+    if(vAngle != 0.0) { /* a simple fast test because it's probably the main case */
+      v_motoGame->SetBlockRotation(p->Id(), addvAngle + p->DynamicRotation());
+    }
+  }
 }
 
 std::string SDynamicBlockMove::getObjectId() {
@@ -203,10 +281,38 @@ SDynamicBlockTranslation::SDynamicBlockTranslation(std::string pBlock, float pX,
 SDynamicBlockTranslation::~SDynamicBlockTranslation() {
 }
 
-void SDynamicBlockRotation::performXY(float *vx, float *vy) {
+void SDynamicBlockRotation::performXY(float *vx, float *vy, float *vAngle) {
+  *vAngle = 0.0;
   SDynamicRotation::performXY(vx, vy);
 }
 
-void SDynamicBlockTranslation::performXY(float *vx, float *vy) {
+void SDynamicBlockTranslation::performXY(float *vx, float *vy, float *vAngle) {
+  *vAngle = 0.0;
   SDynamicTranslation::performXY(vx, vy);
 }
+
+SDynamicBlockSelfRotation::SDynamicBlockSelfRotation(std::string pBlock, float pPeriod, int p_startTime, int p_endTime)
+  : SDynamicBlockMove(pBlock, p_startTime, p_endTime, pPeriod), SDynamicSelfRotation(pPeriod){
+  
+}
+
+SDynamicBlockSelfRotation::~SDynamicBlockSelfRotation() {
+}
+
+void SDynamicBlockSelfRotation::performXY(float *vx, float *vy, float *vAngle) {
+  *vx = *vy = 0.0;
+  SDynamicSelfRotation::performXY(vAngle);
+}
+
+SDynamicEntitySelfRotation::SDynamicEntitySelfRotation(std::string pEntity, float pPeriod, int p_startTime, int p_endTime)
+  : SDynamicEntityMove(pEntity, p_startTime, p_endTime, pPeriod), SDynamicSelfRotation(pPeriod){
+  
+}
+
+SDynamicEntitySelfRotation::~SDynamicEntitySelfRotation() {
+}
+
+void SDynamicEntitySelfRotation::performXY(float *vx, float *vy, float *vAngle) {
+  *vx = *vy = 0.0;
+  SDynamicSelfRotation::performXY(vAngle);
+}
