@@ -46,6 +46,7 @@ namespace vapp {
   //
   //SoundPlayer *Sound::m_pPlayers[16];
   std::vector<SoundSample *> Sound::m_Samples;
+  Mix_Music *Sound::m_pMenuMusic;
 
   /*===========================================================================
   Init and uninit
@@ -158,6 +159,8 @@ namespace vapp {
         SDL_PauseAudio(0);
       #endif
     #endif
+	
+	m_pMenuMusic = NULL;
   }
   
   void Sound::uninit(void) {  
@@ -189,165 +192,14 @@ namespace vapp {
         SDL_QuitSubSystem(SDL_INIT_AUDIO);
       }
     #endif
+
+    if(m_pMenuMusic != NULL) {
+      Mix_FreeMusic(m_pMenuMusic);
+    }
+
+      m_bEnable = false;
   }
   
-  ///*===========================================================================
-  //Audio callback (from seperate thread, so please be careful)
-  //===========================================================================*/
-  //void Sound::audioCallback(void *pvUserData,unsigned char *pcStream,int nLen) {    
-  //  /* Go trough player slots... */
-  //  for(int i=0;i<16;i++) {
-  //    if(m_pPlayers[i] != NULL) {
-  //      m_pPlayers[i]->fillBuffer(pcStream,nLen);
-  //      break;
-  //    }
-  //  }
-  //}
-  //
-  ///*===========================================================================
-  //Opening and closing of vorbis file
-  //===========================================================================*/
-  //void VorbisSound::openFile(std::string File) {    
-  //  /* Open source */
-  //  m_pfh = FS::openIFile(File);
-  //  if(m_pfh == NULL) throw Exception("(1) failed to open ogg stream");
-  //  
-  //  /* Create decoding handle */
-  //  VorbisSoundHandle *pHandle = new VorbisSoundHandle;
-  //  pHandle->pfh = m_pfh;
-  //  pHandle->pVorbis = this;
-  //  pHandle->pvf = &m_vf;
-  //  m_ph = pHandle;
-  //  
-  //  memset(&m_vf,0,sizeof(m_vf));
-  //  
-  //  /* Init vorbis libraries */
-  //	if(ov_open_callbacks(pHandle,&m_vf,NULL,0,m_Callbacks) < 0) {
-  //	  delete pHandle;
-  //	  Log("** Warning ** : invalid ogg stream '%s'",File.c_str());
-  //	  throw Exception("(2) failed to open ogg stream");
-  //	}    
-  //	
-  //	m_nCurrentSection = 0;
-
-	 // /* Extract info */
-	 // vorbis_info *vi=ov_info(&m_vf,-1);
-  //	
-	 // m_nSampleBits=16;
-	 // m_nChannels=vi->channels;
-	 // m_nSampleRate=vi->rate;
-	 // m_nAvgBitRate=vi->bitrate_nominal;
-
-	 // /* Calculate length */
-	 // int nBytesPerSample=m_nChannels * (m_nSampleBits/8);
-	 // m_nLength=ov_pcm_total(&m_vf,-1) * nBytesPerSample;
-
-  //  /* We are now ready to stream */	  
-  //  m_bEnd = false;
-  //}
-  //
-  //void VorbisSound::closeFile(void) {
-  //  if(m_ph) {
-  //    delete m_ph;
-  //  }    
-  //	ov_clear(&m_vf);  	
-  //	if(m_pfh) {
-  //	  FS::closeFile(m_pfh);  	    	  
-  //	}
-  //	
-  //	m_ph = NULL;
-  //	m_pfh = NULL;
-  //	m_bEnd = true;
-  //}
-  //
-  ///*==============================================================================
-  //Decoding
-  //==============================================================================*/
-  //int VorbisSound::decode(unsigned char *pcBuf,int nBufSize) {
-  //  int nRem = nBufSize;
-  //  int i = 0;
-  //  while(nRem>0) {
-  //    int nNumRead = ov_read(&m_vf,(char *)&pcBuf[i],nRem,0,2,1,&m_nCurrentSection);
-  //    if(nNumRead == 0) {
-  //      /* EOF */
-  //      m_bEnd = true;
-  //      break;
-  //    }
-  //    else if(nNumRead < 0) {
-  //      /* Error */
-  //      m_bEnd = true;
-  //      break;
-  //    }
-  //    else {
-  //      nRem-=nNumRead;
-  //      i+=nNumRead;
-  //    }
-  //  }
-  //  //printf("got %d bytes from stream\n",i);
-  //  return i;
-  //}
-  //
-  //bool VorbisSound::endOfStream(void) {
-  //  return m_bEnd;
-  //}
-
-  ///*==============================================================================
-  //Vorbis callbacks
-  //==============================================================================*/
-  //size_t VorbisSound_readc(void *pvPtr,size_t Size,size_t Blocks,void *pvDataSource) {
-  //  VorbisSoundHandle *pHandle = (VorbisSoundHandle *)pvDataSource;
-  //  if(pHandle != NULL) {
-  //    int nRem = FS::getLength(pHandle->pfh) - FS::getOffset(pHandle->pfh);
-  //    int nRead = nRem < Blocks ? nRem : Blocks;
-  //    if(FS::readBuf(pHandle->pfh,(char *)pvPtr,Size*nRead)) {
-  //      return nRead;
-  //    }
-  //  }
-	 // return 0;
-  //}
-
-  //int VorbisSound_seekc(void *pvDataSource,ogg_int64_t Offset,int nWhence) {
-  //  VorbisSoundHandle *pHandle = (VorbisSoundHandle *)pvDataSource;
-  //  if(pHandle != NULL) {
-		//  switch(nWhence) {
-		//	  case SEEK_SET:
-		//	    FS::setOffset(pHandle->pfh,Offset);
-		//		  break;
-		//	  case SEEK_END:
-		//	    FS::setOffset(pHandle->pfh,FS::getLength(pHandle->pfh) + Offset);
-		//		  break;
-		//	  case SEEK_CUR:
-		//	    FS::setOffset(pHandle->pfh,FS::getOffset(pHandle->pfh) + Offset);
-		//		  break;
-		//  }
-		//  return 0;
-  //  }
-	 // return -1;	
-  //}
-
-  //int VorbisSound_closec(void *pvDataSource) {
-  //  VorbisSoundHandle *pHandle = (VorbisSoundHandle *)pvDataSource;
-  //  if(pHandle != NULL) {
-  //  }    
-	 // return 0;
-  //}
-
-  //long VorbisSound_tellc(void *pvDataSource) {
-  //  VorbisSoundHandle *pHandle = (VorbisSoundHandle *)pvDataSource;
-  //  if(pHandle != NULL) {
-  //    return FS::getOffset(pHandle->pfh);
-  //  }
-	 // return 0;
-  //}
-  //
-  ///* Callbacks struct */
-  //ov_callbacks VorbisSound::m_Callbacks={  
-  //  VorbisSound_readc,
-  //  VorbisSound_seekc,
-  //  VorbisSound_closec,
-  //  VorbisSound_tellc
-  //};
-  
   /*==============================================================================
   Update sound system
   ==============================================================================*/
@@ -454,7 +306,7 @@ namespace vapp {
       FileHandle *pf = FS::openIFile(File);
       if(pf == NULL) {
         SDL_FreeRW(pOps);
-        throw Exception("failed to open sample file");
+        throw Exception("failed to open sample file " + File);
       }
       
       pOps->hidden.unknown.data1 = (void *)pf;
@@ -557,142 +409,14 @@ namespace vapp {
       playSample(pSample,fVolume);
     }
   }
-      
-//  /*==============================================================================
-//  Stream playing
-//  ==============================================================================*/
-//  void Sound::playStream(std::string File) {
-//    #if USE_SDL_MIXER
-//      /* .. */
-//    #else /* Not using SDL_mixer */
-//      int nSlot = _GetFreePlayerSlot();
-//      if(nSlot<0) return; /* TODO: add request to some kind of queue, so we can
-//                                  play it later */    
-//      StreamPlayer *pStream = new StreamPlayer;
-//      pStream->initStream(File,1024*100);
-//      m_pPlayers[nSlot] = (SoundPlayer *)pStream;
-//    #endif
-//  }
-//
-//  void StreamPlayer::initStream(std::string File,int nBufSize) {
-//    m_vs.openFile(File);
-//    //printf("Streaming '%s'...\n",File.c_str());
-//    m_pcBuf = new unsigned char[nBufSize];
-//    m_nWritePos = m_nReadPos = 0;
-//    m_nBufSize = nBufSize;
-//    m_bNew = true;
-//    
-//    /* Build conversion structure */    
-//    Uint16 nSrcFormat,nDstFormat;
-//    Uint8 nSrcChannels,nDstChannels;
-//    int nSrcRate,nDstRate;
-//    
-//    if(Sound::getSampleBits() == 8) nDstFormat = AUDIO_S8;
-//    else nDstFormat = AUDIO_S16;
-//    
-//    nDstChannels = Sound::getChannels();
-//    nDstRate = Sound::getSampleRate();
-//    
-//    if(m_vs.getSampleBits() == 8) nSrcFormat = AUDIO_S8;
-//    else nSrcFormat = AUDIO_S16;
-//    
-//    nSrcChannels = m_vs.getChannels();
-//    nSrcRate = m_vs.getSampleRate();
-//    
-//    //printf("STREAM:  [%d-bit, %dhz, %d-channel] -> [%d-bit, %dhz, %d-channel]\n",
-//    //        nSrcFormat&0xff,nSrcRate,nSrcChannels,nDstFormat&0xff,nDstRate,nDstChannels);
-//            
-//    if(SDL_BuildAudioCVT(&m_cvt,nSrcFormat,nSrcChannels,nSrcRate,nDstFormat,nDstChannels,nDstRate) < 0) {
-//      m_vs.closeFile();
-//      Log("** Warning ** : StreamPlayer::initStream() : Failed to prepare audio conversion for '%s'!",File.c_str());
-//      throw Exception("unsupported audio format");
-//    }    
-//    
-//    /* Fill buffer */
-//    update();
-//  }
-//
-//  int StreamPlayer::fillBuffer(unsigned char *pcBuffer,int nSize) {    
-//    int nRem = m_nBufSize - m_nReadPos;
-//    
-//    if(nSize <= nRem) {
-//      memcpy(pcBuffer,&m_pcBuf[m_nReadPos],nSize);
-//      m_nReadPos += nSize;
-//      m_nReadPos = m_nReadPos % m_nBufSize;
-//      return nSize;
-//    }
-//    
-//    memcpy(pcBuffer,&m_pcBuf[m_nReadPos],nRem);
-//    memcpy(&pcBuffer[nRem],&m_pcBuf[0],nSize - nRem);
-//    m_nReadPos = nSize - nRem; 
-//    return nSize;
-//  }
-//  
-//  void StreamPlayer::update(void) {
-//    /* Determine how much we should decode, if anything */
-//    int nDecode;    
-//    if(m_nWritePos <= m_nReadPos) {
-//      nDecode = m_nReadPos - m_nWritePos;
-//    }
-//    else {
-//      nDecode = m_nBufSize - m_nWritePos + m_nReadPos;
-//    }
-//    
-//    if(m_bNew) {
-//      nDecode = m_nBufSize;
-//      m_bNew = false;
-//    }
-//    
-//    static unsigned char cLocalBuffer[400000]; /* static because ::update() of
-//                                                  streams are not called at the same time */
-//    if(nDecode > 0) {
-//      /* How much SOURCE data should we fetch then? */
-//      int nSrcDecode = nDecode / m_cvt.len_ratio;
-//    
-//      int nDecodedSrc = m_vs.decode(cLocalBuffer,nSrcDecode);
-//      
-//      m_cvt.buf = (Uint8 *)cLocalBuffer;
-//      m_cvt.len = nDecodedSrc;
-//      
-//      SDL_ConvertAudio(&m_cvt);
-//      
-//      int nActualNewData = nDecodedSrc * m_cvt.len_ratio;
-////      printf("got %d bytes in dest-format\n",nActualNewData);
-//      
-//      int nRem = m_nBufSize - m_nWritePos;
-//      if(nActualNewData <= nRem) {
-//        memcpy(&m_pcBuf[m_nWritePos],cLocalBuffer,nActualNewData);
-//        m_nWritePos += nActualNewData;
-//        m_nWritePos = m_nWritePos % m_nBufSize;
-//      }
-//      else {
-//        memcpy(&m_pcBuf[m_nWritePos],cLocalBuffer,nRem);
-//        memcpy(&m_pcBuf[0],&cLocalBuffer[nRem],nActualNewData-nRem);
-//        m_nWritePos = nActualNewData-nRem;
-//      }            
-//    }
-//  }  
-//  
-//  void StreamPlayer::shutdownPlayer(void) {
-//    delete [] m_pcBuf;
-//    m_vs.closeFile();    
-//  }
-//  
-//  /*==============================================================================
-//  Helpers
-//  ==============================================================================*/
-//  int Sound::_GetFreePlayerSlot(void) {
-//    for(int i=0;i<16;i++) {
-//      if(m_pPlayers[i] == NULL) return i;
-//    }
-//    return -1;
-//  }
 
   /*==============================================================================
   Engine sound simulator
   ==============================================================================*/
   void EngineSoundSimulator::update(float fTime) {
     if(Sound::isEnabled()) {
+      if(fTime < m_fLastBangTime) m_fLastBangTime = fTime; /* manage back in the past */
+
       if(m_fRPM > 100.0f) {
         /* Calculate the delay between the samples */
         float fInterval = (60.0f / m_fRPM) * 1.0f;
@@ -701,19 +425,57 @@ namespace vapp {
           /* Stroke! Determine a random sample to use */
           float x = ((float)rand()) / (float)RAND_MAX; /* linux likes insanely high RAND_MAX'es */
           int i = (int)(((float)m_BangSamples.size())*x);
-          if(i<0) i = 0;
-          if(i>=m_BangSamples.size()) i = m_BangSamples.size()-1;
+	  if(i < 0) i = 0;
+	  if(i>=m_BangSamples.size()) i = m_BangSamples.size()-1;
           
-          #if defined(USE_SDL_MIXER)
-            /* Play it */
-            Mix_PlayChannel(-1,m_BangSamples[i]->pChunk,0);
-          #endif
-          
-          m_fLastBangTime = fTime;
-        }
+#if defined(USE_SDL_MIXER)
+	  /* Play it */
+	  Mix_PlayChannel(-1,m_BangSamples[i]->pChunk,0);
+#endif
+	  m_fLastBangTime = fTime;
+	}
       }
     }
   }
-  
+
+  void Sound::playMusic(std::string i_musicPath) {
+    if(Mix_PlayingMusic() || Mix_FadingMusic()) {
+      stopMusic();
+    }
+
+    if(m_pMenuMusic != NULL) {
+      Mix_FreeMusic(m_pMenuMusic);
+      m_pMenuMusic = NULL;
+    }
+
+    /* No music available, try loading */
+#if defined(WIN32) /* this works around a bug in SDL_mixer 1.2.7 on Windows */
+    SDL_RWops *rwfp;
+    rwfp = SDL_RWFromFile(i_musicPath.c_str(), "rb");
+    if(rwfp != NULL) {
+      m_pMenuMusic = Mix_LoadMUS_RW(rwfp);
+    }
+#else
+    m_pMenuMusic = Mix_LoadMUS(i_musicPath.c_str());
+#endif
+
+    if(m_pMenuMusic == NULL) {
+      throw Exception("No music played !");
+    }
+
+    if(Mix_PlayMusic(m_pMenuMusic, -1) < 0) {
+      throw Exception("No music played !");
+    }
+  }
+
+  void Sound::stopMusic() {
+    if(m_pMenuMusic != NULL && Mix_PlayingMusic()) {
+      Mix_HaltMusic();
+    }
+  }
+
+  bool Sound::isPlayingMusic() {
+    return Mix_PlayingMusic();
+  }
 }
 
