@@ -23,8 +23,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "helpers/VExcept.h"
 #include "VXml.h"
 #include "VApp.h"
-#include "BuiltInFont.h"
 #include "md5sum/md5file.h"
+#include "db/xmDatabase.h"
 
 class vapp::App;
 
@@ -32,56 +32,11 @@ std::vector<Sprite*> Theme::getSpritesList() {
   return m_sprites;
 }
 
-void Theme::initDefaultFont() {
-    CBuiltInFont Fnt;
-
-    /* Create texture */
-    int nImgWidth = 256, nImgHeight = 256;
-    vapp::Color *pImgData = new vapp::Color[nImgWidth * nImgHeight];
-    memset(pImgData,0,nImgWidth * nImgHeight * sizeof(vapp::Color));
-  
-    /* Fill texture with glyphs */
-    int cx=0,cy=0,w=Fnt.getCharWidth(),h=Fnt.getCharHeight();
-    for(int i=0;i<256;i++) {
-      unsigned char *pc = Fnt.fetchChar(i);
-      for(int y=0;y<h;y++) {
-        for(int x=0;x<w;x++) {
-          unsigned char *pcT = (unsigned char *)&pImgData[(cx+x) + (cy+y)*nImgWidth];
-          pcT[0] = 255;
-          pcT[1] = 255;
-          pcT[2] = 255;
-          pcT[3] = pc[x+y*w];
-        }
-      }
-      
-      cx += w;
-      if(cx+w > nImgWidth) {
-        cx = 0;
-        cy += h;
-        if(cy+h > nImgHeight) {
-          delete [] pImgData;
-          throw vapp::TextureError("default font does not fit in texture");
-        }
-      }        
-    }
-    
-    /* Load it */
-    m_pDefaultFontTexture = m_texMan.createTexture("default-font",(unsigned char *)pImgData,
-               256,256,true);
-      
-    delete [] pImgData;
-}
-
-vapp::Texture* Theme::getDefaultFont() {
-  if(m_pDefaultFontTexture == NULL) {
-    initDefaultFont();
-  }
-  return m_pDefaultFontTexture;
+std::vector<Sound*> Theme::getSoundsList() {
+  return m_sounds;
 }
 
 Theme::Theme() {
-  m_pDefaultFontTexture = NULL;
-
   m_player = new BikerTheme(this,
           THEME_PLAYER_BODY,
           THEME_PLAYER_FRONT,
@@ -96,7 +51,6 @@ Theme::Theme() {
           THEME_PLAYER_UGLYWHEELCOLOR
           );
 
-#if defined(ALLOW_GHOST)
   m_ghost = new BikerTheme(this,
          THEME_GHOST_BODY,
          THEME_GHOST_FRONT,
@@ -110,7 +64,6 @@ Theme::Theme() {
          THEME_GHOST_UGLYRIDERCOLOR,
          THEME_GHOST_UGLYWHEELCOLOR
          );
-#endif
 }
 
 Theme::~Theme() {
@@ -118,7 +71,9 @@ Theme::~Theme() {
   delete m_ghost;
 
   cleanSprites();
-  
+  cleanMusics();
+  cleanSounds();
+
   /* Kill textures */
   m_texMan.unloadTextures();
 }
@@ -139,6 +94,8 @@ void Theme::load(std::string p_themeFile) {
   vapp::Log(std::string("Loading theme from file " + p_themeFile).c_str());
 
   cleanSprites(); /* removing existing sprites */
+  cleanMusics();
+  cleanSounds();
 
   vapp::XMLDocument v_ThemeXml;
   TiXmlDocument *v_ThemeXmlData;
@@ -174,6 +131,21 @@ void Theme::load(std::string p_themeFile) {
   }
 }
 
+bool Theme::isAFileOutOfDate(const std::string& i_file) {
+
+  /* files that you can found in themes files for compatibility reason but no need to download */
+
+  if(i_file == "Textures/UI/NewLevelsAvail.png") return true;
+  if(i_file == "Textures/Effects/Sky1.jpg")      return true;
+  if(i_file == "Textures/Effects/Sky2.jpg")      return true;
+  if(i_file == "Textures/Effects/Sky2Drift.jpg") return true;
+  if(i_file == "Textures/Fonts/MFont.png")       return true;
+  if(i_file == "Textures/Fonts/SFont.png")       return true;
+  if(i_file == "Textures/UI/Loading.png")        return true;
+
+  return false;
+}
+
 void Theme::loadSpritesFromXML(TiXmlElement *p_ThemeXmlDataElement) {
   std::string v_spriteType;
   bool v_isAnimation;
@@ -217,13 +189,60 @@ void Theme::loadSpritesFromXML(TiXmlElement *p_ThemeXmlDataElement) {
       vapp::Log("Warning: unknown type '%s' in theme file !", v_spriteType.c_str());
     }
   }
+
+  /* get musics */
+  std::string v_musicName;
+  std::string v_musicFile;
+
+  for(TiXmlElement *pVarElem = p_ThemeXmlDataElement->FirstChildElement("music");
+      pVarElem!=NULL;
+      pVarElem = pVarElem->NextSiblingElement("music")
+      ) {
+
+    pc = pVarElem->Attribute("name");
+    if(pc == NULL) { continue; }
+    v_musicName = pc;
+
+    pc = pVarElem->Attribute("file");
+    if(pc == NULL) { continue; }
+    v_musicFile = pc;
+
+    if(isAFileOutOfDate(THEME_MUSICS_FILE_DIR + std::string("/") + v_musicFile) == false) {
+      m_musics.push_back(new Music(this, v_musicName, v_musicFile));
+      m_requiredFiles.push_back(THEME_MUSICS_FILE_DIR + std::string("/") + v_musicFile);    
+    }
+  }
+
+  /* get sounds */
+  std::string v_soundName;
+  std::string v_soundFile;
+
+  for(TiXmlElement *pVarElem = p_ThemeXmlDataElement->FirstChildElement("sound");
+      pVarElem!=NULL;
+      pVarElem = pVarElem->NextSiblingElement("sound")
+      ) {
+
+    pc = pVarElem->Attribute("name");
+    if(pc == NULL) { continue; }
+    v_soundName = pc;
+
+    pc = pVarElem->Attribute("file");
+    if(pc == NULL) { continue; }
+    v_soundFile = pc;
+
+    if(isAFileOutOfDate(THEME_SOUNDS_FILE_DIR + std::string("/") + v_soundFile) == false) {
+      m_sounds.push_back(new Sound(this, v_soundName, v_soundFile));
+      m_requiredFiles.push_back(THEME_SOUNDS_FILE_DIR + std::string("/") + v_soundFile);    
+    }
+  }
+
 }
 
 Sprite* Theme::getSprite(enum SpriteType pSpriteType, std::string pName) {
   for(unsigned int i=0; i<m_sprites.size(); i++) {
     if(m_sprites[i]->getType() == pSpriteType) {
       if(m_sprites[i]->getName() == pName) {
-  return m_sprites[i];
+	return m_sprites[i];
       }
     }
   }
@@ -231,6 +250,24 @@ Sprite* Theme::getSprite(enum SpriteType pSpriteType, std::string pName) {
   return NULL;
 }
 
+Music* Theme::getMusic(std::string i_name) {
+  for(unsigned int i=0; i<m_musics.size(); i++) {
+    if(m_musics[i]->Name() == i_name) {
+      return m_musics[i];
+    }
+  }
+  throw Exception("Music " + i_name + " not found");
+}
+
+Sound* Theme::getSound(std::string i_name) {
+  for(unsigned int i=0; i<m_sounds.size(); i++) {
+    if(m_sounds[i]->Name() == i_name) {
+      return m_sounds[i];
+    }
+  }
+  throw Exception("Sound " + i_name + " not found");
+}
+
 void Theme::cleanSprites() {
   for(unsigned int i=0; i<m_sprites.size(); i++) {
     delete m_sprites[i];
@@ -238,6 +275,20 @@ void Theme::cleanSprites() {
   m_sprites.clear();
 }
 
+void Theme::cleanMusics() {
+   for(unsigned int i=0; i<m_musics.size(); i++) {
+    delete m_musics[i];
+  }
+  m_musics.clear(); 
+}
+
+void Theme::cleanSounds() {
+   for(unsigned int i=0; i<m_sounds.size(); i++) {
+    delete m_sounds[i];
+  }
+  m_sounds.clear(); 
+}
+
 void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
   std::string v_name;
   std::string v_fileBase;
@@ -252,15 +303,15 @@ void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
   float global_delay   = 0.1;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Animation with no name"); return;}
   v_name = pc;
   
   pc = pVarElem->Attribute("fileBase");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Animation with no fileBase"); return;}
   v_fileBase = pc;
 
   pc = pVarElem->Attribute("fileExtension");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Animation with no fileExtension"); return;}
   v_fileExtension = pc;
 
   pc = pVarElem->Attribute("centerX");
@@ -308,13 +359,17 @@ void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
     pc = pVarSubElem->Attribute("delay");
     if(pc != NULL) {v_delay = atof(pc);} else {v_delay = global_delay;}
 
-    v_anim->addFrame(v_centerX, v_centerY, v_width, v_height, v_delay);
+    if(isAFileOutOfDate(THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
+			v_fileBase + std::string(buf) + std::string(".") + v_fileExtension) == false) {
+      v_anim->addFrame(v_centerX, v_centerY, v_width, v_height, v_delay);
 
-    if(n < 100) {
-      sprintf(buf, "%02i", n);
-      m_requiredFiles.push_back(THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") + v_fileBase + std::string(buf) + std::string(".") + v_fileExtension);
+      if(n < 100) {
+	sprintf(buf, "%02i", n);
+	m_requiredFiles.push_back(THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
+				  v_fileBase + std::string(buf) + std::string(".") + v_fileExtension);
+      }
+      n++;
     }
-    n++;
   }
 }
 
@@ -324,62 +379,67 @@ void Theme::newBikerPartSpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** BikerPart with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** BikerPart with no file"); return;}
   v_fileName = pc;
 
-  m_sprites.push_back(new BikerPartSprite(this, v_name, v_fileName));
-  m_requiredFiles.push_back(THEME_BIKERPART_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_BIKERPART_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new BikerPartSprite(this, v_name, v_fileName));
+    m_requiredFiles.push_back(THEME_BIKERPART_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newDecorationSpriteFromXML(TiXmlElement *pVarElem) {
   std::string v_name;
   std::string v_fileName;
-  std::string v_width;
-  std::string v_height;
-  std::string v_centerX;
-  std::string v_centerY;
+  float v_width;
+  float v_height;
+  float v_centerX;
+  float v_centerY;
   std::string v_blendmode = "default";
   const char *pc;
 
+  float global_centerX = 0.5;
+  float global_centerY = 0.5;
+  float global_width   = 1.0;
+  float global_height  = 1.0;
+
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Sprite with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Sprite with no name"); return;}
   v_fileName = pc;
 
   pc = pVarElem->Attribute("width");
-  if(pc == NULL) {return;}
-  v_width = pc;
+  if(pc != NULL) {v_width = atof(pc);} else {v_width = global_width;}
 
   pc = pVarElem->Attribute("height");
-  if(pc == NULL) {return;}
-  v_height = pc;
+  if(pc != NULL) {v_height = atof(pc);} else {v_height = global_height;}
 
   pc = pVarElem->Attribute("centerX");
-  if(pc == NULL) {return;}
-  v_centerX = pc;
+  if(pc != NULL) {v_centerX = atof(pc);} else {v_centerX = global_centerX;}
 
   pc = pVarElem->Attribute("centerY");
-  if(pc == NULL) {return;}
-  v_centerY = pc;
+  if(pc != NULL) {v_centerY = atof(pc);} else {v_centerY = global_centerY;}
   
   pc = pVarElem->Attribute("blendmode");
   if(pc != NULL) v_blendmode = pc;
 
-  m_sprites.push_back(new DecorationSprite(this, v_name, v_fileName,
-             atof(v_width.c_str()),
-             atof(v_height.c_str()),
-             atof(v_centerX.c_str()),
-             atof(v_centerY.c_str()),
-             strToBlendMode(v_blendmode)
-             ));
-  m_requiredFiles.push_back(THEME_DECORATION_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_DECORATION_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new DecorationSprite(this, v_name, v_fileName,
+					     v_width,
+					     v_height,
+					     v_centerX,
+					     v_centerY,
+					     strToBlendMode(v_blendmode)
+					     ));
+    m_requiredFiles.push_back(THEME_DECORATION_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newEffectSpriteFromXML(TiXmlElement *pVarElem) {
@@ -388,15 +448,17 @@ void Theme::newEffectSpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Effect with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Effect with no file"); return;}
   v_fileName = pc;
 
-  m_sprites.push_back(new EffectSprite(this, v_name, v_fileName));
-  m_requiredFiles.push_back(THEME_EFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_EFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new EffectSprite(this, v_name, v_fileName));
+    m_requiredFiles.push_back(THEME_EFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newEdgeEffectSpriteFromXML(TiXmlElement *pVarElem) {
@@ -407,25 +469,27 @@ void Theme::newEdgeEffectSpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Edge with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Edge with no file"); return;}
   v_fileName = pc;
 
   pc = pVarElem->Attribute("scale");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Edge with no scale"); return;}
   v_scale = pc;
 
   pc = pVarElem->Attribute("depth");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("Edge with no depth"); return;}
   v_depth = pc;
 
-  m_sprites.push_back(new EdgeEffectSprite(this, v_name, v_fileName,
-             atof(v_scale.c_str()),
-             atof(v_depth.c_str())));
-  m_requiredFiles.push_back(THEME_EDGEEFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_EDGEEFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new EdgeEffectSprite(this, v_name, v_fileName,
+					     atof(v_scale.c_str()),
+					     atof(v_depth.c_str())));
+    m_requiredFiles.push_back(THEME_EDGEEFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newFontSpriteFromXML(TiXmlElement *pVarElem) {
@@ -434,15 +498,17 @@ void Theme::newFontSpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Font with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("Font with no file"); return;}
   v_fileName = pc;
 
-  m_sprites.push_back(new FontSprite(this, v_name, v_fileName));
-  m_requiredFiles.push_back(THEME_FONT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_FONT_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new FontSprite(this, v_name, v_fileName));
+    m_requiredFiles.push_back(THEME_FONT_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newMiscSpriteFromXML(TiXmlElement *pVarElem) {
@@ -451,15 +517,17 @@ void Theme::newMiscSpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Misc with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Misc with no file"); return;}
   v_fileName = pc;
 
-  m_sprites.push_back(new MiscSprite(this, v_name, v_fileName));
-  m_requiredFiles.push_back(THEME_MISC_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_MISC_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new MiscSprite(this, v_name, v_fileName));
+    m_requiredFiles.push_back(THEME_MISC_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newUISpriteFromXML(TiXmlElement *pVarElem) {
@@ -468,15 +536,17 @@ void Theme::newUISpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** UI with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** UI with no file"); return;}
   v_fileName = pc;
 
-  m_sprites.push_back(new UISprite(this, v_name, v_fileName));
-  m_requiredFiles.push_back(THEME_UI_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_UI_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new UISprite(this, v_name, v_fileName));
+    m_requiredFiles.push_back(THEME_UI_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 void Theme::newTextureSpriteFromXML(TiXmlElement *pVarElem) {
@@ -485,26 +555,26 @@ void Theme::newTextureSpriteFromXML(TiXmlElement *pVarElem) {
   const char *pc;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Texture with no name"); return;}
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {return;}
+  if(pc == NULL) {vapp::Log("** Texture with no file"); return;}
   v_fileName = pc;
 
-  m_sprites.push_back(new TextureSprite(this, v_name, v_fileName));
-  m_requiredFiles.push_back(THEME_TEXTURE_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  if(isAFileOutOfDate(THEME_TEXTURE_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
+    m_sprites.push_back(new TextureSprite(this, v_name, v_fileName));
+    m_requiredFiles.push_back(THEME_TEXTURE_SPRITE_FILE_DIR + std::string("/") + v_fileName);
+  }
 }
 
 BikerTheme* Theme::getPlayerTheme() {
   return m_player;
 }
 
-#if defined(ALLOW_GHOST)
 BikerTheme* Theme::getGhostTheme() {
   return m_ghost;
 }
-#endif
 
 Sprite::Sprite(Theme* p_associated_theme, std::string v_name) {
   m_associated_theme = p_associated_theme;
@@ -520,22 +590,15 @@ vapp::Texture* Sprite::getTexture(bool bSmall, bool bClamp, vapp::FilterMode eFi
   v_currentTexture = getCurrentTexture();
   if(v_currentTexture == NULL) {
     v_currentTexture = m_associated_theme->loadTexture(getCurrentTextureFileName(),
-                   bSmall,
-                   bClamp,
-                   eFilterMode);
-    if(v_currentTexture == NULL) { 
-      throw Exception("Unable to load texture '" + getCurrentTextureFileName() + "'");
-    }
+						       bSmall,
+						       bClamp,
+						       eFilterMode);
     setCurrentTexture(v_currentTexture);
   }
-
+  
   return v_currentTexture;
 }
 
-std::string Sprite::getName() {
-  return m_name;
-}
-
 SpriteBlendMode Sprite::getBlendMode() {
   return m_blendmode;
 }
@@ -553,16 +616,13 @@ AnimationSprite::AnimationSprite(Theme* p_associated_theme, std::string p_name,
   m_fileBase      = p_fileBase;
   m_fileExtension = p_fileExtention;
   m_fFrameTime    = 0.0;
+  m_type          = SPRITE_TYPE_ANIMATION;
 }
 
 AnimationSprite::~AnimationSprite() {
   cleanFrames();
 }
 
-enum SpriteType AnimationSprite::getType() {
-  return SPRITE_TYPE_ANIMATION;
-}
-
 std::string AnimationSprite::getFileDir() {
   return THEME_ANIMATION_SPRITE_FILE_DIR;
 }
@@ -671,6 +731,7 @@ float AnimationSpriteFrame::getDelay() const {
 }
 
 BikerPartSprite::BikerPartSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
+  m_type = SPRITE_TYPE_BIKERPART;
 }
 
 BikerPartSprite::~BikerPartSprite() {
@@ -680,15 +741,12 @@ std::string BikerPartSprite::getFileDir() {
   return THEME_BIKERPART_SPRITE_FILE_DIR;
 }
 
-enum SpriteType BikerPartSprite::getType() {
-  return SPRITE_TYPE_BIKERPART;
-}
-
 DecorationSprite::DecorationSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName, float p_width, float p_height, float p_centerX, float p_centerY, SpriteBlendMode BlendMode) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
   m_width   = p_width;
   m_height  = p_height;
   m_centerX = p_centerX;
   m_centerY = p_centerY;
+  m_type    = SPRITE_TYPE_DECORATION;
   
   setBlendMode(BlendMode);
 }
@@ -696,10 +754,6 @@ DecorationSprite::DecorationSprite(Theme* p_associated_theme, std::string p_name
 DecorationSprite::~DecorationSprite() {
 }
 
-enum SpriteType DecorationSprite::getType() {
-  return SPRITE_TYPE_DECORATION;
-}
-
 std::string DecorationSprite::getFileDir() {
   return THEME_DECORATION_SPRITE_FILE_DIR;
 }
@@ -721,15 +775,12 @@ float DecorationSprite::getCenterY() {
 }
 
 EffectSprite::EffectSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
+  m_type = SPRITE_TYPE_EFFECT;
 }
 
 EffectSprite::~EffectSprite() {
 }
 
-enum SpriteType EffectSprite::getType() {
-  return SPRITE_TYPE_EFFECT;
-}
-
 std::string EffectSprite::getFileDir() {
   return THEME_EFFECT_SPRITE_FILE_DIR;
 }
@@ -737,15 +788,12 @@ std::string EffectSprite::getFileDir() {
 EdgeEffectSprite::EdgeEffectSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename, float p_fScale, float p_fDepth) : SimpleFrameSprite(p_associated_theme, p_name, p_filename) {
   m_fScale = p_fScale;
   m_fDepth = p_fDepth;
+  m_type   = SPRITE_TYPE_EDGEEFFECT;
 }
 
 EdgeEffectSprite::~EdgeEffectSprite() {
 }
 
-enum SpriteType EdgeEffectSprite::getType() {
-  return SPRITE_TYPE_EDGEEFFECT;
-}
-
 std::string EdgeEffectSprite::getFileDir() {
   return THEME_EDGEEFFECT_SPRITE_FILE_DIR;
 }
@@ -758,58 +806,50 @@ float EdgeEffectSprite::getDepth() const {
   return m_fDepth;
 }
 
+vapp::Texture* EdgeEffectSprite::getTexture(bool bSmall, bool bClamp, vapp::FilterMode eFilterMode) {
+  Sprite::getTexture(bSmall, bClamp, eFilterMode);
+}
+
 FontSprite::FontSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
+  m_type = SPRITE_TYPE_FONT;
 }
 
 FontSprite::~FontSprite() {
 }
 
-enum SpriteType FontSprite::getType() {
-  return SPRITE_TYPE_FONT;
-}
-
 std::string FontSprite::getFileDir() {
   return THEME_FONT_SPRITE_FILE_DIR;
 }
 
 MiscSprite::MiscSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
+  m_type = SPRITE_TYPE_MISC;
 }
 
 MiscSprite::~MiscSprite() {
 }
 
-enum SpriteType MiscSprite::getType() {
-  return SPRITE_TYPE_MISC;
-}
-
 std::string MiscSprite::getFileDir() {
   return THEME_MISC_SPRITE_FILE_DIR;
 }
 
 UISprite::UISprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
+  m_type = SPRITE_TYPE_UI;
 }
 
 UISprite::~UISprite() {
 }
 
-enum SpriteType UISprite::getType() {
-  return SPRITE_TYPE_UI;
-}
-
 std::string UISprite::getFileDir() {
   return THEME_UI_SPRITE_FILE_DIR;
 }
 
 TextureSprite::TextureSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
+  m_type = SPRITE_TYPE_TEXTURE;
 }
 
 TextureSprite::~TextureSprite() {
 }
 
-enum SpriteType TextureSprite::getType() {
-  return SPRITE_TYPE_TEXTURE;
-}
-
 std::string TextureSprite::getFileDir() {
   return THEME_TEXTURE_SPRITE_FILE_DIR;
 }
@@ -909,38 +949,15 @@ vapp::Color BikerTheme::getUglyWheelColor() {
   return m_UglyWheelColor;
 }
 
-bool ThemeChoicer::ExistThemeName(std::string p_themeName) {
-  for(unsigned int i=0; i<m_choices.size(); i++) {
-    if(m_choices[i]->ThemeName() == p_themeName) {
-      return true;
-    }
-  }
-  return false;
-}
-
-#if defined(SUPPORT_WEBACCESS)
-  ThemeChoicer::ThemeChoicer(vapp::WWWAppInterface *p_WebApp,
-           const ProxySettings *p_proxy_settings) {                    
-#else
-  ThemeChoicer::ThemeChoicer(void) {  
-#endif
-
-#if defined(SUPPORT_WEBACCESS)
+ThemeChoicer::ThemeChoicer(vapp::WWWAppInterface *p_WebApp,
+			   const ProxySettings *p_proxy_settings) {                    
   m_webThemes = new WebThemes(p_WebApp, p_proxy_settings);
-#endif
-
-  initList();
 }
 
 ThemeChoicer::~ThemeChoicer() {
-  cleanList();
-
-#if defined(SUPPORT_WEBACCESS)
   delete m_webThemes;
-#endif
 }
 
-#if defined(SUPPORT_WEBACCESS)
  void ThemeChoicer::setURL(const std::string &p_url) {
    m_webThemes->setURL(p_url);
  }
@@ -948,108 +965,26 @@ ThemeChoicer::~ThemeChoicer() {
  void ThemeChoicer::setURLBase(const std::string &p_urlBase) {
    m_webThemes->setURLBase(p_urlBase);
  }
-#endif
 
-std::string ThemeChoicer::getFileName(std::string p_themeName) {
-  for(unsigned int i=0; i<m_choices.size(); i++) {
-    if(m_choices[i]->ThemeName() == p_themeName) {
-      return m_choices[i]->ThemeFile();
-    }
-  }
-  return "";
-}
-
-ThemeChoice* ThemeChoicer::getChoiceByName(std::string p_themeName) {
-  for(unsigned int i=0; i<m_choices.size(); i++) {
-    if(m_choices[i]->ThemeName() == p_themeName) {
-      return m_choices[i];
-    }
-  }
-  return NULL;
-}
-
-void ThemeChoicer::cleanList() {
-  for(unsigned int i=0; i<m_choices.size(); i++) {
-    delete m_choices[i];
-  }
-  m_choices.clear();
-}
-
-void ThemeChoicer::initList() {
-  cleanList();
-
-  std::vector<std::string> v_themesFiles = vapp::FS::findPhysFiles(std::string(THEMES_DIRECTORY) + std::string("/*.xml"), true);
+void ThemeChoicer::initThemesFromDir(xmDatabase *i_db) {
+  std::vector<std::string> v_themesFiles = vapp::FS::findPhysFiles(std::string(THEMES_DIRECTORY)
+								   + std::string("/*.xml"), true);
   std::string v_name;
 
-  /* first, load theme which are in the user dir because, a same theme can be stored
-     in files having different name
-  */
+  i_db->themes_add_begin();
   for(unsigned int i=0; i<v_themesFiles.size(); i++) {
     try {
-      if(vapp::FS::isInUserDir(v_themesFiles[i])) {
-  v_name = getThemeNameFromFile(v_themesFiles[i]);
-  if(ExistThemeName(v_name) == false) {
-    m_choices.push_back(new ThemeChoice(v_name, v_themesFiles[i], true));
-  } else {
-    vapp::Log(std::string("Theme " + v_name + " is present several times").c_str());
-  }
-      }
-    } catch(Exception &e) {
-      /* anyway, give up this theme */
-    }
-  }
-
-  /* load the other theme from not the user directory */
-  for(unsigned int i=0; i<v_themesFiles.size(); i++) {
-    try {
-      if(vapp::FS::isInUserDir(v_themesFiles[i]) == false) {
-  v_name = getThemeNameFromFile(v_themesFiles[i]);
-  if(ExistThemeName(v_name) == false) {
-    m_choices.push_back(new ThemeChoice(v_name, v_themesFiles[i], true));
-  } else {
-    // no warning for non user dir
-    // vapp::Log(std::string("Theme " + v_name + " is present several times").c_str());
-  }
-      }
-    } catch(Exception &e) {
-      /* anyway, give up this theme */
-    }
-  }
-
-#if defined(SUPPORT_WEBACCESS)
-  try {
-    /* add available theme not installed */
-    m_webThemes->upgrade();
-    std::vector<WebTheme*> v_availableThemes = m_webThemes->getAvailableThemes();
-        
-    for(unsigned int i=0; i<v_availableThemes.size(); i++) {
-      if(ExistThemeName(v_availableThemes[i]->getName()) == false) {
-  /* this theme is avaible */
-  m_choices.push_back(new ThemeChoice(v_availableThemes[i]->getName(), "", false));
+      v_name = getThemeNameFromFile(v_themesFiles[i]);
+      if(i_db->themes_exists(v_name) == false) {
+	i_db->themes_add(v_name, v_themesFiles[i]);
       } else {
-  /* the theme already exists ; check sum */
-  for(unsigned int j=0; j<m_choices.size(); j++) {
-
-    if(m_choices[j]->ThemeName() == v_availableThemes[i]->getName()) {
-      std::string v_localMd5;
-      try {
-        v_localMd5 = md5file(m_choices[j]->ThemeFile());
-      } catch(Exception &e) {
-        v_localMd5 = "";
-      }
-      
-      if(v_localMd5 != v_availableThemes[i]->getSum()) {
-        /* this theme can be updated */
-        m_choices[j]->setRequireUpdate(true);
+	vapp::Log(std::string("Theme " + v_name + " is present several times").c_str());
       }
+    } catch(Exception &e) {
+      /* anyway, give up this theme */
     }
   }
-      }
-    }
-  } catch(Exception &e) {
-    /* hum, sorry, you will not see avaible theme to download */
-  }
-#endif
+  i_db->themes_add_end();
 }
 
 std::string ThemeChoicer::getThemeNameFromFile(std::string p_themeFile) {
@@ -1081,57 +1016,12 @@ std::string ThemeChoicer::getThemeNameFromFile(std::string p_themeFile) {
   return m_name;
 }
 
-std::vector<ThemeChoice*> ThemeChoicer::getChoices() {
-  return m_choices;
+void ThemeChoicer::updateFromWWW(xmDatabase *i_db) {
+  m_webThemes->update(i_db);
 }
 
-#if defined(SUPPORT_WEBACCESS)
-void ThemeChoicer::updateFromWWW() {
-  m_webThemes->update();
-  initList();
-}
-
-bool ThemeChoicer::isUpdatableThemeFromWWW(ThemeChoice* pThemeChoice) {
-  return m_webThemes->isUpgradable(pThemeChoice);
-}
-
-void ThemeChoicer::updateThemeFromWWW(ThemeChoice* pThemeChoice) {
-  m_webThemes->upgrade(pThemeChoice);
-}
-#endif
-
-ThemeChoice::ThemeChoice(std::string p_themeName, std::string p_themeFile, bool p_hosted) {
-  m_themeName     = p_themeName;
-  m_themeFile     = p_themeFile;
-  m_hosted        = p_hosted;
-  m_requireUpdate = false;
-}
-
-ThemeChoice::~ThemeChoice() {
-}
-
-std::string ThemeChoice::ThemeName() {
-  return m_themeName;
-}
-
-std::string ThemeChoice::ThemeFile() {
-  return m_themeFile;
-}
-
-bool ThemeChoice::getHosted() {
-  return m_hosted;
-}
-
-void ThemeChoice::setRequireUpdate(bool b) {
-  m_requireUpdate = b;
-}
- 
-bool ThemeChoice::getRequireUpdate() {
-  return m_requireUpdate;
-}
-
- void ThemeChoice::setHosted(bool b) {
-   m_hosted = b;
+void ThemeChoicer::updateThemeFromWWW(xmDatabase *i_db, const std::string& i_id_file) {
+  m_webThemes->upgrade(i_db, i_id_file);
 }
 
 SpriteBlendMode Theme::strToBlendMode(const std::string &s) {
@@ -1143,3 +1033,45 @@ std::string Theme::blendModeToStr(SpriteBlendMode Mode) {
   if(Mode == SPRITE_BLENDMODE_ADDITIVE) return "add";
   return "default";
 }
+
+Music::Music(Theme* p_associated_theme, std::string i_name, std::string i_fileName) {
+  m_associated_theme = p_associated_theme;
+  m_name     	      = i_name;
+  m_fileName 	      = i_fileName;
+}
+ 
+Music::~Music() {
+}
+
+std::string Music::Name() const {
+  return m_name;
+}
+
+std::string Music::FileName() const {
+  return m_fileName;
+}
+ 
+std::string Music::FilePath() const {
+  return vapp::FS::FullPath(THEME_MUSICS_FILE_DIR + std::string("/") + m_fileName);
+}
+
+Sound::Sound(Theme* p_associated_theme, std::string i_name, std::string i_fileName) {
+  m_associated_theme = p_associated_theme;
+  m_name     	      = i_name;
+  m_fileName 	      = i_fileName;
+}
+ 
+Sound::~Sound() {
+}
+
+std::string Sound::Name() const {
+  return m_name;
+}
+
+std::string Sound::FileName() const {
+  return m_fileName;
+}
+ 
+std::string Sound::FilePath() const {
+  return THEME_SOUNDS_FILE_DIR + std::string("/") + m_fileName;
+}
