@@ -32,6 +32,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 class ProxySettings;
 class WebThemes;
+class xmDatabase;
 
 #define THEMES_DIRECTORY "Themes"
 #define THEME_SPRITE_FILE_DIR "Textures"
@@ -44,6 +45,8 @@ class WebThemes;
 #define THEME_TEXTURE_SPRITE_FILE_DIR    THEME_SPRITE_FILE_DIR"/Textures"
 #define THEME_FONT_SPRITE_FILE_DIR       THEME_SPRITE_FILE_DIR"/Fonts"
 #define THEME_EDGEEFFECT_SPRITE_FILE_DIR THEME_SPRITE_FILE_DIR"/Effects"
+#define THEME_MUSICS_FILE_DIR            THEME_SPRITE_FILE_DIR"/Musics"
+#define THEME_SOUNDS_FILE_DIR            THEME_SPRITE_FILE_DIR"/Sounds"
 
 #define THEME_PLAYER_BODY     "PlayerBikerBody"
 #define THEME_PLAYER_FRONT    "PlayerBikerFront"
@@ -57,7 +60,6 @@ class WebThemes;
 #define THEME_PLAYER_UGLYRIDERCOLOR MAKE_COLOR(0,255,0,255)
 #define THEME_PLAYER_UGLYWHEELCOLOR MAKE_COLOR(255,0,0,255)
 
-#if defined(ALLOW_GHOST)
 #define THEME_GHOST_BODY     "GhostBikerBody"
 #define THEME_GHOST_FRONT    "GhostBikerFront"
 #define THEME_GHOST_REAR     "GhostBikerRear"
@@ -69,7 +71,6 @@ class WebThemes;
 #define THEME_GHOST_UPPERLEG "GhostUpperLeg"
 #define THEME_GHOST_UGLYRIDERCOLOR MAKE_COLOR(100,100,128,255)
 #define THEME_GHOST_UGLYWHEELCOLOR MAKE_COLOR(100,100,128,255)
-#endif
 
 #define THEME_DEFAULT_THEMENAME "Classic"
 
@@ -82,7 +83,7 @@ class WebThemes;
     SPRITE_TYPE_MISC,
     SPRITE_TYPE_TEXTURE,
     SPRITE_TYPE_UI,
-    SPRITE_TYPE_EDGEEFFECT
+    SPRITE_TYPE_EDGEEFFECT,
   };
   
   enum SpriteBlendMode {
@@ -93,13 +94,49 @@ class WebThemes;
 class Theme;
 class BikerTheme;
 
+class Music {
+ public:
+  Music(Theme* p_associated_theme, std::string i_name, std::string i_fileName);
+  ~Music();
+
+  std::string Name() const;
+  std::string FileName() const;
+  std::string FilePath() const;
+
+ private:
+  Theme* m_associated_theme;
+  std::string m_name;
+  std::string m_fileName;
+};
+
+class Sound {
+ public:
+  Sound(Theme* p_associated_theme, std::string i_name, std::string i_fileName);
+  ~Sound();
+
+  std::string Name() const;
+  std::string FileName() const;
+  std::string FilePath() const;
+
+ private:
+  Theme* m_associated_theme;
+  std::string m_name;
+  std::string m_fileName;
+};
+
 class Sprite {
   public:
   Sprite(Theme* p_associated_theme, std::string v_name);
   virtual ~Sprite();
 
-  virtual enum SpriteType getType() = 0;
-  std::string getName();
+  /* no inline with virtual functions, so, no longer virtual */
+  inline enum SpriteType getType() {
+    return m_type;
+  }
+  /* called many many many times, so we inline it, and make it return a ref */
+  inline std::string& getName() {
+    return m_name;
+  }
   SpriteBlendMode getBlendMode();
   void setBlendMode(SpriteBlendMode Mode);
   void load();
@@ -108,13 +145,14 @@ class Sprite {
      the bSmall, bClamp, bFilter parameters are considerated 
      only the first time that getTexture is called for a given sprite
   */
-  vapp::Texture* getTexture(bool bSmall=false, bool bClamp=false, vapp::FilterMode eFilterMode = vapp::FM_LINEAR);
+  vapp::Texture* getTexture(bool bSmall=false, bool bClamp=false, vapp::FilterMode eFilterMode = vapp::FM_MIPMAP);
 
  protected:
   virtual vapp::Texture* getCurrentTexture() = 0;
   virtual std::string getCurrentTextureFileName() = 0;
   virtual void setCurrentTexture(vapp::Texture *p_texture) = 0;
   virtual std::string getFileDir();
+  enum SpriteType m_type;
 
   private:
   Theme* m_associated_theme;
@@ -141,7 +179,6 @@ class TextureSprite : public SimpleFrameSprite {
  public:
   TextureSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename);
   virtual ~TextureSprite();
-  enum SpriteType getType();
 
  protected:
   std::string getFileDir();
@@ -153,7 +190,6 @@ class BikerPartSprite : public SimpleFrameSprite {
  public:
   BikerPartSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename);
   virtual ~BikerPartSprite();
-  enum SpriteType getType();
 
  protected:
   std::string getFileDir();
@@ -196,7 +232,6 @@ class AnimationSprite : public Sprite {
  public:
   AnimationSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileBase, std::string p_fileExtention);
   virtual ~AnimationSprite();
-  enum SpriteType getType();
   float getCenterX();
   float getCenterY();
   float getWidth();
@@ -224,7 +259,6 @@ class EffectSprite : public SimpleFrameSprite {
  public:
   EffectSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename);
   virtual ~EffectSprite();
-  enum SpriteType getType();
 
  protected:
   std::string getFileDir();
@@ -236,7 +270,8 @@ class EdgeEffectSprite : public SimpleFrameSprite {
  public:
   EdgeEffectSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename, float fScale, float fDepth);
   virtual ~EdgeEffectSprite();
-  enum SpriteType getType();
+  vapp::Texture* getTexture(bool bSmall=false, bool bClamp=false, vapp::FilterMode eFilterMode = vapp::FM_LINEAR);
+
   float getScale() const;
   float getDepth() const;
 
@@ -252,7 +287,6 @@ class FontSprite : public SimpleFrameSprite {
  public:
   FontSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename);
   virtual ~FontSprite();
-  enum SpriteType getType();
 
  protected:
   std::string getFileDir();
@@ -264,7 +298,6 @@ class MiscSprite : public SimpleFrameSprite {
  public:
   MiscSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename);
   virtual ~MiscSprite();
-  enum SpriteType getType();
 
  protected:
   std::string getFileDir();
@@ -276,7 +309,6 @@ class UISprite : public SimpleFrameSprite {
  public:
   UISprite(Theme* p_associated_theme, std::string p_name, std::string p_filename);
   virtual ~UISprite();
-  enum SpriteType getType();
 
  protected:
   std::string getFileDir();
@@ -288,7 +320,6 @@ class DecorationSprite : public SimpleFrameSprite {
  public:
   DecorationSprite(Theme* p_associated_theme, std::string p_name, std::string p_filename, float p_width, float p_height, float p_centerX, float p_centerY, SpriteBlendMode p_blendmode);
   virtual ~DecorationSprite();
-  enum SpriteType getType();
 
   float getWidth();
   float getHeight();
@@ -315,36 +346,36 @@ class Theme {
 
   std::string Name() const;
   Sprite* getSprite(enum SpriteType pSpriteType, std::string pName);
+  Music* getMusic(std::string i_name);
+  Sound* getSound(std::string i_name);
   vapp::Texture* loadTexture(std::string p_fileName,
 			     bool bSmall=false,
 			     bool bClamp=false,
-			     vapp::FilterMode eFilterMode = vapp::FM_LINEAR);
+			     vapp::FilterMode eFilterMode = vapp::FM_MIPMAP);
 
-  vapp::Texture* getDefaultFont();
   std::vector<Sprite*> getSpritesList();
+  std::vector<Sound*> getSoundsList();
   std::vector<std::string>* getRequiredFiles();
 
   BikerTheme* getPlayerTheme();
-#if defined(ALLOW_GHOST)
   BikerTheme* getGhostTheme();
-#endif
 
   private:
-  void initDefaultFont();
-
   vapp::TextureManager m_texMan;
   std::string m_name;
   std::vector<Sprite*> m_sprites;
+  std::vector<Music*> m_musics;
+  std::vector<Sound*> m_sounds;
   std::vector<std::string> m_requiredFiles;
 
-  vapp::Texture *m_pDefaultFontTexture;
+  bool isAFileOutOfDate(const std::string& i_file); // to not download old files for compatibilities
 
   BikerTheme *m_player;
-#if defined(ALLOW_GHOST)
   BikerTheme *m_ghost;
-#endif
 
   void cleanSprites();
+  void cleanMusics();
+  void cleanSounds();
 
   void loadSpritesFromXML(TiXmlElement *p_ThemeXmlDataElement);
 
@@ -409,61 +440,23 @@ class BikerTheme {
   std::string m_UpperLeg;
 };
 
-class ThemeChoice {
- public:
-  ThemeChoice(std::string p_themeName, std::string p_themeFile, bool p_hosted);
-  ~ThemeChoice();
-  std::string ThemeName();
-  std::string ThemeFile();
-
-  void setRequireUpdate(bool b);
-  bool getRequireUpdate();
-  void setHosted(bool b);
-  bool getHosted();
-
- private:
-  std::string m_themeName;
-  std::string m_themeFile;
-  bool m_hosted;
-  bool m_requireUpdate;
-};
-
 class ThemeChoicer {
  public:
 
-#if defined(SUPPORT_WEBACCESS)
   ThemeChoicer(vapp::WWWAppInterface *p_WebApp = NULL,
 	       const ProxySettings *p_proxy_settings = NULL);	       
-#else
-  ThemeChoicer(void);	       
-#endif
-
   ~ThemeChoicer();
 
-  bool ExistThemeName(std::string p_themeName);
-  std::string getFileName(std::string p_themeName);
-  std::vector<ThemeChoice*> getChoices();
-  ThemeChoice* getChoiceByName(std::string p_themeName);
-
-#if defined(SUPPORT_WEBACCESS)
-  void updateFromWWW();
-  void updateThemeFromWWW(ThemeChoice* pThemeChoice);
-  bool isUpdatableThemeFromWWW(ThemeChoice* pThemeChoice);
+  void updateFromWWW(xmDatabase *i_db);
+  void updateThemeFromWWW(xmDatabase *i_db, const std::string& i_id_theme);
 
   void setURL(const std::string &p_url);
   void setURLBase(const std::string &p_urlBase);
-#endif
+  void initThemesFromDir(xmDatabase *i_db);
 
  private:
-  void cleanList();
-  void initList();
-  std::string getThemeNameFromFile(std::string p_themeFile);
-  
-#if defined(SUPPORT_WEBACCESS)
   WebThemes *m_webThemes;
-#endif
-
-  std::vector<ThemeChoice*> m_choices;
+  std::string getThemeNameFromFile(std::string p_themeFile);
 };
 
 #endif /* __THEME_H__ */
