@@ -26,6 +26,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #ifdef WIN32
   #include <io.h>
   #include <direct.h>
+  #include <windows.h>
+  #include <userenv.h>
+  #include <winbase.h>
 #else
   #include <unistd.h>
   #include <sys/types.h>
@@ -37,6 +40,42 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "VApp.h"
 #include "VFileIO.h"
 #include "helpers/SwapEndian.h"
+#include "md5sum/md5file.h"
+
+#ifdef WIN32
+std::string win32_getUserDir(bool i_asUtf8 = false) {
+  HANDLE hToken;
+  HANDLE proc;
+  std::string v_dest;
+  DWORD cchPath = 1024;
+
+  proc = GetCurrentProcess();
+
+  if(!OpenProcessToken(proc, TOKEN_QUERY, &hToken)) {
+    throw Exception("Can't determine user directory (OpenProcessToken)");
+  }
+
+  if(i_asUtf8) {
+    WCHAR szProfilePath[1024];
+    char v_tmp[1024];
+
+    if (!GetUserProfileDirectoryW( hToken, szProfilePath, &cchPath ) ) {
+      throw Exception("Can't determine user directory (GetUserProfileDirectory)");
+    }
+    WideCharToMultiByte(CP_UTF8, 0, szProfilePath, wcslen(szProfilePath), v_tmp, 1024-1, NULL, NULL);
+    v_dest = v_tmp;
+
+  } else {
+    TCHAR szProfilePath[1024];
+    if (!GetUserProfileDirectory( hToken, szProfilePath, &cchPath ) ) {
+      throw Exception("Can't determine user directory (GetUserProfileDirectory)");
+    }
+    v_dest = szProfilePath;
+  }
+
+  return v_dest;
+}
+#endif
 
 namespace vapp {
 
@@ -51,11 +90,11 @@ namespace vapp {
   #ifndef TRUE
   #define TRUE 1
   #endif
-  #if !defined(_MSC_VER)
-    void strlwr(char *pc) {
-      for(unsigned int i=0; i<strlen(pc); i++) pc[i] = tolower(pc[i]);
-    }
-  #endif
+
+  void strlwr(char *pc) {
+    for(unsigned int i=0; i<strlen(pc); i++) pc[i] = tolower(pc[i]);
+  }
+
   mbool str_match_wildcard(char *pcMWildcard,char *pcMString,mbool CaseSensitive) {
     int nPos=0;
     mbool PrevIsWildcard=FALSE;
@@ -215,10 +254,177 @@ namespace vapp {
   std::vector<std::string> FS::findPhysFiles(std::string Files,bool bRecurse) {
     std::vector<std::string> Result;
     std::string Wildcard;
-    std::string DirToSearch,AltDirToSearch,UDirToSearch = "";
+    std::string DataDirToSearch,AltDirToSearch,UDirToSearch = "";
     
-    //printf("FIND PHYS FILES [%s]\n",Files.c_str());
+    /* First seperate directory from wildcard */
+    int n = Files.find_last_of('/');
+    if(n<0) {
+      /* No directory specified */
+      DataDirToSearch = m_DataDir;
+      AltDirToSearch = "";
+      Wildcard = Files;
+    }
+    else {
+      /* Seperate dir and wildcard */
+      DataDirToSearch = m_DataDir + std::string("/") + Files.substr(0,n+1);
+      UDirToSearch = m_UserDir + std::string("/") + Files.substr(0,n+1);
+      AltDirToSearch = Files.substr(0,n+1);
+      Wildcard = Files.substr(n+1);
+    }    
+
+    /* 1. find in user dir */
+    /* 2. find in data dir */
+    /* 3. find in package  */
+
+    /* Windows? */
+#ifdef WIN32
+    if(UDirToSearch != "") {
+      if(bRecurse) {
+	_FindFilesRecursive(UDirToSearch,Wildcard,Result);
+      } else {
+	long fh;
+	struct _finddata_t fd;
+	
+	if((fh = _findfirst((UDirToSearch + Wildcard).c_str(),&fd)) != -1L) {
+	  do {
+	    if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
+	      std::string F = UDirToSearch + std::string(fd.name);
+	      bool bFound = false;
+	      for(int k = 0;k<Result.size();k++) {
+		if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+		  bFound = true;
+		  break;
+		}
+	      }
+	      if(!bFound)
+		Result.push_back(F);
+	    }
+	  } while(_findnext(fh,&fd)==0);
+	  _findclose(fh);
+	}
+      }
+    }
+
+    if(bRecurse) {
+      _FindFilesRecursive(DataDirToSearch,Wildcard,Result);
+    } else {
+      long fh;
+      struct _finddata_t fd;
+      
+      if((fh = _findfirst((DataDirToSearch + Wildcard).c_str(),&fd)) != -1L) {
+	do {
+	  if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
+	    std::string F = DataDirToSearch + std::string(fd.name);
+	    bool bFound = false;
+	    for(int k = 0;k<Result.size();k++) {
+	      if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+		bFound = true;
+		break;
+	      }
+	    }
+	    if(!bFound)
+	      Result.push_back(F);
+	  }
+	} while(_findnext(fh,&fd)==0);
+	_findclose(fh);
+      }
+    }
+#else /* Assume linux, unix... yalla yalla... */
+    if(UDirToSearch != "") {
+      if(bRecurse) {
+	_FindFilesRecursive(UDirToSearch,Wildcard,Result);        
+      } else {
+	struct dirent *dp;    
+	DIR *dirp = opendir(UDirToSearch.c_str());
+	while(dirp) {
+	  if((dp = readdir(dirp)) != NULL) {
+	    if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
+	      /* Match! */
+	      if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
+		std::string F = UDirToSearch + std::string(dp->d_name);
+		bool bFound = false;
+		for(unsigned int k = 0;k<Result.size();k++) {
+		  if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+		    bFound = true;
+		    break;
+		  }
+		}
+		if(!bFound)
+		  Result.push_back(F);
+	      }
+	    }
+	  } else {
+	    closedir(dirp);
+	    break;        
+	  }
+	}
+      }
+    }
     
+    if(AltDirToSearch != UDirToSearch) {
+      if(bRecurse) {
+	_FindFilesRecursive(AltDirToSearch,Wildcard,Result);
+      } else {
+	struct dirent *dp;    
+	DIR *dirp = opendir(AltDirToSearch.c_str());
+	while(dirp) {
+	  if((dp = readdir(dirp)) != NULL) {
+	    if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
+	      /* Match! */
+	      if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
+		std::string F = AltDirToSearch + std::string(dp->d_name);
+		bool bFound = false;
+		for(unsigned int k = 0;k<Result.size();k++) {
+		  if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+		    bFound = true;
+		    break;
+		  }
+		}
+		if(!bFound)
+		  Result.push_back(F);
+	      }
+	    }
+	  } else {
+	    closedir(dirp);
+	    break;        
+	  }
+	}
+      }
+    }
+    
+    if(DataDirToSearch != AltDirToSearch && DataDirToSearch != UDirToSearch) {
+      if(bRecurse) {
+	_FindFilesRecursive(DataDirToSearch,Wildcard,Result);
+      } else {
+	/* Search directories */
+	struct dirent *dp;    
+        DIR *dirp = opendir(DataDirToSearch.c_str());
+        while(dirp) {
+          if((dp = readdir(dirp)) != NULL) {
+            if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
+              /* Match! */
+              if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
+                std::string F = DataDirToSearch + std::string(dp->d_name);
+                bool bFound = false;
+                for(unsigned int k = 0;k<Result.size();k++) {
+                  if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
+                    bFound = true;
+                    break;
+                  }
+                }
+                if(!bFound)
+                  Result.push_back(F);
+              }             
+            }
+          } else {
+            closedir(dirp);
+            break;        
+          }
+        }
+      }
+		}
+    #endif
+		
     /* Look in package */
     for(int i=0;i<m_nNumPackFiles;i++) {
       /* Make sure about the directory... */
@@ -233,158 +439,9 @@ namespace vapp {
         Result.push_back(m_PackFiles[i].Name);
       }
     }
-
-    /* First seperate directory from wildcard */
-    int n = Files.find_last_of('/');
-    if(n<0) {
-      /* No directory specified */
-      DirToSearch = m_DataDir;
-      AltDirToSearch = m_DataDir;      
-      Wildcard = Files;
-    }
-    else {
-      /* Seperate dir and wildcard */
-      DirToSearch = m_DataDir + std::string("/") + Files.substr(0,n+1);
-      UDirToSearch = m_UserDir + std::string("/") + Files.substr(0,n+1);
-      AltDirToSearch = Files.substr(0,n+1);
-      Wildcard = Files.substr(n+1);
-    }    
-
-    /* TODO: !!!! Quick hack, clean up please */
     
-//    printf("W=%s   D=%s  AD=%s\n",Wildcard.c_str(),DirToSearch.c_str(),AltDirToSearch.c_str());    
-    /* Windows? */
-    #ifdef WIN32
-      if(bRecurse) {
-        _FindFilesRecursive(DirToSearch,Wildcard,Result);
-      }
-      else {
-        long fh;
-        struct _finddata_t fd;
-        
-        if((fh = _findfirst((DirToSearch + Wildcard).c_str(),&fd)) != -1L) {
-          do {
-            if(strcmp(fd.name,".") && strcmp(fd.name,"..")) {
-              std::string F = DirToSearch + std::string(fd.name);
-              bool bFound = false;
-              for(int k = 0;k<Result.size();k++) {
-                if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
-                  bFound = true;
-                  break;
-                }
-              }
-              if(!bFound)
-                Result.push_back(F);
-            }
-          } while(_findnext(fh,&fd)==0);
-          _findclose(fh);
-        }
-      }
-    #else /* Assume linux, unix... yalla yalla... */
-      if(bRecurse) {
-        _FindFilesRecursive(DirToSearch,Wildcard,Result);
-      }
-      else {
-        /* Search directories */
-        struct dirent *dp;    
-        DIR *dirp = opendir(DirToSearch.c_str());
-        while(dirp) {
-          if((dp = readdir(dirp)) != NULL) {
-            if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
-              /* Match! */
-              if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
-                std::string F = DirToSearch + std::string(dp->d_name);
-                bool bFound = false;
-                for(unsigned int k = 0;k<Result.size();k++) {
-                  if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
-                    bFound = true;
-                    break;
-                  }
-                }
-                if(!bFound)
-                  Result.push_back(F);
-              }             
-            }
-          }
-          else {
-            closedir(dirp);
-            break;        
-          }
-        }
-      }
-      
-      if(UDirToSearch != "") {
-        if(bRecurse) {
-          _FindFilesRecursive(UDirToSearch,Wildcard,Result);        
-        }
-        else {
-          struct dirent *dp;    
-          DIR *dirp = opendir(UDirToSearch.c_str());
-          DirToSearch = UDirToSearch;
-          while(dirp) {
-            if((dp = readdir(dirp)) != NULL) {
-              if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
-                /* Match! */
-                if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
-                  std::string F = DirToSearch + std::string(dp->d_name);
-                  bool bFound = false;
-                  for(unsigned int k = 0;k<Result.size();k++) {
-                    if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
-                      bFound = true;
-                      break;
-                    }
-                  }
-                  if(!bFound)
-                    Result.push_back(F);
-                }
-              }
-            }
-            else {
-              closedir(dirp);
-              break;        
-            }
-          }
-        }
-      }
+		//	for(int i=0;i<Result.size();i++) printf("%s\n",Result[i].c_str());
 
-      if(AltDirToSearch != DirToSearch) {
-        if(bRecurse) {
-          _FindFilesRecursive(AltDirToSearch,Wildcard,Result);
-        }
-        else {
-          struct dirent *dp;    
-          DIR *dirp = opendir(AltDirToSearch.c_str());
-          DirToSearch = AltDirToSearch;
-          while(dirp) {
-            if((dp = readdir(dirp)) != NULL) {
-              if(str_match_wildcard((char *)Wildcard.c_str(),(char *)dp->d_name,true)) {
-                /* Match! */
-                if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
-                  std::string F = DirToSearch + std::string(dp->d_name);
-                  bool bFound = false;
-                  for(unsigned int k = 0;k<Result.size();k++) {
-                    if(FS::getFileBaseName(Result[k]) == FS::getFileBaseName(F)) {  
-                      bFound = true;
-                      break;
-                    }
-                  }
-                  if(!bFound)
-                    Result.push_back(F);
-                }
-              }
-            }
-            else {
-              closedir(dirp);
-              break;        
-            }
-          }
-        }
-      }
-    #endif
-    
-    //printf("\n");
-    //for(int i=0;i<Result.size();i++) printf("%s\n",Result[i].c_str());
-    
     /* Return file listing */
     return Result;
   }
@@ -395,11 +452,12 @@ namespace vapp {
     /* Is it an absolute path? */
     if(isPathAbsolute(Path)) {    
       /* Yup, not much to do here then */
+      mkArborescence(Path);
       pfh->fp = fopen(Path.c_str(),"wb");
     }    
     else {
-      /* Nope, try the user dir. We are not going to create files relative
-         to the working-dir, that would be f*cked :) */
+      /* Nope, try the user dir */
+      mkArborescence(m_UserDir + std::string("/") + Path);
       pfh->fp = fopen((m_UserDir + std::string("/") + Path).c_str(),"wb");
     }
     
@@ -413,7 +471,7 @@ namespace vapp {
     return NULL;
   }
   
-  FileHandle *FS::openIFile(std::string Path) {
+  FileHandle *FS::openIFile(std::string Path, bool i_includeCurrentDir) {
     FileHandle *pfh = new FileHandle;
     
     /* Okay. Absolute path? */
@@ -421,19 +479,26 @@ namespace vapp {
       /* Yes ma'am */
       pfh->fp = fopen(Path.c_str(),"rb");      
       pfh->Type = FHT_STDIO;
-    }
-    else {
-      /* Try current working dir */
-      pfh->fp = fopen(Path.c_str(),"rb");
+    } else {
+      /* user dir ? */
+      pfh->fp = fopen((m_UserDir + std::string("/") + Path).c_str(),"rb");        
+
       if(pfh->fp == NULL) {
-        /* No luck. Try the user-dir then */
-        pfh->fp = fopen((m_UserDir + std::string("/") + Path).c_str(),"rb");        
-        if(pfh->fp == NULL && m_bGotDataDir) {
-          /* Not there either, the data-dir is our last chance */
-          pfh->fp = fopen((m_DataDir + std::string("/") + Path).c_str(),"rb");        
-        }
+	if(m_bGotDataDir) { /* data dir ? */
+	  pfh->fp = fopen((m_DataDir + std::string("/") + Path).c_str(),"rb");        
+	}
+
+	if(pfh->fp == NULL) {
+	  if(i_includeCurrentDir) { /* current working dir ? */
+	    pfh->fp = fopen(Path.c_str(),"rb");
+	  }
+	}
+      }
+
+      /* got something ? */
+      if(pfh->fp != NULL) {
+	pfh->Type = FHT_STDIO;
       }
-      pfh->Type = FHT_STDIO;
     }
     
     if(pfh->fp == NULL) {
@@ -455,20 +520,21 @@ namespace vapp {
       }      
     }
     
-    if(pfh->fp != NULL) { 
-      pfh->bRead = true;
-      pfh->Name = Path;
-      
-      if(pfh->Type == FHT_STDIO) {
-        fseek(pfh->fp,0,SEEK_END);
-        pfh->nSize = ftell(pfh->fp);
-        fseek(pfh->fp,0,SEEK_SET);        
-      }
-      return pfh;
+    if(pfh->fp == NULL) { 
+      delete pfh;
+      return NULL;
     }
-    
-    delete pfh;
-    return NULL;
+
+
+    pfh->bRead = true;
+    pfh->Name = Path;
+      
+    if(pfh->Type == FHT_STDIO) {
+      fseek(pfh->fp,0,SEEK_END);
+      pfh->nSize = ftell(pfh->fp);
+      fseek(pfh->fp,0,SEEK_SET);        
+    }
+    return pfh;
   }
     
   void FS::closeFile(FileHandle *pfh) {
@@ -608,9 +674,9 @@ namespace vapp {
   } 
   
   int FS::readInt(FileHandle *pfh) {
-    int v;
-    if(!readBuf(pfh,(char *)&v,sizeof(v))) _ThrowFileError(pfh,"readInt -> failed");
-    return v;
+    int32_t nv;
+    if(!readBuf(pfh,(char *)&nv,sizeof(4))) _ThrowFileError(pfh,"readInt -> failed");
+    return static_cast<int>(nv);
   } 
   
   float FS::readFloat(FileHandle *pfh) {
@@ -657,7 +723,9 @@ namespace vapp {
   }
   
   void FS::writeInt(FileHandle *pfh,int v) {
-    if(!writeBuf(pfh,(char *)&v,sizeof(v))) _ThrowFileError(pfh,"writeInt -> failed");
+    int32_t nv;
+    nv = static_cast<int32_t>(v);
+    if(!writeBuf(pfh,(char *)&nv, 4)) _ThrowFileError(pfh,"writeInt -> failed");
   }
   
   void FS::writeFloat(FileHandle *pfh,float v) {
@@ -778,6 +846,13 @@ namespace vapp {
     return FName.substr(0,n);
   }
 
+  std::string FS::getFileExtension(std::string Path) {
+    int n;
+    n = Path.find_last_of(".");
+    if(n<0) return "";
+    return Path.substr(n+1, Path.length() - n - 1);
+  }
+
   /*=========================================================================== 
   Find out how old this file thing is... (physical file system only)
   ===========================================================================*/
@@ -977,15 +1052,17 @@ namespace vapp {
   /*===========================================================================
   Initialize file system fun
   ===========================================================================*/
-  std::string FS::m_UserDir="",FS::m_DataDir; /* Globals */
+  std::string FS::m_UserDir="",FS::m_UserDirUTF8="",FS::m_DataDir; /* Globals */
   bool FS::m_bGotDataDir;
   std::string FS::m_BinDataFile = "";
   int FS::m_nNumPackFiles = 0;
   PackFile FS::m_PackFiles[MAX_PACK_FILES];
-    
+  std::string FS::m_binCheckSum = "";
+
   void FS::init(std::string AppDir) {    
     m_bGotDataDir = false;
     m_UserDir = "";
+    m_UserDirUTF8 = "";
     m_DataDir = "";
   
     #if defined(WIN32) /* Windoze... */
@@ -1003,7 +1080,8 @@ namespace vapp {
       /* Valid path? */
       if(isDir(cModulePath)) {
         /* Alright, use this dir */    
-        m_UserDir = cModulePath;
+        m_UserDir     = win32_getUserDir()     + std::string("/.") + AppDir;
+        m_UserDirUTF8 = win32_getUserDir(true) + std::string("/.") + AppDir;
         m_DataDir = cModulePath;     
         
         m_bGotDataDir = true;
@@ -1018,51 +1096,8 @@ namespace vapp {
         throw Exception("invalid user home directory");
       
       m_UserDir = m_UserDir + std::string("/.") + AppDir;
-      
-      /* If user-dir isn't there, try making it */
-      if(!isDir(m_UserDir)) {
-        if(mkdir(m_UserDir.c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
-          Log("** Warning ** : failed to create user directory '%s'!",m_UserDir.c_str());
-        }
-        if(!isDir(m_UserDir)) {
-          /* Still no dir... */
-          throw Exception("could not create user directory");
-        }
-      }
-      
-      /* If there is no Replay-dir in user-dir try making that too */
-      if(!isDir(getReplaysDir())) {
-        if(mkdir(getReplaysDir().c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
-          Log("** Warning ** : failed to create user replay directory '%s'!",(m_UserDir + std::string("/Replays")).c_str());
-        }
-        if(!isDir(getReplaysDir())) {
-          /* Still no dir... */
-          throw Exception("could not create user replay directory");
-        }
-      }
+      m_UserDirUTF8 = m_UserDir;
 
-      /* Make sure we got a level cache dir */
-      if(!isDir(m_UserDir + std::string("/LCache"))) {
-        if(mkdir((m_UserDir + std::string("/LCache")).c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
-          Log("** Warning ** : failed to create user LCache directory '%s'!",(m_UserDir + std::string("/LCache")).c_str());
-        }
-        if(!isDir(m_UserDir + std::string("/LCache"))) {
-          /* Still no dir... */
-          throw Exception("could not create user LCache directory");
-        }
-      }
-
-      /* The same goes for the /Levels dir */
-      if(!isDir(getLevelsDir())) {
-        if(mkdir(getLevelsDir().c_str(),S_IRUSR|S_IWUSR|S_IRWXU)) { /* drwx------ */
-          Log("** Warning ** : failed to create user levels directory '%s'!",(m_UserDir + std::string("/Levels")).c_str());
-        }
-        if(!isDir(getLevelsDir())) {
-          /* Still no dir... */
-          throw Exception("could not create user levels directory");
-        }
-      }
-      
       /* And the data dir? */
       m_DataDir = std::string(GAMEDATADIR);
       if(isDir(m_DataDir)) {
@@ -1071,6 +1106,20 @@ namespace vapp {
       }
     #endif    
 
+      /* If user-dir isn't there, try making it */
+      if(isDir(m_UserDir) == false) {
+	mkArborescenceDir(m_UserDir);
+      }
+      if(isDir(getReplaysDir()) == false) {
+	mkArborescenceDir(getReplaysDir());
+      }
+      if(isDir(m_UserDir + std::string("/LCache")) == false) {
+	mkArborescenceDir(m_UserDir + std::string("/LCache"));
+      } 
+      if(isDir(getLevelsDir()) == false) {
+	mkArborescenceDir(getLevelsDir());
+      } 
+
     /* Delete old log */    
     remove( (m_UserDir + std::string("/xmoto.log")).c_str() );
     
@@ -1080,81 +1129,85 @@ namespace vapp {
       Log("Data directory: %s",m_DataDir.c_str());
       m_BinDataFile = m_DataDir + std::string("/xmoto.bin");
     }
-    else {
-      Log("Data directory: (local)");    
-      m_BinDataFile = "xmoto.bin";
-
-      /* If it is UNIX and we're running in local data mode, AND we can't find the
-         xmoto.bin file, then we can safely assume the user haven't read the
-         instructions :) */
-      #if !defined(WIN32)
-        FILE *fptest = fopen(m_BinDataFile.c_str(),"rb");
-        if(fptest == NULL) {
-          Log("Failed to find the data files! Please either install the program\n"
-              "with 'make install' or 'cd' into the 'bin' directory, and start\n"
-              "the program from there.\n");
-         
-          throw Exception("Can't find data!");
-        }
-        fclose(fptest);
-      #endif
-    }
           
     /* Initialize binary data package if any */
     FILE *fp = fopen(m_BinDataFile.c_str(),"rb");
     m_nNumPackFiles = 0;
-    if(fp != NULL) {
-      Log("Initializing binary data package...\n");
-      
-      char cBuf[256];
-      fread(cBuf,4,1,fp);
-      if(!strncmp(cBuf,"XBI1",4)) {
-        int nNameLen;
-        while((nNameLen=fgetc(fp)) >= 0) {
-//          printf("%d  \n",nNameLen);
-          /* Read file name */
-          fread(cBuf,nNameLen,1,fp);
-          cBuf[nNameLen] = '\0';
-//          printf("      %s\n",cBuf);
-          
-          /* Read file size */
-          int nSize;
+    if(fp == NULL) {
+			throw Exception("Package xmoto.bin not found !");
+		}
 
-          /* Patch by Michel Daenzer (applied 2005-10-25) */
-          unsigned char nSizeBuf[4];
-          fread(nSizeBuf,4,1,fp);
-          nSize = nSizeBuf[0] | nSizeBuf[1] << 8 | nSizeBuf[2] << 16 | nSizeBuf[3] << 24;
-          
-          if(m_nNumPackFiles < MAX_PACK_FILES) {
-            m_PackFiles[m_nNumPackFiles].Name = cBuf;
-            m_PackFiles[m_nNumPackFiles].nOffset = ftell(fp);
-            m_PackFiles[m_nNumPackFiles].nSize = nSize;
-            m_nNumPackFiles ++;
-          }
-          else
-            Log("** Warning ** : Too many files in binary data package! (Limit=%d)",MAX_PACK_FILES);          
-
-          fseek(fp,nSize,SEEK_CUR);
-        }
-      }
-      else
-        Log("** Warning ** : Invalid binary data package format!");
+		Log("Initializing binary data package...");
       
-      fclose(fp);
-    }        
-  }
+		char cBuf[256];
+		char md5sum[256];
+		fread(cBuf,4,1,fp);
+		if(!strncmp(cBuf,"XBI3",4)) {
+			int nNameLen;
+			int md5sumLen;
+
+			/* get bin checksum of the package.list */
+			md5sumLen = fgetc(fp);
+			fread(md5sum,md5sumLen,1,fp);
+			md5sum[md5sumLen] = '\0';			
+			m_binCheckSum = md5sum;
+
+			while((nNameLen=fgetc(fp)) >= 0) {
+				//          printf("%d  \n",nNameLen);
+				/* Read file name */
+				fread(cBuf,nNameLen,1,fp);
+				cBuf[nNameLen] = '\0';
+
+				md5sumLen = fgetc(fp);
+				fread(md5sum,md5sumLen,1,fp);
+				md5sum[md5sumLen] = '\0';
+				//          printf("      %s\n",cBuf);
+				
+				/* Read file size */
+				int nSize;
+				
+				/* Patch by Michel Daenzer (applied 2005-10-25) */
+				unsigned char nSizeBuf[4];
+				fread(nSizeBuf,4,1,fp);
+				nSize = nSizeBuf[0] | nSizeBuf[1] << 8 | nSizeBuf[2] << 16 | nSizeBuf[3] << 24;
+				
+				if(m_nNumPackFiles < MAX_PACK_FILES) {
+					m_PackFiles[m_nNumPackFiles].Name = cBuf;
+					//printf("%s\n", m_PackFiles[m_nNumPackFiles].Name.c_str());
+					m_PackFiles[m_nNumPackFiles].md5sum = md5sum;
+					//printf("md5sum(%s) = %s\n",
+					//   m_PackFiles[m_nNumPackFiles].Name.c_str(),
+					//   m_PackFiles[m_nNumPackFiles].md5sum.c_str());
+
+					m_PackFiles[m_nNumPackFiles].nOffset = ftell(fp);
+					m_PackFiles[m_nNumPackFiles].nSize = nSize;
+					m_nNumPackFiles ++;
+				}
+				else
+					Log("** Warning ** : Too many files in binary data package! (Limit=%d)",MAX_PACK_FILES);          
+				
+				fseek(fp,nSize,SEEK_CUR);
+			}
+		} else
+			Log("** Warning ** : Invalid binary data package format!");
+		fclose(fp);
+	}     
 
   /*===========================================================================
   Hmm, for some reason I have to do this. Don't ask me why.
   ===========================================================================*/
   int FS::mkDir(const char *pcPath) {
-    #if defined(_MSC_VER)
+    #if defined(WIN32)
       return _mkdir(pcPath);
     #else
       return mkdir(pcPath,S_IRUSR|S_IWUSR|S_IRWXU);
     #endif
   }
 
+  std::string FS::binCheckSum() {
+    return FS::m_binCheckSum;
+  }
+
   /*===========================================================================
   Endian-safe I/O helpers
   ===========================================================================*/
@@ -1200,19 +1253,20 @@ namespace vapp {
     return big ? SwapEndian::BigFloat(v) : SwapEndian::LittleFloat(v);
   }
   
-  
-  bool FS::isInUserDir(std::string p_filepath) {
-    std::string v_userDir;
+  bool FS::isFileInDir(std::string p_dirpath, std::string p_filepath) {
     std::string v_fileDir;
 
-    v_userDir = getUserDir();
     v_fileDir = getFileDir(p_filepath);
 
-    if(v_userDir.length() > v_fileDir.length()) {
+    if(p_dirpath.length() > v_fileDir.length()) {
       return false;
     }
 
-    return v_fileDir.substr(0, v_userDir.length()) == v_userDir;
+    return v_fileDir.substr(0, p_dirpath.length()) == p_dirpath;
+  }
+
+  bool FS::isInUserDir(std::string p_filepath) {
+    return isFileInDir(getUserDir(), p_filepath);
   }
 
   bool FS::doesDirectoryExist(std::string p_path) {
@@ -1241,10 +1295,51 @@ namespace vapp {
     }
 
     mkArborescence(v_parentDir);
-    if(mkDir(v_parentDir.c_str()) != 0) {
+    if(FS::mkDir(v_parentDir.c_str()) != 0) {
       throw Exception("Can't create directory " + v_parentDir);
     }
   }
 
+  void FS::mkArborescenceDir(std::string v_dirpath) {
+    mkArborescence(v_dirpath + "/file.tmp");
+  }
+
+  std::string FS::md5sum(std::string i_filePath) {
+    /* is it a file from the pack or a real file ? */
+    if(FS::isFileReal(i_filePath)) {
+      return md5file(i_filePath);
+    }
+
+    /* package */
+    for(int i=0; i<m_nNumPackFiles; i++) {              
+      if(m_PackFiles[i].Name == i_filePath ||
+	 (std::string("./") + m_PackFiles[i].Name) == i_filePath) {
+	   /* Found it, yeah. */
+	   //printf("md5sum(%s) = %s\n", m_PackFiles[i].Name.c_str(), m_PackFiles[i].md5sum.c_str());
+	   return m_PackFiles[i].md5sum;
+	 }
+    }
+    return "";
+  }
+
+
+  bool FS::isFileReal(std::string i_filePath) {
+    FILE *fp;
+
+    fp = fopen(i_filePath.c_str(), "rb");
+    if(fp == NULL) {
+      return false;
+    }
+    fclose(fp);
+
+    return true;
+  }
+
+  std::string FS::FullPath(const std::string& i_relative_path) {
+    if(fileExists(getUserDir() + std::string("/") + i_relative_path)) {
+      return getUserDir() + std::string("/") + i_relative_path;
+    }
+    return getDataDir() + std::string("/") + i_relative_path;
+  }
 }
 
