@@ -54,6 +54,7 @@ namespace vapp {
     }
   
     std::string Name;
+    std::string md5sum;
     int nSize,nOffset;
   };
 
@@ -99,7 +100,7 @@ namespace vapp {
       
       static void writeLog(const std::string &s);
       static FileHandle *openOFile(std::string Path);
-      static FileHandle *openIFile(std::string Path);
+      static FileHandle *openIFile(std::string Path, bool i_includeCurrentDir = false);
       static void closeFile(FileHandle *pfh);
 
       static bool readBuf(FileHandle *pfh,char *pcBuf, unsigned int nBufSize);
@@ -154,7 +155,8 @@ namespace vapp {
       /* File name mangling */
       static std::string getFileDir(std::string Path);
       static std::string getFileBaseName(std::string Path);
-      
+      static std::string getFileExtension(std::string Path);      
+
       /* Misc */
       static bool isDir(std::string AppDir);
       static int getFileTimeStamp(const std::string &Path);
@@ -164,6 +166,7 @@ namespace vapp {
       
       /* Data interfaces */
       static std::string getUserDir(void) {return m_UserDir;}
+      static std::string getUserDirUTF8(void) {return m_UserDirUTF8;}
       static std::string getDataDir(void) {return m_DataDir;}
       static bool isDataDirAvailable(void) {return m_bGotDataDir;}
       
@@ -174,9 +177,19 @@ namespace vapp {
       static bool isFileReadable(std::string p_filename);
       static bool fileExists(std::string p_filename);
       static void mkArborescence(std::string v_filepath);
+      static void mkArborescenceDir(std::string v_dirpath);
 
       // return true if p_filepath is a path from user dir
       static bool isInUserDir(std::string p_filepath);
+      static bool isFileInDir(std::string p_dirpath, std::string p_filepath);
+
+      /* return false if the file is in the package */
+      static bool isFileReal(std::string i_filePath);
+      static std::string md5sum(std::string i_filePath);
+
+      /* return user_dir + i_relative_path if exists or data_dir + i_relative_path */
+      static std::string FullPath(const std::string& i_relative_path);
+      static std::string binCheckSum();
 
     private:
       /* Helper functions */
@@ -184,12 +197,13 @@ namespace vapp {
       static void _FindFilesRecursive(const std::string &Dir,const std::string &Wildcard,std::vector<std::string> &List);
       
       /* Data */
-      static std::string m_UserDir,m_DataDir;
+      static std::string m_UserDir, m_UserDirUTF8, m_DataDir;
       static bool m_bGotDataDir;
       
       static std::string m_BinDataFile;      
       static int m_nNumPackFiles;
       static PackFile m_PackFiles[MAX_PACK_FILES];
+      static std::string m_binCheckSum;
   };
 
 }
