@@ -1,7 +1,6 @@
 /* file contributed by Nicolas Adenis-Lamarre */
 
 #include "BuildConfig.h"
-#if defined(SUPPORT_WEBACCESS)
 
 /* I (rasmus) made these (among others) changes to the files:  
     - File names changed to WWW.cpp and WWW.h
@@ -21,6 +20,7 @@
 #include "helpers/VExcept.h"
 #include "VXml.h"
 #include <curl/curl.h>
+#include "GameText.h"
 
 #ifdef WIN32
   #include <io.h>
@@ -33,6 +33,7 @@
 #include <vector>
 #include "helpers/FileCompression.h"
 #include "md5sum/md5file.h"
+#include "db/xmDatabase.h"
 
 struct f_curl_download_data {
   vapp::WWWAppInterface *v_WebApp;
@@ -118,96 +119,13 @@ bool ProxySettings::useDefaultAuthentification() const {
   return m_authUser == "";
 }
 
-WebHighscore::WebHighscore(WebRoom *p_room,
-         std::string p_levelId,
-         std::string p_playerName,
-         std::string p_time,
-         std::string p_rplUrl,
-         const ProxySettings *p_proxy_settings) {
-  m_playerName  = p_playerName;
-  m_time        = p_time;
-  m_levelId     = p_levelId;
-  m_rplUrl      = p_rplUrl;
-  m_rplFilename = vapp::FS::getReplaysDir()
+void WebRoom::downloadReplay(const std::string& i_url) {
+  std::string i_rplFilename = vapp::FS::getReplaysDir()
     + "/" 
-    + vapp::FS::getFileBaseName(m_rplUrl) 
+    + vapp::FS::getFileBaseName(i_url) 
     + ".rpl";
 
-  m_proxy_settings = p_proxy_settings;
-  m_room = p_room;
-}
-
-WebHighscore::~WebHighscore() {
-}
-
-void WebHighscore::download() {
-  FSWeb::downloadFile(m_rplFilename, m_rplUrl, NULL, NULL, m_proxy_settings);
-}
-
-std::string WebHighscore::getReplayName() {
-  return vapp::FS::getFileBaseName(m_rplFilename);
-}
-
-std::string WebHighscore::getPlayerName() const {
-  return m_playerName;
-}
-
-std::string WebHighscore::getLevelId() const {
-  return m_levelId;
-}
-
-std::string WebHighscore::getTime() const {
-  char cTime[256];
-  int n1=0,n2=0,n3=0;
-  
-  sscanf(m_time.c_str(),"%d:%d:%d",&n1,&n2,&n3);
-  sprintf(cTime,"%02d:%02d:%02d",n1,n2,n3);
-  
-  return cTime;
-}
-
-float WebHighscore::getFTime() const {
-  std::string::size_type v_pos;
-  std::string v_min, v_sec, v_hun;
-  std::string v_rest;
-  float v_fmin, v_fsec, v_fhun;
-
-  v_rest = m_time;
-
-  /* search min */
-  v_pos = v_rest.find(":", 0);
-  if(v_pos == std::string::npos || v_pos == 0) {
-    throw Exception("error : invalid time in webhighscore");
-  }
-  v_min = v_rest.substr(0, v_pos);
-  if(v_pos == v_rest.length() -1) {
-    throw Exception("error : invalid time in webhighscore");
-  }
-  v_rest = v_rest.substr(v_pos+1, v_rest.length() -v_pos -1);
-
-  /* search sec */
-  v_pos = v_rest.find(":", 0);
-  if(v_pos == std::string::npos || v_pos == 0) {
-    throw Exception("error : invalid time in webhighscore");
-  }
-  v_sec = v_rest.substr(0, v_pos);
-  if(v_pos == v_rest.length() -1) {
-    throw Exception("error : invalid time in webhighscore");
-  }
-  v_rest = v_rest.substr(v_pos+1, v_rest.length() -v_pos -1);
-
-  /* search hun */
-  v_hun = v_rest;
-
-  v_fmin = atof(v_min.c_str());
-  v_fsec = atof(v_sec.c_str());
-  v_fhun = atof(v_hun.c_str());
-
-  return v_fmin * 60.0 + v_fsec + v_fhun / 100.0;
-}
-
-WebRoom* WebHighscore::getRoom() const {
-  return m_room;
+  FSWeb::downloadFile(i_rplFilename, i_url, NULL, NULL, m_proxy_settings);
 }
 
 WebRoom::WebRoom(const ProxySettings *p_proxy_settings) {
@@ -219,109 +137,18 @@ WebRoom::WebRoom(const ProxySettings *p_proxy_settings) {
   m_webhighscores_url = DEFAULT_WEBHIGHSCORES_URL;
 
   m_proxy_settings = p_proxy_settings;
-  m_roomName = "";
 }
 
 WebRoom::~WebRoom() {
-  cleanHash();
 }
 
-std::string WebRoom::getRoomName() const {
-  return m_roomName;
+std::string WebRoom::getRoomId() const {
+  return m_roomId;
 }
 
-void WebRoom::setWebsiteURL(std::string p_webhighscores_url) {
-  m_webhighscores_url = p_webhighscores_url;
-}
-
-WebHighscore* WebRoom::getHighscoreFromLevel(const std::string &p_levelId) {
-  #if defined(USE_HASH_MAP)
-    return m_webhighscores[p_levelId.c_str()];
-  #else
-    for(int i=0;i<m_webhighscores.size();i++)
-      if(m_webhighscores[i]->getLevelId() == p_levelId) return m_webhighscores[i];
-    return NULL;  
-  #endif
-}
-
-void WebRoom::cleanHash() {
-#if defined(USE_HASH_MAP)
-  HashNamespace::hash_map<const char*, WebHighscore*, HashNamespace::hash<const char*>, highscore_str, std::allocator<WebHighscore*> >::iterator it;
-
-  for (it = m_webhighscores.begin(); it != m_webhighscores.end(); it++) {
-    /* can be NULL, because search on unexisting value create an entry */
-    if(it->second != NULL) { 
-      delete it->second; /* delete highscore */
-    }
-  }
-#else
-  for(int i=0; i<m_webhighscores.size(); i++) {
-    delete m_webhighscores[i];
-  }
-#endif
-
-  /* clear hash table */
-  m_webhighscores.clear();
-}
-
-void WebRoom::fillHash() {
-  /* clean hash table */
-  cleanHash();
-
-  /* fillHashTable */
-  vapp::XMLDocument v_webHSXml;
-  TiXmlDocument *v_webHSXmlData;
-  TiXmlElement *v_webHSXmlDataElement;
-  const char *pc;
-  std::string v_levelId, v_playerName, v_time, v_rplUrl;
-  WebHighscore *wh;
-
-  v_webHSXml.readFromFile(m_userFilename);
-  v_webHSXmlData = v_webHSXml.getLowLevelAccess();
-
-  if(v_webHSXmlData == NULL) {
-    throw Exception("error : unable to analyze xml highscore file");
-  }
-
-  v_webHSXmlDataElement = v_webHSXmlData->FirstChildElement("xmoto_worldrecords");
-
-  if(v_webHSXmlDataElement != NULL) {
-    /* get Room name */
-    pc = v_webHSXmlDataElement->Attribute("roomname");
-    if(pc != NULL) {
-      m_roomName = pc;
-    }
-
-    for(TiXmlElement *pVarElem = v_webHSXmlDataElement->FirstChildElement("worldrecord");
-  pVarElem!=NULL;
-  pVarElem = pVarElem->NextSiblingElement("worldrecord")
-  ) {
-
-      pc = pVarElem->Attribute("level_id");
-      if(pc == NULL) { continue; }
-      v_levelId = pc;
-
-      pc = pVarElem->Attribute("player");
-      if(pc == NULL) { continue; }
-      v_playerName = pc;   
-
-      pc = pVarElem->Attribute("time");
-      if(pc == NULL) { continue; }
-      v_time = pc;   
-
-      pc = pVarElem->Attribute("replay");
-      if(pc == NULL) { continue; }
-      v_rplUrl = pc;
-      
-      wh = new WebHighscore(this, v_levelId, v_playerName, v_time, v_rplUrl, m_proxy_settings);
-      
-      #if defined(USE_HASH_MAP)
-        m_webhighscores[wh->getLevelId().c_str()] = wh;
-      #else
-        m_webhighscores.push_back(wh);
-      #endif      
-    }
-  }
+void WebRoom::setWebsiteInfos(const std::string& i_id_room, const std::string& i_webhighscores_url) {
+  m_webhighscores_url = i_webhighscores_url;
+  m_roomId = i_id_room;
 }
 
 void WebRoom::update() {
@@ -333,8 +160,8 @@ void WebRoom::update() {
          m_proxy_settings);
 }
 
-void WebRoom::upgrade() {
-  fillHash();
+void WebRoom::upgrade(xmDatabase *i_db) {
+  i_db->webhighscores_updateDB(m_userFilename, m_webhighscores_url);
 }
 
 size_t FSWeb::writeData(void *ptr, size_t size, size_t nmemb, FILE *stream) {
@@ -546,6 +373,8 @@ void FSWeb::uploadReplay(std::string p_replayFilename,
   std::string v_proxy_auth_str;
   std::string v_www_agent = WWW_AGENT;
   f_curl_upload_data v_data;
+  struct curl_slist *v_headers;
+  std::string v_accept_language;
 
   FILE *v_destinationFile;
   std::string v_local_file;
@@ -595,6 +424,12 @@ void FSWeb::uploadReplay(std::string p_replayFilename,
   curl_easy_setopt(v_curl, CURLOPT_USERAGENT,  v_www_agent.c_str());
   curl_easy_setopt(v_curl, CURLOPT_FAILONERROR, 1);
   curl_easy_setopt(v_curl, CURLOPT_FOLLOWLOCATION, 1);
+  curl_easy_setopt(v_curl, CURLOPT_ENCODING, "gzip,deflate");
+
+  v_headers = NULL;
+  v_accept_language = "Accept-Language: " + std::string(WEB_LANGUAGE);
+  v_headers = curl_slist_append(v_headers, v_accept_language.c_str()); 
+  curl_easy_setopt(v_curl, CURLOPT_HTTPHEADER, v_headers);
 
   /* set proxy settings */
   if(p_proxy_settings != NULL) {
@@ -640,6 +475,10 @@ void FSWeb::uploadReplay(std::string p_replayFilename,
 
   fclose(v_destinationFile);
 
+  /* free the headers */
+  curl_slist_free_all(v_headers);
+
+
   /* CURLE_ABORTED_BY_CALLBACK is not considered as an error */
   if(v_res != CURLE_ABORTED_BY_CALLBACK) {
     if(v_res != CURLE_OK) {
@@ -714,41 +553,6 @@ void FSWeb::uploadReplayAnalyseMsg(std::string p_filename,
   p_msg = pc;
 }
 
-WebLevel::WebLevel(std::string p_id, std::string p_name, std::string p_url) {
-  m_id   = p_id;
-  m_name = p_name;
-  m_url  = p_url;
-  m_require_update = false;
-}
-
-std::string WebLevel::getId() const {
-  return m_id;
-}
-
-std::string WebLevel::getName() const {
-  return m_name;
-}
-
-std::string WebLevel::getUrl() const {
-  return m_url;
-}
-
-bool WebLevel::requireUpdate() const {
-  return m_require_update;
-}
-
-void WebLevel::setRequireUpdate(bool p_require_update) {
-  m_require_update = p_require_update;
-}
-
-void WebLevel::setCurrentPath(std::string p_current_path) {
-  m_current_path = p_current_path;
-}
-
-std::string WebLevel::getCurrentPath() const {
-  return m_current_path;
-}
-
 WebLevels::WebLevels(vapp::WWWAppInterface *p_WebLevelApp,
          const ProxySettings *p_proxy_settings) {
   m_WebLevelApp    = p_WebLevelApp;
@@ -757,11 +561,6 @@ WebLevels::WebLevels(vapp::WWWAppInterface *p_WebLevelApp,
 }
 
 WebLevels::~WebLevels() {
-  /* delete levels information */
-  std::vector<WebLevel*>::iterator it; 
-  for(it = m_webLevels.begin(); it != m_webLevels.end(); it++) {
-    delete *it;
-  }
 }
 
 std::string WebLevels::getXmlFileName() {
@@ -785,70 +584,7 @@ void WebLevels::createDestinationDirIfRequired() {
 
   if(vapp::FS::isDir(v_destination_dir) == false) {
     /* no mkdir() with microsoft C */
-    if(vapp::FS::mkDir(v_destination_dir.c_str()) != 0) { /* drwx------ */
-      throw Exception("error : unable to make directory "
-          + v_destination_dir); 
-    }
-  }
-}
-
-void WebLevels::extractLevelsToDownloadFromXml() {
-  vapp::XMLDocument v_webLXml;
-  TiXmlDocument *v_webLXmlData;
-  TiXmlElement *v_webLXmlDataElement;
-  const char *pc;
-  std::string v_levelId, v_levelName, v_url, v_MD5sum_web;
-  
-  v_webLXml.readFromFile(getXmlFileName());
-  v_webLXmlData = v_webLXml.getLowLevelAccess();
-
-  if(v_webLXmlData == NULL) {
-    throw Exception("error : unable to analyze xml level file");
-  }
-
-  v_webLXmlDataElement = v_webLXmlData->FirstChildElement("xmoto_levels");
-  
-  if(v_webLXmlDataElement == NULL) {
-    throw Exception("error : unable to analyze xml level file");
-  }
-
-  TiXmlElement *pVarElem = v_webLXmlDataElement->FirstChildElement("level");
-  while(pVarElem != NULL) {
-    
-    pc = pVarElem->Attribute("level_id");
-    if(pc != NULL) {
-      v_levelId = pc;
-  
-      pc = pVarElem->Attribute("name");
-      if(pc != NULL) {
-  v_levelName = pc;
-  
-  pc = pVarElem->Attribute("url");
-  if(pc != NULL) {
-    v_url = pc;  
-    
-    pc = pVarElem->Attribute("sum");
-    if(pc != NULL) {
-      v_MD5sum_web = pc;  
-
-      /* if it doesn't exist */
-      if(m_WebLevelApp->doesLevelExist(v_levelId) == false) {
-        m_webLevels.push_back(new WebLevel(v_levelId, v_levelName, v_url));
-      } else { /* or it md5sum if different */
-        if(m_WebLevelApp->levelPathForUpdate(v_levelId) != "") {
-    if(m_WebLevelApp->levelMD5Sum(v_levelId) != v_MD5sum_web) {
-      WebLevel *v_lvl = new WebLevel(v_levelId, v_levelName, v_url);
-      v_lvl->setCurrentPath(m_WebLevelApp->levelPathForUpdate(v_levelId));
-      v_lvl->setRequireUpdate(true);
-      m_webLevels.push_back(v_lvl);
-    }
-        }
-      }
-    }
-  }
-      }
-      pVarElem = pVarElem->NextSiblingElement("level");
-    }
+    vapp::FS::mkArborescence(v_destination_dir + "/file");
   }
 }
 
@@ -856,16 +592,9 @@ std::string WebLevels::getDestinationFile(std::string p_url) {
   return getDestinationDir() + "/" + vapp::FS::getFileBaseName(p_url) + ".lvl";
 }
 
-void WebLevels::update() {
-  /* delete levels information */
-  std::vector<WebLevel*>::iterator it; 
-  for(it = m_webLevels.begin(); it != m_webLevels.end(); it++) {
-    delete *it;
-  }
-  m_webLevels.clear();
-
+void WebLevels::update(xmDatabase *i_db) {
   downloadXml();
-  extractLevelsToDownloadFromXml();
+  i_db->weblevels_updateDB(getXmlFileName());
 }
 
 int FSWeb::f_curl_progress_callback_upload(void *clientp,
@@ -927,82 +656,106 @@ int FSWeb::f_curl_progress_callback_download(void *clientp,
   return 0;
 }
 
-bool WebLevels::exists(const std::string p_id) {
-  std::vector<WebLevel*>::iterator it;
-  it = m_webLevels.begin();
-  while(it != m_webLevels.end()) {
-    if((*it)->getId() == p_id) {
-      return true;
-    }
-    it++;
-  }
-
-  return false;
+int WebLevels::nbLevelsToGet(xmDatabase *i_db) const {
+  char **v_result;
+  int nrow;
+  v_result = i_db->readDB("SELECT a.name FROM weblevels AS a "
+			  "LEFT OUTER JOIN levels AS b ON a.id_level=b.id_level "
+			  "WHERE b.id_level IS NULL OR a.checkSum <> b.checkSum;",
+			  nrow);
+  i_db->read_DB_free(v_result);
+  return nrow;
 }
 
-void WebLevels::upgrade() {
-  std::vector<WebLevel*>::iterator it;
+void WebLevels::upgrade(xmDatabase *i_db) {
+  char **v_result;
+  int nrow;
+  std::string v_levelId;
+  std::string v_levelName;
+  std::string v_urlFile;
+  bool        v_isAnUpdate;
+  std::string v_filePath;
   f_curl_download_data v_data;
+
+  int v_nb_levels_to_download;
+  int v_nb_levels_performed  = 0;
+  float v_percentage;
   bool to_download;
 
   createDestinationDirIfRequired();
 
-  int v_nb_levels_to_download = m_webLevels.size();
-  int v_nb_levels_performed  = 0;
-
+  v_result = i_db->readDB("SELECT a.id_level, a.name, a.fileUrl, b.filepath "
+			  "FROM weblevels AS a "
+			  "LEFT OUTER JOIN levels AS b ON a.id_level=b.id_level "
+			  "WHERE b.id_level IS NULL OR a.checkSum <> b.checkSum;",
+			  nrow);
+  v_nb_levels_to_download = nrow;
   v_data.v_WebApp = m_WebLevelApp;
   v_data.v_nb_files_to_download = v_nb_levels_to_download;
 
-  /* download levels */
-  it = m_webLevels.begin();
-  while(it != m_webLevels.end() && m_WebLevelApp->isCancelAsSoonAsPossible() == false) {
-    std::string v_url = (*it)->getUrl();
-    float v_percentage = (((float)v_nb_levels_performed) * 100.0) / ((float)v_nb_levels_to_download);
+  m_webLevelsNewDownloadedOK.clear();
+  m_webLevelsUpdatedDownloadedOK.clear();
 
-    m_WebLevelApp->setTaskProgress(v_percentage);
-    m_WebLevelApp->setBeingDownloadedInformation((*it)->getName(), (*it)->requireUpdate());
+  try {
+    /* download levels */
+    for(unsigned int i=0; i<nrow; i++) {
+      if(m_WebLevelApp->isCancelAsSoonAsPossible()) break;
 
-    /* should the level be updated */
-    to_download = true;
-    if((*it)->requireUpdate()) {
-      /* does the user want to update the level ? */
-      to_download = m_WebLevelApp->shouldLevelBeUpdated((*it)->getId());
+      v_levelId    = i_db->getResult(v_result, 4, i, 0);
+      v_levelName  = i_db->getResult(v_result, 4, i, 1);
+      v_urlFile    = i_db->getResult(v_result, 4, i, 2);
+      v_isAnUpdate = i_db->getResult(v_result, 4, i, 3) != NULL;
+      if(v_isAnUpdate) {
+	v_filePath = i_db->getResult(v_result, 4, i, 3);
+      }
+      v_percentage = (((float)v_nb_levels_performed) * 100.0) / ((float)v_nb_levels_to_download);
+	
+      m_WebLevelApp->setTaskProgress(v_percentage);
+      m_WebLevelApp->setBeingDownloadedInformation(v_levelName, v_isAnUpdate);
+      
+      /* should the level be updated */
+      to_download = true;
+      if(v_isAnUpdate) {
+	/* does the user want to update the level ? */
+	to_download = m_WebLevelApp->shouldLevelBeUpdated(v_levelId);
+      }
+      
+      if(to_download) {
+	v_data.v_nb_files_performed = v_nb_levels_performed;
+	std::string v_destFile;
+	
+	if(v_isAnUpdate) {
+	  if(vapp::FS::isInUserDir(v_filePath)) {
+	    v_destFile = v_filePath;
+	  } else {
+	    v_destFile = WebLevels::getDestinationFile(v_urlFile);
+	  }
+	} else {
+	  v_destFile = getDestinationFile(v_urlFile);
+	}
+	
+	FSWeb::downloadFileBz2(v_destFile,
+			       v_urlFile,
+			       FSWeb::f_curl_progress_callback_download,
+			       &v_data,
+			       m_proxy_settings);
+	
+	if(v_isAnUpdate) {
+	  m_webLevelsUpdatedDownloadedOK.push_back(v_destFile);
+	} else {
+	  m_webLevelsNewDownloadedOK.push_back(v_destFile);
+	}
+      }    
+      
+      v_nb_levels_performed++;
+      m_WebLevelApp->readEvents(); 
     }
-
-    if(to_download) {
-      v_data.v_nb_files_performed = v_nb_levels_performed;
-      std::string v_destFile;
-      
-      if((*it)->requireUpdate()) {
-  v_destFile = m_WebLevelApp->levelPathForUpdate((*it)->getId());
-      } else {
-  v_destFile = getDestinationFile(v_url);
-      }
-      
-      FSWeb::downloadFileBz2(v_destFile,
-           v_url,
-           FSWeb::f_curl_progress_callback_download,
-           &v_data,
-           m_proxy_settings);
-      
-      if((*it)->requireUpdate()) {
-  m_webLevelsUpdatedDownloadedOK.push_back(v_destFile);
-      } else {
-  m_webLevelsNewDownloadedOK.push_back(v_destFile);
-      }
-    }    
-
-    v_nb_levels_performed++;
-    m_WebLevelApp->readEvents(); 
-    it++;
+  } catch(Exception &e) {
+    i_db->read_DB_free(v_result);
+    throw e;
   }
-
   m_WebLevelApp->setTaskProgress(100.0);
-}
-
-void WebLevels::getUpdateInfo(int *pnUBytes,int *pnULevels) {
-  if(pnULevels != NULL) *pnULevels = m_webLevels.size();
-  if(pnUBytes != NULL) *pnUBytes = -1;
+  i_db->read_DB_free(v_result);
 }
 
 const std::vector<std::string> &WebLevels::getNewDownloadedLevels(void) {
@@ -1013,27 +766,6 @@ const std::vector<std::string> &WebLevels::getUpdatedDownloadedLevels(void) {
   return m_webLevelsUpdatedDownloadedOK;
 }
 
-WebTheme::WebTheme(std::string pName, std::string pUrl, std::string pSum) {
-  m_name = pName;
-  m_url  = pUrl;
-  m_sum  = pSum;
-}
-
-WebTheme::~WebTheme() {
-}
-
-std::string WebTheme::getName() const {
-  return m_name;
-}
-
-std::string WebTheme::getUrl() const {
-  return m_url;
-}
-
-std::string WebTheme::getSum() const {
-  return m_sum;
-}
-
 WebThemes::WebThemes(vapp::WWWAppInterface *p_WebApp,
          const ProxySettings *p_proxy_settings) {
   m_proxy_settings = p_proxy_settings;
@@ -1043,138 +775,77 @@ WebThemes::WebThemes(vapp::WWWAppInterface *p_WebApp,
 }
 
 WebThemes::~WebThemes() {
-  clean();
 }
 
-void WebThemes::update() {
+void WebThemes::update(xmDatabase *i_db) {
   FSWeb::downloadFileBz2UsingMd5(getXmlFileName(),
          m_themes_url,
          NULL,
          NULL,
          m_proxy_settings);
-}
-
-void WebThemes::upgrade() {
-  clean();
-  extractThemesAvailableFromXml();
+  i_db->webthemes_updateDB(getXmlFileName());
 }
 
 std::string WebThemes::getXmlFileName() {
   return vapp::FS::getUserDir() + "/" + DEFAULT_WEBTHEMES_FILENAME;
 }
 
-void WebThemes::extractThemesAvailableFromXml() {
-  vapp::XMLDocument v_webTXml;
-  TiXmlDocument *v_webTXmlData;
-  TiXmlElement *v_webTXmlDataElement;
-  const char *pc;
-  std::string v_themeName, v_url, v_MD5sum_web;
-  
-  v_webTXml.readFromFile(getXmlFileName());
-  v_webTXmlData = v_webTXml.getLowLevelAccess();
-
-  if(v_webTXmlData == NULL) {
-    throw Exception("error : unable to analyze xml theme file");
-  }
-
-  v_webTXmlDataElement = v_webTXmlData->FirstChildElement("xmoto_themes");
-  
-  if(v_webTXmlDataElement == NULL) {
-    throw Exception("error : unable to analyze xml theme file");
-  }
-
-  TiXmlElement *pVarElem = v_webTXmlDataElement->FirstChildElement("theme");
-  while(pVarElem != NULL) {
-    
-    pc = pVarElem->Attribute("name");
-    if(pc != NULL) {
-      v_themeName = pc;
-  
-      pc = pVarElem->Attribute("url");
-      if(pc != NULL) {
-  v_url = pc;  
-    
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) {
-    v_MD5sum_web = pc;
-
-    m_availableThemes.push_back(new WebTheme(v_themeName, v_url, v_MD5sum_web));
-      
-  }
-      }
-      pVarElem = pVarElem->NextSiblingElement("theme");
-    }
-  }
-}
-
-void WebThemes::clean() {
-  for(unsigned int i=0;i<m_availableThemes.size();i++) {
-    delete m_availableThemes[i];
-  }    
-
-  m_availableThemes.clear();
-}
-
-const std::vector<WebTheme*> &WebThemes::getAvailableThemes() {
-  return m_availableThemes;
-}
-
-bool WebThemes::isUpgradable(ThemeChoice *p_themeChoice) {
-  for(unsigned int i=0; i<m_availableThemes.size(); i++) {
-    if(m_availableThemes[i]->getName() == p_themeChoice->ThemeName()) {
-      return true;
-    }
-  }
-  return false;
-}
-
-void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
+void WebThemes::upgrade(xmDatabase *i_db, const std::string& i_id_theme) {
   std::string v_destinationFile;
   std::string v_sourceFile;
-  WebTheme *v_associated_webtheme;
   f_curl_download_data v_data;
-  
-  /* no update required */
-  if(p_themeChoice->getHosted() && p_themeChoice->getRequireUpdate() == false) {
-    // do nothing : it give a possibility to download file removed by mistake
-  }
+  char **v_result;
+  int nrow;
+  std::string v_fileUrl;
+  std::string v_filePath;
+  std::string v_themeFile;
+  bool v_onDisk = false;
 
-  /* find the associated webtheme */
-  v_associated_webtheme = NULL;
-  for(unsigned int i=0; i<m_availableThemes.size(); i++) {
-    if(m_availableThemes[i]->getName() == p_themeChoice->ThemeName()) {
-      v_associated_webtheme = m_availableThemes[i];
-    }
-  }
-
-  if(v_associated_webtheme == NULL) {
+  /* download even if the the theme is uptodate
+     it give a possibility to download file removed by mistake
+  */
+  v_result = i_db->readDB("SELECT a.fileUrl, b.filepath "
+			  "FROM webthemes AS a LEFT OUTER JOIN themes AS b "
+			  "ON a.id_theme = b.id_theme "
+			  "WHERE a.id_theme=\"" + xmDatabase::protectString(i_id_theme) + "\";",
+			  nrow);
+  if(nrow != 1) {
+    i_db->read_DB_free(v_result);
     throw Exception("error : unable to find this theme on the web");
   }
+
+  v_fileUrl = i_db->getResult(v_result, 2, 0, 0);
+  if(i_db->getResult(v_result, 2, 0, 1) != NULL) {
+    v_filePath = i_db->getResult(v_result, 2, 0, 1);
+    v_onDisk = true;
+  }  
+  i_db->read_DB_free(v_result);
+
   // theme avaible on the web get it
 
   /* the destination file must be in the user dir */
-  if(vapp::FS::isInUserDir(p_themeChoice->ThemeFile())) {
-    v_destinationFile = p_themeChoice->ThemeFile();
+  if(v_filePath != "") {
+    if(vapp::FS::isInUserDir(v_filePath)) {
+      v_destinationFile = v_filePath;
+    }
   }
 
   if(v_destinationFile == "") {
     /* determine destination file */
     v_destinationFile = 
       vapp::FS::getUserDir() + "/" + THEMES_DIRECTORY + "/" + 
-      vapp::FS::getFileBaseName(v_associated_webtheme->getUrl()) + ".xml";
-  }    
+      vapp::FS::getFileBaseName(v_fileUrl) + ".xml";
+  } 
   
+  v_themeFile = v_destinationFile;
+
   /* download the theme file */
   vapp::FS::mkArborescence(v_destinationFile);
   FSWeb::downloadFileBz2(v_destinationFile,
-       v_associated_webtheme->getUrl(),
-       NULL,
-       NULL,
-       m_proxy_settings);
-
-  if(v_destinationFile == "") {
-    throw Exception("error : unable to determine theme destination file");
-  }
+			 v_fileUrl,
+			 NULL,
+			 NULL,
+			 m_proxy_settings);
 
   /* download all the files required */
   Theme *v_theme = new Theme();
@@ -1189,9 +860,8 @@ void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
     }
   }
   if(v_nb_files_to_download != 0) {
-
     int v_nb_files_performed  = 0;
-    
+
     v_data.v_WebApp = m_WebApp;
     v_data.v_nb_files_to_download = v_nb_files_to_download;
 
@@ -1200,58 +870,38 @@ void WebThemes::upgrade(ThemeChoice *p_themeChoice) {
       // download v_required_files[i]     
       v_destinationFile = vapp::FS::getUserDir() + std::string("/") + (*v_required_files)[i];
       v_sourceFile = m_themes_urlBase + 
-  std::string("/") + (*v_required_files)[i];
-      
+	std::string("/") + (*v_required_files)[i];
+
       if(vapp::FS::fileExists((*v_required_files)[i]) == false) {
-  v_data.v_nb_files_performed = v_nb_files_performed;
+	v_data.v_nb_files_performed = v_nb_files_performed;
   
-  float v_percentage = (((float)v_nb_files_performed) * 100.0) / ((float)v_nb_files_to_download);
-  m_WebApp->setTaskProgress(v_percentage);
-  m_WebApp->setBeingDownloadedInformation((*v_required_files)[i], true);
+	float v_percentage = (((float)v_nb_files_performed) * 100.0) / ((float)v_nb_files_to_download);
+	m_WebApp->setTaskProgress(v_percentage);
+	m_WebApp->setBeingDownloadedInformation((*v_required_files)[i], true);
 
-  vapp::FS::mkArborescence(v_destinationFile);
+	vapp::FS::mkArborescence(v_destinationFile);
 
-  FSWeb::downloadFile(v_destinationFile,
-          v_sourceFile,
-          FSWeb::f_curl_progress_callback_download,
-          &v_data,
-          m_proxy_settings);
+	FSWeb::downloadFile(v_destinationFile,
+			    v_sourceFile,
+			    FSWeb::f_curl_progress_callback_download,
+			    &v_data,
+			    m_proxy_settings);
   
-  v_nb_files_performed++;
+	v_nb_files_performed++;
       }
       m_WebApp->readEvents();
       i++;
     }
     m_WebApp->setTaskProgress(100.0);
   }
-
+  
   delete v_theme;
 
-  p_themeChoice->setRequireUpdate(false);
-  p_themeChoice->setHosted(true);
-}
-
-WebRoomInfos::WebRoomInfos(std::string p_id,
-         std::string p_name,
-         std::string p_urlHighscores) {
-  m_id           = p_id;
-  m_name         = p_name;
-  m_urlHighscore = p_urlHighscores;
-}
-
-WebRoomInfos::~WebRoomInfos() {
-}
-
-std::string WebRoomInfos::getId() {
-  return m_id;
-}
-
-std::string WebRoomInfos::getName() {
-  return m_name;
-}
-
-std::string WebRoomInfos::getUrlHighscores() {
-  return m_urlHighscore;
+  if(v_onDisk) {
+    i_db->themes_update(i_id_theme, v_themeFile);
+  } else {
+    i_db->themes_add(i_id_theme, v_themeFile);
+  }
 }
 
 WebRooms::WebRooms(const ProxySettings *p_proxy_settings) {
@@ -1259,7 +909,6 @@ WebRooms::WebRooms(const ProxySettings *p_proxy_settings) {
 }
 
 WebRooms::~WebRooms() {
-  clean();
 }
 
 void WebRooms::update() {
@@ -1274,67 +923,6 @@ std::string WebRooms::getXmlFileName() {
   return vapp::FS::getUserDir() + "/" + DEFAULT_WEBROOMS_FILENAME;
 }
 
-void WebRooms::upgrade() {
-  clean();
-
-  try {
-    vapp::XMLDocument v_webRXml;
-    TiXmlDocument *v_webRXmlData;
-    TiXmlElement *v_webRXmlDataElement;
-    const char *pc;
-    std::string v_RoomName, v_RoomHighscoreUrl, v_RoomId;
-  
-    v_webRXml.readFromFile(getXmlFileName());
-    v_webRXmlData = v_webRXml.getLowLevelAccess();
-
-    if(v_webRXmlData == NULL) {
-      throw Exception("error : unable to analyze xml rooms list file");
-    }
-
-    v_webRXmlDataElement = v_webRXmlData->FirstChildElement("xmoto_rooms");
-    
-    if(v_webRXmlDataElement == NULL) {
-      throw Exception("error : unable to analyze xml rooms list file");
-    }
-    
-    TiXmlElement *pVarElem = v_webRXmlDataElement->FirstChildElement("room");
-    while(pVarElem != NULL) {
-      
-      pc = pVarElem->Attribute("name");
-      if(pc != NULL) {
-  v_RoomName = pc;
-  
-  pc = pVarElem->Attribute("highscores_url");
-  if(pc != NULL) {
-    v_RoomHighscoreUrl = pc;   
-    
-    pc = pVarElem->Attribute("id");
-    if(pc != NULL) {
-      v_RoomId = pc;
-      m_availableRooms.push_back(new WebRoomInfos(v_RoomId,
-              v_RoomName,
-              v_RoomHighscoreUrl));
-    }
-  }
-  pVarElem = pVarElem->NextSiblingElement("room");
-      }
-    }
-    
-  } catch(Exception &e) {
-    /* by default, room is highscore */
-    m_availableRooms.push_back(new WebRoomInfos(DEFAULT_WEBROOM_ID, DEFAULT_WEBROOM_NAME, DEFAULT_WEBHIGHSCORES_URL));
-  }
+void WebRooms::upgrade(xmDatabase *i_db) {
+  i_db->webrooms_updateDB(getXmlFileName());
 }
-
-const std::vector<WebRoomInfos*> &WebRooms::getAvailableRooms() {
-  return m_availableRooms;
-}
-
-void WebRooms::clean() {
-  for(unsigned int i=0;i<m_availableRooms.size();i++) {
-    delete m_availableRooms[i];
-  }    
-  m_availableRooms.clear();
-}
-
-#endif
