@@ -25,13 +25,12 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define __WWW_H__
 
 #include "BuildConfig.h"
-#if defined(SUPPORT_WEBACCESS)
 
 #include <string>
 #include <vector>
 #include <stdio.h>
 
-#if !defined(_MSC_VER) && !defined(__APPLE__) && !defined(__MACH__)
+#if !defined(WIN32) && !defined(__APPLE__) && !defined(__MACH__)
   //#define USE_HASH_MAP // removed because it seems to segfault i don't know why when i refresh levels using F5 and quit
 #endif
 
@@ -42,7 +41,7 @@ class ThemeChoice;
 #define DEFAULT_WEBHIGHSCORES_URL         "http://xmoto.tuxfamily.org/highscores.xml"
 #define DEFAULT_WEBHIGHSCORES_FILENAME    "webhighscores.xml"
 #define DEFAULT_TRANSFERT_TIMEOUT         240
-#define DEFAULT_TRANSFERT_CONNECT_TIMEOUT 7
+#define DEFAULT_TRANSFERT_CONNECT_TIMEOUT 15
 #define DEFAULT_WEBLEVELS_URL             "http://xmoto.tuxfamily.org/levels.xml"
 #define DEFAULT_WEBLEVELS_FILENAME        "weblevels.xml"
 #define DEFAULT_WEBLEVELS_DIR             "downloaded"
@@ -69,12 +68,8 @@ class ThemeChoice;
   #endif
   #else // #ifdef __GNUC__
   #include <hash_map>
-  #if (_MSC_VER >= 1300)
-  namespace HashNamespace=stdext;
-  #else
   namespace HashNamespace=std;
   #endif
-  #endif
 
   struct highscore_str {
     bool operator()(const char* s1, const char* s2) {
@@ -84,6 +79,7 @@ class ThemeChoice;
 #endif
 
 class WebRoom;
+class xmDatabase;
 
 class ProxySettings {
  public:
@@ -178,78 +174,25 @@ class FSWeb {
 				       std::string &p_msg);
 };
 
-class WebHighscore {
- public:
-  WebHighscore(WebRoom *p_room,
-	       std::string p_levelId,
-	       std::string p_playerName,
-	       std::string p_time,
-	       std::string p_rplUrl,
-	       const ProxySettings *p_proxy_settings);
-  ~WebHighscore();
-  void download(); /* throws exceptions */
-  /* return the path to the replay, even if it's not downloaded */
-  std::string getReplayName();
-
-  std::string getPlayerName() const;
-  std::string getTime() const;
-  float       getFTime() const; /* throws exceptions */
-  std::string getLevelId() const;
-  WebRoom* getRoom() const;
-
- private:
-  std::string m_playerName;
-  std::string m_time;
-  std::string m_levelId;
-  std::string m_rplUrl;
-  std::string m_rplFilename;
-  const ProxySettings *m_proxy_settings;
-  WebRoom *m_room;
-};
-
 class WebRoom {
  public:
   WebRoom(const ProxySettings *p_proxy_settings);
   ~WebRoom();
 
   void update(); /* throws exceptions */
-  void upgrade(); /* throws exceptions */
+  void upgrade(xmDatabase *i_db); /* throws exceptions */
 
   /* return NULL if no data found */
-  WebHighscore* getHighscoreFromLevel(const std::string &p_levelId);
-  void setWebsiteURL(std::string p_webhighscores_url);
-  std::string getRoomName() const;
+  void setWebsiteInfos(const std::string& i_id_room, const std::string& i_webhighscores_url);
+  std::string getRoomId() const;
+
+  void downloadReplay(const std::string& i_url);
 
  private:
-  #if defined(USE_HASH_MAP)
-    HashNamespace::hash_map<const char*, WebHighscore*, HashNamespace::hash<const char*>, highscore_str> m_webhighscores;
-  #else
-    std::vector<WebHighscore *> m_webhighscores;
-  #endif
   std::string m_userFilename;
   std::string m_webhighscores_url;
-  std::string m_roomName;
+  std::string m_roomId;
   const ProxySettings *m_proxy_settings;
-
-  void fillHash();
-  void cleanHash();
-};
-
-class WebRoomInfos {
- public:
-  WebRoomInfos(std::string p_id,
-	       std::string p_name,
-	       std::string p_urlHighscores);
-  ~WebRoomInfos();
-
-  std::string getId();
-  std::string getName();
-  std::string getUrlHighscores();
-
- private:
-  std::string m_id;
-  std::string m_name;
-  std::string m_urlHighscore;
 };
 
 class WebRooms {
@@ -261,43 +204,17 @@ class WebRooms {
   void update(); /* throws exceptions */
 
   /* fill the list of avaible rooms ; does not required an internet connexion */
-  void upgrade();
+  void upgrade(xmDatabase *i_db);
 
   void setURL(const std::string &p_url) {m_rooms_url = p_url;}
-  const std::vector<WebRoomInfos*> &getAvailableRooms();
 
  private:
   const ProxySettings *m_proxy_settings;
-  std::vector<WebRoomInfos*> m_availableRooms;
   std::string m_rooms_url;
 
-  void clean();
   std::string getXmlFileName();
 };
 
-class WebLevel {
-public:
-  WebLevel(std::string p_id, std::string p_name, std::string p_url);
-  std::string getId() const;
-  std::string getName() const;
-  std::string getUrl() const;
-
-  /* true if the level should be updated from the website but exists */  
-  bool requireUpdate() const; /* false by default */
-  void setRequireUpdate(bool p_require_update);
-
-  /* if level require update, these methods have a sense */
-  void setCurrentPath(std::string p_current_path);
-  std::string getCurrentPath() const;
-
-private:
-  std::string m_id;
-  std::string m_name;
-  std::string m_url;
-  bool m_require_update;
-  std::string m_current_path;
-};
-
 class WebLevels {
  public:
   WebLevels(vapp::WWWAppInterface *p_WebLevelApp,
@@ -305,31 +222,28 @@ class WebLevels {
   ~WebLevels();
 
   /* check for new levels to download */
-  void update(); /* throws exceptions */
+  void update(xmDatabase *i_db); /* throws exceptions */
 
   /* download new levels */
-  void upgrade(); /* throws exceptions */
-  
-  /* after update() you might want to know how much is going to be downloaded */
-  void getUpdateInfo(int *pnUBytes,int *pnULevels);
+  void upgrade(xmDatabase *i_db); /* throws exceptions */
   
   /* Get names of new files downloaded OK */
   const std::vector<std::string> &getNewDownloadedLevels();
   
   /* Get names of updated files downloaded OK*/
   const std::vector<std::string> &getUpdatedDownloadedLevels();
-
-  /* Get IDs of updated levels downloaded OK*/
-  const std::vector<std::string> &getUpdatedDownloadedLevelIDs();
   
   /* Set URL */
   void setURL(const std::string &p_url) {m_levels_url = p_url;}
 
-  bool exists(const std::string p_id);
+  static std::string getDestinationFile(std::string p_url);
+
+  /* return the number of level files required to download for an upgrade */
+  int nbLevelsToGet(xmDatabase *i_db) const;
 
  private:
   vapp::WWWAppInterface *m_WebLevelApp;
-  std::vector<WebLevel*> m_webLevels;
+
   std::vector<std::string> m_webLevelsNewDownloadedOK; /* file names of those levels 
            which where downloaded OK (so we can load them right away) and which are new */
   std::vector<std::string> m_webLevelsUpdatedDownloadedOK;
@@ -340,24 +254,8 @@ class WebLevels {
 
   std::string getXmlFileName();
   void downloadXml(); /* throw exceptions */
-  std::string getDestinationDir();
-  std::string getDestinationFile(std::string p_url);
+  static std::string getDestinationDir();
   void createDestinationDirIfRequired();
-  void extractLevelsToDownloadFromXml(); /* throw exceptions */				      
-};
-
-class WebTheme {
- public:
-  WebTheme(std::string pName, std::string pUrl, std::string pSum);
-  ~WebTheme();
-  std::string getName() const;
-  std::string getUrl() const;
-  std::string getSum() const;
-
- private:
-  std::string m_name;
-  std::string m_url;
-  std::string m_sum;
 };
 
 class WebThemes {
@@ -367,33 +265,22 @@ class WebThemes {
   ~WebThemes();
 
   /* check for new themes to download */
-  void update(); /* throws exceptions */
-
-  /* fill the list of avaible theme ; does not required an internet connexion */
-  void upgrade();
+  void update(xmDatabase *i_db); /* throws exceptions */
 
   /* download a theme or just update it */
-  void upgrade(ThemeChoice *p_themeChoice);
-  bool isUpgradable(ThemeChoice *p_themeChoice);
-
-  const std::vector<WebTheme*> &getAvailableThemes();
+  void upgrade(xmDatabase *i_db, const std::string& i_id_theme);
 
   void setURL(const std::string &p_url) {m_themes_url = p_url;}
   void setURLBase(const std::string &p_urlBase) {m_themes_urlBase = p_urlBase;}
 
  private:
   std::string getXmlFileName();
-  void extractThemesAvailableFromXml();
-  void clean();
 
   vapp::WWWAppInterface *m_WebApp;
   std::string m_themes_url;
   std::string m_themes_urlBase;
   const ProxySettings *m_proxy_settings;
-  std::vector<WebTheme*> m_availableThemes;
 };
 
-#endif
-
 #endif /* WEBSTUFFS */
 
