@@ -29,7 +29,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #define ENTITY_DEFAULT_SPRITE_NAME ""
 #define ENTITY_DEFAULT_SIZE        1.0
-#define ENTITY_DEFAULT_Z           1.0
+#define ENTITY_DEFAULT_Z           -1.0
 #define PARTICLES_SOURCE_SMOKE_TIME_INCREMENT  0.050
 #define PARTICLES_SOURCE_STAR_TIME_INCREMENT   0.025
 #define PARTICLES_SOURCE_FIRE_TIME_INCREMENT   0.040
@@ -55,9 +55,12 @@ class Entity {
   float Z() const;
   float Width() const;
   float Height() const;
+  float DrawAngle() const;
+  bool  DrawReversed() const;
   const TColor& Color() const;
   std::string SpriteName() const; /* PlayerStart, Flower, EndOfLevel, Bird, ... */
-  
+  bool  isAlive() const;
+
   virtual void loadToPlay();
   virtual void unloadToPlay();
   void setDynamicPosition(const Vector2f& i_dynamicPosition);
@@ -66,9 +69,12 @@ class Entity {
   void setSize(float i_size);
   void setWidth(float i_width);
   void setHeight(float i_height);
+  void setDrawAngle(float i_drawAngle);
+  void setDrawReversed(bool i_drawReversed);
   void setZ(float i_z);
   void setSpriteName(const std::string& i_spriteName);
   void setColor(const TColor& i_color);
+  void setAlive(bool alive);
 
   void saveXml(vapp::FileHandle *i_pfh);
   void saveBinary(vapp::FileHandle *i_pfh);
@@ -81,7 +87,8 @@ class Entity {
   void setSpeciality(EntitySpeciality i_speciality);
 
   virtual bool updateToTime(float i_time, Vector2f i_gravity);
-  virtual void clearAfterRewind();
+
+  AABB& getAABB();
 
  private:
   std::string m_id;              /** Its own identifer */
@@ -90,11 +97,18 @@ class Entity {
   Vector2f    m_dynamicPosition; /** Current position */
   float       m_size;            /** Size (radius) */
   float       m_width, m_height; /** size of the picture, negativ it the theme size must be used */
+  float       m_drawAngle;
+  bool        m_drawReversed;
   float       m_z;               /** deep coord */
   bool        m_doesKill;
   bool        m_doesMakeWin;
   bool        m_isToTake;
+  /* Use to know if a script shall update the pos of the entity*/
+  bool        m_isAlive;
   TColor      m_color;
+
+  AABB        m_BBox;
+  bool        m_isBBoxDirty;
 };
 
 class ParticlesSource : public Entity {
@@ -105,7 +119,6 @@ class ParticlesSource : public Entity {
   virtual void loadToPlay();
   virtual void unloadToPlay();
   virtual bool updateToTime(float i_time, Vector2f i_gravity);
-  virtual void clearAfterRewind();
   void addParticle(Vector2f i_velocity, float i_killTime);
   virtual void addParticle(Vector2f i_velocity, float i_killTime, std::string i_spriteName) = 0;
   std::vector<EntityParticle *> &Particles();
@@ -115,7 +128,7 @@ class ParticlesSource : public Entity {
   std::vector<EntityParticle *> m_particles;
 
  private:
-  float       m_nextParticleTime;
+  float       m_lastParticleTime;
   float       m_particleTime_increment;
 
   void deleteParticles();
