@@ -28,8 +28,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "Level.h"
 #include "../VFileIO.h"
 #include "../VXml.h"
-#include "../md5sum/md5file.h"
 #include "../helpers/Color.h"
+#include "../db/xmDatabase.h";
+
+#define CACHE_LEVEL_FORMAT_VERSION 18
 
 Level::Level() {
   m_xmotoTooOld = false;
@@ -37,7 +39,11 @@ Level::Level() {
   m_rightLimit  = 0.0;
   m_topLimit    = 0.0;
   m_bottomLimit = 0.0;
-  m_xmlSource = NULL;
+  m_xmlSource   = NULL;
+  m_nbEntitiesToTake = 0;
+  m_borderTexture = "";
+  m_numberLayer   = 0;
+  m_isScripted    = false;
 }
 
 Level::~Level() {
@@ -65,6 +71,10 @@ std::string Level::Description() const {
   return m_description;
 }
 
+std::string Level::Music() const {
+  return m_music;
+}
+
 bool Level::isXMotoTooOld() const {
   return m_xmotoTooOld;
 }
@@ -121,7 +131,7 @@ std::string Level::Checksum() const {
 }
 
 bool Level::isScripted() const {
-  return (m_scriptFileName != "" || m_scriptSource != "");
+  return m_isScripted;
 }
 
 Block& Level::getBlockById(const std::string& i_id) {
@@ -190,8 +200,8 @@ void Level::setAuthor(const std::string& i_author) {
   m_author = i_author;
 }
 
-void Level::setSky(const std::string& i_sky) {
-  m_sky = i_sky;
+void Level::setCollisionSystem(vapp::CollisionSystem* p_CollisionSystem) {
+  m_pCollisionSystem = p_CollisionSystem;
 }
 
 std::string Level::scriptFileName() const {
@@ -214,13 +224,17 @@ std::vector<Zone *> &Level::Zones() {
   return m_zones;
 }
 
-std::string Level::Sky() const {
+const SkyApparence& Level::Sky() const {
   return m_sky;
 }
 
 void Level::killEntity(const std::string& i_entityId) {
   for(unsigned int i=0; i<m_entities.size(); i++) {
     if(m_entities[i]->Id() == i_entityId) {
+      if(m_entities[i]->IsToTake()){
+	m_nbEntitiesToTake--;
+      }
+      m_entities[i]->setAlive(false);
       m_entitiesDestroyed.push_back(m_entities[i]);
       m_entities.erase(m_entities.begin() + i);
       return;
@@ -234,21 +248,25 @@ std::vector<Entity *>& Level::Entities() {
 }
 
 unsigned int Level::countToTakeEntities() {
-  unsigned int n = 0;
-  
-  for(unsigned int i=0; i<m_entities.size(); i++) {
-    if(m_entities[i]->IsToTake()) {
-      n++;
-    }
-  }
-  return n;
+  return m_nbEntitiesToTake;
 }
 
 void Level::revertEntityDestroyed(const std::string& i_entityId) {
   for(unsigned int i=0; i<m_entitiesDestroyed.size(); i++) {
     if(m_entitiesDestroyed[i]->Id() == i_entityId) {
+      m_entitiesDestroyed[i]->setAlive(true);
+
+      if(m_entitiesDestroyed[i]->IsToTake()){
+	m_nbEntitiesToTake++;
+      }
+
       m_entities.push_back(m_entitiesDestroyed[i]);
+
+      /* add it back to the collision system */
+      m_pCollisionSystem->addEntity(m_entitiesDestroyed[i]);
+
       m_entitiesDestroyed.erase(m_entitiesDestroyed.begin() + i);
+
       return;
     }
   }
@@ -296,9 +314,81 @@ void Level::saveXML(void) {
   vapp::FS::writeLineF(pfh,"\t\t<description>%s</description>",m_description.c_str());
   vapp::FS::writeLineF(pfh,"\t\t<author>%s</author>",m_author.c_str());
   vapp::FS::writeLineF(pfh,"\t\t<date>%s</date>",m_date.c_str());
-  vapp::FS::writeLineF(pfh,"\t\t<sky>%s</sky>",m_sky.c_str());
+  if(m_sky.Drifted()) {
+    vapp::FS::writeLineF(pfh,
+			 "\t\t<sky zoom=\"%f\" offset=\"%f\" color_r=\"%i\" color_g=\"%i\" color_b=\"%i\" color_a=\"%i\" drifted=\"true\" driftZoom=\"%f\" driftColor_r=\"%i\" driftColor_g=\"%i\" driftColor_b=\"%i\" driftColor_a=\"%i\">%s</sky>",
+			 m_sky.Zoom(),
+			 m_sky.Offset(),
+			 m_sky.TextureColor().Red(),
+			 m_sky.TextureColor().Green(),
+			 m_sky.TextureColor().Blue(),
+			 m_sky.TextureColor().Alpha(),
+			 m_sky.DriftZoom(),
+			 m_sky.DriftTextureColor().Red(),
+			 m_sky.DriftTextureColor().Green(),
+			 m_sky.DriftTextureColor().Blue(),
+			 m_sky.DriftTextureColor().Alpha(),
+			 m_sky.Texture().c_str());
+  } else {
+    vapp::FS::writeLineF(pfh,
+			 "\t\t<sky zoom=\"%f\" offset=\"%f\" color_r=\"%i\" color_g=\"%i\" color_b=\"%i\" color_a=\"%i\">%s</sky>",
+			 m_sky.Zoom(),
+			 m_sky.Offset(),
+			 m_sky.TextureColor().Red(),
+			 m_sky.TextureColor().Green(),
+			 m_sky.TextureColor().Blue(),
+			 m_sky.TextureColor().Alpha(),
+			 m_sky.Texture().c_str());
+  }
+  vapp::FS::writeLineF(pfh,"\t\t<border texture=\"%s\" />",m_borderTexture.c_str());
+  vapp::FS::writeLineF(pfh,"\t\t<music name=\"%s\" />", m_music.c_str());
   vapp::FS::writeLineF(pfh,"\t</info>");
+
+  if(m_numberLayer > 0){
+    vapp::FS::writeLineF(pfh, "\t\t<layeroffsets>");
+    for(int i=0; i<m_numberLayer; i++){
+      std::string front = "";
+      if(m_isLayerFront[i] == true){
+	front = " frontlayer=\"true\"";
+      }
+
+      vapp::FS::writeLineF(pfh, "\t\t\t<layeroffset x=\"%f\" y=\"%f\"%s/>",
+			   m_layerOffsets[i].x, m_layerOffsets[i].y, front.c_str());
+    }
+    vapp::FS::writeLineF(pfh, "\t\t</layeroffsets>");
+  }
   
+  /* replacement sprites */
+  if(m_rSpriteForStrawberry 	 != "" ||
+     m_rSpriteForWecker     	 != "" ||
+     m_rSpriteForFlower     	 != "" ||
+     m_rSpriteForStar       	 != "" ||
+     m_rSoundForPickUpStrawberry != ""
+     ) {
+    vapp::FS::writeLineF(pfh,"\t<theme_replacements>");
+    if(m_rSpriteForStrawberry != "") {
+      vapp::FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Strawberry\" new_name=\"%s\" />",
+			   m_rSpriteForStrawberry.c_str());
+    }
+    if(m_rSpriteForWecker != "") {
+      vapp::FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Wrecker\" new_name=\"%s\" />",
+			   m_rSpriteForWecker.c_str());
+    }
+    if(m_rSpriteForFlower != "") {
+      vapp::FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Flower\" new_name=\"%s\" />",
+			   m_rSpriteForFlower.c_str());
+    }
+    if(m_rSpriteForStar != "") {
+      vapp::FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Star\" new_name=\"%s\" />",
+			   m_rSpriteForStar.c_str());
+    }
+    if(m_rSoundForPickUpStrawberry != "") {
+      vapp::FS::writeLineF(pfh,"\t\t<sound_replacement old_name=\"PickUpStrawberry\" new_name=\"%s\" />",
+			   m_rSoundForPickUpStrawberry.c_str());
+    }
+    vapp::FS::writeLineF(pfh,"\t</theme_replacements>");
+  }
+
   /* MISC */
   if(m_scriptFileName != "" && m_scriptSource != "") {
     vapp::FS::writeLineF(pfh,"\t<script source=\"%s\">",m_scriptFileName.c_str());
@@ -358,8 +448,8 @@ void Level::loadXML(void) {
     m_xmlSource = new vapp::XMLDocument();
   }
 
-  m_xmlSource->readFromFile( m_fileName, NULL );    
-  
+  m_xmlSource->readFromFile(m_fileName, NULL);
+
   TiXmlDocument *pDoc = m_xmlSource->getLowLevelAccess();
   if(pDoc == NULL) throw Exception("failed to load level XML");
   
@@ -388,16 +478,22 @@ void Level::loadXML(void) {
   m_date = "";
   m_description = "";
   m_author = "";
-  m_sky = "sky1";
+  m_music = "";
   
   m_scriptFileName = "";
   m_scriptSource = "";
-  
+  m_borderTexture = "";
+
   m_bottomLimit = m_leftLimit = -50.0f;
   m_topLimit    = m_rightLimit = 50.0f;
   
   m_playerStart = Vector2f(0.0, 0.0);
+  m_numberLayer = 0;
   
+  m_isScripted = false;
+
+  m_sky.reInit();
+
   if(!m_xmotoTooOld) {    
     /* Get level pack */
     m_pack = vapp::XML::getOption(pLevelElem,"levelpack");
@@ -409,7 +505,7 @@ void Level::loadXML(void) {
       /* Name */
       std::string Tmp = vapp::XML::getElementText(*m_xmlSource, pInfoElem,"name");
       if(Tmp != "") m_name = Tmp;
-      
+
       /* Author */
       Tmp = vapp::XML::getElementText(*m_xmlSource, pInfoElem,"author");
       if(Tmp != "") m_author = Tmp;
@@ -424,12 +520,151 @@ void Level::loadXML(void) {
       
       /* Sky */
       Tmp = vapp::XML::getElementText(*m_xmlSource, pInfoElem,"sky");
-      if(Tmp != "") m_sky = Tmp;
+      if(Tmp != "") m_sky.setTexture(Tmp);
+
+      /* advanced sky parameters ? */
+      bool v_useAdvancedOptions = false;
+      TiXmlElement *pSkyElem = vapp::XML::findElement(*m_xmlSource, NULL, std::string("sky"));
+      std::string v_skyValue;
+      if(pSkyElem != NULL) {
+	v_skyValue = vapp::XML::getOption(pSkyElem, "zoom");
+	if(v_skyValue != "") {
+	  m_sky.setZoom(atof(v_skyValue.c_str()));
+	  v_useAdvancedOptions = true;
+	}
+	v_skyValue = vapp::XML::getOption(pSkyElem, "offset");
+	if(v_skyValue != "") {
+	  m_sky.setOffset(atof(v_skyValue.c_str()));
+	  v_useAdvancedOptions = true;
+	}
+
+	int v_r = -1, v_g = -1, v_b = -1, v_a = -1;
+	v_skyValue = vapp::XML::getOption(pSkyElem, "color_r");
+	if(v_skyValue != "") v_r = atoi(v_skyValue.c_str());
+	v_skyValue = vapp::XML::getOption(pSkyElem, "color_g");
+	if(v_skyValue != "") v_g = atoi(v_skyValue.c_str());
+	v_skyValue = vapp::XML::getOption(pSkyElem, "color_b");
+	if(v_skyValue != "") v_b = atoi(v_skyValue.c_str());
+	v_skyValue = vapp::XML::getOption(pSkyElem, "color_a");
+	if(v_skyValue != "") v_a = atoi(v_skyValue.c_str());
+	if(v_r != -1 || v_g != -1 || v_b != -1 || v_a != -1) {
+	  if(v_r == -1) v_r = 0;
+	  if(v_g == -1) v_g = 0;
+	  if(v_b == -1) v_b = 0;
+	  if(v_a == -1) v_a = 0;
+	  m_sky.setTextureColor(TColor(v_r, v_g, v_b, v_a));
+	  v_useAdvancedOptions = true;
+	}
+
+	v_skyValue = vapp::XML::getOption(pSkyElem, "drifted");
+	if(v_skyValue == "true") {
+	  m_sky.setDrifted(true);
+	  v_useAdvancedOptions = true;
+
+	  v_skyValue = vapp::XML::getOption(pSkyElem, "driftZoom");
+	  if(v_skyValue != "") m_sky.setDriftZoom(atof(v_skyValue.c_str()));
+
+	  int v_r = -1, v_g = -1, v_b = -1, v_a = -1;
+	  v_skyValue = vapp::XML::getOption(pSkyElem, "driftColor_r");
+	  if(v_skyValue != "") v_r = atoi(v_skyValue.c_str());
+	  v_skyValue = vapp::XML::getOption(pSkyElem, "driftColor_g");
+	  if(v_skyValue != "") v_g = atoi(v_skyValue.c_str());
+	  v_skyValue = vapp::XML::getOption(pSkyElem, "driftColor_b");
+	  if(v_skyValue != "") v_b = atoi(v_skyValue.c_str());
+	  v_skyValue = vapp::XML::getOption(pSkyElem, "driftColor_a");
+	  if(v_skyValue != "") v_a = atoi(v_skyValue.c_str());
+	  if(v_r != -1 || v_g != -1 || v_b != -1 || v_a != -1) {
+	    if(v_r == -1) v_r = 0;
+	    if(v_g == -1) v_g = 0;
+	    if(v_b == -1) v_b = 0;
+	    if(v_a == -1) v_a = 0;
+	    m_sky.setDriftTextureColor(TColor(v_r, v_g, v_b, v_a)); 
+	  }
+	}
+      }
+      if(v_useAdvancedOptions == false) {
+	/* set old values in case no option is used */
+	m_sky.setOldXmotoValuesFromTextureName();
+      }
+
+      /* Border */
+      TiXmlElement *pBorderElem = vapp::XML::findElement(*m_xmlSource, NULL, std::string("border"));
+      if(pBorderElem != NULL) {
+	m_borderTexture = vapp::XML::getOption(pBorderElem, "texture");  
+      }
+
+      /* Music */
+      TiXmlElement *pMusicElem = vapp::XML::findElement(*m_xmlSource, NULL, std::string("music"));
+      if(pMusicElem != NULL) {
+	m_music = vapp::XML::getOption(pMusicElem, "name");  
+      }
     }
-    
+
+    /* background level offsets */
+    TiXmlElement* pLayerOffsets = vapp::XML::findElement(*m_xmlSource, NULL, std::string("layeroffsets"));
+    if(pLayerOffsets == NULL){
+      m_numberLayer = 0;
+    }
+    else {
+      for(TiXmlElement* pElem = pLayerOffsets->FirstChildElement("layeroffset");
+	  pElem!=NULL;
+	  pElem=pElem->NextSiblingElement("layeroffset")) {
+	Vector2f offset;
+	offset.x = atof(vapp::XML::getOption(pElem, "x").c_str());
+	offset.y = atof(vapp::XML::getOption(pElem, "y").c_str());
+	m_numberLayer++;
+	m_layerOffsets.push_back(offset);
+	m_isLayerFront.push_back(vapp::XML::getOption(pElem, "frontlayer", "false") == "true");
+      }
+      for(int i=0; i<m_numberLayer; i++){
+	vapp::Log("Level::loadXML offsets layer %d: %f, %f isfront: %s",
+		  i, m_layerOffsets[i].x, m_layerOffsets[i].y,
+		  m_isLayerFront[i]?"true":"false");
+      }
+    }
+
+    /* replacement theme */
+    m_rSpriteForStrawberry 	= "";
+    m_rSpriteForWecker 	   	= "";
+    m_rSpriteForFlower 	   	= "";
+    m_rSpriteForStar   	   	= "";
+    m_rSoundForPickUpStrawberry = "";
+
+    TiXmlElement *pThemeReplaceElem = vapp::XML::findElement(*m_xmlSource, pLevelElem,std::string("theme_replacements"));
+    if(pThemeReplaceElem != NULL) {
+      /* Get replacements  for sprites */
+      for(TiXmlElement *pElem = pThemeReplaceElem->FirstChildElement("sprite_replacement"); pElem!=NULL;
+	  pElem=pElem->NextSiblingElement("sprite_replacement")) {
+	std::string v_old_name = vapp::XML::getOption(pElem, "old_name");
+	std::string v_new_name = vapp::XML::getOption(pElem, "new_name");
+	/* for efficacity and before other are not required, only change main ones */
+	if        (v_old_name == "Strawberry") {
+	  m_rSpriteForStrawberry = v_new_name;
+	} else	if(v_old_name == "Wrecker") {
+	  m_rSpriteForWecker     = v_new_name;
+	} else 	if(v_old_name == "Flower") {
+	  m_rSpriteForFlower     = v_new_name;
+	} else 	if(v_old_name == "Star") {
+	  m_rSpriteForStar       = v_new_name;
+	}
+      }
+      /* Get replacements  for sounds */
+      for(TiXmlElement *pElem = pThemeReplaceElem->FirstChildElement("sound_replacement"); pElem!=NULL;
+	  pElem=pElem->NextSiblingElement("sound_replacement")) {
+	std::string v_old_name = vapp::XML::getOption(pElem, "old_name");
+	std::string v_new_name = vapp::XML::getOption(pElem, "new_name");
+	/* for efficacity and before other are not required, only change main ones */
+	if        (v_old_name == "PickUpStrawberry") {
+	  m_rSoundForPickUpStrawberry = v_new_name;
+	}
+      }
+    }
+
     /* Get script */
     TiXmlElement *pScriptElem = vapp::XML::findElement(*m_xmlSource, pLevelElem,std::string("script"));
     if(pScriptElem != NULL) {
+      m_isScripted = true;
+
       /* External script file specified? */
       m_scriptFileName = vapp::XML::getOption(pScriptElem,"source");      
       
@@ -477,6 +712,19 @@ void Level::loadXML(void) {
     }  
   }
 
+  /* if blocks with layerid but level with no layer, throw an exception */
+  for(int i=0; i<m_blocks.size(); i++){
+    int blockLayer = m_blocks[i]->getLayer();
+    if(blockLayer != -1){
+      if((getNumberLayer() == 0) || (blockLayer >= getNumberLayer())){
+	vapp::Log("** Warning ** : Level '%s' Block '%s' as a layer, but the level has no layer", m_id.c_str(), m_blocks[i]->Id().c_str());
+	throw Exception("the block has a layer but the level is without layers");
+      }
+    }
+  }
+
+  addLimits();
+
   delete m_xmlSource;
   m_xmlSource = NULL;
 
@@ -485,33 +733,29 @@ void Level::loadXML(void) {
 
 /* Load using the best way possible. File name must already be set!
   *  Return whether or not it was loaded from the cache. */
-bool Level::loadReducedFromFile(bool cacheEnabled) {
+bool Level::loadReducedFromFile() {
   std::string cacheFileName;
 
-  m_checkSum = md5file(FileName());
+  m_checkSum = vapp::FS::md5sum(FileName());
 
   // First try to load it from the cache
   bool cached = false;
-  if (cacheEnabled) {
-    /* Determine name in cache */
-    std::string LevelFileBaseName = vapp::FS::getFileBaseName(FileName());
-    cacheFileName = getNameInCache();
+  /* Determine name in cache */
+  std::string LevelFileBaseName = vapp::FS::getFileBaseName(FileName());
+  cacheFileName = getNameInCache();
     
-    try {
-      cached = importBinaryHeaderFromFile(cacheFileName, m_checkSum);
-    } catch (Exception &e) {
-      vapp::Log("** Warning **: Exception while loading binary level, will load "
-                "XML instead for '%s' (%s)", FileName().c_str(),
-                e.getMsg().c_str());
-    }
+  try {
+    cached = importBinaryHeaderFromFile(cacheFileName, m_checkSum);
+  } catch (Exception &e) {
+    vapp::Log("** Warning **: Exception while loading binary level, will load "
+	      "XML instead for '%s' (%s)", FileName().c_str(),
+	      e.getMsg().c_str());
   }
   
   // If we couldn't get it from the cache, then load from (slow) XML
   if (!cached) {
     loadXML();
-    if (cacheEnabled) {
-      exportBinary(cacheFileName, m_checkSum); /* Cache it now */
-    }
+    exportBinary(cacheFileName, m_checkSum); /* Cache it now */
   }
   
   unloadLevelBody(); /* remove body datas */
@@ -523,6 +767,24 @@ std::string Level::getNameInCache() const {
   return "LCache/" + Checksum() + vapp::FS::getFileBaseName(FileName()) + ".blv";
 }
 
+void Level::removeFromCache(xmDatabase *i_db, const std::string& i_id_level) {
+  char **v_result;
+  int nrow;
+  v_result = i_db->readDB("SELECT checkSum, filepath FROM levels WHERE id_level=\"" +
+			  xmDatabase::protectString(i_id_level) + "\";",
+			  nrow);
+  if(nrow == 1) {
+    try {
+      std::string v_checkSum = i_db->getResult(v_result, 2, 0, 0);
+      std::string v_filePath = i_db->getResult(v_result, 2, 0, 1);
+      vapp::FS::deleteFile("LCache/" + v_checkSum + vapp::FS::getFileBaseName(v_filePath) + ".blv");
+    } catch(Exception &e) {
+      /* ok, it was perhaps not in cache */
+    }
+  }
+  i_db->read_DB_free(v_result);
+}
+
 /*===========================================================================
   Export binary level file
   ===========================================================================*/
@@ -538,14 +800,41 @@ void Level::exportBinary(const std::string &FileName, const std::string& pSum) {
   else {
     exportBinaryHeader(pfh);
 
-    vapp::FS::writeString(pfh,m_sky);
+    vapp::FS::writeString(pfh,   m_sky.Texture());
+    vapp::FS::writeFloat_LE(pfh, m_sky.Zoom());
+    vapp::FS::writeFloat_LE(pfh, m_sky.Offset());
+    vapp::FS::writeInt_LE(pfh,   m_sky.TextureColor().Red());
+    vapp::FS::writeInt_LE(pfh,   m_sky.TextureColor().Green());
+    vapp::FS::writeInt_LE(pfh,   m_sky.TextureColor().Blue());
+    vapp::FS::writeInt_LE(pfh,   m_sky.TextureColor().Alpha());
+    vapp::FS::writeBool(pfh,     m_sky.Drifted());
+    vapp::FS::writeFloat_LE(pfh, m_sky.DriftZoom());
+    vapp::FS::writeInt_LE(pfh,   m_sky.DriftTextureColor().Red());
+    vapp::FS::writeInt_LE(pfh,   m_sky.DriftTextureColor().Green());
+    vapp::FS::writeInt_LE(pfh,   m_sky.DriftTextureColor().Blue());
+    vapp::FS::writeInt_LE(pfh,   m_sky.DriftTextureColor().Alpha());
+
+    vapp::FS::writeString(pfh,m_borderTexture);
     vapp::FS::writeString(pfh,m_scriptFileName);
-      
+
     vapp::FS::writeFloat_LE(pfh,m_leftLimit);
     vapp::FS::writeFloat_LE(pfh,m_rightLimit);
     vapp::FS::writeFloat_LE(pfh,m_topLimit);
     vapp::FS::writeFloat_LE(pfh,m_bottomLimit);
-      
+
+    vapp::FS::writeString(pfh,m_rSpriteForStrawberry);
+    vapp::FS::writeString(pfh,m_rSpriteForFlower);
+    vapp::FS::writeString(pfh,m_rSpriteForWecker);
+    vapp::FS::writeString(pfh,m_rSpriteForStar);
+    vapp::FS::writeString(pfh,m_rSoundForPickUpStrawberry);
+
+    vapp::FS::writeInt_LE(pfh, m_numberLayer);
+    for(int i=0; i<m_numberLayer; i++){
+      vapp::FS::writeFloat_LE(pfh, m_layerOffsets[i].x);
+      vapp::FS::writeFloat_LE(pfh, m_layerOffsets[i].y);
+      vapp::FS::writeBool(pfh,     m_isLayerFront[i]);
+    }    
+
     /* Write script (if any) */
     vapp::FS::writeInt_LE(pfh,m_scriptSource.length());
     vapp::FS::writeBuf(pfh,(char *)m_scriptSource.c_str(),m_scriptSource.length());
@@ -591,14 +880,9 @@ void Level::importBinaryHeader(vapp::FileHandle *pfh) {
   m_playerStart  = Vector2f(0.0, 0.0);
   m_xmotoTooOld  = false;
 
-  char cTag[5];
-  vapp::FS::readBuf(pfh,(char *)cTag,4);
-  cTag[4] = '\0';
-  int nFormat = 0;
-  if(!strcmp(cTag,"XBL5"))
-    nFormat = 5;
+  int nFormat = vapp::FS::readInt_LE(pfh);
   
-  if(nFormat != 5) {
+  if(nFormat != CACHE_LEVEL_FORMAT_VERSION) {
     throw Exception("Old file format");
   }
   
@@ -610,6 +894,37 @@ void Level::importBinaryHeader(vapp::FileHandle *pfh) {
   m_description = vapp::FS::readString(pfh);
   m_author 	= vapp::FS::readString(pfh);
   m_date   	= vapp::FS::readString(pfh);
+  m_music       = vapp::FS::readString(pfh);
+  m_isScripted  = vapp::FS::readBool(pfh);
+}
+
+void Level::importHeader(const std::string& i_id,
+			 const std::string& i_checkSum,
+			 const std::string& i_pack,
+			 const std::string& i_packNum,
+			 const std::string& i_name,
+			 const std::string& i_description,
+			 const std::string& i_author,
+			 const std::string& i_date,
+			 const std::string& i_music,
+			 bool i_isScripted
+			 ) {
+  unloadLevelBody();
+
+  m_isBodyLoaded = false;  
+  m_playerStart  = Vector2f(0.0, 0.0);
+  m_xmotoTooOld  = false;
+
+  m_id      	= i_id;
+  m_checkSum    = i_checkSum;
+  m_pack    	= i_pack;
+  m_packNum 	= i_packNum;
+  m_name    	= i_name;
+  m_description = i_description;
+  m_author 	= i_author;
+  m_date   	= i_date;
+  m_music       = i_music;
+  m_isScripted  = i_isScripted;
 }
 
   /*===========================================================================
@@ -646,7 +961,7 @@ void Level::exportBinaryHeader(vapp::FileHandle *pfh) {
   /* 4 -> includes level pack num */
   /* 5 -> clean code */
   /* Write CRC32 of XML */
-  vapp::FS::writeBuf   (pfh,"XBL5", 4);
+  vapp::FS::writeInt_LE(pfh, CACHE_LEVEL_FORMAT_VERSION);
   vapp::FS::writeString(pfh	    , m_checkSum);
   vapp::FS::writeString(pfh	    , m_id);
   vapp::FS::writeString(pfh	    , m_pack);
@@ -655,6 +970,8 @@ void Level::exportBinaryHeader(vapp::FileHandle *pfh) {
   vapp::FS::writeString(pfh	    , m_description);
   vapp::FS::writeString(pfh	    , m_author);
   vapp::FS::writeString(pfh	    , m_date);
+  vapp::FS::writeString(pfh	    , m_music);
+  vapp::FS::writeBool(  pfh	    , m_isScripted);
 }
 
 bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
@@ -672,14 +989,9 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
   }
   else {
     /* Read tag - it tells something about the format */
-    char cTag[5];
-    vapp::FS::readBuf(pfh,(char *)cTag,4);
-    cTag[4] = '\0';
-    int nFormat = 0;
-    if(!strcmp(cTag,"XBL5"))
-      nFormat = 5;
+    int nFormat = vapp::FS::readInt_LE(pfh);
     
-    if(nFormat == 5) { /* reject other formats */
+    if(nFormat == CACHE_LEVEL_FORMAT_VERSION) { /* reject other formats */
       /* Read "format 1" / "format 2" binary level */
       /* Right */
       std::string md5sum = vapp::FS::readString(pfh);
@@ -696,8 +1008,33 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
         m_description = vapp::FS::readString(pfh);
         m_author = vapp::FS::readString(pfh);
         m_date = vapp::FS::readString(pfh);
+        m_music = vapp::FS::readString(pfh);
+	m_isScripted = vapp::FS::readBool(pfh);
+
+	/* sky */
+        m_sky.setTexture(vapp::FS::readString(pfh));
+	m_sky.setZoom(vapp::FS::readFloat_LE(pfh));
+	m_sky.setOffset(vapp::FS::readFloat_LE(pfh));
+
+	int v_r, v_g, v_b, v_a;
+	v_r = vapp::FS::readInt_LE(pfh);
+	v_g = vapp::FS::readInt_LE(pfh);
+	v_b = vapp::FS::readInt_LE(pfh);
+	v_a = vapp::FS::readInt_LE(pfh);
+	m_sky.setTextureColor(TColor(v_r, v_g, v_b, v_a));
+
+	m_sky.setDrifted(vapp::FS::readBool(pfh));
+	m_sky.setDriftZoom(vapp::FS::readFloat_LE(pfh));
+
+	v_r = vapp::FS::readInt_LE(pfh);
+	v_g = vapp::FS::readInt_LE(pfh);
+	v_b = vapp::FS::readInt_LE(pfh);
+	v_a = vapp::FS::readInt_LE(pfh);
+	m_sky.setDriftTextureColor(TColor(v_r, v_g, v_b, v_a));
+	/* *** */
+
+        m_borderTexture = vapp::FS::readString(pfh);
 
-        m_sky = vapp::FS::readString(pfh);
         m_scriptFileName = vapp::FS::readString(pfh);
 
         m_leftLimit = vapp::FS::readFloat_LE(pfh);
@@ -705,6 +1042,23 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
         m_topLimit = vapp::FS::readFloat_LE(pfh);
         m_bottomLimit = vapp::FS::readFloat_LE(pfh);
 
+	/* theme replacements */
+	m_rSpriteForStrawberry 	    = vapp::FS::readString(pfh);
+	m_rSpriteForFlower     	    = vapp::FS::readString(pfh);
+	m_rSpriteForWecker     	    = vapp::FS::readString(pfh);
+	m_rSpriteForStar       	    = vapp::FS::readString(pfh);
+	m_rSoundForPickUpStrawberry = vapp::FS::readString(pfh);
+
+	/* layers */
+	m_numberLayer = vapp::FS::readInt_LE(pfh);
+	for(int i=0; i<m_numberLayer; i++){
+	  Vector2f offset;
+	  offset.x = vapp::FS::readFloat_LE(pfh);
+	  offset.y = vapp::FS::readFloat_LE(pfh);
+	  m_layerOffsets.push_back(offset);
+	  m_isLayerFront.push_back(vapp::FS::readBool(pfh));
+	}
+
         /* Read embedded script */
         int nScriptSourceLen = vapp::FS::readInt_LE(pfh);
         if(nScriptSourceLen > 0) {
@@ -845,15 +1199,13 @@ int Level::compareVersionNumbers(const std::string &v1,const std::string &v2) {
   return 0;    
 }
 
-int Level::loadToPlay(vapp::CollisionSystem& p_CollisionSystem) {
+int Level::loadToPlay() {
   int v_nbErrors = 0;
 
-  loadToPlayLimits();
-  
   /* preparing blocks */
   for(unsigned int i=0; i<m_blocks.size(); i++) {
     try {
-      v_nbErrors += m_blocks[i]->loadToPlay(p_CollisionSystem);
+      v_nbErrors += m_blocks[i]->loadToPlay(*m_pCollisionSystem);
     } catch(Exception &e) {
       throw Exception("Fail to load block '" + m_blocks[i]->Id() + "' :\n" + e.getMsg());
     }
@@ -862,6 +1214,13 @@ int Level::loadToPlay(vapp::CollisionSystem& p_CollisionSystem) {
   /* Spawn initial entities */
   for(unsigned int i=0; i<m_entities.size(); i++) {
     m_entities[i]->loadToPlay();
+    Vector2f v = m_entities[i]->DynamicPosition();
+
+    m_pCollisionSystem->addEntity(m_entities[i]);
+
+    if(m_entities[i]->IsToTake()){
+      m_nbEntitiesToTake++;
+    }
   }
   
   return v_nbErrors;
@@ -888,9 +1247,11 @@ void Level::unloadToPlay() {
     delete m_entitiesExterns[i];
   }
   m_entitiesExterns.clear();
+
+  m_nbEntitiesToTake = 0;
 }
 
-void Level::loadToPlayLimits() {
+void Level::addLimits() {
   Block *pBlock;
   Vector2f v_P;
 
@@ -898,56 +1259,62 @@ void Level::loadToPlayLimits() {
   float fVMargin = 20,fHMargin = 20;
 
   pBlock = new Block("LEVEL_TOP");
+  if(m_borderTexture != "") {
+    pBlock->setTexture(m_borderTexture);
+  }
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit()  - fHMargin, TopLimit() + fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit() + fHMargin, TopLimit() + fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit()           , TopLimit())));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit()            , TopLimit())));
+  /* move border block to the second static blocks layer */
+  pBlock->setIsLayer(true);
   Blocks().push_back(pBlock);
     
   pBlock = new Block("LEVEL_BOTTOM");
+  if(m_borderTexture != "") {
+    pBlock->setTexture(m_borderTexture);
+  }
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit()           , BottomLimit())));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit() + fHMargin, BottomLimit() - fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit()  - fHMargin, BottomLimit() - fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit()            , BottomLimit())));
+  pBlock->setIsLayer(true);
   Blocks().push_back(pBlock);
 
   pBlock = new Block("LEVEL_LEFT");
+  if(m_borderTexture != "") {
+    pBlock->setTexture(m_borderTexture);
+  }
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit(), TopLimit())));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit(), BottomLimit())));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit() - fHMargin, BottomLimit() - fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(LeftLimit() - fHMargin, TopLimit() + fVMargin))); 
+  pBlock->setIsLayer(true);
   Blocks().push_back(pBlock);
     
   pBlock = new Block("LEVEL_RIGHT");
+  if(m_borderTexture != "") {
+    pBlock->setTexture(m_borderTexture);
+  }
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit(), TopLimit())));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit() + fHMargin, TopLimit() + fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit() + fHMargin, BottomLimit() - fVMargin)));
   pBlock->Vertices().push_back(new BlockVertex(Vector2f(RightLimit(), BottomLimit())));
+  pBlock->setIsLayer(true);
   Blocks().push_back(pBlock);
 }
 
-void Level::clearAfterRewind() {
-  for(unsigned int i=0; i<m_entities.size(); i++) {
-    m_entities[i]->clearAfterRewind();
-  } 
-}
-
 void Level::spawnEntity(Entity *v_entity) {
   m_entitiesExterns.push_back(v_entity);
+  if(v_entity->IsToTake()){
+    m_nbEntitiesToTake++;
+  }
 }
 
 std::vector<Entity *>& Level::EntitiesExterns() {
   return m_entitiesExterns;
 }
 
-std::string Level::PathForUpdate() const {
-  /* If level path is not absolute, this level is not updatable */
-  if(vapp::FS::isPathAbsolute(FileName())) {
-    return FileName();
-  }
-  return "";
-}
-
 void Level::unloadLevelBody() {
   unloadToPlay();
 
@@ -974,4 +1341,47 @@ void Level::unloadLevelBody() {
   if(m_xmlSource != NULL) {
     delete m_xmlSource;
   }
+
+  m_numberLayer = 0;
+  m_layerOffsets.clear();
+  m_isLayerFront.clear();
+}
+
+void Level::rebuildCache() {
+  exportBinary(getNameInCache(), m_checkSum);
+}
+
+std::string Level::SpriteForStrawberry() const {
+  if(m_rSpriteForStrawberry != "") {
+    return m_rSpriteForStrawberry;
+  }
+  return "Strawberry";
+}
+
+std::string Level::SpriteForWecker() const {
+  if(m_rSpriteForWecker != "") {
+    return m_rSpriteForWecker;
+  }
+  return "Wrecker";
+}
+
+std::string Level::SpriteForFlower() const {
+  if(m_rSpriteForFlower != "") {
+    return m_rSpriteForFlower;
+  }
+  return "Flower";
+}
+
+std::string Level::SpriteForStar() const {
+  if(m_rSpriteForStar != "") {
+    return m_rSpriteForStar;
+  }
+  return "Star";
+}
+
+std::string Level::SoundForPickUpStrawberry() const {
+  if(m_rSoundForPickUpStrawberry != "") {
+    return m_rSoundForPickUpStrawberry;
+  }
+  return "PickUpStrawberry";
 }
