@@ -24,18 +24,19 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "../VApp.h"
 #include "../VXml.h"
-#include "../PhysSettings.h"
-#include "../MotoGame.h"
+#include "Scene.h"
 #include "Block.h"
 #include "Zone.h"
 #include "Entity.h"
 #include "BasicSceneStructs.h"
+#include "SkyApparence.h"
 
 class Block;
 class Entity;
 namespace vapp {
   class MotoGame;
 }
+class xmDatabase;
 
 /*===========================================================================
   Level source object - holds all stored information about a level
@@ -45,18 +46,29 @@ class Level {
   Level();
   ~Level();
 
-  bool loadReducedFromFile(bool i_cacheEnabled);
+  bool loadReducedFromFile();
   void loadFullyFromFile();
   bool isFullyLoaded() const;
   void exportBinaryHeader(vapp::FileHandle *pfh);
   void importBinaryHeader(vapp::FileHandle *pfh);
+  void importHeader(const std::string& i_id,
+		    const std::string& i_checkSum,
+		    const std::string& i_pack,
+		    const std::string& i_packNum,
+		    const std::string& i_name,
+		    const std::string& i_description,
+		    const std::string& i_author,
+		    const std::string& i_date,
+		    const std::string& i_music,
+		    bool i_isScripted);
+  void rebuildCache();
 
   void loadXML();
   void saveXML();
 
   /* load level so that it is possible to play */
   /* to replay a level, unload then, reload it */
-  int loadToPlay(vapp::CollisionSystem& p_CollisionSystem); /* return the number of errors found */
+  int loadToPlay(); /* return the number of errors found */
   void unloadToPlay();
 
   std::string Id()          const;
@@ -64,6 +76,7 @@ class Level {
   std::string Author()      const;
   std::string Date()        const;
   std::string Description() const;
+  std::string Music()       const;
   bool isXMotoTooOld()      const;
   std::string getRequiredVersion() const;
   std::string Pack()        const;
@@ -73,7 +86,7 @@ class Level {
   float TopLimit()          const;
   float BottomLimit()       const;
   Vector2f PlayerStart()    const;
-  std::string Sky()         const;
+  const SkyApparence& Sky()        const;
   void setLimits(float v_leftLimit, float v_rightLimit, float v_topLimit, float v_bottomLimit);
 
   std::string FileName() const;
@@ -91,7 +104,7 @@ class Level {
   void setDescription(const std::string& i_description);
   void setDate(const std::string& i_date);
   void setAuthor(const std::string& i_author);
-  void setSky(const std::string& i_sky);
+  void setCollisionSystem(vapp::CollisionSystem* p_CollisionSystem);
 
   std::string scriptFileName() const;
   std::string scriptSource() const;
@@ -115,23 +128,44 @@ class Level {
   /* because some objects like entities have an internal movement */
   void updateToTime(vapp::MotoGame& i_scene);
   /* this method calls objects because rewind in replay can required some actions (like removing particles) */
-  void clearAfterRewind();
 
   /* the entity will be destroyed by the level */
   void spawnEntity(Entity *v_entity);
   
-  std::string PathForUpdate() const;
+  std::string SpriteForStrawberry() const;
+  std::string SpriteForWecker() const;
+  std::string SpriteForFlower() const;
+  std::string SpriteForStar() const;
+  std::string SoundForPickUpStrawberry() const;
+
+  int getNumberLayer() {
+    return m_numberLayer;
+  }
+
+  Vector2f getLayerOffset(int layer) {
+    return m_layerOffsets[layer];
+  }
+
+  bool isLayerFront(int layer){
+    return m_isLayerFront[layer];
+  }
+
+  std::vector<Vector2f>& getLayerOffsets(){
+    return m_layerOffsets;
+  }
+
+  static void removeFromCache(xmDatabase *i_db, const std::string& i_id_level);
 
  private:
   std::string m_id;                 /* Level ID */
-  std::string m_name;             /* Name of level */
-  std::string m_author;           /* Author of level */
-  std::string m_date;             /* When it was crafted */
-  std::string m_description;      /* Description */
+  std::string m_name;               /* Name of level */
+  std::string m_author;             /* Author of level */
+  std::string m_date;               /* When it was crafted */
+  std::string m_description;        /* Description */
+  std::string m_music;              /* music */
   std::string m_requiredVersion;    /* Required X-Moto version */
   std::string m_pack;               /* In this level pack */
   std::string m_packNum;            /* value to sort levels */
-  std::string m_sky;                /* Level sky */
   std::string m_fileName;
   vapp::XMLDocument *m_xmlSource;    /* Plain XML source */      
   std::string m_scriptFileName;     /* Script file name */      
@@ -147,8 +181,25 @@ class Level {
   std::vector<Entity *> m_entitiesExterns;
   Entity               *m_startEntity; /* entity where the player start */
   bool m_isBodyLoaded;
+  vapp::CollisionSystem* m_pCollisionSystem;
+  /* to avoid calculate it each frame */
+  int  m_nbEntitiesToTake;
+  std::string m_borderTexture;
+  SkyApparence m_sky;
+  bool m_isScripted;
 
-  void loadToPlayLimits();
+  int m_numberLayer;
+  /* vector is the offset, and if bool == true, then it's a front layer*/
+  std::vector<Vector2f> m_layerOffsets;
+  std::vector<bool>     m_isLayerFront;
+
+  std::string m_rSpriteForStrawberry;
+  std::string m_rSpriteForWecker;
+  std::string m_rSpriteForFlower;
+  std::string m_rSpriteForStar;
+  std::string m_rSoundForPickUpStrawberry;
+
+  void addLimits();
   void exportBinary(const std::string &i_fileName, const std::string& i_sum);
   bool importBinary(const std::string &i_fileName, const std::string& i_sum);
   bool importBinaryHeaderFromFile(const std::string &i_fileName, const std::string& i_sum);
